@@ -20,194 +20,64 @@
 #ifndef _cspublic_h_
 #define _cspublic_h_
 
-#include "tds_sysdep_public.h"
+#include <cstypes.h>
 
+#undef TDS_STATIC_CAST
 #ifdef __cplusplus
+#define TDS_STATIC_CAST(type, a) static_cast<type>(a)
 extern "C"
 {
 #if 0
 }
 #endif
+#else
+#define TDS_STATIC_CAST(type, a) ((type)(a))
 #endif
 
-static const char rcsid_cspublic_h[] = "$Id: cspublic.h,v 1.53 2004/10/28 12:42:11 freddy77 Exp $";
+static const char rcsid_cspublic_h[] = "$Id: cspublic.h,v 1.58 2007/06/25 09:48:20 freddy77 Exp $";
 static const void *const no_unused_cspublic_h_warn[] = { rcsid_cspublic_h, no_unused_cspublic_h_warn };
 
-typedef int CS_RETCODE;
-
 #define CS_PUBLIC
 #define CS_STATIC static
 
-typedef tds_sysdep_int32_type CS_INT;
-typedef unsigned tds_sysdep_int32_type CS_UINT;
-typedef tds_sysdep_int16_type CS_SMALLINT;
-typedef unsigned char CS_TINYINT;
-typedef char CS_CHAR;
-typedef unsigned char CS_BYTE;
-typedef struct _cs_numeric
-{
-	unsigned char precision;
-	unsigned char scale;
-	unsigned char array[33];
-} CS_NUMERIC;
-typedef tds_sysdep_real32_type CS_REAL;
-typedef tds_sysdep_real64_type CS_FLOAT;
-typedef char CS_BOOL;
-typedef void CS_VOID;
-typedef struct _cs_varbinary
-{
-	CS_INT len;
-	CS_CHAR array[256];
-} CS_VARBINARY;
-typedef CS_NUMERIC CS_DECIMAL;
-typedef unsigned char CS_IMAGE;
-typedef unsigned char CS_TEXT;
-typedef unsigned char CS_LONGBINARY;
-typedef unsigned char CS_LONGCHAR;
-typedef tds_sysdep_int32_type CS_LONG;
-typedef unsigned char CS_BINARY;
-typedef unsigned tds_sysdep_int16_type CS_USHORT;
-typedef unsigned char CS_BIT;
+#define CS_SUCCEED		1
+#define CS_FAIL			0
+#define CS_MEM_ERROR		-1
+#define CS_PENDING 		-2
+#define CS_QUIET 		-3
+#define CS_BUSY			-4
+#define CS_INTERRUPT 		-5
+#define CS_BLK_HAS_TEXT		-6
+#define CS_CONTINUE		-7
+#define CS_FATAL		-8
+#define CS_RET_HAFAILOVER	-9
+#define CS_UNSUPPORTED		-10
+
+#define CS_CANCELED	-202
+#define CS_ROW_FAIL	-203
+#define CS_END_DATA	-204
+#define CS_END_RESULTS	-205
+#define CS_END_ITEM	-206
+#define CS_NOMSG	-207
+#define CS_TIMED_OUT 	-208
 
-#define CS_FAIL	   0
-#define CS_SUCCEED 1
 #define CS_SIZEOF(x) sizeof(x)
-#define CS_NOMSG   -99
 
 #define CS_LAYER(x)    (((x) >> 24) & 0xFF)
 #define CS_ORIGIN(x)   (((x) >> 16) & 0xFF)
 #define CS_SEVERITY(x) (((x) >>  8) & 0xFF)
 #define CS_NUMBER(x)   ((x) & 0xFF)
 
-#define CS_OBJ_NAME 400		/* full name length */
-#define CS_TP_SIZE  16		/* text pointer */
-#define CS_TS_SIZE  8		/* length of timestamp */
-
 /* forward declarations */
-typedef struct _cs_config CS_CONFIG;
-typedef struct _cs_context CS_CONTEXT;
-typedef struct _cs_clientmsg CS_CLIENTMSG;
-typedef struct _cs_connection CS_CONNECTION;
-typedef struct _cs_servermsg CS_SERVERMSG;
-typedef struct _cs_locale CS_LOCALE;
-typedef struct _cs_command CS_COMMAND;
 typedef CS_RETCODE(*CS_CSLIBMSG_FUNC) (CS_CONTEXT *, CS_CLIENTMSG *);
 typedef CS_RETCODE(*CS_CLIENTMSG_FUNC) (CS_CONTEXT *, CS_CONNECTION *, CS_CLIENTMSG *);
 typedef CS_RETCODE(*CS_SERVERMSG_FUNC) (CS_CONTEXT *, CS_CONNECTION *, CS_SERVERMSG *);
 
 
-#define CS_IODATA          (CS_INT)1600
-
-typedef struct _cs_iodesc
-{
-	CS_INT iotype;
-	CS_INT datatype;
-	CS_LOCALE *locale;
-	CS_INT usertype;
-	CS_INT total_txtlen;
-	CS_INT offset;
-	CS_BOOL log_on_update;
-	CS_CHAR name[CS_OBJ_NAME];
-	CS_INT namelen;
-	CS_BYTE timestamp[CS_TS_SIZE];
-	CS_INT timestamplen;
-	CS_BYTE textptr[CS_TP_SIZE];
-	CS_INT textptrlen;
-} CS_IODESC;
-
-#define CS_MAX_MSG 1024
-#define CS_MAX_NAME 132
-#define CS_MAX_SCALE 77
-#define CS_MAX_PREC 77		/* used by php */
-#define CS_SQLSTATE_SIZE 8
-
-
+#define CS_IODATA          TDS_STATIC_CAST(CS_INT, 1600)
 #define CS_SRC_VALUE   -2562
 
-typedef struct _cs_datafmt
-{
-	CS_CHAR name[CS_MAX_NAME];
-	CS_INT namelen;
-	CS_INT datatype;
-	CS_INT format;
-	CS_INT maxlength;
-	CS_INT scale;
-	CS_INT precision;
-	CS_INT status;
-	CS_INT count;
-	CS_INT usertype;
-	CS_LOCALE *locale;
-} CS_DATAFMT;
 
-typedef struct _cs_money
-{
-	CS_INT mnyhigh;
-	CS_UINT mnylow;
-} CS_MONEY;
-
-typedef struct _cs_money4
-{
-	CS_INT mny4;
-} CS_MONEY4;
-
-typedef struct _cs_datetime
-{
-	CS_INT dtdays;
-	CS_INT dttime;
-} CS_DATETIME;
-
-typedef struct _cs_datetime4
-{
-	CS_USHORT days;
-	CS_USHORT minutes;
-} CS_DATETIME4;
-
-typedef struct _cs_daterec
-{
-	CS_INT dateyear;
-	CS_INT datemonth;
-	CS_INT datedmonth;
-	CS_INT datedyear;
-	CS_INT datedweek;
-	CS_INT datehour;
-	CS_INT dateminute;
-	CS_INT datesecond;
-	CS_INT datemsecond;
-	CS_INT datetzone;
-} CS_DATEREC;
-
-typedef CS_INT CS_MSGNUM;
-
-struct _cs_clientmsg
-{
-	CS_INT severity;
-	CS_MSGNUM msgnumber;
-	CS_CHAR msgstring[CS_MAX_MSG];
-	CS_INT msgstringlen;
-	CS_INT osnumber;
-	CS_CHAR osstring[CS_MAX_MSG];
-	CS_INT osstringlen;
-	CS_INT status;
-	CS_BYTE sqlstate[CS_SQLSTATE_SIZE];
-	CS_INT sqlstatelen;
-};
-
-struct _cs_servermsg
-{
-	CS_MSGNUM msgnumber;
-	CS_INT state;
-	CS_INT severity;
-	CS_CHAR text[CS_MAX_MSG];
-	CS_INT textlen;
-	CS_CHAR svrname[CS_MAX_NAME];
-	CS_INT svrnlen;
-	CS_CHAR proc[CS_MAX_NAME];
-	CS_INT proclen;
-	CS_INT line;
-	CS_INT status;
-	CS_BYTE sqlstate[CS_SQLSTATE_SIZE];
-	CS_INT sqlstatelen;
-};
 
 /* status bits for CS_SERVERMSG */
 #define CS_HASEED 0x01
@@ -265,9 +135,9 @@ typedef struct _cs_blkdesc CS_BLKDESC;
 #define CS_PROTO_DYNPROC	48
 #define CS_DATA_FLTN	49
 #define CS_DATA_BITN	50
-#define CS_DATA_INT8	51
-#define CS_DATA_VOID	52
-#define CS_OPTION_GET	53
+#define CS_OPTION_GET	51
+#define CS_DATA_INT8	52
+#define CS_DATA_VOID	53
 
 /* CS_CAP_RESPONSE values */
 #define CS_RES_NOMSG	1
@@ -302,54 +172,88 @@ typedef struct _cs_blkdesc CS_BLKDESC;
 #define CS_PROTO_NOBULK	30
 #define CS_DATA_NOSENSITIVITY	31
 #define CS_DATA_NOBOUNDARY	32
-#define CS_DATA_NOTDSDEBUG	33
+#define CS_RES_NOTDSDEBUG	33
 #define CS_RES_NOSTRIPBLANKS	34
 #define CS_DATA_NOINT8	35
 
 /* Properties */
 enum
 {
-	CS_USERNAME = 1,
 /*
  * These defines looks weird but programs can test support for defines,
  * compiler can check enum and there are no define side effecs
  */
+	CS_USERNAME = 9100,
 #define CS_USERNAME CS_USERNAME
-	CS_PASSWORD,
+	CS_PASSWORD = 9101,
 #define CS_PASSWORD CS_PASSWORD
-	CS_APPNAME,
+	CS_APPNAME = 9102,
 #define CS_APPNAME CS_APPNAME
-	CS_HOSTNAME,
+	CS_HOSTNAME = 9103,
 #define CS_HOSTNAME CS_HOSTNAME
-	CS_PACKETSIZE,
-#define CS_PACKETSIZE CS_PACKETSIZE
-	CS_SEC_ENCRYPTION,
-#define CS_SEC_ENCRYPTION CS_SEC_ENCRYPTION
-	CS_LOC_PROP,
-#define CS_LOC_PROP CS_LOC_PROP
-	CS_SEC_CHALLENGE,
-#define CS_SEC_CHALLENGE CS_SEC_CHALLENGE
-	CS_SEC_NEGOTIATE,
-#define CS_SEC_NEGOTIATE CS_SEC_NEGOTIATE
-	CS_TDS_VERSION,
+	CS_LOGIN_STATUS = 9104,
+#define CS_LOGIN_STATUS CS_LOGIN_STATUS
+	CS_TDS_VERSION = 9105,
 #define CS_TDS_VERSION CS_TDS_VERSION
-	CS_NETIO,
-#define CS_NETIO CS_NETIO
-	CS_IFILE,
-#define CS_IFILE CS_IFILE
-	CS_USERDATA,
-#define CS_USERDATA CS_USERDATA
-	CS_SEC_APPDEFINED,
-#define CS_SEC_APPDEFINED CS_SEC_APPDEFINED
-	CS_CHARSETCNV,
+	CS_CHARSETCNV = 9106,
 #define CS_CHARSETCNV CS_CHARSETCNV
-	CS_ANSI_BINDS,
+	CS_PACKETSIZE = 9107,
+#define CS_PACKETSIZE CS_PACKETSIZE
+	CS_USERDATA = 9108,
+#define CS_USERDATA CS_USERDATA
+	CS_NETIO = 9110,
+#define CS_NETIO CS_NETIO
+	CS_TEXTLIMIT = 9112,
+#define CS_TEXTLIMIT CS_TEXTLIMIT
+	CS_HIDDEN_KEYS = 9113,
+#define CS_HIDDEN_KEYS CS_HIDDEN_KEYS
+	CS_VERSION = 9114,
+#define CS_VERSION CS_VERSION
+	CS_IFILE = 9115,
+#define CS_IFILE CS_IFILE
+	CS_LOGIN_TIMEOUT = 9116,
+#define CS_LOGIN_TIMEOUT CS_LOGIN_TIMEOUT
+	CS_TIMEOUT = 9117,
+#define CS_TIMEOUT CS_TIMEOUT
+	CS_MAX_CONNECT = 9118,
+#define CS_MAX_CONNECT CS_MAX_CONNECT
+	CS_EXPOSE_FMTS = 9120,
+#define CS_EXPOSE_FMTS CS_EXPOSE_FMTS
+	CS_EXTRA_INF = 9121,
+#define CS_EXTRA_INF CS_EXTRA_INF
+	CS_ANSI_BINDS = 9123,
 #define CS_ANSI_BINDS CS_ANSI_BINDS
-	CS_VER_STRING,
+	CS_BULK_LOGIN = 9124,
+#define CS_BULK_LOGIN CS_BULK_LOGIN
+	CS_LOC_PROP = 9125,
+#define CS_LOC_PROP CS_LOC_PROP
+	CS_PARENT_HANDLE = 9130,
+#define CS_PARENT_HANDLE CS_PARENT_HANDLE
+	CS_EED_CMD = 9131,
+#define CS_EED_CMD CS_EED_CMD
+	CS_DIAG_TIMEOUT = 9132,
+#define CS_DIAG_TIMEOUT CS_DIAG_TIMEOUT
+	CS_DISABLE_POLL = 9133,
+#define CS_DISABLE_POLL CS_DISABLE_POLL
+	CS_SEC_ENCRYPTION = 9135,
+#define CS_SEC_ENCRYPTION CS_SEC_ENCRYPTION
+	CS_SEC_CHALLENGE = 9136,
+#define CS_SEC_CHALLENGE CS_SEC_CHALLENGE
+	CS_SEC_NEGOTIATE = 9137,
+#define CS_SEC_NEGOTIATE CS_SEC_NEGOTIATE
+	CS_CON_STATUS = 9143,
+#define CS_CON_STATUS CS_CON_STATUS
+	CS_VER_STRING = 9144,
 #define CS_VER_STRING CS_VER_STRING
-	CS_SERVERNAME,
+	CS_SERVERNAME = 9146,
 #define CS_SERVERNAME CS_SERVERNAME
-	CS_PORT
+	CS_SEC_APPDEFINED = 9149,
+#define CS_SEC_APPDEFINED CS_SEC_APPDEFINED
+	CS_STICKY_BINDS = 9151,
+#define CS_STICKY_BINDS CS_STICKY_BINDS
+	CS_SERVERADDR = 9206,
+#define CS_SERVERADDR CS_SERVERADDR
+	CS_PORT = 9300
 #define CS_PORT CS_PORT
 };
 
@@ -364,7 +268,7 @@ enum
 
 enum
 {
-	CS_TDS_40 = 1,
+	CS_TDS_40 = 7360,
 	CS_TDS_42,
 	CS_TDS_46,
 	CS_TDS_495,
@@ -374,20 +278,26 @@ enum
 };
 
 /* bit mask values used by CS_DATAFMT.status */
-#define CS_CANBENULL   (1)
-#define CS_HIDDEN      (1 << 1)
-#define CS_IDENTITY    (1 << 2)
-#define CS_KEY         (1 << 3)
-#define CS_VERSION_KEY (1 << 4)
-#define CS_TIMESTAMP   (1 << 5)
-#define CS_UPDATABLE   (1 << 6)
-#define CS_UPDATECOL   (1 << 7)
-#define CS_RETURN      (1 << 8)
+#define CS_HIDDEN      (1 <<  0)
+#define CS_KEY         (1 <<  1)
+#define CS_VERSION_KEY (1 <<  2)
+#define CS_NODATA      (1 <<  3)
+#define CS_UPDATABLE   (1 <<  4)
+#define CS_CANBENULL   (1 <<  5)
+#define CS_DESCIN      (1 <<  6)
+#define CS_DESCOUT     (1 <<  7)
+#define CS_INPUTVALUE  (1 <<  8)
+#define CS_UPDATECOL   (1 <<  9)
+#define CS_RETURN      (1 << 10)
+#define CS_TIMESTAMP   (1 << 13)
+#define CS_NODEFAULT   (1 << 14)
+#define CS_IDENTITY    (1 << 15)
 
 /*
  * DBD::Sybase compares indicator to CS_NULLDATA so this is -1
  * (the documentation states -1)
  */
+#define CS_GOODDATA	0
 #define CS_NULLDATA	(-1)
 
 /* CS_CON_STATUS read-only property bit mask values */
@@ -396,63 +306,81 @@ enum
 
 /*
  * Code added for CURSOR support
- * types accepted by ct_cursor 
+ * types accepted by ct_cursor
  */
-#define CS_CURSOR_DECLARE  1
-#define CS_CURSOR_OPEN     2
-#define CS_CURSOR_ROWS     3
-#define CS_CURSOR_OPTION   4
-#define CS_CURSOR_UPDATE   5
-#define CS_CURSOR_DELETE   6
-#define CS_CURSOR_DEALLOC  7
-#define CS_IMPLICIT_CURSOR 8
-#define CS_CURSOR_CLOSE    9
+#define CS_CURSOR_DECLARE  700
+#define CS_CURSOR_OPEN     701
+#define CS_CURSOR_ROWS     703
+#define CS_CURSOR_UPDATE   704
+#define CS_CURSOR_DELETE   705
+#define CS_CURSOR_CLOSE    706
+#define CS_CURSOR_DEALLOC  707
+#define CS_CURSOR_OPTION   725
+
+#define CS_FOR_UPDATE      TDS_STATIC_CAST(CS_INT, 0x1)
+#define CS_READ_ONLY       TDS_STATIC_CAST(CS_INT, 0x2)
+#define CS_RESTORE_OPEN    TDS_STATIC_CAST(CS_INT, 0x8)
+#define CS_IMPLICIT_CURSOR TDS_STATIC_CAST(CS_INT, 0x40)
+
+
+#define CS_CURSTAT_NONE      TDS_STATIC_CAST(CS_INT, 0x0)
+#define CS_CURSTAT_DECLARED  TDS_STATIC_CAST(CS_INT, 0x1)
+#define CS_CURSTAT_OPEN      TDS_STATIC_CAST(CS_INT, 0x2)
+#define CS_CURSTAT_CLOSED    TDS_STATIC_CAST(CS_INT, 0x4)
+#define CS_CURSTAT_RDONLY    TDS_STATIC_CAST(CS_INT, 0x8)
+#define CS_CURSTAT_UPDATABLE TDS_STATIC_CAST(CS_INT, 0x10)
+#define CS_CURSTAT_ROWCOUNT  TDS_STATIC_CAST(CS_INT, 0x20)
+#define CS_CURSTAT_DEALLOC   TDS_STATIC_CAST(CS_INT, 0x40)
+
+#define CS_CUR_STATUS        TDS_STATIC_CAST(CS_INT, 9126)
+#define CS_CUR_ID            TDS_STATIC_CAST(CS_INT, 9127)
+#define CS_CUR_NAME          TDS_STATIC_CAST(CS_INT, 9128)
+#define CS_CUR_ROWCOUNT      TDS_STATIC_CAST(CS_INT, 9129)
 
 /* options accepted by ct_options() */
-#define CS_OPT_ANSINULL		1
-#define CS_OPT_ANSIPERM		2
-#define CS_OPT_ARITHABORT	3
-#define CS_OPT_ARITHIGNORE	4
-#define CS_OPT_AUTHOFF		5
-#define CS_OPT_AUTHON		6
-#define CS_OPT_CHAINXACTS	7
-#define CS_OPT_CURCLOSEONXACT	8
-#define CS_OPT_CURREAD		9
-#define CS_OPT_CURWRITE		10
-#define CS_OPT_DATEFIRST	11
-#define CS_OPT_DATEFORMAT	12
-#define CS_OPT_FIPSFLAG		13
-#define CS_OPT_FORCEPLAN	14
-#define CS_OPT_FORMATONLY	15
-#define CS_OPT_GETDATA		16
-#define CS_OPT_IDENTITYOFF	17
-#define CS_OPT_IDENTITYON	18
-#define CS_OPT_ISOLATION	19
-#define CS_OPT_NOCOUNT		20
-#define CS_OPT_NOEXEC		21
-#define CS_OPT_PARSEONLY	22
-#define CS_OPT_QUOTED_IDENT	23
-#define CS_OPT_RESTREES		24
-#define CS_OPT_ROWCOUNT		25
-#define CS_OPT_SHOWPLAN		26
-#define CS_OPT_STATS_IO		27
-#define CS_OPT_STATS_TIME	28
-#define CS_OPT_STR_RTRUNC	29
-#define CS_OPT_TEXTSIZE		30
-#define CS_OPT_TRUNCIGNORE	31
+#define CS_OPT_DATEFIRST	5001
+#define CS_OPT_TEXTSIZE		5002
+#define CS_OPT_STATS_TIME	5003
+#define CS_OPT_STATS_IO		5004
+#define CS_OPT_ROWCOUNT		5005
+#define CS_OPT_DATEFORMAT	5007
+#define CS_OPT_ISOLATION	5008
+#define CS_OPT_AUTHON		5009
+#define CS_OPT_SHOWPLAN		5013
+#define CS_OPT_NOEXEC		5014
+#define CS_OPT_ARITHIGNORE	5015
+#define CS_OPT_TRUNCIGNORE	5016
+#define CS_OPT_ARITHABORT	5017
+#define CS_OPT_PARSEONLY	5018
+#define CS_OPT_GETDATA		5020
+#define CS_OPT_NOCOUNT		5021
+#define CS_OPT_FORCEPLAN	5023
+#define CS_OPT_FORMATONLY	5024
+#define CS_OPT_CHAINXACTS	5025
+#define CS_OPT_CURCLOSEONXACT	5026
+#define CS_OPT_FIPSFLAG		5027
+#define CS_OPT_RESTREES		5028
+#define CS_OPT_IDENTITYON	5029
+#define CS_OPT_CURREAD		5030
+#define CS_OPT_CURWRITE		5031
+#define CS_OPT_IDENTITYOFF	5032
+#define CS_OPT_AUTHOFF		5033
+#define CS_OPT_ANSINULL		5034
+#define CS_OPT_QUOTED_IDENT	5035
+#define CS_OPT_ANSIPERM		5036
+#define CS_OPT_STR_RTRUNC	5037
 
 /* options accepted by ct_command() */
 enum ct_command_options
 {
-	CS_MORE,
-	CS_END,
-	CS_UNUSED,
-	CS_RECOMPILE,
+	CS_MORE = 16,
+	CS_END = 32,
+	CS_RECOMPILE = 188,
 	CS_NO_RECOMPILE,
-	CS_COLUMN_DATA,
-	CS_BULK_DATA,
 	CS_BULK_INIT,
-	CS_BULK_CONT
+	CS_BULK_CONT,
+	CS_BULK_DATA,
+	CS_COLUMN_DATA
 };
 
 
@@ -468,8 +396,10 @@ enum
 #define CS_FMT_NULLTERM CS_FMT_NULLTERM
 	CS_FMT_PADNULL = 2,
 #define CS_FMT_PADBLANK CS_FMT_PADBLANK
-	CS_FMT_PADBLANK = 4
+	CS_FMT_PADBLANK = 4,
 #define CS_FMT_PADNULL CS_FMT_PADNULL
+	CS_FMT_JUSTIFY_RT = 8
+#define CS_FMT_JUSTIFY_RT CS_FMT_JUSTIFY_RT
 };
 
 /* callbacks */
@@ -491,94 +421,111 @@ enum
 #define CS_UNUSED	-99999
 
 /* other */
-#define CS_CLEAR	3
-#define CS_SET		4
-#define CS_LANG_CMD	7
-#define CS_ROW_FAIL	9
-#define CS_END_DATA	10
-#define CS_END_ITEM 11
+#define CS_GET		33
+#define CS_SET		34
+#define CS_CLEAR	35
+#define CS_INIT         36
+#define CS_STATUS       37
+#define CS_MSGLIMIT     38
+#define CS_SUPPORTED    40
+
+#define CS_CMD_DONE	4046
 #define CS_CMD_SUCCEED	4047
 #define CS_CMD_FAIL	4048
-#define CS_CMD_DONE	4046
-#define CS_END_RESULTS	15
-#define CS_VERSION_100	16
-#define CS_FORCE_EXIT	17
-#define CS_GET		25
-#define CS_CON_STATUS 26
-#define CS_FORCE_CLOSE 27
-#define CS_SYNC_IO	29
-#define CS_LC_ALL	37
-#define CS_SYB_LANG	38
-#define CS_SYB_CHARSET	39
-#define CS_SV_COMM_FAIL	41
-#define CS_BULK_LOGIN	42
+
+/* commands */
+#define CS_LANG_CMD	 148
+#define CS_RPC_CMD	 149
+#define CS_SEND_DATA_CMD 151
+#define CS_SEND_BULK_CMD 153
+
+#define CS_VERSION_100	112
+#define CS_VERSION_110	1100
+#define CS_VERSION_120	1100
+#define CS_VERSION_125	12500
+#define CS_VERSION_150	15000
+
 #define BLK_VERSION_100 CS_VERSION_100
-#define CS_BLK_IN	43
-#define CS_BLK_OUT	44
-#define CS_BLK_BATCH	45
-#define CS_BLK_ALL	46
-#define CS_BLK_CANCEL	47
-#define CS_CANCEL_ALL	48
-#define CS_NUMDATA	49
-#define CS_CANCEL_ATTN	50
-#define CS_PARENT_HANDLE	51
-#define CS_COMP_ID	52
-#define CS_BYLIST_LEN	53
-#define CS_COMP_BYLIST	54
-#define CS_COMP_OP	55
-#define CS_COMP_COLID	56
+#define BLK_VERSION_110 CS_VERSION_100
+
+#define CS_FORCE_EXIT	300
+#define CS_FORCE_CLOSE  301
+
+#define CS_SYNC_IO	8111
+#define CS_ASYNC_IO	8112
+#define CS_DEFER_IO	8113
+
+#define CS_CANCEL_CURRENT 6000
+#define CS_CANCEL_ALL	  6001
+#define CS_CANCEL_ATTN	  6002
+
+#define CS_ROW_COUNT	800
+#define CS_CMD_NUMBER	801
+#define CS_NUM_COMPUTES	802
+#define CS_NUMDATA	803
+#define CS_NUMORDERCOLS	805
+#define CS_MSGTYPE      806
+#define CS_BROWSE_INFO	807
+#define CS_TRANS_STATE	808
+
+#define CS_TRAN_UNDEFINED   0
+#define CS_TRAN_IN_PROGRESS 1
+#define CS_TRAN_COMPLETED   2
+#define CS_TRAN_FAIL        3
+#define CS_TRAN_STMT_FAIL   4
+
+#define CS_COMP_OP	5350
+#define CS_COMP_ID	5351
+#define CS_COMP_COLID	5352
+#define CS_COMP_BYLIST	5353
+#define CS_BYLIST_LEN	5354
+
 #define CS_NO_COUNT	-1
-#define CS_ROW_COUNT	59
-#define CS_OP_SUM	60
-#define CS_OP_AVG	61
-#define CS_OP_MIN	62
-#define CS_OP_MAX	63
-#define CS_OP_COUNT	64
-#define CS_CANCEL_CURRENT	67
-#define CS_CAPREQUEST	73
-#define CS_EED_CMD	77
-#define CS_LOGIN_TIMEOUT	78
-#define CS_CAP_REQUEST	79
-#define CS_DESCRIBE_INPUT	80
-#define CS_PREPARE	81
-#define CS_EXECUTE	82
-#define CS_DEALLOC	83
-#define CS_CAP_RESPONSE	84
-#define CS_RPC_CMD	85
-/* need correct value for CS_SEND_BULK_CMD  */
-#define CS_SEND_BULK_CMD 0xFFFF
-#define CS_INPUTVALUE	86
-#define CS_GOODDATA	87
-/* define CS_RETURN	88 */
-#define CS_CMD_NUMBER	89
-#define CS_BROWSE_INFO	90
-#define CS_NUMORDERCOLS	91
-#define CS_NUM_COMPUTES	92
-#define CS_NODATA	96
-#define CS_DESCIN	98
-#define CS_DESCOUT	99
-/* define CS_UPDATECOL	100 */
-#define CS_NODEFAULT	102
-#define CS_FMT_JUSTIFY_RT	106
-#define CS_TRANS_STATE	107
-#define CS_TRAN_IN_PROGRESS	108
-#define CS_TRAN_COMPLETED	109
-#define CS_TRAN_STMT_FAIL	110
-#define CS_TRAN_FAIL	111
-#define CS_TRAN_UNDEFINED	112
-#define CS_SV_RETRY_FAIL	114
-#define CS_TIMEOUT	115
-#define CS_CANCELED 	116
-#define CS_NO_RECOMPILE	117
-#define CS_COLUMN_DATA	118
-#define CS_SEND_DATA_CMD	119
-#define CS_SUPPORTED 120
-#define CS_EXPOSE_FMTS 121
-#define CS_DYNAMIC_CMD 122
-#define CS_MSGTYPE 123
-#define CS_VERSION	9114
-#define CS_EXTRA_INF	9121
-#define CS_CUR_CMD 133
+
+#define CS_OP_SUM	5370
+#define CS_OP_AVG	5371
+#define CS_OP_COUNT	5372
+#define CS_OP_MIN	5373
+#define CS_OP_MAX	5374
+
+#define CS_CAP_REQUEST	1
+#define CS_CAP_RESPONSE	2
+
+#define CS_PREPARE	717
+#define CS_EXECUTE	718
+#define CS_DESCRIBE_INPUT	720
+#define CS_DESCRIBE_OUTPUT	721
+
+#define CS_DEALLOC	711
+
+#define CS_LC_ALL	     7
+#define CS_SYB_LANG	     8
+#define CS_SYB_CHARSET	     9
+#define CS_SYB_SORTORDER     10
+#define CS_SYB_COLLATE CS_SYB_SORTORDER
+#define CS_SYB_LANG_CHARSET  11
+
+#define CS_BLK_IN	1
+#define CS_BLK_OUT	2
+
+#define CS_BLK_BATCH	1
+#define CS_BLK_ALL	2
+#define CS_BLK_CANCEL	3
+
+/* to do support these */
+
+#define CS_BLK_ARRAY_MAXLEN 0x1000
+#define CS_DEF_PREC     18
+
+/* Error Severities  */
+#define CS_SV_INFORM        TDS_STATIC_CAST(CS_INT, 0)
+#define CS_SV_API_FAIL      TDS_STATIC_CAST(CS_INT, 1)
+#define CS_SV_RETRY_FAIL    TDS_STATIC_CAST(CS_INT, 2)
+#define CS_SV_RESOURCE_FAIL TDS_STATIC_CAST(CS_INT, 3)
+#define CS_SV_CONFIG_FAIL   TDS_STATIC_CAST(CS_INT, 4)
+#define CS_SV_COMM_FAIL     TDS_STATIC_CAST(CS_INT, 5)
+#define CS_SV_INTERNAL_FAIL TDS_STATIC_CAST(CS_INT, 6)
+#define CS_SV_FATAL         TDS_STATIC_CAST(CS_INT, 7)
 
 /* result_types */
 #define CS_COMPUTE_RESULT	4045
@@ -592,39 +539,49 @@ enum
 #define CS_DESCRIBE_RESULT	4051
 
 /* bind types */
-#define CS_CHAR_TYPE	1
-#define CS_INT_TYPE	2
-#define CS_SMALLINT_TYPE	3
-#define CS_TINYINT_TYPE	4
-#define CS_MONEY_TYPE	5
-#define CS_DATETIME_TYPE	6
-#define CS_NUMERIC_TYPE	7
-#define CS_DECIMAL_TYPE	8
-#define CS_DATETIME4_TYPE	9
-#define CS_MONEY4_TYPE	10
-#define CS_IMAGE_TYPE	11
-#define CS_BINARY_TYPE	12
-#define CS_BIT_TYPE	13
-#define CS_REAL_TYPE	14
-#define CS_FLOAT_TYPE	15
-#define CS_TEXT_TYPE	16
-#define CS_VARCHAR_TYPE	17
-#define CS_VARBINARY_TYPE	18
-#define CS_LONGCHAR_TYPE	19
-#define CS_LONGBINARY_TYPE	20
-#define CS_LONG_TYPE	21
-#define CS_ILLEGAL_TYPE	22
-#define CS_SENSITIVITY_TYPE	23
-#define CS_BOUNDARY_TYPE	24
-#define CS_VOID_TYPE	25
-#define CS_USHORT_TYPE	26
-#define CS_UNIQUE_TYPE	27
-#define CS_UNICHAR_TYPE  28
+#define CS_ILLEGAL_TYPE     TDS_STATIC_CAST(CS_INT, -1)
+#define CS_CHAR_TYPE        TDS_STATIC_CAST(CS_INT, 0)
+#define CS_BINARY_TYPE      TDS_STATIC_CAST(CS_INT, 1)
+#define CS_LONGCHAR_TYPE    TDS_STATIC_CAST(CS_INT, 2)
+#define CS_LONGBINARY_TYPE  TDS_STATIC_CAST(CS_INT, 3)
+#define CS_TEXT_TYPE        TDS_STATIC_CAST(CS_INT, 4)
+#define CS_IMAGE_TYPE       TDS_STATIC_CAST(CS_INT, 5)
+#define CS_TINYINT_TYPE     TDS_STATIC_CAST(CS_INT, 6)
+#define CS_SMALLINT_TYPE    TDS_STATIC_CAST(CS_INT, 7)
+#define CS_INT_TYPE         TDS_STATIC_CAST(CS_INT, 8)
+#define CS_REAL_TYPE        TDS_STATIC_CAST(CS_INT, 9)
+#define CS_FLOAT_TYPE       TDS_STATIC_CAST(CS_INT, 10)
+#define CS_BIT_TYPE         TDS_STATIC_CAST(CS_INT, 11)
+#define CS_DATETIME_TYPE    TDS_STATIC_CAST(CS_INT, 12)
+#define CS_DATETIME4_TYPE   TDS_STATIC_CAST(CS_INT, 13)
+#define CS_MONEY_TYPE       TDS_STATIC_CAST(CS_INT, 14)
+#define CS_MONEY4_TYPE      TDS_STATIC_CAST(CS_INT, 15)
+#define CS_NUMERIC_TYPE     TDS_STATIC_CAST(CS_INT, 16)
+#define CS_DECIMAL_TYPE     TDS_STATIC_CAST(CS_INT, 17)
+#define CS_VARCHAR_TYPE     TDS_STATIC_CAST(CS_INT, 18)
+#define CS_VARBINARY_TYPE   TDS_STATIC_CAST(CS_INT, 19)
+#define CS_LONG_TYPE        TDS_STATIC_CAST(CS_INT, 20)
+#define CS_SENSITIVITY_TYPE TDS_STATIC_CAST(CS_INT, 21)
+#define CS_BOUNDARY_TYPE    TDS_STATIC_CAST(CS_INT, 22)
+#define CS_VOID_TYPE        TDS_STATIC_CAST(CS_INT, 23)
+#define CS_USHORT_TYPE      TDS_STATIC_CAST(CS_INT, 24)
+#define CS_UNICHAR_TYPE     TDS_STATIC_CAST(CS_INT, 25)
+#define CS_BLOB_TYPE        TDS_STATIC_CAST(CS_INT, 26)
+#define CS_DATE_TYPE        TDS_STATIC_CAST(CS_INT, 27)
+#define CS_TIME_TYPE        TDS_STATIC_CAST(CS_INT, 28)
+#define CS_UNITEXT_TYPE     TDS_STATIC_CAST(CS_INT, 29)
+#define CS_BIGINT_TYPE      TDS_STATIC_CAST(CS_INT, 30)
+#define CS_USMALLINT_TYPE   TDS_STATIC_CAST(CS_INT, 31)
+#define CS_UINT_TYPE        TDS_STATIC_CAST(CS_INT, 32)
+#define CS_UBIGINT_TYPE     TDS_STATIC_CAST(CS_INT, 33)
+#define CS_XML_TYPE         TDS_STATIC_CAST(CS_INT, 34)
+#define CS_UNIQUE_TYPE      TDS_STATIC_CAST(CS_INT, 40)
 
+#define CS_USER_TYPE        TDS_STATIC_CAST(CS_INT, 100)
 /* cs_dt_info type values */
 enum
 {
-	CS_MONTH = 1,
+	CS_MONTH = 7340,
 #define CS_MONTH CS_MONTH
 	CS_SHORTMONTH,
 #define CS_SHORTMONTH CS_SHORTMONTH
@@ -641,56 +598,56 @@ enum
 /* DT_CONVFMT types */
 enum
 {
-	CS_DATES_HMS = 1,
-#define CS_DATES_HMS CS_DATES_HMS
-	CS_DATES_SHORT,
+	CS_DATES_SHORT = 0,
 #define CS_DATES_SHORT CS_DATES_SHORT
-	CS_DATES_LONG,
-#define CS_DATES_LONG CS_DATES_LONG
 	CS_DATES_MDY1,
 #define CS_DATES_MDY1 CS_DATES_MDY1
-	CS_DATES_MYD1,
-#define CS_DATES_MYD1 CS_DATES_MYD1
-	CS_DATES_DMY1,
-#define CS_DATES_DMY1 CS_DATES_DMY1
-	CS_DATES_DYM1,
-#define CS_DATES_DYM1 CS_DATES_DYM1
-	CS_DATES_YDM1,
-#define CS_DATES_YDM1 CS_DATES_YDM1
-	CS_DATES_YMD2,
-#define CS_DATES_YMD2 CS_DATES_YMD2
-	CS_DATES_MDY1_YYYY,
-#define CS_DATES_MDY1_YYYY CS_DATES_MDY1_YYYY
-	CS_DATES_DMY1_YYYY,
-#define CS_DATES_DMY1_YYYY CS_DATES_DMY1_YYYY
-	CS_DATES_YMD2_YYYY,
-#define CS_DATES_YMD2_YYYY CS_DATES_YMD2_YYYY
-	CS_DATES_DMY2,
-#define CS_DATES_DMY2 CS_DATES_DMY2
 	CS_DATES_YMD1,
 #define CS_DATES_YMD1 CS_DATES_YMD1
-	CS_DATES_DMY2_YYYY,
-#define CS_DATES_DMY2_YYYY CS_DATES_DMY2_YYYY
-	CS_DATES_YMD1_YYYY,
-#define CS_DATES_YMD1_YYYY CS_DATES_YMD1_YYYY
-	CS_DATES_DMY4,
-#define CS_DATES_DMY4 CS_DATES_DMY4
-	CS_DATES_DMY4_YYYY,
-#define CS_DATES_DMY4_YYYY CS_DATES_DMY4_YYYY
-	CS_DATES_MDY2,
-#define CS_DATES_MDY2 CS_DATES_MDY2
-	CS_DATES_MDY2_YYYY,
-#define CS_DATES_MDY2_YYYY CS_DATES_MDY2_YYYY
+	CS_DATES_DMY1,
+#define CS_DATES_DMY1 CS_DATES_DMY1
+	CS_DATES_DMY2,
+#define CS_DATES_DMY2 CS_DATES_DMY2
 	CS_DATES_DMY3,
 #define CS_DATES_DMY3 CS_DATES_DMY3
+	CS_DATES_DMY4,
+#define CS_DATES_DMY4 CS_DATES_DMY4
+	CS_DATES_MDY2,
+#define CS_DATES_MDY2 CS_DATES_MDY2
+	CS_DATES_HMS,
+#define CS_DATES_HMS CS_DATES_HMS
+	CS_DATES_LONG,
+#define CS_DATES_LONG CS_DATES_LONG
 	CS_DATES_MDY3,
 #define CS_DATES_MDY3 CS_DATES_MDY3
-	CS_DATES_DMY3_YYYY,
-#define CS_DATES_DMY3_YYYY CS_DATES_DMY3_YYYY
-	CS_DATES_MDY3_YYYY,
-#define CS_DATES_MDY3_YYYY CS_DATES_MDY3_YYYY
+	CS_DATES_YMD2,
+#define CS_DATES_YMD2 CS_DATES_YMD2
 	CS_DATES_YMD3,
 #define CS_DATES_YMD3 CS_DATES_YMD3
+	CS_DATES_YDM1,
+#define CS_DATES_YDM1 CS_DATES_YDM1
+	CS_DATES_MYD1,
+#define CS_DATES_MYD1 CS_DATES_MYD1
+	CS_DATES_DYM1,
+#define CS_DATES_DYM1 CS_DATES_DYM1
+	CS_DATES_MDY1_YYYY = 101,
+#define CS_DATES_MDY1_YYYY CS_DATES_MDY1_YYYY
+	CS_DATES_YMD1_YYYY,
+#define CS_DATES_YMD1_YYYY CS_DATES_YMD1_YYYY
+	CS_DATES_DMY1_YYYY,
+#define CS_DATES_DMY1_YYYY CS_DATES_DMY1_YYYY
+	CS_DATES_DMY2_YYYY,
+#define CS_DATES_DMY2_YYYY CS_DATES_DMY2_YYYY
+	CS_DATES_DMY3_YYYY,
+#define CS_DATES_DMY3_YYYY CS_DATES_DMY3_YYYY
+	CS_DATES_DMY4_YYYY,
+#define CS_DATES_DMY4_YYYY CS_DATES_DMY4_YYYY
+	CS_DATES_MDY2_YYYY,
+#define CS_DATES_MDY2_YYYY CS_DATES_MDY2_YYYY
+	CS_DATES_MDY3_YYYY = 110,
+#define CS_DATES_MDY3_YYYY CS_DATES_MDY3_YYYY
+	CS_DATES_YMD2_YYYY,
+#define CS_DATES_YMD2_YYYY CS_DATES_YMD2_YYYY
 	CS_DATES_YMD3_YYYY
 #define CS_DATES_YMD3_YYYY CS_DATES_YMD3_YYYY
 };
@@ -724,17 +681,17 @@ typedef struct _cs_objdata
 /* Eventually, these should be in terms of TDS values */
 enum
 {
-	CS_OPT_SUNDAY,
-	CS_OPT_MONDAY,
+	CS_OPT_MONDAY = 1,
 	CS_OPT_TUESDAY,
 	CS_OPT_WEDNESDAY,
 	CS_OPT_THURSDAY,
 	CS_OPT_FRIDAY,
-	CS_OPT_SATURDAY
+	CS_OPT_SATURDAY,
+	CS_OPT_SUNDAY
 };
 enum
 {
-	CS_OPT_FMTMDY,
+	CS_OPT_FMTMDY = 1,
 	CS_OPT_FMTDMY,
 	CS_OPT_FMTYMD,
 	CS_OPT_FMTYDM,
@@ -743,8 +700,9 @@ enum
 };
 enum
 {
-	CS_OPT_LEVEL0,
+	CS_OPT_LEVEL0 = 0,
 	CS_OPT_LEVEL1,
+	CS_OPT_LEVEL2,
 	CS_OPT_LEVEL3
 };
 
@@ -752,12 +710,8 @@ enum
 #define CS_TRUE	1
 
 #define SRV_PROC	CS_VOID
-typedef struct _cs_blk_row CS_BLK_ROW;
 
 /* constants required for ct_diag (not jet implemented) */
-#define CS_INIT 36
-#define CS_STATUS 37
-#define CS_MSGLIMIT 38
 #define CS_CLIENTMSG_TYPE 4700
 #define CS_SERVERMSG_TYPE 4701
 #define CS_ALLMSG_TYPE 4702
