@@ -1,8 +1,11 @@
 #ifndef DES_H
 #define DES_H
 
-static const char rcsid_des_h[] = "$Id: des.h,v 1.10 2004/10/28 12:42:12 freddy77 Exp $";
-static const void *const no_unused_des_h_warn[] = {	rcsid_des_h, no_unused_des_h_warn };
+/* $Id: des.h,v 1.12 2007/03/12 13:28:50 freddy77 Exp $ */
+
+#if defined(__GNUC__) && __GNUC__ >= 4
+#pragma GCC visibility push(hidden)
+#endif
 
 typedef unsigned char des_cblock[8];
 
@@ -20,4 +23,8 @@ int tds_des_set_key(DES_KEY * dkey, des_cblock user_key, int len);
 void tds_des_encrypt(DES_KEY * key, des_cblock block);
 void _mcrypt_decrypt(DES_KEY * key, unsigned char *block);
 
+#if defined(__GNUC__) && __GNUC__ >= 4
+#pragma GCC visibility pop
+#endif
+
 #endif /* !DES_H */
