@@ -1,8 +1,11 @@
 #ifndef MD4_H
 #define MD4_H
 
-static const char rcsid_md4_h[] = "$Id: md4.h,v 1.5 2004/10/28 12:42:12 freddy77 Exp $";
-static const void *const no_unused_md4_h_warn[] = { rcsid_md4_h, no_unused_md4_h_warn };
+/* $Id: md4.h,v 1.7 2007/03/12 13:28:50 freddy77 Exp $ */
+
+#if defined(__GNUC__) && __GNUC__ >= 4
+#pragma GCC visibility push(hidden)
+#endif
 
 struct MD4Context
 {
@@ -18,4 +21,8 @@ void MD4Transform(TDS_UINT buf[4], TDS_UINT const in[16]);
 
 typedef struct MD4Context MD4_CTX;
 
+#if defined(__GNUC__) && __GNUC__ >= 4
+#pragma GCC visibility pop
+#endif
+
 #endif /* !MD4_H */
