@@ -1,8 +1,11 @@
 #ifndef MD5_H
 #define MD5_H
 
-static const char rcsid_md5_h[] = "$Id: md5.h,v 1.2 2004/10/28 12:42:12 freddy77 Exp $";
-static const void *const no_unused_md5_h_warn[] = { rcsid_md5_h, no_unused_md5_h_warn };
+/* $Id: md5.h,v 1.4 2007/03/12 13:28:50 freddy77 Exp $ */
+
+#if defined(__GNUC__) && __GNUC__ >= 4
+#pragma GCC visibility push(hidden)
+#endif
 
 struct MD5Context {
 	TDS_UINT buf[4];
@@ -21,4 +24,8 @@ void MD5Transform(TDS_UINT buf[4], TDS_UINT const in[16]);
  */
 typedef struct MD5Context MD5_CTX;
 
+#if defined(__GNUC__) && __GNUC__ >= 4
+#pragma GCC visibility pop
+#endif
+
 #endif /* !MD5_H */
