@@ -20,6 +20,29 @@
 #ifndef _replacements_h_
 #define _replacements_h_
 
+/* $Id: replacements.h,v 1.18 2007/12/21 09:06:53 freddy77 Exp $ */
+
+#include <stdarg.h>
+#include "tds_sysdep_public.h"
+
+#ifndef HAVE_READPASSPHRASE
+# include <replacements/readpassphrase.h>
+#else
+# include <readpassphrase.h>
+#endif
+
+/* these headers are needed for basename */
+#ifdef HAVE_STRING_H
+# include <string.h>
+#endif
+#ifdef HAVE_LIBGEN_H
+# include <libgen.h>
+#endif
+
+#if defined(__GNUC__) && __GNUC__ >= 4
+#pragma GCC visibility push(hidden)
+#endif
+
 #ifdef __cplusplus
 extern "C"
 {
@@ -28,13 +51,10 @@ extern "C"
 #endif
 #endif
 
-static const char rcsid_replacements_h[] = "$Id: replacements.h,v 1.8 2004/10/28 12:42:12 freddy77 Exp $";
-static const void *const no_unused_replacements_h_warn[] = { rcsid_replacements_h, no_unused_replacements_h_warn };
-
-#include <stdarg.h>
-#include "tds_sysdep_public.h"
-
-#if !HAVE_VSNPRINTF
+#if defined(HAVE__VSNPRINTF) && !defined(HAVE_VSNPRINTF)
+#undef vsnprintf
+#define vsnprintf _vsnprintf
+#elif !HAVE_VSNPRINTF
 int vsnprintf(char *ret, size_t max, const char *fmt, va_list ap);
 #endif /* !HAVE_VSNPRINTF */
 
@@ -54,6 +74,28 @@ tds_sysdep_int64_type atoll(const char *nptr);
 char *strtok_r(char *str, const char *sep, char **lasts);
 #endif /* !HAVE_STRTOK_R */
 
+#if HAVE_STRLCPY
+#define tds_strlcpy(d,s,l) strlcpy(d,s,l)
+#else
+size_t tds_strlcpy(char *dest, const char *src, size_t len);
+#endif
+
+#if HAVE_STRLCAT
+#define tds_strlcat(d,s,l) strlcat(d,s,l)
+#else
+size_t tds_strlcat(char *dest, const char *src, size_t len);
+#endif
+
+#if HAVE_BASENAME
+#define tds_basename(s) basename(s)
+#else
+char *tds_basename(char *path);
+#endif
+
+#if defined(WIN32)
+int gettimeofday (struct timeval *tv, void *tz);
+#endif
+
 #ifdef __cplusplus
 #if 0
 {
@@ -61,4 +103,8 @@ char *strtok_r(char *str, const char *sep, char **lasts);
 }
 #endif
 
+#if defined(__GNUC__) && __GNUC__ >= 4
+#pragma GCC visibility pop
+#endif
+
 #endif
