@@ -22,9 +22,11 @@
 
 #include <sybfront.h>
 
-static char  rcsid_sqlfront_h [ ] =
-         "$Id: sqlfront.h,v 1.1.1.1 2001/10/12 23:28:56 brianb Exp $";
-static void *no_unused_sqlfront_h_warn[]={rcsid_sqlfront_h, no_unused_sqlfront_h_warn};
+static const char rcsid_sqlfront_h[] = "$Id: sqlfront.h,v 1.4 2007/08/02 11:32:57 freddy77 Exp $";
+static const void *const no_unused_sqlfront_h_warn[] = { rcsid_sqlfront_h, no_unused_sqlfront_h_warn };
 
+typedef DBPROCESS * PDBPROCESS;
+typedef LOGINREC  * PLOGINREC;
+typedef DBCURSOR  * PDBCURSOR;
 
 #endif
