@@ -1,5 +1,6 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004  Brian Bruns
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005  Brian Bruns
+ * Copyright (C) 2004, 2005, 2006, 2007  Frediano Ziglio
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -20,15 +21,14 @@
 #ifndef _sql_h_
 #define _sql_h_
 
-#include <tds.h>
+#include "tds.h"
 
+#if defined(UNIXODBC) || defined(TDS_NO_DM)
+#include <sql.h>
+#include <sqlext.h>
 #ifdef UNIXODBC
-#include <sql.h>
-#include <sqlext.h>
 #include <odbcinst.h>
-#elif defined(TDS_NO_DM)
-#include <sql.h>
-#include <sqlext.h>
+#endif
 #else /* IODBC */
 #include <isql.h>
 #include <isqlext.h>
@@ -46,6 +46,18 @@
 #endif
 #endif
 
+#ifndef HAVE_SQLSETPOSIROW
+#define SQLSETPOSIROW SQLUSMALLINT
+#endif
+
+#ifndef HAVE_SQLROWOFFSET
+#define SQLROWOFFSET SQLLEN
+#endif
+
+#ifndef HAVE_SQLROWSETSIZE
+#define SQLROWSETSIZE SQLULEN
+#endif
+
 #ifdef __cplusplus
 extern "C"
 {
@@ -54,8 +66,17 @@ extern "C"
 #endif
 #endif
 
-static const char rcsid_sql_h[] = "$Id: tdsodbc.h,v 1.74 2004/10/28 12:42:12 freddy77 Exp $";
-static const void *const no_unused_sql_h_warn[] = { rcsid_sql_h, no_unused_sql_h_warn };
+/* $Id: tdsodbc.h,v 1.103.2.1 2008/01/27 10:41:22 freddy77 Exp $ */
+
+#if defined(__GNUC__) && __GNUC__ >= 4
+#pragma GCC visibility push(hidden)
+#define ODBC_API SQL_API __attribute__((externally_visible))
+#else
+#define ODBC_API SQL_API
+#endif
+
+#define ODBC_MAX(a,b) ( (a) > (b) ? (a) : (b) )
+#define ODBC_MIN(a,b) ( (a) < (b) ? (a) : (b) )
 
 struct _sql_error
 {
@@ -73,6 +94,7 @@ struct _sql_errors
 	SQLRETURN lastrc;
 	int num_errors;
 	struct _sql_error *errs;
+	char ranked;
 };
 
 #if ENABLE_EXTRA_CHECKS
@@ -93,7 +115,7 @@ void odbc_check_struct_extra(void *p);
 void odbc_errs_reset(struct _sql_errors *errs);
 
 /** add an error to list */
-void odbc_errs_add(struct _sql_errors *errs, const char *sqlstate, const char *msg, const char *server);
+void odbc_errs_add(struct _sql_errors *errs, const char *sqlstate, const char *msg);
 
 /** Add an error to list. This functions is for error that came from server */
 void odbc_errs_add_rdbms(struct _sql_errors *errs, TDS_UINT native, const char *sqlstate, const char *msg, int linenum,
@@ -207,6 +229,7 @@ struct _hcattr
 	DSTR translate_lib;
 	SQLUINTEGER translate_option;
 	SQLUINTEGER txn_isolation;
+	SQLUINTEGER cursor_type;
 #ifdef TDS_NO_DM
 	SQLUINTEGER trace;
 	DSTR tracefile;
@@ -223,7 +246,11 @@ struct _hdbc
 	TDSSOCKET *tds_socket;
 	DSTR dsn;
 	DSTR server;		/* aka Instance */
-	/** statement executing */
+	/**
+	 * Statement executing. This should be set AFTER sending query
+	 * to avoid race condition and assure to not overwrite it if
+	 * another statement is executing a query.
+	 */
 	struct _hstmt *current_statement;
 	/** list of all statements allocated from this connection */
 	struct _hstmt *stmt_list;
@@ -231,6 +258,9 @@ struct _hdbc
 	struct _hcattr attr;
 	/** descriptors associated to connection */
 	TDS_DESC *uad[TDS_MAX_APP_DESC];
+	/** <>0 if server handle cursors */
+	unsigned int cursor_support;
+	TDS_INT default_query_timeout;
 };
 
 struct _hsattr
@@ -284,6 +314,15 @@ struct _hsattr
 /*	TDS_DESC *imp_param_desc; */
 };
 
+typedef enum
+{
+	NOT_IN_ROW,
+	IN_NORMAL_ROW,
+	IN_COMPUTE_ROW,
+	AFTER_COMPUTE_ROW,
+	PRE_NORMAL_ROW
+} TDS_ODBC_ROW_STATUS;
+
 struct _hstmt
 {
 	SQLSMALLINT htype;	/* do not reorder this field */
@@ -308,12 +347,18 @@ struct _hstmt
 	TDSPARAMINFO *params;
 	/** last valid parameter in params, it's a ODBC index (from 1 relative to descriptor) */
 	int param_num;
+	/** position in prepared query to check parameters, used only in RPC */
+	char *prepared_pos;
+
+	unsigned int curr_param_row, num_param_rows;
 
 	/** number of parameter in current query */
 	unsigned int param_count;
 	int row;
 	/** row count to return */
-	int row_count;
+	TDS_INT8 row_count;
+	/** status of row, it can happen that this flag mark that we are still parsing row, this it's normal */
+	TDS_ODBC_ROW_STATUS row_status;
 	/* do NOT free dynamic, free from socket or attach to connection */
 	TDSDYNAMIC *dyn;
 	struct _sql_errors errs;
@@ -322,6 +367,10 @@ struct _hstmt
 	SQLULEN sql_rowset_size;
 	struct _hsattr attr;
 	DSTR cursor_name;	/* auto generated cursor name */
+	int special_row;
+	/* do NOT free cursor, free from socket or attach to connection */
+	TDSCURSOR *cursor;
+	unsigned char cancel_sent;
 };
 
 typedef struct _henv TDS_ENV;
@@ -334,12 +383,6 @@ typedef struct _hchk TDS_CHK;
 #define IS_HSTMT(x) (((TDS_CHK *)x)->htype == SQL_HANDLE_STMT)
 #define IS_HDESC(x) (((TDS_CHK *)x)->htype == SQL_HANDLE_DESC)
 
-TDS_DESC *desc_alloc(SQLHANDLE parent, int desc_type, int alloc_type);
-SQLRETURN desc_free(TDS_DESC * desc);
-SQLRETURN desc_alloc_records(TDS_DESC * desc, unsigned count);
-SQLRETURN desc_copy(TDS_DESC * dest, TDS_DESC * src);
-SQLRETURN desc_free_records(TDS_DESC * desc);
-
 /* fix a bug in MingW headers */
 #ifdef __MINGW32__
 #if SQL_INTERVAL_YEAR == (100 + SQL_CODE_SECOND)
@@ -375,6 +418,112 @@ SQLRETURN desc_free_records(TDS_DESC * desc);
 #endif
 #endif
 
+#ifdef WIN32
+BOOL get_login_info(HWND hwndParent, TDSCONNECTION * connection);
+#endif
+
+/*
+ * connectparams.h
+ */
+/**
+ * Parses a connection string for SQLDriverConnect().
+ * \param connect_string      point to connection string
+ * \param connect_string_end  point to end of connection string
+ * \param connection          structure where to store informations
+ * \return 0 if error, 1 otherwise
+ */
+int odbc_parse_connect_string(const char *connect_string, const char *connect_string_end, TDSCONNECTION * connection);
+int odbc_get_dsn_info(const char *DSN, TDSCONNECTION * connection);
+
+/*
+ * convert_tds2sql.c
+ */
+TDS_INT convert_tds2sql(TDSCONTEXT * context, int srctype, TDS_CHAR * src, TDS_UINT srclen, int desttype, TDS_CHAR * dest, SQLULEN destlen, const struct _drecord *drec_ixd);
+
+/*
+ * descriptor.c
+ */
+TDS_DESC *desc_alloc(SQLHANDLE parent, int desc_type, int alloc_type);
+SQLRETURN desc_free(TDS_DESC * desc);
+SQLRETURN desc_alloc_records(TDS_DESC * desc, unsigned count);
+SQLRETURN desc_copy(TDS_DESC * dest, TDS_DESC * src);
+SQLRETURN desc_free_records(TDS_DESC * desc);
+
+/*
+ * odbc.c
+ */
+SQLRETURN _SQLRowCount(SQLHSTMT hstmt, SQLLEN FAR * pcrow);
+
+/*
+ * odbc_checks.h
+ */
+#if ENABLE_EXTRA_CHECKS
+/* macro */
+#define CHECK_ENV_EXTRA(env) odbc_check_env_extra(env)
+#define CHECK_DBC_EXTRA(dbc) odbc_check_dbc_extra(dbc)
+#define CHECK_STMT_EXTRA(stmt) odbc_check_stmt_extra(stmt)
+#define CHECK_DESC_EXTRA(desc) odbc_check_desc_extra(desc)
+/* declarations*/
+void odbc_check_env_extra(TDS_ENV * env);
+void odbc_check_dbc_extra(TDS_DBC * dbc);
+void odbc_check_stmt_extra(TDS_STMT * stmt);
+void odbc_check_desc_extra(TDS_DESC * desc);
+#else
+/* macro */
+#define CHECK_ENV_EXTRA(env)
+#define CHECK_DBC_EXTRA(dbc)
+#define CHECK_STMT_EXTRA(stmt)
+#define CHECK_DESC_EXTRA(desc)
+#endif
+
+/*
+ * odbc_util.h
+ */
+int odbc_set_stmt_query(struct _hstmt *stmt, const char *sql, int sql_len);
+int odbc_set_stmt_prepared_query(struct _hstmt *stmt, const char *sql, int sql_len);
+void odbc_set_return_status(struct _hstmt *stmt, unsigned int n_row);
+void odbc_set_return_params(struct _hstmt *stmt, unsigned int n_row);
+
+SQLSMALLINT odbc_server_to_sql_type(int col_type, int col_size);
+int odbc_sql_to_c_type_default(int sql_type);
+int odbc_sql_to_server_type(TDSSOCKET * tds, int sql_type);
+int odbc_c_to_server_type(int c_type);
+
+void odbc_set_sql_type_info(TDSCOLUMN * col, struct _drecord *drec, SQLINTEGER odbc_ver);
+SQLINTEGER odbc_sql_to_displaysize(int sqltype, TDSCOLUMN *col);
+int odbc_get_string_size(int size, SQLCHAR * str);
+void odbc_rdbms_version(TDSSOCKET * tds_socket, char *pversion_string);
+SQLINTEGER odbc_get_param_len(const struct _drecord *drec_axd, const struct _drecord *drec_ixd, const TDS_DESC* axd, unsigned int n_row);
+
+SQLRETURN odbc_set_string(SQLPOINTER buffer, SQLSMALLINT cbBuffer, SQLSMALLINT FAR * pcbBuffer, const char *s, int len);
+SQLRETURN odbc_set_string_i(SQLPOINTER buffer, SQLINTEGER cbBuffer, SQLINTEGER FAR * pcbBuffer, const char *s, int len);
+
+SQLSMALLINT odbc_get_concise_sql_type(SQLSMALLINT type, SQLSMALLINT interval);
+SQLRETURN odbc_set_concise_sql_type(SQLSMALLINT concise_type, struct _drecord *drec, int check_only);
+SQLSMALLINT odbc_get_concise_c_type(SQLSMALLINT type, SQLSMALLINT interval);
+SQLRETURN odbc_set_concise_c_type(SQLSMALLINT concise_type, struct _drecord *drec, int check_only);
+
+SQLLEN odbc_get_octet_len(int c_type, const struct _drecord *drec);
+
+/*
+ * prepare_query.c
+ */
+SQLRETURN prepare_call(struct _hstmt *stmt);
+SQLRETURN native_sql(struct _hdbc *dbc, char *s);
+int parse_prepared_query(struct _hstmt *stmt, int compute_row);
+int start_parse_prepared_query(struct _hstmt *stmt, int compute_row);
+int continue_parse_prepared_query(struct _hstmt *stmt, SQLPOINTER DataPtr, SQLLEN StrLen_or_Ind);
+const char *parse_const_param(const char * s, TDS_SERVER_TYPE *type);
+
+/*
+ * sql2tds.c
+ */
+SQLRETURN sql2tds(TDS_STMT * stmt, const struct _drecord *drec_ixd, const struct _drecord *drec_axd, TDSCOLUMN *curcol, int compute_row, const TDS_DESC* axd, unsigned int n_row);
+
+#if defined(__GNUC__) && __GNUC__ >= 4
+#pragma GCC visibility pop
+#endif
+
 #ifdef __cplusplus
 #if 0
 {
