@@ -20,43 +20,59 @@
 #ifndef _tdsstring_h_
 #define _tdsstring_h_
 
-static const char rcsid_tdsstring_h[] = "$Id: tdsstring.h,v 1.13 2004/10/28 12:42:12 freddy77 Exp $";
-static const void *const no_unused_tdsstring_h_warn[] = { rcsid_tdsstring_h, no_unused_tdsstring_h_warn };
+/* $Id: tdsstring.h,v 1.19 2007/07/01 10:10:52 freddy77 Exp $ */
 
-extern char tds_str_empty[];
+#if defined(__GNUC__) && __GNUC__ >= 4
+#pragma GCC visibility push(hidden)
+#endif
+
+extern const char tds_str_empty[1];
 
 /* TODO do some function and use inline if available */
 
 /** \addtogroup dstring
- *  \@{ 
+ * @{ 
  */
 
 #if ENABLE_EXTRA_CHECKS
 void tds_dstr_init(DSTR * s);
 int tds_dstr_isempty(DSTR * s);
-char *tds_dstr_cstr(DSTR * s);
+char *tds_dstr_buf(DSTR * s);
 size_t tds_dstr_len(DSTR * s);
 #else
 /** init a string with empty */
 #define tds_dstr_init(s) \
-	do { *(s) = (DSTR) &tds_str_empty[0]; } while(0)
+	do { DSTR *_tds_s = (s); _tds_s->dstr_size = 0; _tds_s->dstr_s = (char*) tds_str_empty; } while(0)
 
 /** test if string is empty */
 #define tds_dstr_isempty(s) \
-	((*(s))->dstr_s[0] == '\0')
-#define tds_dstr_cstr(s) \
-	((*(s))->dstr_s)
+	((s)->dstr_size == 0)
+#define tds_dstr_buf(s) \
+	((s)->dstr_s)
 #define tds_dstr_len(s) \
-	strlen((*(s))->dstr_s)
+	((s)->dstr_size)
 #endif
 
+#define tds_dstr_cstr(s) \
+	((const char* ) tds_dstr_buf(s))
+
 void tds_dstr_zero(DSTR * s);
 void tds_dstr_free(DSTR * s);
 
-DSTR tds_dstr_copy(DSTR * s, const char *src);
-DSTR tds_dstr_copyn(DSTR * s, const char *src, unsigned int length);
-DSTR tds_dstr_set(DSTR * s, char *src);
+DSTR* tds_dstr_dup(DSTR * s, const DSTR * src);
+DSTR* tds_dstr_copy(DSTR * s, const char *src);
+DSTR* tds_dstr_copyn(DSTR * s, const char *src, unsigned int length);
+DSTR* tds_dstr_set(DSTR * s, char *src);
 
-/** \@} */
+/** limit length of string, MUST be <= current length */
+DSTR* tds_dstr_setlen(DSTR *s, unsigned int length);
+/** allocate space for length char */
+DSTR* tds_dstr_alloc(DSTR *s, unsigned int length);
+
+/** @} */
+
+#if defined(__GNUC__) && __GNUC__ >= 4
+#pragma GCC visibility pop
+#endif
 
 #endif /* _tdsstring_h_ */
