@@ -1,5 +1,5 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 2004  James K. Lowden
+ * Copyright (C) 2004, 2005  James K. Lowden
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -41,19 +41,11 @@
 #include <string.h>
 #endif
 
-#if HAVE_LIBGEN_H
-#include <libgen.h>
-#endif
-
-#if ! HAVE_BASENAME
-char *basename(char *path);
-#endif
-
 #include <sqlfront.h>
 #include <sybdb.h>
 #include "replacements.h"
 
-static char software_version[] = "$Id: bsqldb.c,v 1.12 2004/11/09 09:54:39 freddy77 Exp $";
+static char software_version[] = "$Id: bsqldb.c,v 1.32 2007/12/06 19:00:24 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 int err_handler(DBPROCESS * dbproc, int severity, int dberr, int oserr, char *dberrstr, char *oserrstr);
@@ -65,18 +57,21 @@ static void print_results(DBPROCESS *dbproc);
 static int get_printable_size(int type, int size);
 static void usage(const char invoked_as[]);
 
-struct METADATA { char *name, *format_string; const char *source; int type, size; };
+struct METADATA { char *name, *format_string; const char *source; int type, size, width; };
 static int set_format_string(struct METADATA * meta, const char separator[]);
 
 typedef struct _options 
 { 
-	int 	fverbose;
-	FILE 	*verbose;
+	int 	fverbose, 
+		fquiet;
+	FILE 	*headers, 
+		*verbose;
 	char 	*servername, 
 		*database, 
 		*appname, 
-		 hostname[128], 
-		*input_filename, 
+		 hostname[128];
+	const char *colsep;
+	char	*input_filename, 
 		*output_filename, 
 		*error_filename; 
 } OPTIONS;
@@ -85,6 +80,7 @@ LOGINREC* get_login(int argc, char *argv[], OPTIONS *poptions);
 
 /* global variables */
 OPTIONS options;
+static const char default_colsep[] = "  ";
 /* end global variables */
 
 
@@ -118,6 +114,7 @@ main(int argc, char *argv[])
 	
 
 	memset(&options, 0, sizeof(options));
+	options.headers = stderr;
 	login = get_login(argc, argv, &options); /* get command-line parameters and call dblogin() */
 	assert(login != NULL);
 
@@ -212,7 +209,8 @@ next_query(DBPROCESS *dbproc)
 			
 	fprintf(options.verbose, "%s:%d: Query:\n", options.appname, __LINE__);
 	
-	/* Normally, a call to dbcmd() clears the buffer the first time it's 
+	/*
+	 * Normally, a call to dbcmd() clears the buffer the first time it's 
 	 * invoked after a call to dbsqlexec() or dbsqlsend().  If fgets(3) 
 	 * returns 0 below, however, we'd indicate "success" without calling
 	 * dbcmd().  This would leave the prior query in the buffer, which 
@@ -226,10 +224,10 @@ next_query(DBPROCESS *dbproc)
 		/* 'go' or 'GO' separates command batches */
 		const char *p = query_line;
 		if (strncasecmp(p, "go", 2) == 0) {
-			while(isspace(*p))
-				++p;
-			if (strcmp(p, "\n") == 0)
-				return 1;
+			for (p+=2; isspace((unsigned char) *p); p++) {
+				if (*p == '\n')
+					return 1;
+			}
 		}
 
 		fprintf(options.verbose, "\t%s", query_line);
@@ -274,6 +272,13 @@ print_results(DBPROCESS *dbproc)
 	int i, c, ret;
 	int iresultset;
 	int ncomputeids = 0, ncols = 0;
+
+	/* 
+	 * If using default column separator, we want columns to line up vertically, 
+	 * 	so we use blank padding (STRINGBIND).  
+	 * For any other separator, we use no padding.
+	 */
+	const int bindtype = (0 == strcmp(options.colsep, default_colsep))? STRINGBIND : NTBSTRINGBIND;
 	
 	/* 
 	 * Set up each result set with dbresults()
@@ -357,7 +362,6 @@ print_results(DBPROCESS *dbproc)
 		fprintf(options.verbose, "%-6s  %-30s  %-30s  %-15s  %-6s  %-6s  \n", "col", "name", "source", "type", "size", "varies");
 		fprintf(options.verbose, "%.6s  %.30s  %.30s  %.15s  %.6s  %.6s  \n", dashes, dashes, dashes, dashes, dashes, dashes);
 		for (c=0; c < ncols; c++) {
-			int width;
 			/* Get and print the metadata.  Optional: get only what you need. */
 			char *name = dbcolname(dbproc, c+1);
 			metadata[c].name = strdup(name ? (const char *) name : empty_string);
@@ -377,11 +381,11 @@ print_results(DBPROCESS *dbproc)
 			 * Build the column header format string, based on the column width. 
 			 * This is just one solution to the question, "How wide should my columns be when I print them out?"
 			 */
-			width = get_printable_size(metadata[c].type, metadata[c].size);
-			if (width < strlen(metadata[c].name))
-				width = strlen(metadata[c].name);
+			metadata[c].width = get_printable_size(metadata[c].type, metadata[c].size);
+			if (metadata[c].width < strlen(metadata[c].name))
+				metadata[c].width = strlen(metadata[c].name);
 				
-			ret = set_format_string(&metadata[c], (c+1 < ncols)? "  " : "\n");
+			ret = set_format_string(&metadata[c], (c+1 < ncols)? options.colsep : "\n");
 			if (ret <= 0) {
 				fprintf(stderr, "%s:%d: asprintf(), column %d failed\n", options.appname, __LINE__, c+1);
 				return;
@@ -398,10 +402,10 @@ print_results(DBPROCESS *dbproc)
 			 * inaccesible to the application.  
 			 */
 
-			data[c].buffer = calloc(1, metadata[c].size);
+			data[c].buffer = calloc(1, metadata[c].width);
 			assert(data[c].buffer);
 
-			erc = dbbind(dbproc, c+1, STRINGBIND, -1, (BYTE *) data[c].buffer);
+			erc = dbbind(dbproc, c+1, bindtype, 0, (BYTE *) data[c].buffer);
 			if (erc == FAIL) {
 				fprintf(stderr, "%s:%d: dbbind(), column %d failed\n", options.appname, __LINE__, c+1);
 				return;
@@ -422,7 +426,7 @@ print_results(DBPROCESS *dbproc)
 			for (c=0; c < metacompute[i]->numalts; c++) {
 				/* read metadata */
 				struct METADATA *meta = &metacompute[i]->meta[c];
-				int nbylist, ibylist;
+				int nby, iby;
 				BYTE *bylist;
 				char *colname, *bynames;
 				int altcolid = dbaltcolid(dbproc, i+1, c+1);
@@ -434,13 +438,13 @@ print_results(DBPROCESS *dbproc)
 				 * Jump through hoops to determine a useful name for the computed column 
 				 * If the query says "compute count(c) by a,b", we get a "by list" indicating a & b.  
 				 */
-				bylist = dbbylist(dbproc, c+1, &nbylist);
+				bylist = dbbylist(dbproc, c+1, &nby);
 
 				bynames = strdup("by (");
-				for (ibylist=0; ibylist < nbylist; ibylist++) {
+				for (iby=0; iby < nby; iby++) {
 					char *s = NULL; 
-					int ret = asprintf(&s, "%s%s%s", bynames, dbcolname(dbproc, bylist[ibylist]), 
-										(ibylist+1 < nbylist)? ", " : ")");
+					int ret = asprintf(&s, "%s%s%s", bynames, dbcolname(dbproc, bylist[iby]), 
+										(iby+1 < nby)? ", " : ")");
 					if (ret < 0) {
 						fprintf(options.verbose, "Insufficient room to create name for column %d:\n", 1+c);
 						break;
@@ -452,13 +456,19 @@ print_results(DBPROCESS *dbproc)
 				if( altcolid == -1 ) {
 					colname = "*";
 				} else {
-					colname = metadata[altcolid].name;
+					assert(0 < altcolid && altcolid <= dbnumcols(dbproc));
+					colname = metadata[--altcolid].name;
 				}
 
 				asprintf(&metacompute[i]->meta[c].name, "%s(%s)", dbprtype(dbaltop(dbproc, i+1, c+1)), colname);
 				assert(metacompute[i]->meta[c].name);
 					
-				ret = set_format_string(meta, (c+1 < metacompute[i]->numalts)? "  " : "\n");
+				metacompute[i]->meta[c].width = get_printable_size(metacompute[i]->meta[c].type, 
+										   metacompute[i]->meta[c].size);
+				if (metacompute[i]->meta[c].width < strlen(metacompute[i]->meta[c].name))
+					metacompute[i]->meta[c].width = strlen(metacompute[i]->meta[c].name);
+
+				ret = set_format_string(meta, (c+1 < metacompute[i]->numalts)? options.colsep : "\n");
 				if (ret <= 0) {
 					free(bynames);
 					fprintf(stderr, "%s:%d: asprintf(), column %d failed\n", options.appname, __LINE__, c+1);
@@ -466,17 +476,17 @@ print_results(DBPROCESS *dbproc)
 				}
 				
 				fprintf(options.verbose, "\tcolumn %d is %s, type %s, size %d %s\n", 
-					c+1, metacompute[i]->meta[c].name, dbprtype(metacompute[i]->meta[c].type), metacompute[i]->meta[c].size, 
-					(nbylist > 0)? bynames : "");
+					c+1, metacompute[i]->meta[c].name, dbprtype(metacompute[i]->meta[c].type),
+					metacompute[i]->meta[c].size, (nby > 0)? bynames : "");
 				free(bynames);
 	
 				/* allocate buffer */
 				assert(metacompute[i]->data);
-				metacompute[i]->data[c].buffer = calloc(1, metacompute[i]->meta[c].size);
+				metacompute[i]->data[c].buffer = calloc(1, metacompute[i]->meta[c].width);
 				assert(metacompute[i]->data[c].buffer);
 				
 				/* bind */
-				erc = dbaltbind(dbproc, i+1, c+1, STRINGBIND, -1, metacompute[i]->data[c].buffer);
+				erc = dbaltbind(dbproc, i+1, c+1, bindtype, -1, (BYTE*) metacompute[i]->data[c].buffer);
 				if (erc == FAIL) {
 					fprintf(stderr, "%s:%d: dbaltbind(), column %d failed\n", options.appname, __LINE__, c+1);
 					return;
@@ -487,16 +497,17 @@ print_results(DBPROCESS *dbproc)
 		fprintf(options.verbose, "\n");
 		fprintf(options.verbose, "Data\n");
 
-		/* Print the column headers to stderr to keep them separate from the data.  */
-		for (c=0; c < ncols; c++) {
-			fprintf(stderr, metadata[c].format_string, metadata[c].name);
-		}
+		if (!options.fquiet) {
+			/* Print the column headers to stderr to keep them separate from the data.  */
+			for (c=0; c < ncols; c++) {
+				fprintf(options.headers, metadata[c].format_string, metadata[c].name);
+			}
 
-		/* Underline the column headers.  */
-		for (c=0; c < ncols; c++) {
-			fprintf(stderr, metadata[c].format_string, dashes);
+			/* Underline the column headers.  */
+			for (c=0; c < ncols; c++) {
+				fprintf(options.headers, metadata[c].format_string, dashes);
+			}
 		}
-
 		/* 
 		 * Print the data to stdout.  
 		 */
@@ -522,6 +533,12 @@ print_results(DBPROCESS *dbproc)
 				assert(row_code != BUF_FULL);
 				break;
 				
+			case FAIL:
+				fprintf(stderr, "bsqldb: fatal error: dbnextrow returned FAIL\n");
+				assert(row_code != FAIL);
+				exit(EXIT_FAILURE);
+				break;
+				
 			default: /* computeid */
 				fprintf(options.verbose, "Data for computeid %d\n", row_code);
 				for (c=0; c < metacompute[row_code-1]->numalts; c++) {
@@ -530,12 +547,12 @@ print_results(DBPROCESS *dbproc)
 					
 					/* left justify the names */
 					strcat(fmt, &meta->format_string[1]);
-					fprintf(stderr, fmt, meta->name);
+					fprintf(options.headers, fmt, meta->name);
 				}
 
 				/* Underline the column headers.  */
 				for (c=0; c < metacompute[row_code-1]->numalts; c++) {
-					fprintf(stderr, metacompute[row_code-1]->meta[c].format_string, dashes);
+					fprintf(options.headers, metacompute[row_code-1]->meta[c].format_string, dashes);
 				}
 					
 				for (c=0; c < metacompute[row_code-1]->numalts; c++) {
@@ -560,21 +577,24 @@ print_results(DBPROCESS *dbproc)
 		}
 
 		/* Check return status */
-		fprintf(options.verbose, "Retrieving return status... ");
-		if (dbhasretstat(dbproc) == TRUE) {
-			fprintf(stderr, "Procedure returned %d\n", dbretstatus(dbproc));
-		} else {
-			fprintf(options.verbose, "none\n");
+		if (!options.fquiet) {
+			fprintf(options.verbose, "Retrieving return status... ");
+			if (dbhasretstat(dbproc) == TRUE) {
+				fprintf(stderr, "Procedure returned %d\n", dbretstatus(dbproc));
+			} else {
+				fprintf(options.verbose, "none\n");
+			}
 		}
 		
 		/* 
 		 * Get row count, if available.   
 		 */
-		if (DBCOUNT(dbproc) > -1)
-			fprintf(stderr, "%d rows affected\n", DBCOUNT(dbproc));
-		else 
-			fprintf(stderr, "@@rowcount not available\n");
-			
+		if (!options.fquiet) {
+			if (DBCOUNT(dbproc) > -1)
+				fprintf(stderr, "%d rows affected\n", DBCOUNT(dbproc));
+			else 
+				fprintf(stderr, "@@rowcount not available\n");
+		}			
 
 		/* 
 		 * Check return parameter values 
@@ -666,14 +686,19 @@ set_format_string(struct METADATA * meta, const char separator[])
 	const char *size_and_width;
 	assert(meta);
 
-	/* right justify numbers, left justify strings */
-	size_and_width = is_character_data(meta->type)? "%%-%d.%ds%s" : "%%%d.%ds%s";
-	
-	width = get_printable_size(meta->type, meta->size);
-	if (width < strlen(meta->name))
-		width = strlen(meta->name);
+	if(0 == strcmp(options.colsep, default_colsep)) { 
+		/* right justify numbers, left justify strings */
+		size_and_width = is_character_data(meta->type)? "%%-%d.%ds%s" : "%%%d.%ds%s";
+		
+		width = get_printable_size(meta->type, meta->size);
+		if (width < strlen(meta->name))
+			width = strlen(meta->name);
 
-	ret = asprintf(&meta->format_string, size_and_width, width, width, separator);
+		ret = asprintf(&meta->format_string, size_and_width, width, width, separator);
+	} else {
+		/* For anything except the default two-space separator, don't justify the strings. */
+		ret = asprintf(&meta->format_string, "%%s%s", separator);
+	}
 		       
 	return ret;
 }
@@ -688,6 +713,43 @@ usage(const char invoked_as[])
 			, invoked_as);
 }
 
+static void unescape(char arg[])
+{
+	char *p = arg;
+	char escaped = '1'; /* any digit will do for an initial value */
+	while ((p = strchr(p, '\\')) != NULL) {
+	
+		switch (p[1]) {
+		case '0':
+			/* FIXME we use strlen() of field/row terminators, which obviously won't work here */
+			fprintf(stderr, "bsqldb, line %d: NULL terminators ('\\0') not yet supported.\n", __LINE__);
+			escaped = '\0';
+			break;
+		case 't':
+			escaped = '\t';
+			break;
+		case 'r':
+			escaped = '\r';
+			break;
+		case 'n':
+			escaped = '\n';
+			break;
+		case '\\':
+			escaped = '\\';
+			break;
+		default:
+			break;
+		}
+			
+		/* Overwrite the backslash with the intended character, and shift everything down one */
+		if (!isdigit((unsigned char) escaped)) {
+			*p++ = escaped;
+			memmove(p, p+1, 1 + strlen(p+1));
+			escaped = '1';
+		}
+	}
+}
+
 LOGINREC *
 get_login(int argc, char *argv[], OPTIONS *options)
 {
@@ -698,7 +760,8 @@ get_login(int argc, char *argv[], OPTIONS *options)
 
 	assert(options && argv);
 	
-	options->appname = basename(argv[0]);
+	options->appname = tds_basename(argv[0]);
+	options->colsep = default_colsep; /* may be overridden by -t */
 	
 	login = dblogin();
 	
@@ -715,7 +778,7 @@ get_login(int argc, char *argv[], OPTIONS *options)
 		DBSETLHOST(login, options->hostname);
 	}
 
-	while ((ch = getopt(argc, argv, "U:P:S:D:i:o:e:v")) != -1) {
+	while ((ch = getopt(argc, argv, "U:P:S:dD:i:o:e:t:hqv")) != -1) {
 		switch (ch) {
 		case 'U':
 			DBSETLUSER(login, optarg);
@@ -726,6 +789,7 @@ get_login(int argc, char *argv[], OPTIONS *options)
 		case 'S':
 			options->servername = strdup(optarg);
 			break;
+		case 'd':
 		case 'D':
 			options->database = strdup(optarg);
 			break;
@@ -738,6 +802,16 @@ get_login(int argc, char *argv[], OPTIONS *options)
 		case 'e':
 			options->error_filename = strdup(optarg);
 			break;
+		case 't':
+			unescape(optarg);
+			options->colsep = strdup(optarg);
+			break;
+		case 'h':
+			options->headers = stdout;
+			break;
+		case 'q':
+			options->fquiet = 1;
+			break;
 		case 'v':
 			options->fverbose = 1;
 			break;
@@ -759,7 +833,6 @@ get_login(int argc, char *argv[], OPTIONS *options)
 int
 err_handler(DBPROCESS * dbproc, int severity, int dberr, int oserr, char *dberrstr, char *oserrstr)
 {
-
 	if (dberr) {
 		fprintf(stderr, "%s: Msg %d, Level %d\n", options.appname, dberr, severity);
 		fprintf(stderr, "%s\n\n", dberrstr);
@@ -776,44 +849,30 @@ err_handler(DBPROCESS * dbproc, int severity, int dberr, int oserr, char *dberrs
 int
 msg_handler(DBPROCESS * dbproc, DBINT msgno, int msgstate, int severity, char *msgtext, char *srvname, char *procname, int line)
 {
-	printf("Msg %ld, Level %d, State %d\n", (long) msgno, severity, msgstate);
+	enum {changed_database = 5701, changed_language = 5703 };
+	
+	if (msgno == changed_database || msgno == changed_language) 
+		return 0;
 
-	if (strlen(srvname) > 0)
-		printf("Server '%s', ", srvname);
-	if (strlen(procname) > 0)
-		printf("Procedure '%s', ", procname);
-	if (line > 0)
-		printf("Line %d", line);
+	if (msgno > 0) {
+		fprintf(stderr, "Msg %ld, Level %d, State %d\n", (long) msgno, severity, msgstate);
 
-	printf("\n\t%s\n", msgtext);
+		if (strlen(srvname) > 0)
+			fprintf(stderr, "Server '%s', ", srvname);
+		if (strlen(procname) > 0)
+			fprintf(stderr, "Procedure '%s', ", procname);
+		if (line > 0)
+			fprintf(stderr, "Line %d", line);
+
+		fprintf(stderr, "\n\t");
+	}
+	fprintf(stderr, "%s\n", msgtext);
+	
+	if (severity > 10) {
+		fprintf(stderr, "%s: error: severity %d >10, exiting\n", options.appname, severity);
+		exit(severity);
+	}
 
 	return 0;
 }
 
-
-#if ! HAVE_BASENAME
-/*
- * Replacement for basename(3)
- *
- * cf: http://www.opengroup.org/onlinepubs/007908799/xsh/basename.html
- *
- * The basename() function may modify the string pointed to by path, 
- * and may return a pointer to static storage that may then be overwritten 
- * by a subsequent call to basename(). 
- */
-char *basename(char *path)
-{
-	char *p;
-	
-	if (path == NULL)
-		return path;
-	
-	for (p = path + strlen(path) - 1; *p == '/'; p--) {
-		*p = '\0';
-	}
-	
-	p = strrchr(path, '/');
-	
-	return (p)? p : path;
-}
-#endif /* HAVE_BASENAME */
