@@ -1,5 +1,5 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004  Brian Bruns
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005  Brian Bruns
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -36,23 +36,27 @@
 #include <strings.h>
 #endif /* HAVE_STRINGS_H */
 
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include "tds.h"
 #include <sybfront.h>
 #include <sybdb.h>
-#include "dblib.h"
 #include "freebcp.h"
 
-static char software_version[] = "$Id: freebcp.c,v 1.34.2.1 2005/02/03 09:39:14 freddy77 Exp $";
+static char software_version[] = "$Id: freebcp.c,v 1.48 2006/12/01 21:51:11 jklowden Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 void pusage(void);
 int process_parameters(int, char **, struct pd *);
-static void unescape(char arg[]);
+static int unescape(char arg[]);
 int login_to_database(struct pd *, DBPROCESS **);
 
-int file_character(PARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir);
-int file_native(PARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir);
-int file_formatted(PARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir);
+int file_character(BCPPARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir);
+int file_native(BCPPARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir);
+int file_formatted(BCPPARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir);
+int setoptions (DBPROCESS * dbproc, BCPPARAMDATA * params);
 
 int err_handler(DBPROCESS * dbproc, int severity, int dberr, int oserr, char *dberrstr, char *oserrstr);
 int msg_handler(DBPROCESS * dbproc, DBINT msgno, int msgstate, int severity, char *msgtext, char *srvname, char *procname,
@@ -61,11 +65,11 @@ int msg_handler(DBPROCESS * dbproc, DBINT msgno, int msgstate, int severity, cha
 int
 main(int argc, char **argv)
 {
-	PARAMDATA params;
+	BCPPARAMDATA params;
 	DBPROCESS *dbproc;
 	int ok = FALSE;
 
-	memset(&params, '\0', sizeof(PARAMDATA));
+	memset(&params, '\0', sizeof(params));
 
 	params.textsize = 4096;	/* our default text size is 4K */
 
@@ -81,17 +85,8 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
-	if (dbfcmd(dbproc, "set textsize %d", params.textsize) == FAIL) {
-		printf("dbfcmd failed\n");
+	if (!setoptions(dbproc, &params)) 
 		return FALSE;
-	}
-
-	if (dbsqlexec(dbproc) == FAIL) {
-		printf("dbsqlexec failed\n");
-		return FALSE;
-	}
-
-	while (NO_MORE_RESULTS != dbresults(dbproc));
 
 	if (params.cflag) {	/* character format file */
 		ok = file_character(&params, dbproc, params.direction);
@@ -109,16 +104,16 @@ main(int argc, char **argv)
 }
 
 
-static void unescape(char arg[])
+static int unescape(char arg[])
 {
-	char *p = arg;
-	char escaped = '1'; /* any digit will do for an initial value */
-	while ((p = strchr(p, '\\')) != NULL) {
-	
+	char *p = arg, *next;
+	char escaped;
+	while ((next = strchr(p, '\\')) != NULL) {
+
+		p = next;
+
 		switch (p[1]) {
 		case '0':
-			/* FIXME we use strlen() of field/row terminators, which obviously won't work here */
-			fprintf(stderr, "freebcp, line %d: NULL terminators ('\\0') not yet supported.\n", __LINE__);
 			escaped = '\0';
 			break;
 		case 't':
@@ -134,53 +129,47 @@ static void unescape(char arg[])
 			escaped = '\\';
 			break;
 		default:
-			break;
+			++p;
+			continue;
 		}
-			
+
 		/* Overwrite the backslash with the intended character, and shift everything down one */
-		if (!isdigit((unsigned char) escaped)) {
-			*p++ = escaped;
-			memmove(p, p+1, 1 + strlen(p+1));
-		}
+		*p++ = escaped;
+		memmove(p, p+1, 1 + strlen(p+1));
 	}
+	return strchr(p, 0) - arg;
 }
 
 int
-process_parameters(int argc, char **argv, PARAMDATA * pdata)
+process_parameters(int argc, char **argv, BCPPARAMDATA *pdata)
 {
-	int state;
-	int i;
+	extern char *optarg;
+	extern int optind;
+	extern int optopt;
+	
+	int ch;
 
-	char arg[FILENAME_MAX + 1];
-
-	if (argc == 1) {
+	if (argc < 6) {
 		pusage();
 		return (FALSE);
 	}
 
-	if (*(1 + argv[1]) == 'v') {
-		printf("freebcp version %s\n", software_version);
-		return (FALSE);
-	}
-
-	if (argc < 6) {
-		fprintf(stderr, "A minimum of 6 parameters must be supplied.\n");
-		return (FALSE);
-	}
-
-	/* set some defaults */
-
+	/* 
+	 * Set some defaults and read the table, file, and direction arguments.  
+	 */
 	pdata->firstrow = 0;
 	pdata->lastrow = 0;
 	pdata->batchsize = 1000;
 	pdata->maxerrors = 10;
 
 	/* argument 1 - the database object */
+	pdata->dbobject = strdup(argv[1]);
+	if (pdata->dbobject == NULL) {
+		fprintf(stderr, "Out of memory!\n");
+		return FALSE;
+	}
 
-	pdata->dbobject = (char *) malloc(strlen(argv[1]) + 1);
-	if (pdata->dbobject != (char *) NULL)
-		strcpy(pdata->dbobject, argv[1]);
-
+	/* argument 2 - the direction */
 	strcpy(pdata->dbdirection, argv[2]);
 
 	if (strcmp(pdata->dbdirection, "in") == 0) {
@@ -194,233 +183,117 @@ process_parameters(int argc, char **argv, PARAMDATA * pdata)
 		return (FALSE);
 	}
 
+	/* argument 3 - the datafile name */
 	strcpy(pdata->hostfilename, argv[3]);
 
-	/* get the rest of the arguments */
-
-	state = GET_NEXTARG;
-
-	for (i = 4; i < argc; i++) {
-		strcpy(arg, argv[i]);
-
-		switch (state) {
-
-		case GET_NEXTARG:
-
-			if (arg[0] != '-') {
-				fprintf(stderr, "Parse error: expected parameter %d to begin with a '-' instead of '%c'.\n", i,
-					arg[0]);
-				return (FALSE);
-			}
-
-			switch (arg[1]) {
-
-			case 'v':
-			case 'V':
-				printf("freebcp version %s\n", software_version);
-				return FALSE;
-				break;
-			case 'm':
-				pdata->mflag++;
-				if (strlen(arg) > 2)
-					pdata->maxerrors = atoi(&arg[2]);
-				else
-					state = GET_MAXERRORS;
-				break;
-			case 'f':
-				pdata->fflag++;
-				if (strlen(arg) > 2)
-					strcpy(pdata->formatfile, &arg[2]);
-				else
-					state = GET_FORMATFILE;
-				break;
-			case 'e':
-				pdata->eflag++;
-				if (strlen(arg) > 2) {
-					pdata->errorfile = (char *) malloc(strlen(arg));
-					strcpy(pdata->errorfile, &arg[2]);
-				} else
-					state = GET_ERRORFILE;
-				break;
-			case 'F':
-				pdata->Fflag++;
-				if (strlen(arg) > 2)
-					pdata->firstrow = atoi(&arg[2]);
-				else
-					state = GET_FIRSTROW;
-				break;
-			case 'L':
-				pdata->Lflag++;
-				if (strlen(arg) > 2)
-					pdata->lastrow = atoi(&arg[2]);
-				else
-					state = GET_LASTROW;
-				break;
-			case 'b':
-				pdata->bflag++;
-				if (strlen(arg) > 2)
-					pdata->batchsize = atoi(&arg[2]);
-				else
-					state = GET_BATCHSIZE;
-				break;
-			case 'n':
-				pdata->nflag++;
-				break;
-			case 'c':
-				pdata->cflag++;
-				break;
-			case 'E':
-				pdata->Eflag++;
-				break;
-			case 'd':
-				tdsdump_open(NULL);
-				break;
-			case 't':
-				pdata->tflag++;
-				if (strlen(arg) > 2) {
-					pdata->fieldterm = (char *) malloc(strlen(arg));
-					strcpy(pdata->fieldterm, &arg[2]);
-					unescape(pdata->fieldterm);
-				} else
-					state = GET_FIELDTERM;
-				break;
-			case 'r':
-				pdata->rflag++;
-				if (strlen(arg) > 2) {
-					pdata->rowterm = (char *) malloc(strlen(arg));
-					strcpy(pdata->rowterm, &arg[2]);
-					unescape(pdata->rowterm);
-				} else
-					state = GET_ROWTERM;
-				break;
-			case 'U':
-				pdata->Uflag++;
-				if (strlen(arg) > 2) {
-					pdata->user = (char *) malloc(strlen(arg));
-					strcpy(pdata->user, &arg[2]);
-				} else
-					state = GET_USER;
-				break;
-			case 'P':
-				pdata->Pflag++;
-				if (strlen(arg) > 2) {
-					pdata->pass = (char *) malloc(strlen(arg));
-					strcpy(pdata->pass, &arg[2]);
-				} else
-					state = GET_PASS;
-				break;
-			case 'I':
-				pdata->Iflag++;
-				if (strlen(arg) > 2)
-					strcpy(pdata->interfacesfile, &arg[2]);
-				else
-					state = GET_INTERFACESFILE;
-				break;
-			case 'S':
-				pdata->Sflag++;
-				if (strlen(arg) > 2) {
-					pdata->server = (char *) malloc(strlen(arg));
-					strcpy(pdata->server, &arg[2]);
-				} else
-					state = GET_SERVER;
-				break;
-			case 'h':	/* hints */
-				if (strlen(arg) > 2) {
-					pdata->hint = (char *) malloc(strlen(arg));
-					strcpy(pdata->hint, &arg[2]);
-				} else
-					state = GET_HINT;
-				break;
-			case 'T':
-				pdata->Tflag++;
-				if (strlen(arg) > 2)
-					pdata->textsize = atoi(&arg[2]);
-				else
-					state = GET_TEXTSIZE;
-				break;
-			case 'A':
-				pdata->Aflag++;
-				if (strlen(arg) > 2)
-					pdata->packetsize = atoi(&arg[2]);
-				else
-					state = GET_PACKETSIZE;
-				break;
-			default:
-				fprintf(stderr, "Unknown option '%c'.\n", arg[1]);
-				return (FALSE);
-
+	/* 
+	 * Get the rest of the arguments 
+	 */
+	optind = 4; /* start processing options after table, direction, & filename */
+	while ((ch = getopt(argc, argv, "m:f:e:F:L:b:t:r:U:P:I:S:h:T:A:O:0:ncEdvV")) != -1) {
+		switch (ch) {
+		case 'v':
+		case 'V':
+			printf("freebcp version %s\n", software_version);
+			return FALSE;
+			break;
+		case 'm':
+			pdata->mflag++;
+			pdata->maxerrors = atoi(optarg);
+			break;
+		case 'f':
+			pdata->fflag++;
+			strcpy(pdata->formatfile, optarg);
+			break;
+		case 'e':
+			pdata->eflag++;
+			pdata->errorfile = strdup(optarg);
+			break;
+		case 'F':
+			pdata->Fflag++;
+			pdata->firstrow = atoi(optarg);
+			break;
+		case 'L':
+			pdata->Lflag++;
+			pdata->lastrow = atoi(optarg);
+			break;
+		case 'b':
+			pdata->bflag++;
+			pdata->batchsize = atoi(optarg);
+			break;
+		case 'n':
+			pdata->nflag++;
+			break;
+		case 'c':
+			pdata->cflag++;
+			break;
+		case 'E':
+			pdata->Eflag++;
+			break;
+		case 'd':
+			tdsdump_open(NULL);
+			break;
+		case 't':
+			pdata->tflag++;
+			pdata->fieldterm = strdup(optarg);
+			pdata->fieldtermlen = unescape(pdata->fieldterm);
+			break;
+		case 'r':
+			pdata->rflag++;
+			pdata->rowterm = strdup(optarg);
+			pdata->rowtermlen = unescape(pdata->rowterm);
+			break;
+		case 'U':
+			pdata->Uflag++;
+			pdata->user = strdup(optarg);
+			break;
+		case 'P':
+			pdata->Pflag++;
+			if ((strcmp(optarg, "-")) == 0) {
+				char pwd[255], *nl;
+				memset(pwd, 0, 255);
+				fgets(pwd, 255, stdin);
+				nl = strchr(pwd, '\n');
+				if(nl) *nl = '\0';
+				pdata->pass = strdup(pwd);
+			} else {
+				pdata->pass = strdup(optarg);
 			}
 			break;
-		case GET_MAXERRORS:
-			pdata->maxerrors = atoi(arg);
-			state = GET_NEXTARG;
+		case 'I':
+			pdata->Iflag++;
+			strcpy(pdata->interfacesfile, optarg);
 			break;
-		case GET_FORMATFILE:
-			strcpy(pdata->formatfile, arg);
-			state = GET_NEXTARG;
+		case 'S':
+			pdata->Sflag++;
+			pdata->server = strdup(optarg);
 			break;
-		case GET_ERRORFILE:
-			pdata->errorfile = (char *) malloc(strlen(arg) + 1);
-			strcpy(pdata->errorfile, arg);
-			state = GET_NEXTARG;
+		case 'h':
+			pdata->hint = strdup(optarg);
 			break;
-		case GET_FIRSTROW:
-			pdata->firstrow = atoi(arg);
-			state = GET_NEXTARG;
+		case 'O':
+		case '0':
+			pdata->options = strdup(optarg);
 			break;
-		case GET_LASTROW:
-			pdata->lastrow = atoi(arg);
-			state = GET_NEXTARG;
+		case 'T':
+			pdata->Tflag++;
+			pdata->textsize = atoi(optarg);
 			break;
-		case GET_BATCHSIZE:
-			pdata->batchsize = atoi(arg);
-			state = GET_NEXTARG;
+		case 'A':
+			pdata->Aflag++;
+			pdata->packetsize = atoi(optarg);
 			break;
-		case GET_FIELDTERM:
-			pdata->fieldterm = (char *) malloc(strlen(arg) + 1);
-			strcpy(pdata->fieldterm, arg);
-			state = GET_NEXTARG;
-			break;
-		case GET_ROWTERM:
-			pdata->rowterm = (char *) malloc(strlen(arg) + 1);
-			strcpy(pdata->rowterm, arg);
-			state = GET_NEXTARG;
-			break;
-		case GET_USER:
-			pdata->user = (char *) malloc(strlen(arg) + 1);
-			strcpy(pdata->user, arg);
-			state = GET_NEXTARG;
-			break;
-		case GET_PASS:
-			pdata->pass = (char *) malloc(strlen(arg) + 1);
-			strcpy(pdata->pass, arg);
-			state = GET_NEXTARG;
-			break;
-		case GET_INTERFACESFILE:
-			strcpy(pdata->interfacesfile, arg);
-			state = GET_NEXTARG;
-			break;
-		case GET_SERVER:
-			pdata->server = (char *) malloc(strlen(arg) + 1);
-			strcpy(pdata->server, arg);
-			state = GET_NEXTARG;
-			break;
-		case GET_TEXTSIZE:
-			pdata->textsize = atoi(arg);
-			state = GET_NEXTARG;
-			break;
-		case GET_PACKETSIZE:
-			pdata->packetsize = atoi(arg);
-			state = GET_NEXTARG;
-			break;
-
+		case '?':
 		default:
-			break;
-
+			pusage();
+			return (FALSE);
 		}
 	}
 
-
+	/* 
+	 * Check for required/disallowed option combinations 
+	 */
+	 
 	/* these must be specified */
 	if (!pdata->Uflag || !pdata->Pflag || !pdata->Sflag) {
 		fprintf(stderr, "All 3 options -U, -P, -S must be supplied.\n");
@@ -433,17 +306,16 @@ process_parameters(int argc, char **argv, PARAMDATA * pdata)
 		return (FALSE);
 	}
 
-	/* character mode file */
+	/* character mode file: Fill in some default values*/
 	if (pdata->cflag) {
-		/* Fill in some default values */
 
 		if (!pdata->tflag || !pdata->fieldterm) {	/* field terminator not specified */
-			pdata->fieldterm = (char *) malloc(2);
-			strcpy(pdata->fieldterm, "\t");
+			pdata->fieldterm = "\t";
+			pdata->fieldtermlen = 1;
 		}
-		if (!pdata->rflag || !pdata->rowterm) {	/* row terminator not specified */
-			pdata->rowterm = (char *) malloc(2);
-			strcpy(pdata->rowterm, "\n");
+		if (!pdata->rflag || !pdata->rowterm) {		/* row terminator not specified */
+			pdata->rowterm =  "\n";
+			pdata->rowtermlen = 1;
 		}
 	}
 
@@ -452,7 +324,7 @@ process_parameters(int argc, char **argv, PARAMDATA * pdata)
 }
 
 int
-login_to_database(PARAMDATA * pdata, DBPROCESS ** pdbproc)
+login_to_database(BCPPARAMDATA * pdata, DBPROCESS ** pdbproc)
 {
 	LOGINREC *login;
 
@@ -461,7 +333,8 @@ login_to_database(PARAMDATA * pdata, DBPROCESS ** pdbproc)
 	if (dbinit() == FAIL)
 		return (FALSE);
 
-	/* Install the user-supplied error-handling and message-handling
+	/*
+	 * Install the user-supplied error-handling and message-handling
 	 * routines. They are defined at the bottom of this source file.
 	 */
 
@@ -497,7 +370,7 @@ login_to_database(PARAMDATA * pdata, DBPROCESS ** pdbproc)
 	 * ** Get a connection to the database.
 	 */
 
-	if ((*pdbproc = dbopen(login, pdata->server)) == (DBPROCESS *) NULL) {
+	if ((*pdbproc = dbopen(login, pdata->server)) == NULL) {
 		fprintf(stderr, "Can't connect to server \"%s\".\n", pdata->server);
 		return (FALSE);
 	}
@@ -516,7 +389,7 @@ login_to_database(PARAMDATA * pdata, DBPROCESS ** pdbproc)
 }
 
 int
-file_character(PARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
+file_character(BCPPARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 {
 	DBINT li_rowsread = 0;
 	int i;
@@ -556,7 +429,7 @@ file_character(PARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 
 	if (pdata->Eflag) {
 
-		dbproc->bcpinfo->identity_insert_on = 1;
+		bcp_control(dbproc, BCPKEEPIDENTITY, 1);
 	
 		if (dbfcmd(dbproc, "set identity_insert %s on", pdata->dbobject) == FAIL) {
 			printf("dbfcmd failed\n");
@@ -571,32 +444,32 @@ file_character(PARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 		while (NO_MORE_RESULTS != dbresults(dbproc));
 	}
 
-	dbproc->hostfileinfo->firstrow = pdata->firstrow;
-	dbproc->hostfileinfo->lastrow = pdata->lastrow;
-	dbproc->hostfileinfo->maxerrs = pdata->maxerrors;
+	bcp_control(dbproc, BCPFIRST, pdata->firstrow);
+	bcp_control(dbproc, BCPLAST, pdata->lastrow);
+	bcp_control(dbproc, BCPMAXERRS, pdata->maxerrors);
 
 	if (bcp_columns(dbproc, li_numcols) == FAIL) {
 		printf("Error in bcp_columns.\n");
 		return FALSE;
 	}
 
-	for (i = 1; i <= li_numcols - 1; i++) {
+	for (i = 1; i < li_numcols; ++i) {
 		if (bcp_colfmt(dbproc, i, SYBCHAR, 0, -1, (const BYTE *) pdata->fieldterm,
-			       strlen(pdata->fieldterm), i) == FAIL) {
+			       pdata->fieldtermlen, i) == FAIL) {
 			printf("Error in bcp_colfmt col %d\n", i);
 			return FALSE;
 		}
 	}
 
 	if (bcp_colfmt(dbproc, li_numcols, SYBCHAR, 0, -1, (const BYTE *) pdata->rowterm,
-		       strlen(pdata->rowterm), li_numcols) == FAIL) {
+		       pdata->rowtermlen, li_numcols) == FAIL) {
 		printf("Error in bcp_colfmt col %d\n", li_numcols);
 		return FALSE;
 	}
 
 	bcp_control(dbproc, BCPBATCH, pdata->batchsize);
 
-	printf("\nStarting copy...\n\n");
+	printf("\nStarting copy...\n");
 
 	if (FAIL == bcp_exec(dbproc, &li_rowsread)) {
 		fprintf(stderr, "bcp copy %s failed\n", (dir == DB_IN) ? "in" : "out");
@@ -609,7 +482,7 @@ file_character(PARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 }
 
 int
-file_native(PARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
+file_native(BCPPARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 {
 	DBINT li_rowsread = 0;
 	int i;
@@ -651,7 +524,7 @@ file_native(PARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 
 	if (pdata->Eflag) {
 
-		dbproc->bcpinfo->identity_insert_on = 1;
+		bcp_control(dbproc, BCPKEEPIDENTITY, 1);
 	
 		if (dbfcmd(dbproc, "set identity_insert %s on", pdata->dbobject) == FAIL) {
 			printf("dbfcmd failed\n");
@@ -666,9 +539,9 @@ file_native(PARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 		while (NO_MORE_RESULTS != dbresults(dbproc));
 	}
 
-	dbproc->hostfileinfo->firstrow = pdata->firstrow;
-	dbproc->hostfileinfo->lastrow = pdata->lastrow;
-	dbproc->hostfileinfo->maxerrs = pdata->maxerrors;
+	bcp_control(dbproc, BCPFIRST, pdata->firstrow);
+	bcp_control(dbproc, BCPLAST, pdata->lastrow);
+	bcp_control(dbproc, BCPMAXERRS, pdata->maxerrors);
 
 	if (bcp_columns(dbproc, li_numcols) == FAIL) {
 		printf("Error in bcp_columns.\n");
@@ -679,7 +552,7 @@ file_native(PARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 		li_coltype = dbcoltype(dbproc, i);
 		li_collen = dbcollen(dbproc, i);
 
-		if (bcp_colfmt(dbproc, i, li_coltype, -1, -1, (BYTE *) NULL, -1, i) == FAIL) {
+		if (bcp_colfmt(dbproc, i, li_coltype, -1, -1, NULL, -1, i) == FAIL) {
 			printf("Error in bcp_colfmt col %d\n", i);
 			return FALSE;
 		}
@@ -699,7 +572,7 @@ file_native(PARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 }
 
 int
-file_formatted(PARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
+file_formatted(BCPPARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 {
 
 	int li_rowsread;
@@ -709,7 +582,7 @@ file_formatted(PARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 
 	if (pdata->Eflag) {
 
-		dbproc->bcpinfo->identity_insert_on = 1;
+		bcp_control(dbproc, BCPKEEPIDENTITY, 1);
 	
 		if (dbfcmd(dbproc, "set identity_insert %s on", pdata->dbobject) == FAIL) {
 			printf("dbfcmd failed\n");
@@ -724,9 +597,9 @@ file_formatted(PARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 		while (NO_MORE_RESULTS != dbresults(dbproc));
 	}
 
-	dbproc->hostfileinfo->firstrow = pdata->firstrow;
-	dbproc->hostfileinfo->lastrow = pdata->lastrow;
-	dbproc->hostfileinfo->maxerrs = pdata->maxerrors;
+	bcp_control(dbproc, BCPFIRST, pdata->firstrow);
+	bcp_control(dbproc, BCPLAST, pdata->lastrow);
+	bcp_control(dbproc, BCPMAXERRS, pdata->maxerrors);
 
 	if (FAIL == bcp_readfmt(dbproc, pdata->formatfile))
 		return FALSE;
@@ -744,6 +617,65 @@ file_formatted(PARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 	return TRUE;
 }
 
+
+int
+setoptions(DBPROCESS * dbproc, BCPPARAMDATA * params){
+	FILE *optFile;
+	char optBuf[256];
+	RETCODE fOK;
+
+	if (dbfcmd(dbproc, "set textsize %d ", params->textsize) == FAIL) {
+		fprintf(stderr, "setoptions() could not set textsize at %s:%d\n", __FILE__, __LINE__);
+		return FALSE;
+	}
+
+	/* 
+	 * If the option is a filename, read the SQL text from the file.  
+	 * Else pass the option verbatim to the server.
+	 */
+	if (params->options) {
+		if ((optFile = fopen(params->options, "r")) == NULL) {
+			if (dbfcmd(dbproc, params->options) == FAIL) {
+				fprintf(stderr, "setoptions() failed preparing options at %s:%d\n", __FILE__, __LINE__);
+				return FALSE;
+			}
+		} else {
+			while (fgets (optBuf, sizeof(optBuf), optFile) != NULL) {
+				if (dbfcmd(dbproc, optBuf) == FAIL) {
+					fprintf(stderr, "setoptions() failed preparing options at %s:%d\n", __FILE__, __LINE__);
+					return FALSE;
+				}
+			}
+			if (!feof (optFile)) {
+				perror("freebcp");
+        			fprintf(stderr, "error reading options file \"%s\" at %s:%d\n", params->options, __FILE__, __LINE__);
+				return FALSE;
+			}
+			fclose(optFile);
+		}
+	
+	}
+	
+	if (dbsqlexec(dbproc) == FAIL) {
+		fprintf(stderr, "setoptions() failed sending options at %s:%d\n", __FILE__, __LINE__);
+		return FALSE;
+	}
+	
+	while ((fOK = dbresults(dbproc)) == SUCCEED) {
+		while ((fOK = dbnextrow(dbproc)) == REG_ROW);
+		if (fOK == FAIL) {
+			fprintf(stderr, "setoptions() failed sending options at %s:%d\n", __FILE__, __LINE__);
+			return FALSE;
+		}
+	}
+	if (fOK == FAIL) {
+		fprintf(stderr, "setoptions() failed sending options at %s:%d\n", __FILE__, __LINE__);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 void
 pusage(void)
 {
@@ -752,7 +684,7 @@ pusage(void)
 	fprintf(stderr, "        [-F firstrow] [-L lastrow] [-b batchsize]\n");
 	fprintf(stderr, "        [-n] [-c] [-t field_terminator] [-r row_terminator]\n");
 	fprintf(stderr, "        [-U username] [-P password] [-I interfaces_file] [-S server]\n");
-	fprintf(stderr, "        [-v] [-d] [-h \"hint [,...]\" \n");
+	fprintf(stderr, "        [-v] [-d] [-h \"hint [,...]\" [-O \"set connection_option on|off, ...]\"\n");
 	fprintf(stderr, "        [-A packet size] [-T text or image size] [-E]\n");
 	fprintf(stderr, "        \n");
 	fprintf(stderr, "example: freebcp testdb.dbo.inserttest in inserttest.txt -S mssql -U guest -P password -c\n");
@@ -761,7 +693,14 @@ pusage(void)
 int
 err_handler(DBPROCESS * dbproc, int severity, int dberr, int oserr, char *dberrstr, char *oserrstr)
 {
+	static int sent = 0;
 
+	if (dberr == SYBEBBCI) { /* Batch successfully bulk copied to the server */
+		int batch = bcp_getbatchsize(dbproc);
+		printf("%d rows sent to SQL Server.\n", sent += batch);
+		return INT_CANCEL;
+	}
+	
 	if (dberr) {
 		fprintf(stderr, "Msg %d, Level %d\n", dberr, severity);
 		fprintf(stderr, "%s\n\n", dberrstr);
@@ -772,15 +711,15 @@ err_handler(DBPROCESS * dbproc, int severity, int dberr, int oserr, char *dberrs
 		fprintf(stderr, "%s\n", dberrstr);
 	}
 
-	return (INT_CANCEL);
+	return INT_CANCEL;
 }
 
 int
 msg_handler(DBPROCESS * dbproc, DBINT msgno, int msgstate, int severity, char *msgtext, char *srvname, char *procname, int line)
 {
 	/*
-	 * ** If it's a database change message, we'll ignore it.
-	 * ** Also ignore language change message.
+	 * If it's a database change message, we'll ignore it.
+	 * Also ignore language change message.
 	 */
 	if (msgno == 5701 || msgno == 5703)
 		return (0);
