@@ -1,4 +1,4 @@
-static char rcsid_freebcp_h[] = "$Id: freebcp.h,v 1.9 2004/06/19 05:56:29 jklowden Exp $";
+static char rcsid_freebcp_h[] = "$Id: freebcp.h,v 1.12 2006/10/06 21:28:20 jklowden Exp $";
 static void *no_unused_freebcp_h_warn[] = { rcsid_freebcp_h, no_unused_freebcp_h_warn };
 
 enum states
@@ -43,11 +43,14 @@ typedef struct pd
 	int maxerrors;
 	int textsize;
 	char *fieldterm;
+	int fieldtermlen;
 	char *rowterm;
+	int rowtermlen;
 	char *user;
 	char *pass;
 	char *server;
 	char *hint;
+	char *options;
 	int packetsize;
 	int mflag;
 	int fflag;
@@ -67,4 +70,4 @@ typedef struct pd
 	int Aflag;
 	int Eflag;
 }
-PARAMDATA;
+BCPPARAMDATA;
