@@ -1,5 +1,5 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 1998-2004  Brian Bruns, Bill Thompson
+ * Copyright (C) 1998-2004, 2005  Brian Bruns, Bill Thompson
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,13 +21,8 @@
 #include <config.h>
 #endif
 
-#include "bkpublic.h"
-
-#include "ctpublic.h"
-#include "ctlib.h"
-#include "replacements.h"
-
-#include <stdlib.h>
+#include <stdarg.h>
+#include <stdio.h>
 
 #if HAVE_STDLIB_H
 #include <stdlib.h>
@@ -37,19 +32,24 @@
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
+#include "bkpublic.h"
+
+#include "ctpublic.h"
+#include "ctlib.h"
+#include "replacements.h"
+
 typedef struct _pbcb
 {
-	unsigned char *pb;
-	int cb;
+	char *pb;
+	unsigned int cb;
 } TDS_PBCB;
 
-static char software_version[] = "$Id: blk.c,v 1.20 2004/10/13 11:06:07 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+TDS_RCSID(var, "$Id: blk.c,v 1.43 2007/12/31 10:06:49 freddy77 Exp $");
 
 static CS_RETCODE _blk_get_col_data(CS_BLKDESC *, TDSCOLUMN *, int );
 static int _blk_add_variable_columns(CS_BLKDESC * blkdesc, int offset, unsigned char * rowbuffer, int start, int *var_cols);
 static CS_RETCODE _blk_add_fixed_columns(CS_BLKDESC * blkdesc, int offset, unsigned char * rowbuffer, int start);
-static CS_RETCODE _blk_build_bcp_record(CS_BLKDESC *blkdesc, CS_INT offset, TDS_INT *record_len);
+static CS_RETCODE _blk_build_bcp_record(CS_BLKDESC *blkdesc, CS_INT offset);
 static CS_RETCODE _blk_send_colmetadata(CS_BLKDESC * blkdesc);
 static CS_RETCODE _blk_build_bulk_insert_stmt(TDS_PBCB * clause, TDSCOLUMN * bcpcol, int first);
 static CS_RETCODE _rowxfer_in_init(CS_BLKDESC * blkdesc);
@@ -75,18 +75,16 @@ CS_RETCODE
 blk_bind(CS_BLKDESC * blkdesc, CS_INT item, CS_DATAFMT * datafmt, CS_VOID * buffer, CS_INT * datalen, CS_SMALLINT * indicator)
 {
 	TDSCOLUMN *colinfo;
-	TDSSOCKET *tds;
-	CS_CONNECTION *con = blkdesc->con;
+	CS_CONNECTION *con;
 	CS_INT bind_count;
 	int i;
 
-	tds = (TDSSOCKET *) blkdesc->con->tds_socket;
-
 	tdsdump_log(TDS_DBG_FUNC, "blk_bind()\n");
 
 	if (!blkdesc) {
 		return CS_FAIL;
 	}
+	con = blkdesc->con;
 
 	if (item == CS_UNUSED) {
 		/* clear all bindings */
@@ -108,7 +106,7 @@ blk_bind(CS_BLKDESC * blkdesc, CS_INT item, CS_DATAFMT * datafmt, CS_VOID * buff
 	/* check item value */
 
 	if (item < 1 || item > blkdesc->bindinfo->num_cols) {
-		_ctclient_msg(blkdesc->con, "blk_bind", 2, 5, 1, 141, "%s, %d", "colnum", item);
+		_ctclient_msg(con, "blk_bind", 2, 5, 1, 141, "%s, %d", "colnum", item);
 		return CS_FAIL;
 	}
 
@@ -183,12 +181,10 @@ blk_default(CS_BLKDESC * blkdesc, CS_INT colnum, CS_VOID * buffer, CS_INT buflen
 CS_RETCODE
 blk_describe(CS_BLKDESC * blkdesc, CS_INT item, CS_DATAFMT * datafmt)
 {
-	TDSSOCKET *tds;
 	TDSCOLUMN *curcol;
 	int len;
 
 	tdsdump_log(TDS_DBG_FUNC, "blk_describe()\n");
-	tds = blkdesc->con->tds_socket;
 
 	if (item < 1 || item > blkdesc->bindinfo->num_cols) {
 		_ctclient_msg(blkdesc->con, "blk_describe", 2, 5, 1, 141, "%s, %d", "colnum", item);
@@ -241,7 +237,8 @@ blk_done(CS_BLKDESC * blkdesc, CS_INT type, CS_INT * outrow)
 	case CS_BLK_BATCH:
 
 		tds_flush_packet(tds);
-		tds->state = TDS_QUERYING;
+		/* TODO correct ?? */
+		tds_set_state(tds, TDS_PENDING);
 		if (tds_process_simple_query(tds) != TDS_SUCCEED) {
 			_ctclient_msg(blkdesc->con, "blk_done", 2, 5, 1, 140, "");
 			return CS_FAIL;
@@ -256,7 +253,7 @@ blk_done(CS_BLKDESC * blkdesc, CS_INT type, CS_INT * outrow)
 			return CS_FAIL;
 		}
 
-		tds->out_flag = 0x07;
+		tds->out_flag = TDS_BULK;
 
 		if (IS_TDS7_PLUS(tds)) {
 			_blk_send_colmetadata(blkdesc);
@@ -267,7 +264,8 @@ blk_done(CS_BLKDESC * blkdesc, CS_INT type, CS_INT * outrow)
 	case CS_BLK_ALL:
 
 		tds_flush_packet(tds);
-		tds->state = TDS_QUERYING;
+		/* TODO correct ?? */
+		tds_set_state(tds, TDS_PENDING);
 		if (tds_process_simple_query(tds) != TDS_SUCCEED) {
 			_ctclient_msg(blkdesc->con, "blk_done", 2, 5, 1, 140, "");
 			return CS_FAIL;
@@ -307,10 +305,8 @@ blk_drop(CS_BLKDESC * blkdesc)
 	if (!blkdesc)
 		return CS_SUCCEED;
 
-	if (blkdesc->tablename)
-		free(blkdesc->tablename);
-	if (blkdesc->insert_stmt)
-		free(blkdesc->insert_stmt);
+	free(blkdesc->tablename);
+	free(blkdesc->insert_stmt);
 	tds_free_results(blkdesc->bindinfo);
 	free(blkdesc);
 
@@ -382,6 +378,7 @@ blk_init(CS_BLKDESC * blkdesc, CS_INT direction, CS_CHAR * tablename, CS_INT tna
 
 	/* string can be no-nul terminated so copy with memcpy */
 	blkdesc->tablename = (char *) malloc(tnamelen + 1);
+	/* FIXME malloc can fail */
 	memcpy(blkdesc->tablename, tablename, tnamelen);
 	blkdesc->tablename[tnamelen] = 0;
 
@@ -398,7 +395,7 @@ blk_init(CS_BLKDESC * blkdesc, CS_INT direction, CS_CHAR * tablename, CS_INT tna
 		return CS_FAIL;
 	}
 
-	while ((rc = tds_process_result_tokens(tds, &result_type, NULL))
+	while ((rc = tds_process_tokens(tds, &result_type, NULL, TDS_TOKEN_RESULTS))
 		   == TDS_SUCCEED) {
 	}
 	if (rc != TDS_NO_MORE_RESULTS) {
@@ -439,6 +436,10 @@ blk_init(CS_BLKDESC * blkdesc, CS_INT direction, CS_CHAR * tablename, CS_INT tna
 		curcol->on_server.column_size = resinfo->columns[i]->on_server.column_size;
 		curcol->char_conv = resinfo->columns[i]->char_conv;
 		memcpy(curcol->column_name, resinfo->columns[i]->column_name, resinfo->columns[i]->column_namelen);
+		if (curcol->table_column_name)
+			TDS_ZERO_FREE(curcol->table_column_name);
+		if (resinfo->columns[i]->table_column_name)
+			curcol->table_column_name = strdup(resinfo->columns[i]->table_column_name);
 		curcol->column_nullable = resinfo->columns[i]->column_nullable;
 		curcol->column_identity = resinfo->columns[i]->column_identity;
 		curcol->column_timestamp = resinfo->columns[i]->column_timestamp;
@@ -454,7 +455,8 @@ blk_init(CS_BLKDESC * blkdesc, CS_INT direction, CS_CHAR * tablename, CS_INT tna
 		}
 	}
 
-	bindinfo->current_row = tds_alloc_row(bindinfo);
+	/* TODO check */
+	tds_alloc_row(bindinfo);
 
 	blkdesc->bindinfo = bindinfo;
 	blkdesc->bind_count = CS_UNUSED;
@@ -466,7 +468,7 @@ blk_init(CS_BLKDESC * blkdesc, CS_INT direction, CS_CHAR * tablename, CS_INT tna
 			return CS_FAIL;
 		}
 	
-		while ((rc = tds_process_result_tokens(tds, &result_type, NULL))
+		while ((rc = tds_process_tokens(tds, &result_type, NULL, TDS_TOKEN_RESULTS))
 			   == TDS_SUCCEED) {
 		}
 		if (rc != TDS_NO_MORE_RESULTS) {
@@ -509,15 +511,13 @@ blk_props(CS_BLKDESC * blkdesc, CS_INT action, CS_INT property, CS_VOID * buffer
 			break;
 		default:
 			_ctclient_msg(blkdesc->con, "blk_props", 2, 5, 1, 141, "%s, %d", "action", action);
-			return CS_FAIL;
 			break;
 		}
 		break;
 
 	default:
 		_ctclient_msg(blkdesc->con, "blk_props", 2, 5, 1, 141, "%s, %d", "property", property);
-		return CS_FAIL;
-
+		break;
 	}
 	return CS_FAIL;
 }
@@ -611,18 +611,16 @@ _blk_rowxfer_out(CS_BLKDESC * blkdesc, CS_INT rows_to_xfer, CS_INT * rows_xferre
 	TDSSOCKET *tds;
 	TDS_INT result_type;
 	TDS_INT ret;
-	TDS_INT rowtype;
-	TDS_INT computeid;
 	TDS_INT temp_count;
 	TDS_INT row_of_query;
 	TDS_INT rows_written;
 
 	tdsdump_log(TDS_DBG_FUNC, "blk_rowxfer_out()\n");
 
-	if (!blkdesc)
+	if (!blkdesc || !blkdesc->con)
 		return CS_FAIL;
 
-	tds = (TDSSOCKET *) blkdesc->con->tds_socket;
+	tds = blkdesc->con->tds_socket;
 
 	/*
 	 * the first time blk_xfer called after blk_init()
@@ -637,7 +635,7 @@ _blk_rowxfer_out(CS_BLKDESC * blkdesc, CS_INT rows_to_xfer, CS_INT * rows_xferre
 			return CS_FAIL;
 		}
 	
-		while ((ret = tds_process_result_tokens(tds, &result_type, NULL)) == TDS_SUCCEED) {
+		while ((ret = tds_process_tokens(tds, &result_type, NULL, TDS_TOKEN_RESULTS)) == TDS_SUCCEED) {
 			if (result_type == TDS_ROW_RESULT)
 				break;
 		}
@@ -658,28 +656,29 @@ _blk_rowxfer_out(CS_BLKDESC * blkdesc, CS_INT rows_to_xfer, CS_INT * rows_xferre
 
 	for (temp_count = 0; temp_count < rows_to_xfer; temp_count++) {
 
-		ret = tds_process_row_tokens_ct(tds, &rowtype, &computeid);
+		ret = tds_process_tokens(tds, &result_type, NULL, TDS_STOPAT_ROWFMT|TDS_STOPAT_DONE|TDS_RETURN_ROW|TDS_RETURN_COMPUTE);
 
 		tdsdump_log(TDS_DBG_FUNC, "blk_rowxfer_out() process_row_tokens returned %d\n", ret);
 
 		switch (ret) {
-			case TDS_SUCCEED: 
-				if (rowtype == TDS_REG_ROW) {
+		case TDS_SUCCEED:
+			if (result_type == TDS_ROW_RESULT || result_type == TDS_COMPUTE_RESULT) {
+				if (result_type == TDS_ROW_RESULT) {
 					if (_ct_bind_data( blkdesc->con->ctx, tds->current_results, blkdesc->bindinfo, temp_count))
 						return CS_ROW_FAIL;
 					if (rows_xferred)
 						*rows_xferred = *rows_xferred + 1;
 				}
 				break;
-		
-			case TDS_NO_MORE_ROWS: 
-				return CS_END_DATA;
-				break;
+			}
+		case TDS_NO_MORE_RESULTS: 
+			return CS_END_DATA;
+			break;
 
-			default:
-				_ctclient_msg(blkdesc->con, "blk_rowxfer", 2, 5, 1, 140, "");
-				return CS_FAIL;
-				break;
+		default:
+			_ctclient_msg(blkdesc->con, "blk_rowxfer", 2, 5, 1, 140, "");
+			return CS_FAIL;
+			break;
 		}
 	} 
 
@@ -692,13 +691,11 @@ _blk_rowxfer_in(CS_BLKDESC * blkdesc, CS_INT rows_to_xfer, CS_INT * rows_xferred
 
 	TDSSOCKET *tds;
 	TDS_INT each_row;
-	TDS_INT record_len;
-	unsigned char row_token = 0xd1;
 
 	if (!blkdesc)
 		return CS_FAIL;
 
-	tds = (TDSSOCKET *) blkdesc->con->tds_socket;
+	tds = blkdesc->con->tds_socket;
 
 	/*
 	 * the first time blk_xfer called after blk_init()
@@ -717,7 +714,7 @@ _blk_rowxfer_in(CS_BLKDESC * blkdesc, CS_INT rows_to_xfer, CS_INT * rows_xferred
 
 
 		/* set packet type to send bulk data */
-		tds->out_flag = 0x07;
+		tds->out_flag = TDS_BULK;
 
 		if (IS_TDS7_PLUS(tds)) {
 			_blk_send_colmetadata(blkdesc);
@@ -728,12 +725,8 @@ _blk_rowxfer_in(CS_BLKDESC * blkdesc, CS_INT rows_to_xfer, CS_INT * rows_xferred
 
 	for (each_row = 0; each_row < rows_to_xfer; each_row++ ) {
 
-		if (_blk_build_bcp_record(blkdesc, each_row, &record_len) == CS_SUCCEED) {
+		if (_blk_build_bcp_record(blkdesc, each_row) == CS_SUCCEED) {
 	
-			if (IS_TDS7_PLUS(tds)) {
-				tds_put_byte(tds, row_token);	/* 0xd1 */
-				tds_put_n(tds, blkdesc->bindinfo->current_row, record_len);
-			} 
 		}
 	}
 
@@ -756,7 +749,7 @@ _rowxfer_in_init(CS_BLKDESC * blkdesc)
 	int bcp_record_size       = 0;
 
 	char *query;
-	unsigned char clause_buffer[4096] = { 0 };
+	char clause_buffer[4096] = { 0 };
 
 	TDS_PBCB colclause;
 
@@ -793,6 +786,7 @@ _rowxfer_in_init(CS_BLKDESC * blkdesc)
 		}
 
 	} else {
+		/* NOTE: if we use "with nodescribe" for following inserts server do not send describe */
 		if (asprintf(&query, "insert bulk %s", blkdesc->tablename) < 0) {
 			return CS_FAIL;
 		}
@@ -813,6 +807,9 @@ _rowxfer_in_init(CS_BLKDESC * blkdesc)
 		return CS_FAIL;
 	}
 
+	/* FIXME find a better way, some other thread could change state here */
+	tds_set_state(tds, TDS_QUERYING);
+
 	/* 
 	 * Work out the number of "variable" columns.  These are either nullable or of 
 	 * varying length type e.g. varchar.   
@@ -825,7 +822,7 @@ _rowxfer_in_init(CS_BLKDESC * blkdesc)
 			bcpcol = blkdesc->bindinfo->columns[i];
 
 			/*
-			 * work out storage required for thsi datatype
+			 * work out storage required for this datatype
 			 * blobs always require 16, numerics vary, the
 			 * rest can be taken from the server
 			 */
@@ -1010,8 +1007,8 @@ _blk_build_bulk_insert_stmt(TDS_PBCB * clause, TDSCOLUMN * bcpcol, int first)
 		return CS_FAIL;
 	}
 
-	if (clause->cb < strlen(clause->pb) + strlen(bcpcol->column_name) + strlen(column_type) + ((first) ? 2 : 4)) {
-		unsigned char *temp = malloc(2 * clause->cb);
+	if (clause->cb < strlen(clause->pb) + strlen(bcpcol->column_name) + strlen(column_type) + ((first) ? 2u : 4u)) {
+		char *temp = malloc(2 * clause->cb);
 
 		if (!temp)
 			return CS_FAIL;
@@ -1070,7 +1067,10 @@ _blk_send_colmetadata(CS_BLKDESC * blkdesc)
 			continue;
 		}
 
-		tds_put_smallint(tds, bcpcol->column_usertype);
+		if (IS_TDS90(tds))
+			tds_put_int(tds, bcpcol->column_usertype);
+		else
+			tds_put_smallint(tds, bcpcol->column_usertype);
 		tds_put_smallint(tds, bcpcol->column_flags);
 		tds_put_byte(tds, bcpcol->on_server.column_type);
 
@@ -1094,7 +1094,7 @@ _blk_send_colmetadata(CS_BLKDESC * blkdesc)
 			tds_put_byte(tds, bcpcol->column_prec);
 			tds_put_byte(tds, bcpcol->column_scale);
 		}
-		if (IS_TDS80(tds)
+		if (IS_TDS8_PLUS(tds)
 			&& is_collate_type(bcpcol->on_server.column_type)) {
 			tds_put_n(tds, bcpcol->column_collation, 5);
 		}
@@ -1102,6 +1102,7 @@ _blk_send_colmetadata(CS_BLKDESC * blkdesc)
 			tds_put_smallint(tds, strlen(blkdesc->tablename));
 			tds_put_string(tds, blkdesc->tablename, strlen(blkdesc->tablename));
 		}
+		/* FIXME support multibyte string */
 		tds_put_byte(tds, bcpcol->column_namelen);
 		tds_put_string(tds, bcpcol->column_name, bcpcol->column_namelen);
 
@@ -1110,7 +1111,7 @@ _blk_send_colmetadata(CS_BLKDESC * blkdesc)
 }
 
 static CS_RETCODE
-_blk_build_bcp_record(CS_BLKDESC *blkdesc, CS_INT offset, TDS_INT *record_len)
+_blk_build_bcp_record(CS_BLKDESC *blkdesc, CS_INT offset)
 {
 	TDSSOCKET  *tds = blkdesc->con->tds_socket;
 	TDSCOLUMN  *bindcol;
@@ -1123,6 +1124,7 @@ _blk_build_bcp_record(CS_BLKDESC *blkdesc, CS_INT offset, TDS_INT *record_len)
 	static const unsigned char timestamp[] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };
 
 	static const TDS_TINYINT textptr_size = 16;
+	const unsigned char row_token = 0xd1;
 
 	unsigned char *record;
 	TDS_INT	 old_record_size;
@@ -1168,9 +1170,9 @@ _blk_build_bcp_record(CS_BLKDESC *blkdesc, CS_INT offset, TDS_INT *record_len)
 	 			return CS_FAIL;
 			}
 			tdsdump_log(TDS_DBG_INFO1, "gotten column %d length %d null %d\n",
-					i + 1, bindcol->bcp_column_data->datalen, bindcol->bcp_column_data->null_column);
+					i + 1, bindcol->bcp_column_data->datalen, bindcol->bcp_column_data->is_null);
 	
-			if (bindcol->bcp_column_data->null_column) {
+			if (bindcol->bcp_column_data->is_null) {
 				if (bindcol->column_nullable) {
 					switch (bindcol->on_server.column_type) {
 					case XSYBCHAR:
@@ -1231,14 +1233,12 @@ _blk_build_bcp_record(CS_BLKDESC *blkdesc, CS_INT offset, TDS_INT *record_len)
 
 #if WORDS_BIGENDIAN
 				tds_swap_datatype(bindcol->on_server.column_type, bindcol->bcp_column_data->data);
-#else
-				if (is_numeric_type(bindcol->on_server.column_type)) {
-					tds_swap_datatype(bindcol->on_server.column_type, bindcol->bcp_column_data->data);
-				}
 #endif
 
 				if (is_numeric_type(bindcol->on_server.column_type)) {
 					CS_NUMERIC *num = (CS_NUMERIC *) bindcol->bcp_column_data->data;
+					if (IS_TDS7_PLUS(tds))
+						tds_swap_numeric((TDS_NUMERIC *) num);
 					memcpy(record, num->array, tds_numeric_bytes_per_prec[num->precision]);
 					record += tds_numeric_bytes_per_prec[num->precision]; 
 					new_record_size += tds_numeric_bytes_per_prec[num->precision];
@@ -1251,6 +1251,9 @@ _blk_build_bcp_record(CS_BLKDESC *blkdesc, CS_INT offset, TDS_INT *record_len)
 			tdsdump_log(TDS_DBG_INFO1, "old_record_size = %d new size = %d \n",
 					old_record_size, new_record_size);
 		}
+
+		tds_put_byte(tds, row_token);   /* 0xd1 */
+		tds_put_n(tds, blkdesc->bindinfo->current_row, new_record_size);
 	}  /* IS_TDS7_PLUS */
 	else {
 			memset(record, '\0', old_record_size);	/* zero the rowbuffer */
@@ -1311,9 +1314,7 @@ _blk_build_bcp_record(CS_BLKDESC *blkdesc, CS_INT offset, TDS_INT *record_len)
 
 				}
 			}
-			return CS_SUCCEED;
 	}
-	*record_len = new_record_size;
 
 	return CS_SUCCEED;
 }
@@ -1342,7 +1343,7 @@ _blk_add_fixed_columns(CS_BLKDESC * blkdesc, int offset, unsigned char * rowbuff
 		 		return CS_FAIL;
 			}
 
-			if (bcpcol->bcp_column_data->null_column) {
+			if (bcpcol->bcp_column_data->is_null) {
 				/* No value or default value available and NULL not allowed. col = %d row = %d. */
 				_ctclient_msg(blkdesc->con, "blk_rowxfer", 2, 7, 1, 142, "%d, %d",  i + 1, offset + 1);
 				return CS_FAIL;
@@ -1426,7 +1427,7 @@ _blk_add_variable_columns(CS_BLKDESC * blkdesc, int offset, unsigned char * rowb
 			 * throw an error
 			 */
 
-			if (!(bcpcol->column_nullable) && bcpcol->bcp_column_data->null_column) {
+			if (!(bcpcol->column_nullable) && bcpcol->bcp_column_data->is_null) {
 				/* No value or default value available and NULL not allowed. col = %d row = %d. */
 				_ctclient_msg(blkdesc->con, "blk_rowxfer", 2, 7, 1, 142, "%d, %d",  i + 1, offset + 1);
 				return CS_FAIL;
@@ -1441,7 +1442,7 @@ _blk_add_variable_columns(CS_BLKDESC * blkdesc, int offset, unsigned char * rowb
 				memcpy(&rowbuffer[row_pos], num->array, cpbytes);
 			} else {
 				/* compute the length to copy to the row ** buffer */
-				if (bcpcol->bcp_column_data->null_column) {
+				if (bcpcol->bcp_column_data->is_null) {
 					cpbytes = 0;
 				} else {
 					cpbytes = bcpcol->bcp_column_data->datalen > bcpcol->column_size ? bcpcol->column_size : bcpcol->bcp_column_data->datalen;
@@ -1584,7 +1585,8 @@ _blk_get_col_data(CS_BLKDESC *blkdesc, TDSCOLUMN *bindcol, int offset)
 					case CS_SMALLINT_TYPE:  srclen = 2; break;
 					case CS_TINYINT_TYPE:   srclen = 1; break;
 					default:
-						printf("error not fixed length type (%d) and datalen not specified\n", bindcol->column_bindtype);
+						printf("error not fixed length type (%d) and datalen not specified\n",
+							bindcol->column_bindtype);
 						return CS_FAIL;
 				}
 
@@ -1612,14 +1614,14 @@ _blk_get_col_data(CS_BLKDESC *blkdesc, TDSCOLUMN *bindcol, int offset)
 	
 			/* if convert return FAIL mark error but process other columns */
 			if ((result = cs_convert(ctx, &srcfmt, (CS_VOID *) src, 
-									&destfmt, (CS_VOID *) bindcol->bcp_column_data->data, &destlen) != CS_SUCCEED)) {
+						 &destfmt, (CS_VOID *) bindcol->bcp_column_data->data, &destlen)) != CS_SUCCEED) {
 				tdsdump_log(TDS_DBG_INFO1, "convert failed for %d \n", srcfmt.datatype);
 				return CS_FAIL;
 			}
 		}
 
 		bindcol->bcp_column_data->datalen = destlen;
-		bindcol->bcp_column_data->null_column = null_column;
+		bindcol->bcp_column_data->is_null = null_column;
 
 		return CS_SUCCEED;
 
