@@ -1,5 +1,5 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004  Brian Bruns
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005 Brian Bruns
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,6 +21,8 @@
 #include <config.h>
 #endif
 
+#include <stdarg.h>
+
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
@@ -47,8 +49,7 @@
 #include "tdsconvert.h"
 #include "replacements.h"
 
-static char software_version[] = "$Id: cs.c,v 1.53 2004/10/14 08:16:43 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+TDS_RCSID(var, "$Id: cs.c,v 1.65 2007/12/23 21:12:02 jklowden Exp $");
 
 static int _cs_datatype_length(int dtype);
 static CS_INT cs_diag_storemsg(CS_CONTEXT *context, CS_CLIENTMSG *message);
@@ -56,8 +57,10 @@ static CS_INT cs_diag_clearmsg(CS_CONTEXT *context, CS_INT type);
 static CS_INT cs_diag_getmsg(CS_CONTEXT *context, CS_INT idx, CS_CLIENTMSG *message);
 static CS_INT cs_diag_countmsg(CS_CONTEXT *context, CS_INT *count);
 
-/* 	returns the fixed length of the specified data type, or 0 if not a 
-	fixed length data type */
+/**
+ * returns the fixed length of the specified data type, or 0 if not a 
+ * fixed length data type
+ */
 static int 
 _cs_datatype_length(int dtype)
 {
@@ -190,6 +193,119 @@ _csclient_msg(CS_CONTEXT * ctx, const char *funcname, int layer, int origin, int
 	va_end(ap);
 }
 
+/**
+ * Allocate new CS_LOCALE and initialize it
+ * returns NULL on out of memory errors 
+ */
+static CS_LOCALE *
+_cs_locale_alloc(void)
+{
+	return (CS_LOCALE *) calloc(1, sizeof(CS_LOCALE));
+}
+
+static void
+_cs_locale_free_contents(CS_LOCALE *locale)
+{
+	/* free strings */
+	free(locale->language);
+	locale->language = NULL;
+	free(locale->charset);
+	locale->charset = NULL;
+	free(locale->time);
+	locale->time = NULL;
+	free(locale->collate);
+	locale->collate = NULL;
+}
+
+void 
+_cs_locale_free(CS_LOCALE *locale)
+{
+	/* free contents */
+	_cs_locale_free_contents(locale);
+
+	/* free the data structure */
+	free(locale);
+}
+
+/* returns 0 on out of memory errors, 1 for OK */
+int
+_cs_locale_copy_inplace(CS_LOCALE *new_locale, CS_LOCALE *orig)
+{
+	_cs_locale_free_contents(new_locale);
+	if (orig->language) {
+		new_locale->language = strdup(orig->language);
+		if (!new_locale->language)
+			goto Cleanup;
+	}
+
+	if (orig->charset) {
+		new_locale->charset = strdup(orig->charset);
+		if (!new_locale->charset)
+			goto Cleanup;
+	}
+
+	if (orig->time) {
+		new_locale->time = strdup(orig->time);
+		if (!new_locale->time)
+			goto Cleanup;
+	}
+
+	if (orig->collate) {
+		new_locale->collate = strdup(orig->collate);
+		if (!new_locale->collate)
+			goto Cleanup;
+	}
+
+	return 1;
+
+Cleanup:
+	_cs_locale_free_contents(new_locale);
+	return 0;
+}
+
+/* returns NULL on out of memory errors */
+CS_LOCALE *
+_cs_locale_copy(CS_LOCALE *orig)
+{
+	CS_LOCALE *new_locale;
+
+	new_locale = _cs_locale_alloc();
+	if (!new_locale)
+		return NULL;
+
+	if (orig->language) {
+		new_locale->language = strdup(orig->language);
+		if (!new_locale->language)
+			goto Cleanup;
+	}
+
+	if (orig->charset) {
+		new_locale->charset = strdup(orig->charset);
+		if (!new_locale->charset)
+			goto Cleanup;
+	}
+
+	if (orig->time) {
+		new_locale->time = strdup(orig->time);
+		if (!new_locale->time)
+			goto Cleanup;
+	}
+
+	if (orig->collate) {
+		new_locale->collate = strdup(orig->collate);
+		if (!new_locale->collate)
+			goto Cleanup;
+	}
+
+	return new_locale;
+
+Cleanup:
+	_cs_locale_free(new_locale);
+	return NULL;
+}
+
+extern const char STD_DATETIME_FMT[];
+
 CS_RETCODE
 cs_ctx_alloc(CS_INT version, CS_CONTEXT ** ctx)
 {
@@ -197,12 +313,15 @@ TDSCONTEXT *tds_ctx;
 
 	*ctx = (CS_CONTEXT *) malloc(sizeof(CS_CONTEXT));
 	memset(*ctx, '\0', sizeof(CS_CONTEXT));
-	tds_ctx = tds_alloc_context();
-	tds_ctx_set_parent(tds_ctx, *ctx);
+	tds_ctx = tds_alloc_context(*ctx);
+	if (!tds_ctx) {
+		free(*ctx);
+		return CS_FAIL;
+	}
 	(*ctx)->tds_ctx = tds_ctx;
 	if (tds_ctx->locale && !tds_ctx->locale->date_fmt) {
 		/* set default in case there's no locale file */
-		tds_ctx->locale->date_fmt = strdup("%b %e %Y %I:%M%p");
+		tds_ctx->locale->date_fmt = strdup(STD_DATETIME_FMT);
 	}
 	return CS_SUCCEED;
 }
@@ -228,8 +347,7 @@ cs_ctx_drop(CS_CONTEXT * ctx)
 {
 	if (ctx) {
 		_ct_diag_clearmsg(ctx, CS_ALLMSG_TYPE);
-		if (ctx->userdata)
-			free(ctx->userdata);
+		free(ctx->userdata);
 		if (ctx->tds_ctx)
 			tds_free_context(ctx->tds_ctx);
 		free(ctx);
@@ -268,35 +386,64 @@ CS_INT maxcp;
 			break;
 		}
 	}
-	/* CS_SET */
-	switch (property) {
-	case CS_MESSAGE_CB:
-		if ( ctx->cs_errhandletype == _CS_ERRHAND_INLINE) {
-			cs_diag_clearmsg(ctx, CS_UNUSED);
-		}
-		ctx->_cslibmsg_cb = (CS_CSLIBMSG_FUNC) buffer;
-		ctx->cs_errhandletype = _CS_ERRHAND_CB;
-		return CS_SUCCEED;
-	case CS_USERDATA:
-		if (ctx->userdata)
+	if (action == CS_SET) {
+		switch (property) {
+		case CS_MESSAGE_CB:
+			if ( ctx->cs_errhandletype == _CS_ERRHAND_INLINE) {
+				cs_diag_clearmsg(ctx, CS_UNUSED);
+			}
+			ctx->_cslibmsg_cb = (CS_CSLIBMSG_FUNC) buffer;
+			ctx->cs_errhandletype = _CS_ERRHAND_CB;
+			return CS_SUCCEED;
+		case CS_USERDATA:
 			free(ctx->userdata);
-
-		if (buflen == CS_NULLTERM) {
-			maxcp = strlen(buffer) + 1;
-		} else {
-			maxcp = buflen;
+	
+			if (buflen == CS_NULLTERM) {
+				maxcp = strlen(buffer) + 1;
+			} else {
+				maxcp = buflen;
+			}
+			ctx->userdata = (void *) malloc(maxcp);
+			if ( ctx->userdata == NULL) {
+				return CS_FAIL;
+			}
+			ctx->userdata_len = maxcp;
+	
+			if (buffer) {
+				memcpy(ctx->userdata, buffer, maxcp);
+			} else {
+				return CS_FAIL;
+			}
+			return CS_SUCCEED;
+	
+		case CS_EXTRA_INF:
+		case CS_LOC_PROP:
+		case CS_VERSION:
+			return CS_FAIL;
+			break;
+		}
+	}
+	if (action == CS_CLEAR) {
+		switch (property) {
+		case CS_MESSAGE_CB:
+			if ( ctx->cs_errhandletype == _CS_ERRHAND_INLINE) {
+				cs_diag_clearmsg(ctx, CS_UNUSED);
+			}
+			ctx->_cslibmsg_cb = NULL;
+			ctx->cs_errhandletype = 0;
+			return CS_SUCCEED;
+		case CS_USERDATA:
+			free(ctx->userdata);
+			ctx->userdata = NULL;
+	
+			return CS_SUCCEED;
+	
+		case CS_EXTRA_INF:
+		case CS_LOC_PROP:
+		case CS_VERSION:
+			return CS_FAIL;
+			break;
 		}
-		ctx->userdata = (void *) malloc(maxcp);
-		ctx->userdata_len = maxcp;
-
-		memcpy(ctx->userdata, buffer, maxcp);
-		return CS_SUCCEED;
-
-	case CS_EXTRA_INF:
-	case CS_LOC_PROP:
-	case CS_VERSION:
-		return CS_FAIL;
-		break;
 	}
 	return CS_FAIL;
 }
@@ -335,7 +482,7 @@ CS_RETCODE ret;
 
 		tdsdump_log(TDS_DBG_FUNC, "cs_convert() srcdata is null\n");
 		memset(dest, '\0', destlen);
-		if (resultlen != (CS_INT *) NULL)
+		if (resultlen != NULL)
 			*resultlen = 0;
 		return CS_SUCCEED;
 
@@ -360,13 +507,13 @@ CS_RETCODE ret;
 					memcpy(dest, srcdata, src_len);
 					for (i = src_len; i < destlen; i++)
 						dest[i] = '\0';
-					if (resultlen != (CS_INT *) NULL)
+					if (resultlen != NULL)
 						*resultlen = destlen;
 					ret = CS_SUCCEED;
 					break;
 				case CS_FMT_UNUSED:
 					memcpy(dest, srcdata, src_len);
-					if (resultlen != (CS_INT *) NULL)
+					if (resultlen != NULL)
 						*resultlen = src_len;
 					ret = CS_SUCCEED;
 					break;
@@ -394,7 +541,7 @@ CS_RETCODE ret;
 					} else {
 						memcpy(dest, srcdata, src_len);
 						dest[src_len] = '\0';
-						if (resultlen != (CS_INT *) NULL)
+						if (resultlen != NULL)
 							*resultlen = src_len + 1;
 						ret = CS_SUCCEED;
 					}
@@ -404,7 +551,7 @@ CS_RETCODE ret;
 					memcpy(dest, srcdata, src_len);
 					for (i = src_len; i < destlen; i++)
 						dest[i] = ' ';
-					if (resultlen != (CS_INT *) NULL)
+					if (resultlen != NULL)
 						*resultlen = destlen;
 					ret = CS_SUCCEED;
 					break;
@@ -413,13 +560,13 @@ CS_RETCODE ret;
 					memcpy(dest, srcdata, src_len);
 					for (i = src_len; i < destlen; i++)
 						dest[i] = '\0';
-					if (resultlen != (CS_INT *) NULL)
+					if (resultlen != NULL)
 						*resultlen = destlen;
 					ret = CS_SUCCEED;
 					break;
 				case CS_FMT_UNUSED:
 					memcpy(dest, srcdata, src_len);
-					if (resultlen != (CS_INT *) NULL)
+					if (resultlen != NULL)
 						*resultlen = src_len;
 					ret = CS_SUCCEED;
 					break;
@@ -442,19 +589,20 @@ CS_RETCODE ret;
 		case SYBDATETIME:
 		case SYBDATETIME4:
 			memcpy(dest, srcdata, _cs_datatype_length(src_type));
-			if (resultlen != (CS_INT *) NULL)
+			if (resultlen != NULL)
 				*resultlen = _cs_datatype_length(src_type);
 			ret = CS_SUCCEED;
 			break;
 
-		case SYBBITN:
 		case SYBNUMERIC:
 		case SYBDECIMAL:
+			src_len = tds_numeric_bytes_per_prec[((TDS_NUMERIC *) srcdata)->precision] + 2;
+		case SYBBITN:
 			if (src_len > destlen) {
 				ret = CS_FAIL;
 			} else {
 				memcpy(dest, srcdata, src_len);
-				if (resultlen != (CS_INT *) NULL)
+				if (resultlen != NULL)
 					*resultlen = src_len;
 				ret = CS_SUCCEED;
 			}
@@ -527,7 +675,7 @@ CS_RETCODE ret;
 			free(cres.ib);
 			for (i = len; i < destlen; i++)
 				dest[i] = '\0';
-			if (resultlen != (CS_INT *) NULL)
+			if (resultlen != NULL)
 				*resultlen = destlen;
 			ret = CS_SUCCEED;
 		}
@@ -537,37 +685,37 @@ CS_RETCODE ret;
 		/* fall trough, act same way of TINYINT */
 	case SYBINT1:
 		memcpy(dest, &(cres.ti), 1);
-		if (resultlen != (CS_INT *) NULL)
+		if (resultlen != NULL)
 			*resultlen = 1;
 		ret = CS_SUCCEED;
 		break;
 	case SYBINT2:
 		memcpy(dest, &(cres.si), 2);
-		if (resultlen != (CS_INT *) NULL)
+		if (resultlen != NULL)
 			*resultlen = 2;
 		ret = CS_SUCCEED;
 		break;
 	case SYBINT4:
 		memcpy(dest, &(cres.i), 4);
-		if (resultlen != (CS_INT *) NULL)
+		if (resultlen != NULL)
 			*resultlen = 4;
 		ret = CS_SUCCEED;
 		break;
 	case SYBINT8:
 		memcpy(dest, &(cres.bi), 8);
-		if (resultlen != (CS_INT *) NULL)
+		if (resultlen != NULL)
 			*resultlen = 8;
 		ret = CS_SUCCEED;
 		break;
 	case SYBFLT8:
 		memcpy(dest, &(cres.f), 8);
-		if (resultlen != (CS_INT *) NULL)
+		if (resultlen != NULL)
 			*resultlen = 8;
 		ret = CS_SUCCEED;
 		break;
 	case SYBREAL:
 		memcpy(dest, &(cres.r), 4);
-		if (resultlen != (CS_INT *) NULL)
+		if (resultlen != NULL)
 			*resultlen = 4;
 		ret = CS_SUCCEED;
 		break;
@@ -575,33 +723,34 @@ CS_RETCODE ret;
 
 		tdsdump_log(TDS_DBG_FUNC, "cs_convert() copying %d bytes to src\n", (int) sizeof(TDS_MONEY));
 		memcpy(dest, &(cres.m), sizeof(TDS_MONEY));
-		if (resultlen != (CS_INT *) NULL)
+		if (resultlen != NULL)
 			*resultlen = sizeof(TDS_MONEY);
 		ret = CS_SUCCEED;
 		break;
 	case SYBMONEY4:
 		memcpy(dest, &(cres.m4), sizeof(TDS_MONEY4));
-		if (resultlen != (CS_INT *) NULL)
+		if (resultlen != NULL)
 			*resultlen = sizeof(TDS_MONEY4);
 		ret = CS_SUCCEED;
 		break;
 	case SYBDATETIME:
 		memcpy(dest, &(cres.dt), sizeof(TDS_DATETIME));
-		if (resultlen != (CS_INT *) NULL)
+		if (resultlen != NULL)
 			*resultlen = sizeof(TDS_DATETIME);
 		ret = CS_SUCCEED;
 		break;
 	case SYBDATETIME4:
 		memcpy(dest, &(cres.dt4), sizeof(TDS_DATETIME4));
-		if (resultlen != (CS_INT *) NULL)
+		if (resultlen != NULL)
 			*resultlen = sizeof(TDS_DATETIME4);
 		ret = CS_SUCCEED;
 		break;
 	case SYBNUMERIC:
 	case SYBDECIMAL:
-		memcpy(dest, &(cres.n), sizeof(TDS_NUMERIC));
-		if (resultlen != (CS_INT *) NULL)
-			*resultlen = sizeof(TDS_NUMERIC);
+		src_len = tds_numeric_bytes_per_prec[cres.n.precision] + 2;
+		memcpy(dest, &(cres.n), src_len);
+		if (resultlen != NULL)
+			*resultlen = src_len;
 		ret = CS_SUCCEED;
 		break;
 	case SYBCHAR:
@@ -621,7 +770,7 @@ CS_RETCODE ret;
 				} else {
 					memcpy(dest, cres.c, len);
 					dest[len] = 0;
-					if (resultlen != (CS_INT *) NULL)
+					if (resultlen != NULL)
 						*resultlen = len + 1;
 					ret = CS_SUCCEED;
 				}
@@ -633,7 +782,7 @@ CS_RETCODE ret;
 				memcpy(dest, cres.c, len);
 				for (i = len; i < destlen; i++)
 					dest[i] = ' ';
-				if (resultlen != (CS_INT *) NULL)
+				if (resultlen != NULL)
 					*resultlen = destlen;
 				ret = CS_SUCCEED;
 				break;
@@ -644,14 +793,14 @@ CS_RETCODE ret;
 				memcpy(dest, cres.c, len);
 				for (i = len; i < destlen; i++)
 					dest[i] = '\0';
-				if (resultlen != (CS_INT *) NULL)
+				if (resultlen != NULL)
 					*resultlen = destlen;
 				ret = CS_SUCCEED;
 				break;
 			case CS_FMT_UNUSED:
 				tdsdump_log(TDS_DBG_FUNC, "cs_convert() FMT_UNUSED\n");
 				memcpy(dest, cres.c, len);
-				if (resultlen != (CS_INT *) NULL)
+				if (resultlen != NULL)
 					*resultlen = len;
 				ret = CS_SUCCEED;
 				break;
@@ -706,19 +855,207 @@ TDSDATEREC dr;
 CS_RETCODE
 cs_loc_alloc(CS_CONTEXT * ctx, CS_LOCALE ** locptr)
 {
+	CS_LOCALE *tds_csloc;
+
+	tds_csloc = _cs_locale_alloc();
+	if (!tds_csloc)
+		return CS_FAIL;
+
+	*locptr = tds_csloc;
 	return CS_SUCCEED;
 }
 
 CS_RETCODE
 cs_loc_drop(CS_CONTEXT * ctx, CS_LOCALE * locale)
 {
+	if (!locale)
+		return CS_FAIL;
+
+	_cs_locale_free(locale);
 	return CS_SUCCEED;
 }
 
 CS_RETCODE
 cs_locale(CS_CONTEXT * ctx, CS_INT action, CS_LOCALE * locale, CS_INT type, CS_VOID * buffer, CS_INT buflen, CS_INT * outlen)
 {
-	return CS_SUCCEED;
+	CS_RETCODE code = CS_FAIL;
+
+	if (action == CS_SET) {
+		switch (type) {
+		case CS_LC_ALL:
+			/* what to do here if there is locale data? */
+			if (!buffer) {
+				code = CS_SUCCEED;
+			}
+			break;
+
+		case CS_SYB_CHARSET:
+			if (buflen == CS_NULLTERM) {
+				buflen = strlen((char *)buffer);
+			}
+			
+			free(locale->charset);
+			locale->charset = (char *)malloc(buflen + 1);
+			if (!locale->charset)
+				break;
+
+			strncpy(locale->charset, (char *)buffer, buflen);
+			locale->charset[buflen] = '\0';
+			code = CS_SUCCEED;
+			break;
+
+		case CS_SYB_LANG:
+			if (buflen == CS_NULLTERM) {
+				buflen = strlen((char *)buffer);
+			}
+			
+			free(locale->language);
+			locale->language = (char *)malloc(buflen + 1);
+			if (!locale->language)
+				break;
+
+			strncpy(locale->language, (char *)buffer, buflen);
+			locale->language[buflen] = '\0';
+			code = CS_SUCCEED;
+			break;
+
+		case CS_SYB_LANG_CHARSET:
+		{
+			int i;
+			char *b = (char *)buffer;
+
+			if (buflen == CS_NULLTERM) {
+				buflen = strlen(b);
+			}
+
+			/* find '.' character */
+			for (i = 0; i < buflen; ++i) {
+				if (b[i] == '.') {
+					break;
+				}					
+			}
+			/* not found */
+			if (i == buflen) {
+				break;
+			}
+			if (i) {
+				free(locale->language);
+				locale->language = (char *)malloc(i + 1);
+				if (!locale->language)
+					break;
+
+				strncpy(locale->language, b, i);
+				locale->language[i] = '\0';
+			}
+			if (i != (buflen - 1)) {
+				free(locale->charset);
+				locale->charset = (char *)malloc(buflen - i);
+				if (!locale->charset)
+					break;
+				
+				strncpy(locale->charset, b + i + 1, buflen - i - 1);
+				locale->charset[buflen - i - 1] = '\0';
+			}
+			code = CS_SUCCEED;
+			break;
+		}
+
+		/* TODO commented out until the code works end-to-end
+		case CS_SYB_SORTORDER:
+			if (buflen == CS_NULLTERM) {
+				buflen = strlen((char *)buffer);
+			}
+			
+			free(locale->collate);
+			locale->collate = (char *)malloc(buflen + 1);
+			if (!locale->collate)
+				break;
+
+			strncpy(locale->collate, (char *)buffer, buflen);
+			locale->collate[buflen] = '\0';
+			code = CS_SUCCEED;
+			break;
+		*/
+		}
+	}
+	else if (action == CS_GET)
+	{
+		int tlen;
+
+		switch (type) {
+		case CS_SYB_CHARSET:
+			tlen = (locale->charset ? strlen(locale->charset) : 0) + 1;
+			if (buflen < tlen)
+			{
+				if (outlen)
+					*outlen = tlen;
+				break;
+			}
+			if (locale->charset)
+				strcpy((char *)buffer, locale->charset);
+			else
+				((char *)buffer)[0] = '\0';
+			code = CS_SUCCEED;
+			break;
+
+		case CS_SYB_LANG:
+			tlen = (locale->language ? strlen(locale->language) : 0) + 1;
+			if (buflen < tlen)
+			{
+				if (outlen)
+					*outlen = tlen;
+				break;
+			}
+			if (locale->language)
+				strcpy((char *)buffer, locale->language);
+			else
+				((char *)buffer)[0] = '\0';
+			code = CS_SUCCEED;
+			break;
+
+		case CS_SYB_LANG_CHARSET:
+		{
+			int clen;
+
+			tlen = (locale->language ? strlen(locale->language) : 0) + 1;
+			clen = (locale->charset ? strlen(locale->charset) : 0) + 1;
+			
+			if (buflen < (tlen + clen))
+			{
+				if (outlen)
+					*outlen = tlen + clen;
+				break;
+			}
+			if (locale->language)
+				strcpy((char *)buffer, locale->language);
+			else
+				((char *)buffer)[0] = '\0';
+			strcat((char *)buffer, ".");
+			if (locale->charset) {
+				tlen = strlen((char *)buffer);
+				strcpy((char *)buffer + tlen, locale->charset);
+			}
+			code = CS_SUCCEED;
+			break;
+		}
+
+		case CS_SYB_SORTORDER:
+			tlen = (locale->collate ? strlen(locale->collate) : 0) + 1;
+			if (buflen < tlen)
+			{
+				if (outlen)
+					*outlen = tlen;
+				break;
+			}
+			if (locale->collate)
+				strcpy((char *)buffer, locale->collate);
+			else
+				((char *)buffer)[0] = '\0';
+			code = CS_SUCCEED;
+			break;
+		}
+	}
+	return code;
 }
 
 CS_RETCODE
@@ -897,7 +1234,7 @@ CS_INT msg_count = 0;
 	/* if we already have a list of messages, */
 	/* go to the end of the list...           */
 
-	while (*curptr != (struct cs_diag_msg *)NULL) {
+	while (*curptr != NULL) {
 		msg_count++;
 		curptr = &((*curptr)->next);
 	}
@@ -911,12 +1248,12 @@ CS_INT msg_count = 0;
 	}
 
 	*curptr = (struct cs_diag_msg *) malloc(sizeof(struct cs_diag_msg));
-	if (*curptr == (struct cs_diag_msg *)NULL) { 
+	if (*curptr == NULL) { 
 		return CS_FAIL;
 	} else {
-		(*curptr)->next = (struct cs_diag_msg *)NULL;
+		(*curptr)->next = NULL;
 		(*curptr)->msg  = malloc(sizeof(CS_CLIENTMSG));
-		if ((*curptr)->msg == (CS_CLIENTMSG *) NULL) {
+		if ((*curptr)->msg == NULL) {
 			return CS_FAIL;
 		} else {
 			memcpy((*curptr)->msg, message, sizeof(CS_CLIENTMSG));
@@ -938,7 +1275,7 @@ CS_INT msg_count = 0, msg_found = 0;
 	/* if we already have a list of messages, */
 	/* go to the end of the list...           */
 
-	while (curptr != (struct cs_diag_msg *)NULL) {
+	while (curptr != NULL) {
 		msg_count++;
 		if (msg_count == idx) {
 			msg_found++;
@@ -963,11 +1300,10 @@ struct cs_diag_msg *curptr, *freeptr;
 	curptr = context->msgstore;
 	context->msgstore = NULL;
 
-	while (curptr != (struct cs_diag_msg *)NULL ) {
+	while (curptr != NULL ) {
         	freeptr = curptr;
 		curptr = freeptr->next;
-        	if (freeptr->msg)
-        		free(freeptr->msg);
+        	free(freeptr->msg);
         	free(freeptr);
 	}
 	return CS_SUCCEED;
@@ -981,7 +1317,7 @@ CS_INT msg_count = 0;
 
 	curptr = context->msgstore;
 
-	while (curptr != (struct cs_diag_msg *)NULL) {
+	while (curptr != NULL) {
 		msg_count++;
 		curptr = curptr->next;
 	}
