@@ -28,11 +28,11 @@
 #include "cspublic.h"
 #include "ctlib.h"
 #include "tds.h"
+#include "replacements.h"
 /* #include "fortify.h" */
 
 
-static char software_version[] = "$Id: ctutil.c,v 1.22 2004/03/23 08:51:32 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+TDS_RCSID(var, "$Id: ctutil.c,v 1.28 2007/01/07 16:13:17 jklowden Exp $");
 
 /*
  * test include consistency 
@@ -86,9 +86,12 @@ TEST_ATTRIBUTE(t33,TDS_VARBINARY,len,CS_VARBINARY,len);
 TEST_ATTRIBUTE(t34,TDS_VARBINARY,array,CS_VARBINARY,array);
 #endif
 
-/* error handler */
+/* 
+ * error handler 
+ * This callback function should be invoked only from libtds through tds_ctx->err_handler.  
+ */
 int
-_ct_handle_client_message(TDSCONTEXT * ctx_tds, TDSSOCKET * tds, TDSMESSAGE * msg)
+_ct_handle_client_message(const TDSCONTEXT * ctx_tds, TDSSOCKET * tds, TDSMESSAGE * msg)
 {
 	CS_CLIENTMSG errmsg;
 	CS_CONNECTION *con = NULL;
@@ -100,7 +103,7 @@ _ct_handle_client_message(TDSCONTEXT * ctx_tds, TDSSOCKET * tds, TDSMESSAGE * ms
 	}
 
 	memset(&errmsg, '\0', sizeof(errmsg));
-	errmsg.msgnumber = msg->msg_number;
+	errmsg.msgnumber = msg->msgno;
 	strcpy(errmsg.msgstring, msg->message);
 	errmsg.msgstringlen = strlen(msg->message);
 	errmsg.osstring[0] = '\0';
@@ -114,12 +117,33 @@ _ct_handle_client_message(TDSCONTEXT * ctx_tds, TDSSOCKET * tds, TDSMESSAGE * ms
 		ret = con->_clientmsg_cb(con->ctx, con, &errmsg);
 	else if (con->ctx->_clientmsg_cb)
 		ret = con->ctx->_clientmsg_cb(con->ctx, con, &errmsg);
-	return ret;
+		
+	/*
+	 * The return code from the error handler is either CS_SUCCEED or CS_FAIL.
+	 * This function was called by libtds with some kind of communications failure, and there are
+	 * no cases in which "succeed" could mean anything: In most cases, the function is going to fail
+	 * no matter what.  
+	 *
+	 * Timeouts are a different matter; it's up to the client to decide whether to continue
+	 * waiting or to abort the operation and close the socket.  ct-lib applications do their 
+	 * own cancel processing -- they can call ct_cancel from within the error handler -- so 
+	 * they don't need to return TDS_INT_TIMEOUT.  They can, however, return TDS_INT_CONTINUE
+	 * or TDS_INT_CANCEL.  We map the client's return code to those. 
+	 *
+	 * Only for timeout errors does TDS_INT_CANCEL cause libtds to break the connection. 
+	 */
+	if (msg->msgno == TDSETIME) {
+		switch (ret) {
+		case CS_SUCCEED:	return TDS_INT_CONTINUE;
+		case CS_FAIL:		return TDS_INT_CANCEL;
+		}
+	}
+	return TDS_INT_CANCEL;
 }
 
 /* message handler */
 int
-_ct_handle_server_message(TDSCONTEXT * ctx_tds, TDSSOCKET * tds, TDSMESSAGE * msg)
+_ct_handle_server_message(const TDSCONTEXT * ctx_tds, TDSSOCKET * tds, TDSMESSAGE * msg)
 {
 	CS_SERVERMSG errmsg;
 	CS_CONNECTION *con = NULL;
@@ -131,26 +155,23 @@ _ct_handle_server_message(TDSCONTEXT * ctx_tds, TDSSOCKET * tds, TDSMESSAGE * ms
 	}
 
 	memset(&errmsg, '\0', sizeof(errmsg));
-	errmsg.msgnumber = msg->msg_number;
-	strncpy(errmsg.text, msg->message, sizeof(errmsg.text));
-	errmsg.text[sizeof(errmsg.text)-1] = 0;
+	errmsg.msgnumber = msg->msgno;
+	tds_strlcpy(errmsg.text, msg->message, sizeof(errmsg.text));
 	errmsg.textlen = strlen(errmsg.text);
 	errmsg.sqlstate[0] = 0;
-	if (msg->sql_state) {
-		strncpy(errmsg.sqlstate, msg->sql_state, sizeof(errmsg.sqlstate));
-		errmsg.sqlstate[sizeof(errmsg.sqlstate)-1] = 0;
-	}
-	errmsg.sqlstatelen = strlen(errmsg.sqlstate);
-	errmsg.state = msg->msg_state;
-	errmsg.severity = msg->msg_level;
+	if (msg->sql_state)
+		tds_strlcpy((char *) errmsg.sqlstate, msg->sql_state, sizeof(errmsg.sqlstate));
+	errmsg.sqlstatelen = strlen((char *) errmsg.sqlstate);
+	errmsg.state = msg->state;
+	errmsg.severity = msg->severity;
 	errmsg.line = msg->line_number;
 	if (msg->server) {
 		errmsg.svrnlen = strlen(msg->server);
-		strncpy(errmsg.svrname, msg->server, CS_MAX_NAME);
+		tds_strlcpy(errmsg.svrname, msg->server, CS_MAX_NAME);
 	}
 	if (msg->proc_name) {
 		errmsg.proclen = strlen(msg->proc_name);
-		strncpy(errmsg.proc, msg->proc_name, CS_MAX_NAME);
+		tds_strlcpy(errmsg.proc, msg->proc_name, CS_MAX_NAME);
 	}
 	/* if there is no connection, attempt to call the context handler */
 	if (!con) {
