@@ -18,7 +18,7 @@
 #define MAX(X,Y)      (((X) > (Y)) ? (X) : (Y))
 #define MIN(X,Y)      (((X) < (Y)) ? (X) : (Y))
 
-static char software_version[] = "$Id: rpc_ct_setparam.c,v 1.4 2003/03/31 15:55:35 freddy77 Exp $";
+static char software_version[] = "$Id: rpc_ct_setparam.c,v 1.9 2006/12/29 19:00:33 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 CS_RETCODE ex_clientmsg_cb(CS_CONTEXT * context, CS_CONNECTION * connection, CS_CLIENTMSG * errmsg);
@@ -29,7 +29,7 @@ static CS_INT ex_display_results(CS_COMMAND * cmd);
 
 typedef struct _ex_column_data
 {
-	CS_INT indicator;
+	CS_SMALLINT indicator;
 	CS_CHAR *value;
 	CS_INT valuelen;
 }
@@ -48,6 +48,7 @@ main(int argc, char *argv[])
 
 	CS_INT datalength;
 	CS_SMALLINT nullind;
+	CS_SMALLINT notnullind;
 
 	CS_CHAR cmdbuf[4096];
 
@@ -80,7 +81,7 @@ main(int argc, char *argv[])
 	ct_callback(ctx, NULL, CS_SET, CS_SERVERMSG_CB, (CS_VOID *) ex_servermsg_cb);
 
 	/* do not test error */
-	ret = run_command(cmd, "DROP PROCEDURE sample_rpc");
+	ret = run_command(cmd, "IF EXISTS(SELECT * FROM SYSOBJECTS WHERE name = 'sample_rpc' AND type = 'P') DROP PROCEDURE sample_rpc");
 
 	strcpy(cmdbuf, "create proc sample_rpc (@intparam int, \
         @sintparam smallint output, @floatparam float output, \
@@ -107,7 +108,7 @@ main(int argc, char *argv[])
 	}
 
 	/*
-	 * ** Assign values to the variables used for parameter passing.
+	 * Assign values to the variables used for parameter passing.
 	 */
 
 	intvar = 2;
@@ -118,7 +119,7 @@ main(int argc, char *argv[])
 	strcpy(moneystring, "300.90");
 
 	/*
-	 * ** Clear and setup the CS_DATAFMT structures used to convert datatypes.
+	 * Clear and setup the CS_DATAFMT structures used to convert datatypes.
 	 */
 
 	memset(&srcfmt, 0, sizeof(CS_DATAFMT));
@@ -136,9 +137,9 @@ main(int argc, char *argv[])
 	destfmt.locale = NULL;
 
 	/*
-	 * ** Convert the string representing the money value
-	 * ** to a CS_MONEY variable. Since this routine does not have the
-	 * ** context handle, we use the property functions to get it.
+	 * Convert the string representing the money value
+	 * to a CS_MONEY variable. Since this routine does not have the
+	 * context handle, we use the property functions to get it.
 	 */
 	if ((ret = ct_cmd_props(cmd, CS_GET, CS_PARENT_HANDLE, &conn, CS_UNUSED, NULL)) != CS_SUCCEED) {
 		fprintf(stderr, "ct_cmd_props() failed");
@@ -155,16 +156,18 @@ main(int argc, char *argv[])
 	}
 
 	/*
-	 * ** Send the RPC command for our stored procedure.
+	 * Send the RPC command for our stored procedure.
 	 */
 	if ((ret = ct_command(cmd, CS_RPC_CMD, rpc_name, CS_NULLTERM, CS_NO_RECOMPILE)) != CS_SUCCEED) {
 		fprintf(stderr, "ct_command(CS_RPC_CMD) failed");
 		return 1;
 	}
 
+	nullind    = -1;
+	notnullind = 0;
 	/*
-	 * ** Clear and setup the CS_DATAFMT structure, then pass
-	 * ** each of the parameters for the RPC.
+	 * Clear and setup the CS_DATAFMT structure, then pass
+	 * each of the parameters for the RPC.
 	 */
 	memset(&datafmt, 0, sizeof(datafmt));
 	strcpy(datafmt.name, "@intparam");
@@ -175,9 +178,8 @@ main(int argc, char *argv[])
 	datafmt.locale = NULL;
 
 	datalength = CS_SIZEOF(CS_INT);
-	nullind = -1;
 
-	if ((ret = ct_setparam(cmd, &datafmt, (CS_VOID *) & intvar, &datalength, &nullind)) != CS_SUCCEED) {
+	if ((ret = ct_setparam(cmd, &datafmt, (CS_VOID *) & intvar, &datalength, &notnullind)) != CS_SUCCEED) {
 		fprintf(stderr, "ct_setparam(int) failed");
 		return 1;
 	}
@@ -190,9 +192,8 @@ main(int argc, char *argv[])
 	datafmt.locale = NULL;
 
 	datalength = CS_SIZEOF(CS_SMALLINT);
-	nullind = -1;
 
-	if ((ret = ct_setparam(cmd, &datafmt, (CS_VOID *) & smallintvar, &datalength, &nullind)) != CS_SUCCEED) {
+	if ((ret = ct_setparam(cmd, &datafmt, (CS_VOID *) & smallintvar, &datalength, &notnullind)) != CS_SUCCEED) {
 		fprintf(stderr, "ct_setparam(smallint) failed");
 		return 1;
 	}
@@ -205,9 +206,8 @@ main(int argc, char *argv[])
 	datafmt.locale = NULL;
 
 	datalength = CS_SIZEOF(CS_FLOAT);
-	nullind = -1;
 
-	if ((ret = ct_setparam(cmd, &datafmt, (CS_VOID *) & floatvar, &datalength, &nullind)) != CS_SUCCEED) {
+	if ((ret = ct_setparam(cmd, &datafmt, (CS_VOID *) & floatvar, &datalength, &notnullind)) != CS_SUCCEED) {
 		fprintf(stderr, "ct_setparam(float) failed");
 		return 1;
 	}
@@ -221,9 +221,8 @@ main(int argc, char *argv[])
 	datafmt.locale = NULL;
 
 	datalength = CS_SIZEOF(CS_MONEY);
-	nullind = -1;
 
-	if ((ret = ct_setparam(cmd, &datafmt, (CS_VOID *) & moneyvar, &datalength, &nullind)) != CS_SUCCEED) {
+	if ((ret = ct_setparam(cmd, &datafmt, (CS_VOID *) & moneyvar, &datalength, &notnullind)) != CS_SUCCEED) {
 		fprintf(stderr, "ct_setparam(money) failed");
 		return 1;
 	}
@@ -236,12 +235,11 @@ main(int argc, char *argv[])
 	datafmt.locale = NULL;
 
 	/*
-	 * ** The datetime variable is filled in by the RPC so pass NULL for
-	 * ** the data, 0 for data length, and -1 for the indicator arguments.
+	 * The datetime variable is filled in by the RPC so pass NULL for
+	 * the data, 0 for data length, and -1 for the indicator arguments.
 	 */
 
 	datalength = 0;
-	nullind = -1;
 
 	if ((ret = ct_setparam(cmd, &datafmt, NULL, &datalength, &nullind)) != CS_SUCCEED) {
 		fprintf(stderr, "ct_setparam(datetime4) failed");
@@ -256,11 +254,10 @@ main(int argc, char *argv[])
 
 
 	datalength = 0;
-	nullind = -1;
 
 	/*
-	 * ** The character string variable is filled in by the RPC so pass NULL
-	 * ** for the data 0 for data length, and -1 for the indicator arguments.
+	 * The character string variable is filled in by the RPC so pass NULL
+	 * for the data 0 for data length, and -1 for the indicator arguments.
 	 */
 	if ((ret = ct_setparam(cmd, &datafmt, NULL, &datalength, &nullind)) != CS_SUCCEED) {
 		fprintf(stderr, "ct_setparam(char) failed");
@@ -277,13 +274,13 @@ main(int argc, char *argv[])
 	datalength = CS_SIZEOF(CS_BINARY);
 	nullind = -1;
 
-	if ((ret = ct_setparam(cmd, &datafmt, (CS_VOID *) & binaryvar, &datalength, &nullind)) != CS_SUCCEED) {
+	if ((ret = ct_setparam(cmd, &datafmt, (CS_VOID *) & binaryvar, &datalength, &notnullind)) != CS_SUCCEED) {
 		fprintf(stderr, "ct_setparam(binary) failed");
 		return 1;
 	}
 
 	/*
-	 * ** Send the command to the server
+	 * Send the command to the server
 	 */
 	if (ct_send(cmd) != CS_SUCCEED) {
 		fprintf(stderr, "ct_send(RPC) failed");
@@ -309,6 +306,8 @@ main(int argc, char *argv[])
 		return 1;
 	}
 
+	run_command(cmd, "DROP PROCEDURE sample_rpc");
+
 	if (verbose) {
 		fprintf(stdout, "Trying logout\n");
 	}
@@ -337,7 +336,7 @@ CS_SMALLINT msg_id;
 int i, j;
 
 	/*
-	 * ** Process the results of the RPC.
+	 * Process the results of the RPC.
 	 */
 	while ((ret = ct_results(cmd, &res_type)) == CS_SUCCEED) {
 		switch ((int) res_type) {
@@ -345,7 +344,7 @@ int i, j;
 		case CS_PARAM_RESULT:
 		case CS_STATUS_RESULT:
 			/*
-			 * ** Print the result header based on the result type.
+			 * Print the result header based on the result type.
 			 */
 			switch ((int) res_type) {
 			case CS_ROW_RESULT:
@@ -363,14 +362,14 @@ int i, j;
 			fflush(stdout);
 
 			/*
-			 * ** All three of these result types are fetchable.
-			 * ** Since the result model for rpcs and rows have
-			 * ** been unified in the New Client-Library, we
-			 * ** will use the same routine to display them
+			 * All three of these result types are fetchable.
+			 * Since the result model for rpcs and rows have
+			 * been unified in the New Client-Library, we
+			 * will use the same routine to display them
 			 */
 
 			/*
-			 * ** Find out how many columns there are in this result set.
+			 * Find out how many columns there are in this result set.
 			 */
 			ret = ct_res_info(cmd, CS_NUMDATA, &num_cols, CS_UNUSED, NULL);
 			if (ret != CS_SUCCEED) {
@@ -379,7 +378,7 @@ int i, j;
 			}
 
 			/*
-			 * ** Make sure we have at least one column
+			 * Make sure we have at least one column
 			 */
 			if (num_cols <= 0) {
 				fprintf(stderr, "ct_res_info(CS_NUMDATA) returned zero columns");
@@ -387,11 +386,11 @@ int i, j;
 			}
 
 			/*
-			 * ** Our program variable, called 'coldata', is an array of
-			 * ** EX_COLUMN_DATA structures. Each array element represents
-			 * ** one column.  Each array element will re-used for each row.
-			 * **
-			 * ** First, allocate memory for the data element to process.
+			 * Our program variable, called 'coldata', is an array of
+			 * EX_COLUMN_DATA structures. Each array element represents
+			 * one column.  Each array element will re-used for each row.
+			 * 
+			 * First, allocate memory for the data element to process.
 			 */
 			coldata = (EX_COLUMN_DATA *) malloc(num_cols * sizeof(EX_COLUMN_DATA));
 			if (coldata == NULL) {
@@ -424,7 +423,7 @@ int i, j;
 				}
 
 				ret = ct_bind(cmd, (i + 1), &outdatafmt[i], coldata[i].value, &coldata[i].valuelen,
-					      (CS_SMALLINT *) & coldata[i].indicator);
+					      & coldata[i].indicator);
 				if (ret != CS_SUCCEED) {
 					fprintf(stderr, "ct_bind failed \n");
 					return 1;
@@ -444,12 +443,12 @@ int i, j;
 			while (((ret = ct_fetch(cmd, CS_UNUSED, CS_UNUSED, CS_UNUSED,
 						&rows_read)) == CS_SUCCEED) || (ret == CS_ROW_FAIL)) {
 				/*
-				 * ** Increment our row count by the number of rows just fetched.
+				 * Increment our row count by the number of rows just fetched.
 				 */
 				row_count = row_count + rows_read;
 
 				/*
-				 * ** Check if we hit a recoverable error.
+				 * Check if we hit a recoverable error.
 				 */
 				if (ret == CS_ROW_FAIL) {
 					fprintf(stdout, "Error on row %d.\n", row_count);
@@ -457,19 +456,19 @@ int i, j;
 				}
 
 				/*
-				 * ** We have a row.  Loop through the columns displaying the
-				 * ** column values.
+				 * We have a row.  Loop through the columns displaying the
+				 * column values.
 				 */
 				for (i = 0; i < num_cols; i++) {
 					/*
-					 * ** Display the column value
+					 * Display the column value
 					 */
 					fprintf(stdout, "%s", coldata[i].value);
 					fflush(stdout);
 
 					/*
-					 * ** If not last column, Print out spaces between this
-					 * ** column and next one.
+					 * If not last column, Print out spaces between this
+					 * column and next one.
 					 */
 					if (i != num_cols - 1) {
 						disp_len = ex_display_dlen(&outdatafmt[i]);
@@ -484,7 +483,7 @@ int i, j;
 			}
 
 			/*
-			 * ** Free allocated space.
+			 * Free allocated space.
 			 */
 			for (i = 0; i < num_cols; i++) {
 				free(coldata[i].value);
@@ -493,13 +492,13 @@ int i, j;
 			free(outdatafmt);
 
 			/*
-			 * ** We're done processing rows.  Let's check the final return
-			 * ** value of ct_fetch().
+			 * We're done processing rows.  Let's check the final return
+			 * value of ct_fetch().
 			 */
 			switch ((int) ret) {
 			case CS_END_DATA:
 				/*
-				 * ** Everything went fine.
+				 * Everything went fine.
 				 */
 				fprintf(stdout, "All done processing rows.\n");
 				fflush(stdout);
@@ -507,7 +506,7 @@ int i, j;
 
 			case CS_FAIL:
 				/*
-				 * ** Something terrible happened.
+				 * Something terrible happened.
 				 */
 				fprintf(stderr, "ct_fetch returned CS_FAIL\n");
 				return 1;
@@ -515,7 +514,7 @@ int i, j;
 
 			default:
 				/*
-				 * ** We got an unexpected return value.
+				 * We got an unexpected return value.
 				 */
 				fprintf(stderr, "ct_fetch returned %d\n", ret);
 				return 1;
@@ -525,9 +524,6 @@ int i, j;
 			break;
 
 		case CS_MSG_RESULT:
-			/*
-			 * **
-			 */
 			ret = ct_res_info(cmd, CS_MSGTYPE, (CS_VOID *) & msg_id, CS_UNUSED, NULL);
 			if (ret != CS_SUCCEED) {
 				fprintf(stderr, "ct_res_info(msg_id) failed");
@@ -539,27 +535,27 @@ int i, j;
 
 		case CS_CMD_SUCCEED:
 			/*
-			 * ** This means no rows were returned.
+			 * This means no rows were returned.
 			 */
 			break;
 
 		case CS_CMD_DONE:
 			/*
-			 * ** Done with result set.
+			 * Done with result set.
 			 */
 			break;
 
 		case CS_CMD_FAIL:
 			/*
-			 * ** The server encountered an error while
-			 * ** processing our command.
+			 * The server encountered an error while
+			 * processing our command.
 			 */
 			fprintf(stderr, "ct_results returned CS_CMD_FAIL.");
 			break;
 
 		default:
 			/*
-			 * ** We got something unexpected.
+			 * We got something unexpected.
 			 */
 			fprintf(stderr, "ct_results returned unexpected result type.");
 			return CS_FAIL;
@@ -567,27 +563,27 @@ int i, j;
 	}
 
 	/*
-	 * ** We're done processing results. Let's check the
-	 * ** return value of ct_results() to see if everything
-	 * ** went ok.
+	 * We're done processing results. Let's check the
+	 * return value of ct_results() to see if everything
+	 * went ok.
 	 */
 	switch ((int) ret) {
 	case CS_END_RESULTS:
 		/*
-		 * ** Everything went fine.
+		 * Everything went fine.
 		 */
 		break;
 
 	case CS_FAIL:
 		/*
-		 * ** Something failed happened.
+		 * Something failed happened.
 		 */
 		fprintf(stderr, "ct_results failed.");
 		break;
 
 	default:
 		/*
-		 * ** We got an unexpected return value.
+		 * We got an unexpected return value.
 		 */
 		fprintf(stderr, "ct_results returned unexpected result type.");
 		break;
