@@ -17,13 +17,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
-static char software_version[] = "$Id: dbopen.c,v 1.6.2.1 2004/11/28 10:48:01 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
 #if HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include "tds.h"
 #include "sybdb.h"
 
 #ifdef DMALLOC
@@ -34,11 +32,13 @@ static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 #undef dbopen
 #endif
 
+TDS_RCSID(var, "$Id: dbopen.c,v 1.11 2007/09/20 15:32:54 freddy77 Exp $");
+
 DBPROCESS *
-dbopen(LOGINREC * login, char *server)
+dbopen(LOGINREC * login, const char *server)
 {
 	/* default it's platform specific */
-#if defined(WIN32) || defined(_WIN32) || defined(__WIN32__)
+#if defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(DOS32X)
 	return tdsdbopen(login, server, 1);
 #else
 	return tdsdbopen(login, server, 0);
