@@ -21,7 +21,9 @@
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
 
+#include <stdarg.h>
 #include <stdio.h>
+#include <assert.h>
 
 #if HAVE_STDLIB_H
 #include <stdlib.h>
@@ -37,8 +39,7 @@
 #include <dmalloc.h>
 #endif
 
-static char software_version[] = "$Id: dbutil.c,v 1.27.2.1 2005/01/06 01:51:18 jklowden Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+TDS_RCSID(var, "$Id: dbutil.c,v 1.42 2007/12/07 05:27:55 jklowden Exp $");
 
 /*
  * test include consistency 
@@ -78,25 +79,24 @@ TEST_ATTRIBUTE(t30,TDS_NUMERIC,array,DBNUMERIC,array);
  * know what it really was.  
  */
 int
-_dblib_handle_info_message(TDSCONTEXT * tds_ctx, TDSSOCKET * tds, TDSMESSAGE * msg)
+_dblib_handle_info_message(const TDSCONTEXT * tds_ctx, TDSSOCKET * tds, TDSMESSAGE * msg)
 {
-	DBPROCESS *dbproc = NULL;
+	DBPROCESS *dbproc = (tds && tds->parent)? (DBPROCESS *) tds->parent : NULL;
 
-	if (tds && tds->parent) {
-		dbproc = (DBPROCESS *) tds->parent;
-	}
+	tdsdump_log(TDS_DBG_FUNC, "_dblib_handle_info_message(%p, %p, %p)\n", tds_ctx, tds, msg);
+	tdsdump_log(TDS_DBG_FUNC, "msgno %d: \"%s\"\n", msg->msgno, msg->message);
 
-	/* now check to see if the user supplied a function,
-	 * if not, ignore the problem
+	/* 
+	 * Check to see if the user supplied a function, else ignore the message. 
 	 */
 	if (_dblib_msg_handler) {
 		_dblib_msg_handler(dbproc,
-				   msg->msg_number,
-				   msg->msg_state,
-				   msg->msg_level, msg->message, msg->server, msg->proc_name, msg->line_number);
+				   msg->msgno,
+				   msg->state,
+				   msg->severity, msg->message, msg->server, msg->proc_name, msg->line_number);
 	}
 
-	if (msg->msg_level > 10) {
+	if (msg->severity > 10 && _dblib_err_handler) {	/* call the application's error handler, if installed. */
 		/*
 		 * Sybase docs say SYBESMSG is generated only in specific
 		 * cases (severity greater than 16, or deadlock occurred, or
@@ -104,78 +104,135 @@ _dblib_handle_info_message(TDSCONTEXT * tds_ctx, TDSSOCKET * tds, TDSMESSAGE * m
 		 * behavior is that SYBESMSG is always generated for
 		 * server messages with severity greater than 10.
 		 */
-		tds_client_msg(tds_ctx, tds, SYBESMSG, EXSERVER, -1, -1,
-			       "General SQL Server error: Check messages from the SQL Server.");
+		/* Cannot call dbperror() here because server messsage numbers (and text) are not in its lookup table. */
+		static char message[] = "General SQL Server error: Check messages from the SQL Server";
+		(*_dblib_err_handler)(dbproc, msg->severity, msg->msgno, 0, message, NULL);
 	}
 	return SUCCEED;
 }
 
+/** \internal
+ *  \dblib_internal
+ *  \brief handle errors generated by libtds
+ *  \param tds_ctx actually a dbproc: contains all information needed by db-lib to manage communications with the server.
+ *  \param tds contains all information needed by libtds to manage communications with the server.
+ *  \param msg the message to send
+ *  \returns 
+ *  \remarks This function is called by libtds via tds_ctx->err_handler.  It exists to convert the db-lib 
+ *	error handler's return code into one that libtds will know  how to handle.  
+ * 	libtds conveniently issues msgno's with the same values and meanings as db-lib and ODBC use.  
+ *	(N.B. ODBC actually uses db-lib msgno numbers for its driver-specific "server errors".    
+ */
+/* 
+ * Stack:
+ *		libtds
+ *			tds_ctx->err_handler (pointer to _dblib_handle_err_message)
+ *			_dblib_handle_err_message
+ *				dbperror
+ *					client (or default) error handler
+ *					returns db-lib defined return code INT_something.
+ *				returns db-lib return code with Sybase semantics (adjusting client's code if need be)
+ *			returns libtds return code
+ *		decides what to do based on the universal libtds return code, thank you. 
+ */	
 int
-_dblib_handle_err_message(TDSCONTEXT * tds_ctx, TDSSOCKET * tds, TDSMESSAGE * msg)
+_dblib_handle_err_message(const TDSCONTEXT * tds_ctx, TDSSOCKET * tds, TDSMESSAGE * msg)
 {
-	DBPROCESS *dbproc = NULL;
+	DBPROCESS *dbproc = (tds && tds->parent)? (DBPROCESS *) tds->parent : NULL;
 	int rc = INT_CANCEL;
 
-	if (tds && tds->parent) {
-		dbproc = (DBPROCESS *) tds->parent;
-	}
-	if (msg->msg_number > 0) {
-		/* now check to see if the user supplied a function,
-		 * if not, ignore the problem
-		 */
-		if (_dblib_err_handler) {
-			rc = _dblib_err_handler(dbproc, msg->msg_level, msg->msg_number, msg->msg_state, msg->message, msg->server);
-		}
-	}
+	assert(_dblib_err_handler);
+	assert(msg);
+
+	rc = dbperror(dbproc, msg->msgno, msg->oserr);
 
 	/*
 	 * Preprocess the return code to handle INT_TIMEOUT/INT_CONTINUE
 	 * for non-SYBETIME errors in the strange and different ways as
 	 * specified by Sybase and Microsoft.
 	 */
-	if (msg->msg_number != SYBETIME) {
+	if (msg->msgno != SYBETIME) {
 		switch (rc) {
 		case INT_TIMEOUT:
 			rc = INT_EXIT;
 			break;
 		case INT_CONTINUE:
-			if (!dbproc || !dbproc->msdblib)
+			if (!dbproc || !dbproc->msdblib) {
 				/* Sybase behavior */
+				assert(0);  /* dbperror() should prevent */
 				rc = INT_EXIT;
-			else
+			} else {
 				/* Microsoft behavior */
 				rc = INT_CANCEL;
+			}
 			break;
 		default:
 			break;
 		}
 	}
 
+	/*
+	 * Convert db-lib return code to libtds return code, and return.
+	 */
 	switch (rc) {
+	case INT_CANCEL:	return TDS_INT_CANCEL;
+	case INT_TIMEOUT:	return TDS_INT_TIMEOUT;
+	case INT_CONTINUE:	return TDS_INT_CONTINUE;
+	
 	case INT_EXIT:
-		exit(EXIT_FAILURE);
-		break;
-	case INT_CANCEL:
-		return SUCCEED;
-		break;
-	case INT_TIMEOUT:
-		/* XXX do something clever */
-		return SUCCEED;
-		break;
-	case INT_CONTINUE:
-		/* XXX do something clever */
-		return SUCCEED;
-		break;
+		assert(0);  /* dbperror() should prevent */
 	default:
 		/* unknown return code from error handler */
-		return FAIL;
+		exit(EXIT_FAILURE);
 		break;
 	}
 
-	/* notreached */
-	return FAIL;
+	/* not reached */
+	assert(0);
+	return TDS_INT_CANCEL;
 }
 
+/**
+ * \ingroup dblib_internal
+ * \brief check interrupts for libtds.
+ * 
+ * \param vdbproc a DBPROCESS pointer, contains all information needed by db-lib to manage communications with the server.
+ * \sa DBDEAD(), dbsetinterrupt().
+ */
+int
+_dblib_check_and_handle_interrupt(void * vdbproc)
+{
+	DBPROCESS * dbproc = (DBPROCESS*) vdbproc;
+	int ret = INT_CONTINUE;
+	
+	assert( ! (dbproc == NULL && DBDEAD(dbproc)) );  /* a non-process can't be a dead process */
+	
+	if (dbproc->chkintr == NULL || dbproc->hndlintr == NULL)
+		return INT_CONTINUE;
+		
+	tdsdump_log(TDS_DBG_FUNC, "tds_int_handler %p [%p, %p]", dbproc, dbproc->chkintr, dbproc->hndlintr);
+
+	if (dbproc->chkintr(dbproc)){
+		switch (ret = dbproc->hndlintr(dbproc)) {
+		case INT_EXIT:
+			tdsdump_log(TDS_DBG_FUNC, "dbproc->hndlintr returned INT_EXIT, goodbye!\n");
+			exit(1);
+		case INT_CANCEL:
+			tdsdump_log(TDS_DBG_FUNC, "dbproc->hndlintr returned INT_CANCEL\n");
+			break;
+		case INT_CONTINUE:
+			tdsdump_log(TDS_DBG_FUNC, "dbproc->hndlintr returned INT_CONTINUE\n");
+			break;
+		default:
+			tdsdump_log(TDS_DBG_FUNC, "dbproc->hndlintr returned an invalid value (%d), returning INT_CONTINUE\n", ret);
+			ret = INT_CONTINUE;
+			break;
+		}
+	}
+	return ret;
+}
+
+
 void
 _dblib_setTDS_version(TDSLOGIN * tds_login, DBINT version)
 {
