@@ -1,6 +1,6 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
  * Copyright (C) 1998, 1999, 2000, 2001  Brian Bruns
- * Copyright (C) 2002, 2003, 2004    James K. Lowden
+ * Copyright (C) 2002, 2003, 2004, 2005    James K. Lowden
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -22,6 +22,7 @@
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
 
+#include <stdarg.h>
 #include <stdio.h>
 
 #ifdef HAVE_UNISTD_H
@@ -36,6 +37,10 @@
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
+#if HAVE_ERRNO_H
+# include <errno.h>
+#endif /* HAVE_ERRNO_H */
+
 #include <assert.h>
 
 #include "tds.h"
@@ -43,13 +48,13 @@
 #include "sybfront.h"
 #include "sybdb.h"
 #include "dblib.h"
+#include "replacements.h"
 
 #ifdef DMALLOC
 #include <dmalloc.h>
 #endif
 
-static char software_version[] = "$Id: rpc.c,v 1.32.2.6 2005/01/10 09:34:59 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+TDS_RCSID(var, "$Id: rpc.c,v 1.64 2007/12/06 18:32:34 freddy77 Exp $");
 
 static void rpc_clear(DBREMOTE_PROC * rpc);
 static void param_clear(DBREMOTE_PROC_PARAM * pparam);
@@ -57,22 +62,29 @@ static void param_clear(DBREMOTE_PROC_PARAM * pparam);
 static TDSPARAMINFO *param_info_alloc(TDSSOCKET * tds, DBREMOTE_PROC * rpc);
 
 /**
- * Initialize a remote procedure call. 
+ * \ingroup dblib_rpc
+ * \brief Initialize a remote procedure call. 
  *
- * Only supported option would be DBRPCRECOMPILE, 
+ * \param dbproc contains all information needed by db-lib to manage communications with the server.
+ * \param rpcname name of the stored procedure to be run.  
+ * \param options Only supported option would be DBRPCRECOMPILE, 
  * which causes the stored procedure to be recompiled before executing.
- * FIXME: I don't know the value for DBRPCRECOMPILE and have not added it to sybdb.h
+ * \remark The RPC functions are the only way to get back OUTPUT parameter data with db-lib 
+ * from modern Microsoft servers.  
+ * \retval SUCCEED normal.
+ * \retval FAIL on error
+ * \sa dbrpcparam(), dbrpcsend()
  */
-
 RETCODE
 dbrpcinit(DBPROCESS * dbproc, char *rpcname, DBSMALLINT options)
 {
 	DBREMOTE_PROC **rpc;
 	int dbrpcrecompile = 0;
 
-	/* sanity */
-	if (dbproc == NULL || rpcname == NULL)
-		return FAIL;
+	tdsdump_log(TDS_DBG_FUNC, "dbrpcinit(%p, %s, %d)\n", dbproc, rpcname, options);
+	CHECK_DBPROC();
+	DBPERROR_RETURN(IS_TDSDEAD(dbproc->tds_socket), SYBEDDNE);
+	CHECK_NULP(rpcname, "dbrpcinit", 2, FAIL);
 
 	if (options & DBRPCRESET) {
 		rpc_clear(dbproc->rpc);
@@ -85,10 +97,7 @@ dbrpcinit(DBPROCESS * dbproc, char *rpcname, DBSMALLINT options)
 	options &= ~DBRPCRECOMPILE;	/* turn that one off, now that we've extracted it */
 
 	/* all other options except DBRPCRECOMPILE are invalid */
-	if (options) {
-		/* should show client error message */
-		return FAIL;
-	}
+	DBPERROR_RETURN3(options, SYBEIPV, (int) options, "options", "dbrpcinit");
 
 	/* to allocate, first find a free node */
 	for (rpc = &dbproc->rpc; *rpc != NULL; rpc = &(*rpc)->next) {
@@ -116,7 +125,7 @@ dbrpcinit(DBPROCESS * dbproc, char *rpcname, DBSMALLINT options)
 
 	/* store */
 	(*rpc)->options = options & DBRPCRECOMPILE;
-	(*rpc)->param_list = (DBREMOTE_PROC_PARAM *) NULL;
+	(*rpc)->param_list = NULL;
 
 	/* completed */
 	tdsdump_log(TDS_DBG_INFO1, "dbrpcinit() added rpcname \"%s\"\n", rpcname);
@@ -125,8 +134,9 @@ dbrpcinit(DBPROCESS * dbproc, char *rpcname, DBSMALLINT options)
 }
 
 /**
- * \ingroup dblib_api
+ * \ingroup dblib_rpc
  * \brief Add a parameter to a remote procedure call.
+ *
  * Call between dbrpcinit() and dbrpcsend()
  * \param dbproc contains all information needed by db-lib to manage communications with the server.
  * \param paramname literal name of the parameter, according to the stored procedure (starts with '@').  Optional.  
@@ -152,54 +162,68 @@ dbrpcparam(DBPROCESS * dbproc, char *paramname, BYTE status, int type, DBINT max
 	DBREMOTE_PROC_PARAM **pparam;
 	DBREMOTE_PROC_PARAM *param;
 
-	/* sanity */
-	if (dbproc == NULL)
-		return FAIL;
-	if (dbproc->rpc == NULL)
-		return FAIL;
+	tdsdump_log(TDS_DBG_FUNC, "dbrpcparam(%p, %s, 0x%x, %d, %d, %d, %p)\n", 
+				   dbproc, paramname, status, type, maxlen, datalen, value);
+	CHECK_DBPROC();
+	DBPERROR_RETURN(IS_TDSDEAD(dbproc->tds_socket), SYBEDDNE);
+	CHECK_PARAMETER(dbproc->rpc, SYBERPCS, FAIL);
 
 	/* validate datalen parameter */
 
 	if (is_fixed_type(type)) {
-		if (datalen > 0) 
-			return FAIL;
-	} else {
-		if (datalen < 0) 
-			return FAIL;
+		if (datalen != 0)
+			datalen = -1; 
+	} else {	/* Sybooks: "Passing datalen as -1 for any of these [non-fixed] datatypes results 
+			 * in the DBPROCESS referenced by dbproc being marked as "dead," or unusable."
+			 */
+		DBPERROR_RETURN(datalen < 0, SYBERPIL);
 	}
 
+	/* "value parameter for dbprcparam() can be NULL, only if the datalen parameter is 0." */
+	DBPERROR_RETURN(value == NULL && datalen != 0, SYBERPNULL);
+	
+	/* nullable types most provide a data length */
+	DBPERROR_RETURN(is_nullable_type(type) && datalen < 0, SYBERPUL);
+
 	/* validate maxlen parameter */
 
 	if (status & DBRPCRETURN) {
 		if (is_fixed_type(type)) {
-			if (maxlen != -1)
-				return FAIL;
+			maxlen = -1;
 		} else {
 			if (maxlen == -1)
 				maxlen = 255;
 		}
 	} else {
-		if (maxlen != -1)
-			return FAIL;
+		/*
+		 * Well, maxlen should be used only for output parameter however it seems
+		 * that ms implementation wrongly require this 0 for NULL variable
+		 * input parameters, so fix it
+		 */
+		DBPERROR_RETURN3(maxlen != -1 && maxlen != 0, SYBEIPV, (int) maxlen, "maxlen", "dbrpcparam");
+		maxlen = -1;
 	}
-
-	/* TODO add other tests for correctness */
+	
+	/* end validation */
 
 	/* allocate */
 	param = (DBREMOTE_PROC_PARAM *) malloc(sizeof(DBREMOTE_PROC_PARAM));
-	if (param == NULL)
+	if (param == NULL) {
+		dbperror(dbproc, SYBEMEM, 0);
 		return FAIL;
+	}
 
 	if (paramname) {
 		name = strdup(paramname);
 		if (name == NULL) {
 			free(param);
+			dbperror(dbproc, SYBEMEM, 0);
 			return FAIL;
 		}
 	}
 
 	/* initialize */
-	param->next = (DBREMOTE_PROC_PARAM *) NULL;	/* NULL signifies end of linked list */
+	param->next = NULL;	/* NULL signifies end of linked list */
 	param->name = name;
 	param->status = status;
 	param->type = type;
@@ -207,10 +231,9 @@ dbrpcparam(DBPROCESS * dbproc, char *paramname, BYTE status, int type, DBINT max
 	param->datalen = datalen;
 
 	/*
-	 * if datalen = 0, value parameter is ignored       
-	 * this is one way to specify a NULL input parameter 
+	 * If datalen = 0, value parameter is ignored.
+	 * This is one way to specify a NULL input parameter. 
 	 */
-
 	if (datalen == 0)
 		param->value = NULL;
 	else
@@ -237,31 +260,38 @@ dbrpcparam(DBPROCESS * dbproc, char *paramname, BYTE status, int type, DBINT max
 }
 
 /**
- * Execute the procedure and free associated memory
+ * \ingroup dblib_rpc
+ * \brief Execute the procedure and free associated memory
+ *
+ * \param dbproc contains all information needed by db-lib to manage communications with the server.
+ * \retval SUCCEED normal.
+ * \retval FAIL on error
+ * \sa dbrpcinit(), dbrpcparam()
  */
 RETCODE
 dbrpcsend(DBPROCESS * dbproc)
 {
 	DBREMOTE_PROC *rpc;
 
+	tdsdump_log(TDS_DBG_FUNC, "dbrpcsend(%p)\n", dbproc);
+	CHECK_DBPROC();
+	DBPERROR_RETURN(IS_TDSDEAD(dbproc->tds_socket), SYBEDDNE);
+	CHECK_PARAMETER(dbproc->rpc, SYBERPCS, FAIL);	/* dbrpcinit should allocate pointer */
+
 	/* sanity */
-	if (dbproc == NULL || dbproc->rpc == NULL	/* dbrpcinit should allocate pointer */
-	    || dbproc->rpc->name == NULL) {	/* can't be ready without a name */
+	if (dbproc->rpc->name == NULL) {	/* can't be ready without a name */
+		tdsdump_log(TDS_DBG_INFO1, "returning FAIL: name is NULL\n");
 		return FAIL;
 	}
 
 	dbproc->dbresults_state = _DB_RES_INIT;
 
-	/* FIXME do stuff */
-	tdsdump_log(TDS_DBG_FUNC, "dbrpcsend()\n");
-
 	for (rpc = dbproc->rpc; rpc != NULL; rpc = rpc->next) {
 		int erc;
 		TDSPARAMINFO *pparam_info = NULL;
 
 		/*
-		 * liam@inodes.org: allow stored procedures to have no
-		 * paramaters 
+		 * liam@inodes.org: allow stored procedures to have no paramaters 
 		 */
 		if (rpc->param_list != NULL) {
 			pparam_info = param_info_alloc(dbproc->tds_socket, rpc);
@@ -270,14 +300,18 @@ dbrpcsend(DBPROCESS * dbproc)
 		}
 		erc = tds_submit_rpc(dbproc->tds_socket, dbproc->rpc->name, pparam_info);
 		tds_free_param_results(pparam_info);
-		if (erc == TDS_FAIL)
+		if (erc == TDS_FAIL) {
+			tdsdump_log(TDS_DBG_INFO1, "returning FAIL: tds_submit_rpc() failed\n");
 			return FAIL;
+		}
 	}
 
 	/* free up the memory */
 	rpc_clear(dbproc->rpc);
 	dbproc->rpc = NULL;
 
+	tdsdump_log(TDS_DBG_FUNC, "dbrpcsend() returning SUCCEED\n");
+
 	return SUCCEED;
 }
 
@@ -287,21 +321,28 @@ dbrpcsend(DBPROCESS * dbproc)
 static const unsigned char *
 param_row_alloc(TDSPARAMINFO * params, TDSCOLUMN * curcol, int param_num, void *value, int size)
 {
-	const unsigned char *row = tds_alloc_param_row(params, curcol);
-	tdsdump_log(TDS_DBG_INFO1, "param_row_alloc, size = %d, offset = %d, row_size = %d\n",
-                size, curcol->column_offset,
-                params->row_size);
-
+	const void *row = tds_alloc_param_data(curcol);
+	tdsdump_log(TDS_DBG_INFO1, "parameter size = %d, data = %p, row_size = %d\n",
+				   size, curcol->column_data, params->row_size);
 	if (!row)
 		return NULL;
-	tdsdump_log(TDS_DBG_FUNC, "param_row_alloc(): doing data from value\n");
 	if (size > 0 && value) {
-		tdsdump_log(TDS_DBG_FUNC, "param_row_alloc(): copying %d bytes of data to parameter #%d\n", size, param_num);
-		memcpy(&params->current_row[curcol->column_offset], value, size);
+		tdsdump_log(TDS_DBG_FUNC, "copying %d bytes of data to parameter #%d\n", size, param_num);
+		if (!is_blob_type(curcol->column_type)) {
+			memcpy(curcol->column_data, value, size);
+		} else {
+			TDSBLOB *blob = (TDSBLOB *) curcol->column_data;
+			blob->textvalue = malloc(size);
+			tdsdump_log(TDS_DBG_FUNC, "blob parameter supported, size %d textvalue pointer is %p\n", 
+						  size, blob->textvalue);
+			if (!blob->textvalue)
+				return NULL;
+			memcpy(blob->textvalue, value, size);
+		}
 	}
 	else {
-		tdsdump_log(TDS_DBG_FUNC, "param_row_alloc(): setting parameter #%d to NULL\n", param_num);
-		tds_set_null(params->current_row, param_num);
+		tdsdump_log(TDS_DBG_FUNC, "setting parameter #%d to NULL\n", param_num);
+		curcol->column_cur_size = -1;
 	}
 
 	return row;
@@ -333,19 +374,18 @@ param_info_alloc(TDSSOCKET * tds, DBREMOTE_PROC * rpc)
 
 		if (!(new_params = tds_alloc_param_result(params))) {
 			tds_free_param_results(params);
-			fprintf(stderr, "out of rpc memory!");
+			tdsdump_log(TDS_DBG_ERROR, "out of rpc memory!");
 			return NULL;
 		}
 		params = new_params;
 
-		/* Determine whether an input parameter is NULL
-		 * or not.
+		/*
+		 * Determine whether an input parameter is NULL or not.
 		 */
-
 		param_is_null = 0;
 		temp_type = p->type;
-		temp_datalen = p->datalen;
 		temp_value = p->value;
+		temp_datalen = p->datalen;
 
 		if (p->datalen == 0)
 			param_is_null = 1; 
@@ -368,8 +408,7 @@ param_info_alloc(TDSSOCKET * tds, DBREMOTE_PROC * rpc)
 
 		/* meta data */
 		if (p->name) {
-			strncpy(pcol->column_name, p->name, sizeof(pcol->column_name));
-			pcol->column_name[sizeof(pcol->column_name) - 1] = 0;
+			tds_strlcpy(pcol->column_name, p->name, sizeof(pcol->column_name));
 			pcol->column_namelen = strlen(pcol->column_name);
 		}
 
@@ -393,7 +432,7 @@ param_info_alloc(TDSSOCKET * tds, DBREMOTE_PROC * rpc)
 
 		if (!prow) {
 			tds_free_param_results(params);
-			fprintf(stderr, "out of memory for rpc row!");
+			tdsdump_log(TDS_DBG_ERROR, "out of memory for rpc row!");
 			return NULL;
 		}
 
@@ -414,8 +453,7 @@ rpc_clear(DBREMOTE_PROC * rpc)
 	while (rpc) {
 		next = rpc->next;
 		param_clear(rpc->param_list);
-		if (rpc->name);
-			free(rpc->name);
+		free(rpc->name);
 		free(rpc);
 		rpc = next;
 	}
@@ -431,8 +469,7 @@ param_clear(DBREMOTE_PROC_PARAM * pparam)
 
 	while (pparam) {
 		next = pparam->next;
-		if (pparam->name)
-			free(pparam->name);
+		free(pparam->name);
 		/* free self */
 		free(pparam);
 		pparam = next;
