@@ -1,25 +1,11 @@
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif /* HAVE_CONFIG_H */
-
-#include <stdio.h>
-
-#if HAVE_STDLIB_H
-#include <stdlib.h>
-#endif /* HAVE_STDLIB_H */
-
-#if HAVE_STRING_H
-#include <string.h>
-#endif /* HAVE_STRING_H */
-
-#include <sqlfront.h>
-#include <sqldb.h>
+/* 
+ * Purpose: Test buffering
+ * Functions: dbclrbuf dbgetrow dbsetopt 
+ */
 
 #include "common.h"
 
-
-
-static char software_version[] = "$Id: t0003.c,v 1.9 2004/09/09 08:54:49 freddy77 Exp $";
+static char software_version[] = "$Id: t0003.c,v 1.13 2007/12/04 02:06:38 jklowden Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 
@@ -71,18 +57,11 @@ main(int argc, char **argv)
 	dbloginfree(login);
 	add_bread_crumb();
 
+#ifdef MICROSOFT_DBLIB
+	dbsetopt(dbproc, DBBUFFER, "100");
+#else
 	dbsetopt(dbproc, DBBUFFER, "100", 0);
-	add_bread_crumb();
-
-	fprintf(stdout, "Dropping table\n");
-	add_bread_crumb();
-	dbcmd(dbproc, "drop table #dblib0003");
-	add_bread_crumb();
-	dbsqlexec(dbproc);
-	add_bread_crumb();
-	while (dbresults(dbproc) != NO_MORE_RESULTS) {
-		/* nop */
-	}
+#endif
 	add_bread_crumb();
 
 	fprintf(stdout, "creating table\n");
@@ -125,9 +104,9 @@ main(int argc, char **argv)
 	}
 
 	add_bread_crumb();
-	dbbind(dbproc, 1, INTBIND, -1, (BYTE *) & testint);
+	dbbind(dbproc, 1, INTBIND, 0, (BYTE *) & testint);
 	add_bread_crumb();
-	dbbind(dbproc, 2, STRINGBIND, -1, (BYTE *) teststr);
+	dbbind(dbproc, 2, STRINGBIND, 0, (BYTE *) teststr);
 	add_bread_crumb();
 
 	add_bread_crumb();
