@@ -1,24 +1,12 @@
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif /* HAVE_CONFIG_H */
-
-#include <stdio.h>
-
-#if HAVE_STDLIB_H
-#include <stdlib.h>
-#endif /* HAVE_STDLIB_H */
-
-#if HAVE_STRING_H
-#include <string.h>
-#endif /* HAVE_STRING_H */
-
-#include <sqlfront.h>
-#include <sqldb.h>
+/* 
+ * Purpose: Test fetching 50 rows in two result sets with a 5000-row buffer.
+ * Functions: dbbind dbcmd dbnextrow dbnumcols dbresults dbsetopt dbsqlexec
+ */
 
 #include "common.h"
 
 
-static char software_version[] = "$Id: t0006.c,v 1.12 2004/09/09 08:54:49 freddy77 Exp $";
+static char software_version[] = "$Id: t0006.c,v 1.16 2007/12/04 02:06:38 jklowden Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 
@@ -103,17 +91,6 @@ main(int argc, char **argv)
 
 	add_bread_crumb();
 
-	fprintf(stdout, "Dropping table\n");
-	add_bread_crumb();
-	dbcmd(dbproc, "drop table #dblib0006");
-	add_bread_crumb();
-	dbsqlexec(dbproc);
-	add_bread_crumb();
-	while (dbresults(dbproc) != NO_MORE_RESULTS) {
-		/* nop */
-	}
-	add_bread_crumb();
-
 	fprintf(stdout, "creating table\n");
 	dbcmd(dbproc, "create table #dblib0006 (i int not null, s char(10) not null)");
 	dbsqlexec(dbproc);
@@ -160,9 +137,9 @@ main(int argc, char **argv)
 	}
 
 	add_bread_crumb();
-	dbbind(dbproc, 1, INTBIND, -1, (BYTE *) & testint);
+	dbbind(dbproc, 1, INTBIND, 0, (BYTE *) & testint);
 	add_bread_crumb();
-	dbbind(dbproc, 2, STRINGBIND, -1, (BYTE *) teststr);
+	dbbind(dbproc, 2, STRINGBIND, 0, (BYTE *) teststr);
 	add_bread_crumb();
 
 	get_results(dbproc, 1);
@@ -186,9 +163,9 @@ main(int argc, char **argv)
 		add_bread_crumb();
 
 		add_bread_crumb();
-		dbbind(dbproc, 1, INTBIND, -1, (BYTE *) & testint);
+		dbbind(dbproc, 1, INTBIND, 0, (BYTE *) & testint);
 		add_bread_crumb();
-		dbbind(dbproc, 2, STRINGBIND, -1, (BYTE *) teststr);
+		dbbind(dbproc, 2, STRINGBIND, 0, (BYTE *) teststr);
 		add_bread_crumb();
 
 		get_results(dbproc, 25);
