@@ -1,25 +1,11 @@
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif /* HAVE_CONFIG_H */
-
-#include <stdio.h>
-
-#if HAVE_STDLIB_H
-#include <stdlib.h>
-#endif /* HAVE_STDLIB_H */
-
-#if HAVE_STRING_H
-#include <string.h>
-#endif /* HAVE_STRING_H */
-
-#include <sqlfront.h>
-#include <sqldb.h>
+/* 
+ * Purpose: Test buffering.  As of April 2005, jkl believes this test is broken. 
+ * Functions: dbclrbuf dbsetopt 
+ */
 
 #include "common.h"
 
-
-
-static char software_version[] = "$Id: t0008.c,v 1.10 2004/09/09 08:54:49 freddy77 Exp $";
+static char software_version[] = "$Id: t0008.c,v 1.16 2007/12/04 02:06:38 jklowden Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 
@@ -75,17 +61,6 @@ main(int argc, char **argv)
 #endif
 	add_bread_crumb();
 
-	fprintf(stdout, "Dropping table\n");
-	add_bread_crumb();
-	dbcmd(dbproc, "drop table #dblib0008");
-	add_bread_crumb();
-	dbsqlexec(dbproc);
-	add_bread_crumb();
-	while (dbresults(dbproc) != NO_MORE_RESULTS) {
-		/* nop */
-	}
-	add_bread_crumb();
-
 	fprintf(stdout, "creating table\n");
 	dbcmd(dbproc, "create table #dblib0008 (i int not null, s char(10) not null)");
 	dbsqlexec(dbproc);
@@ -125,9 +100,9 @@ main(int argc, char **argv)
 	}
 
 	add_bread_crumb();
-	dbbind(dbproc, 1, INTBIND, -1, (BYTE *) & testint);
+	dbbind(dbproc, 1, INTBIND, 0, (BYTE *) & testint);
 	add_bread_crumb();
-	dbbind(dbproc, 2, STRINGBIND, -1, (BYTE *) teststr);
+	dbbind(dbproc, 2, STRINGBIND, 0, (BYTE *) teststr);
 	add_bread_crumb();
 
 	add_bread_crumb();
@@ -145,18 +120,18 @@ main(int argc, char **argv)
 
 		add_bread_crumb();
 		if (REG_ROW != dbnextrow(dbproc)) {
-			fprintf(stderr, "dblib %s for %s\n", last_read == 48 ? "okay" : "failed", __FILE__);
+			fprintf(stderr, "dblib failed for %s, dbnextrow1\n", __FILE__);
 			add_bread_crumb();
 			dbexit();
 			add_bread_crumb();
 
 			free_bread_crumb();
-			return (last_read == 48) ? 0 : 1;
+			return 1;
 		}
 		add_bread_crumb();
 		last_read = testint;
-		if (testint == 49) {
-			fprintf(stderr, "dblib failed for %s\n", __FILE__);
+		if (testint < 1 || testint > rows_to_add) {
+			fprintf(stderr, "dblib failed for %s testint = %d\n", __FILE__, (int) testint);
 			exit(1);
 		}
 		if (testint != i) {
@@ -170,11 +145,24 @@ main(int argc, char **argv)
 		printf("Read a row of data -> %d %s\n", (int) testint, teststr);
 	}
 
+	if (REG_ROW == dbnextrow(dbproc)) {
+		fprintf(stderr, "dblib failed for %s, dbnextrow2\n", __FILE__);
+		add_bread_crumb();
+		dbexit();
+		add_bread_crumb();
+
+		free_bread_crumb();
+		return 1;
+	}
+
 	add_bread_crumb();
 	dbexit();
 	add_bread_crumb();
 
-	fprintf(stderr, "dblib failed for %s\n", __FILE__);
+	if (last_read == rows_to_add - 1)
+		printf("dblib okay for %s\n", __FILE__);
+	else
+		fprintf(stderr, "dblib failed for %s last_read %d\n", __FILE__, (int) last_read);
 	free_bread_crumb();
-	return 1;
+	return (last_read == rows_to_add - 1) ? 0 : 1;
 }
