@@ -1,23 +1,11 @@
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif /* HAVE_CONFIG_H */
-
-#include <stdio.h>
-
-#if HAVE_STDLIB_H
-#include <stdlib.h>
-#endif /* HAVE_STDLIB_H */
-
-#if HAVE_STRING_H
-#include <string.h>
-#endif /* HAVE_STRING_H */
-
-#include <sqlfront.h>
-#include <sqldb.h>
+/* 
+ * Purpose: Test sending and receiving TEXT datatype
+ * Functions: dbmoretext dbreadtext dbtxptr dbtxtimestamp dbwritetext 
+ */
 
 #include "common.h"
 
-static char software_version[] = "$Id: t0013.c,v 1.19 2004/09/09 08:54:49 freddy77 Exp $";
+static char software_version[] = "$Id: t0013.c,v 1.23 2006/07/06 12:48:16 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 #define BLOB_BLOCK_SIZE 4096
@@ -92,13 +80,6 @@ main(int argc, char **argv)
 	fread((void *) blob, isiz, 1, fp);
 	fclose(fp);
 
-	fprintf(stdout, "Dropping table\n");
-	dbcmd(dbproc, "drop table #dblib0013");
-	dbsqlexec(dbproc);
-	while (dbresults(dbproc) != NO_MORE_RESULTS) {
-		/* nop */
-	}
-
 	fprintf(stdout, "creating table\n");
 	dbcmd(dbproc, "create table #dblib0013 (i int not null, PigTure image not null)");
 	dbsqlexec(dbproc);
@@ -133,8 +114,10 @@ main(int argc, char **argv)
 		timeStamp = dbtxtimestamp(dbproc, 1);
 	}
 
-	/* Use #ifdef if you want to test dbmoretext mode (needed for 16-bit apps)
-	 * Use #ifndef for big buffer version (32-bit) */
+	/*
+	 * Use #ifdef if you want to test dbmoretext mode (needed for 16-bit apps)
+	 * Use #ifndef for big buffer version (32-bit)
+	 */
 #ifndef DBWRITE_OK_FOR_OVER_4K
 	if (dbwritetext(dbproc, objname, textPtr, DBTXPLEN, timeStamp, FALSE, isiz, (BYTE*) blob) != SUCCEED)
 		return 5;
