@@ -1,23 +1,11 @@
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif /* HAVE_CONFIG_H */
-
-#include <stdio.h>
-
-#if HAVE_STDLIB_H
-#include <stdlib.h>
-#endif /* HAVE_STDLIB_H */
-
-#if HAVE_STRING_H
-#include <string.h>
-#endif /* HAVE_STRING_H */
-
-#include <sqlfront.h>
-#include <sqldb.h>
+/* 
+ * Purpose: Test sending and receiving TEXT datatype
+ * Functions: dbbind dbmoretext dbreadtext dbtxptr dbtxtimestamp dbwritetext 
+ */
 
 #include "common.h"
 
-static char software_version[] = "$Id: t0014.c,v 1.22 2004/09/09 08:54:49 freddy77 Exp $";
+static char software_version[] = "$Id: t0014.c,v 1.27 2006/07/06 12:48:16 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 #define BLOB_BLOCK_SIZE 4096
@@ -97,7 +85,7 @@ main(int argc, char **argv)
 
 	/* FIXME this test seem to not work using temporary tables (sybase?)... */
 	fprintf(stdout, "Dropping table\n");
-	dbcmd(dbproc, "drop table dblib0014");
+	dbcmd(dbproc, "if object_id('dblib0014') is not null drop table dblib0014");
 	dbsqlexec(dbproc);
 	while (dbresults(dbproc) != NO_MORE_RESULTS) {
 		/* nop */
@@ -140,8 +128,10 @@ main(int argc, char **argv)
 			textPtr = dbtxptr(dbproc, 1);
 			timeStamp = dbtxtimestamp(dbproc, 1);
 
-			/* Use #ifdef if you want to test dbmoretext mode (needed for 16-bit apps)
-			 * Use #ifndef for big buffer version (32-bit) */
+			/*
+			 * Use #ifdef if you want to test dbmoretext mode (needed for 16-bit apps)
+			 * Use #ifndef for big buffer version (32-bit)
+			 */
 #if 1
 /* DBWRITE_OK_FOR_OVER_4K */
 			if (dbwritetext(blobproc, objname, textPtr, DBTXPLEN, timeStamp, TRUE, isiz, (BYTE*) blob) != SUCCEED)
@@ -258,6 +248,14 @@ main(int argc, char **argv)
 	}
 
 	free(blob);
+
+	fprintf(stdout, "Dropping table\n");
+	dbcmd(dbproc, "drop table dblib0014");
+	dbsqlexec(dbproc);
+	while (dbresults(dbproc) != NO_MORE_RESULTS) {
+		/* nop */
+	}
+
 	dbexit();
 
 	fprintf(stdout, "dblib %s on %s\n", (failed ? "failed!" : "okay"), __FILE__);
