@@ -1,23 +1,11 @@
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif /* HAVE_CONFIG_H */
-
-#include <stdio.h>
-
-#if HAVE_STDLIB_H
-#include <stdlib.h>
-#endif /* HAVE_STDLIB_H */
-
-#if HAVE_STRING_H
-#include <string.h>
-#endif /* HAVE_STRING_H */
-
-#include <sqlfront.h>
-#include <sqldb.h>
+/* 
+ * Purpose: Test binding and retrieving strings and ints, and cancelling results 
+ * Functions: dbbind dbcancel dbnextrow dbnumcols dbresults dbsqlexec 
+ */
 
 #include "common.h"
 
-static char software_version[] = "$Id: t0015.c,v 1.13 2004/09/09 08:54:49 freddy77 Exp $";
+static char software_version[] = "$Id: t0015.c,v 1.17 2007/12/04 02:06:38 jklowden Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 
@@ -66,17 +54,6 @@ main(int argc, char **argv)
 	dbloginfree(login);
 	add_bread_crumb();
 
-	fprintf(stdout, "Dropping table\n");
-	add_bread_crumb();
-	dbcmd(dbproc, "drop table #dblib0015");
-	add_bread_crumb();
-	dbsqlexec(dbproc);
-	add_bread_crumb();
-	while (dbresults(dbproc) != NO_MORE_RESULTS) {
-		/* nop */
-	}
-	add_bread_crumb();
-
 	fprintf(stdout, "creating table\n");
 	dbcmd(dbproc, "create table #dblib0015 (i int not null, s char(10) not null)");
 	dbsqlexec(dbproc);
@@ -118,13 +95,13 @@ main(int argc, char **argv)
 	}
 
 	add_bread_crumb();
-	if (SUCCEED != dbbind(dbproc, 1, INTBIND, -1, (BYTE *) & testint)) {
+	if (SUCCEED != dbbind(dbproc, 1, INTBIND, 0, (BYTE *) & testint)) {
 		failed = 1;
 		fprintf(stderr, "Had problem with bind\n");
 		abort();
 	}
 	add_bread_crumb();
-	if (SUCCEED != dbbind(dbproc, 2, STRINGBIND, -1, (BYTE *) teststr)) {
+	if (SUCCEED != dbbind(dbproc, 2, STRINGBIND, 0, (BYTE *) teststr)) {
 		failed = 1;
 		fprintf(stderr, "Had problem with bind\n");
 		abort();
@@ -153,13 +130,13 @@ main(int argc, char **argv)
 	}
 
 	add_bread_crumb();
-	if (SUCCEED != dbbind(dbproc, 1, INTBIND, -1, (BYTE *) & testint)) {
+	if (SUCCEED != dbbind(dbproc, 1, INTBIND, 0, (BYTE *) & testint)) {
 		failed = 1;
 		fprintf(stderr, "Had problem with bind\n");
 		abort();
 	}
 	add_bread_crumb();
-	if (SUCCEED != dbbind(dbproc, 2, STRINGBIND, -1, (BYTE *) teststr)) {
+	if (SUCCEED != dbbind(dbproc, 2, STRINGBIND, 0, (BYTE *) teststr)) {
 		failed = 1;
 		fprintf(stderr, "Had problem with bind\n");
 		abort();
