@@ -1,24 +1,12 @@
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif /* HAVE_CONFIG_H */
-
-#include <stdio.h>
-
-#if HAVE_STDLIB_H
-#include <stdlib.h>
-#endif /* HAVE_STDLIB_H */
-
-#if HAVE_STRING_H
-#include <string.h>
-#endif /* HAVE_STRING_H */
-
-#include <assert.h>
-#include <sqlfront.h>
-#include <sqldb.h>
+/* 
+ * Purpose: Test retrieving compute rows
+ * Functions: dbaltbind dbaltcolid dbaltop dbalttype dbnumalts
+ */
 
 #include "common.h"
+#include <assert.h>
 
-static char software_version[] = "$Id: t0023.c,v 1.8 2004/09/09 08:54:49 freddy77 Exp $";
+static char software_version[] = "$Id: t0023.c,v 1.12 2007/12/04 02:06:38 jklowden Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 
@@ -71,17 +59,6 @@ main(int argc, char *argv[])
 	dbloginfree(login);
 	add_bread_crumb();
 
-	fprintf(stdout, "Dropping table\n");
-	add_bread_crumb();
-	dbcmd(dbproc, "drop table #dblib0023");
-	add_bread_crumb();
-	dbsqlexec(dbproc);
-	add_bread_crumb();
-	while (dbresults(dbproc) == SUCCEED) {
-		/* nop */
-	}
-	add_bread_crumb();
-
 	fprintf(stdout, "creating table\n");
 	dbcmd(dbproc, "create table #dblib0023 (col1 int not null,  col2 char(1) not null, col3 datetime not null)");
 	dbsqlexec(dbproc);
@@ -153,19 +130,19 @@ main(int argc, char *argv[])
 
 	add_bread_crumb();
 	fprintf(stdout, "binding row columns\n");
-	if (SUCCEED != dbbind(dbproc, 1, INTBIND, -1, (BYTE *) & rowint)) {
+	if (SUCCEED != dbbind(dbproc, 1, INTBIND, 0, (BYTE *) & rowint)) {
 		failed = 1;
 		fprintf(stderr, "Had problem with bind col1\n");
 		abort();
 	}
 	add_bread_crumb();
-	if (SUCCEED != dbbind(dbproc, 2, STRINGBIND, -1, (BYTE *) rowchar)) {
+	if (SUCCEED != dbbind(dbproc, 2, STRINGBIND, 0, (BYTE *) rowchar)) {
 		failed = 1;
 		fprintf(stderr, "Had problem with bind col2\n");
 		abort();
 	}
 	add_bread_crumb();
-	if (SUCCEED != dbbind(dbproc, 3, STRINGBIND, -1, (BYTE *) rowdate)) {
+	if (SUCCEED != dbbind(dbproc, 3, STRINGBIND, 0, (BYTE *) rowdate)) {
 		failed = 1;
 		fprintf(stderr, "Had problem with bind col3\n");
 		abort();
@@ -199,7 +176,7 @@ main(int argc, char *argv[])
 		abort();
 	}
 
-	if (SUCCEED != dbaltbind(dbproc, 1, 1, INTBIND, -1, (BYTE *) & computeint)) {
+	if (SUCCEED != dbaltbind(dbproc, 1, 1, INTBIND, 0, (BYTE *) & computeint)) {
 		failed = 1;
 		fprintf(stderr, "Had problem with dbaltbind 1, 1\n");
 		abort();
