@@ -1,5 +1,6 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004  Brian Bruns
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005  Brian Bruns
+ * Copyright (C) 2005 Frediano Ziglio
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,24 +22,22 @@
 #include <config.h>
 #endif
 
-#include <stdlib.h>
+#include <stdarg.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 #include <sys/stat.h>
 
-#include "tds.h"
 #include "tdsodbc.h"
 #include "tdsstring.h"
-#include "connectparams.h"
 #include "replacements.h"
 
 #ifdef DMALLOC
 #include <dmalloc.h>
 #endif
 
-static const char software_version[] = "$Id: connectparams.c,v 1.56 2004/11/07 09:35:53 freddy77 Exp $";
-static const void *const no_unused_var_warn[] = { software_version, no_unused_var_warn };
+TDS_RCSID(var, "$Id: connectparams.c,v 1.72 2007/07/01 10:10:52 freddy77 Exp $");
 
 #if !HAVE_SQLGETPRIVATEPROFILESTRING
 
@@ -96,6 +95,25 @@ static int SQLGetPrivateProfileString(LPCSTR pszSection, LPCSTR pszEntry, LPCSTR
 #define FILENAME_MAX 512
 #endif
 
+static int
+parse_server(char *server, TDSCONNECTION * connection)
+{
+	char ip[64];
+	char *p = (char *) strchr(server, '\\');
+
+	if (p) {
+		if (!tds_dstr_copy(&connection->instance_name, p+1))
+			return 0;
+		*p = 0;
+	}
+
+	tds_lookup_host(server, ip);
+	if (!tds_dstr_copy(&connection->ip_addr, ip))
+		return 0;
+
+	return 1;
+}
+
 /** 
  * Read connection information from given DSN
  * @param DSN           DSN name
@@ -131,8 +149,8 @@ odbc_get_dsn_info(const char *DSN, TDSCONNECTION * connection)
 		if (SQLGetPrivateProfileString(DSN, "Server", "", tmp, FILENAME_MAX, "odbc.ini") > 0) {
 			tds_dstr_copy(&connection->server_name, tmp);
 			if (!address_specified) {
-				tds_lookup_host(tds_dstr_cstr(&connection->server_name), tmp);
-				tds_dstr_copy(&connection->ip_addr, tmp);
+				if (!parse_server(tmp, connection))
+					return 0;
 			}
 		}
 	}
@@ -153,33 +171,9 @@ odbc_get_dsn_info(const char *DSN, TDSCONNECTION * connection)
 	}
 
 	tmp[0] = '\0';
-	if (SQLGetPrivateProfileString(DSN, "Database", "", tmp, FILENAME_MAX, "odbc.ini") > 0) {
+	if (tds_dstr_isempty(&connection->database)
+	    && SQLGetPrivateProfileString(DSN, "Database", "", tmp, FILENAME_MAX, "odbc.ini") > 0)
 		tds_dstr_copy(&connection->database, tmp);
-	}
-#if 0
-	tmp[0] = '\0';
-	if (SQLGetPrivateProfileString(DSN, "Authentication", "Server", tmp, FILENAME_MAX, "odbc.ini") > 0) {
-		if (!strcasecmp(tmp, "Server")) {
-			connection->try_domain_login = 0;
-			connection->try_server_login = 1;
-		} else if (!strcasecmp(tmp, "Domain")) {
-			connection->try_domain_login = 1;
-			connection->try_server_login = 0;
-		} else if (!strcasecmp(tmp, "Both")) {
-			connection->try_server_login = 1;
-			connection->try_domain_login = 1;
-		} else {
-			/* default to server authentication */
-			connection->try_domain_login = 0;
-			connection->try_server_login = 1;
-		}
-	}
-#endif
-
-	tmp[0] = '\0';
-	if (SQLGetPrivateProfileString(DSN, "Domain", "", tmp, FILENAME_MAX, "odbc.ini") > 0) {
-		tds_dstr_copy(&connection->default_domain, tmp);
-	}
 
 	tmp[0] = '\0';
 	if (SQLGetPrivateProfileString(DSN, "TextSize", "", tmp, FILENAME_MAX, "odbc.ini") > 0) {
@@ -208,7 +202,6 @@ odbc_parse_connect_string(const char *connect_string, const char *connect_string
 	DSTR *dest_s, value;
 	int reparse = 0;	/* flag for indicate second parse of string */
 	char option[16];
-	char tmp[256];
 
 	tds_dstr_init(&value);
 	for (p = connect_string; p && *p;) {
@@ -254,15 +247,15 @@ odbc_parse_connect_string(const char *connect_string, const char *connect_string
 			/* ignore if servername or DSN specified */
 			if (!reparse) {
 				dest_s = &connection->server_name;
-				tds_lookup_host(tds_dstr_cstr(&value), tmp);
-				if (!tds_dstr_copy(&connection->ip_addr, tmp)) {
+				/* not that safe cast but works -- freddy77 */
+				if (!parse_server((char *) tds_dstr_cstr(&value), connection)) {
 					tds_dstr_free(&value);
 					return 0;
 				}
 			}
 		} else if (strcasecmp(option, "SERVERNAME") == 0) {
 			if (!reparse) {
-				tds_dstr_copy(&connection->server_name, tds_dstr_cstr(&value));
+				tds_dstr_dup(&connection->server_name, &value);
 				tds_read_conf_file(connection, tds_dstr_cstr(&value));
 				reparse = 1;
 				p = connect_string;
@@ -284,15 +277,13 @@ odbc_parse_connect_string(const char *connect_string, const char *connect_string
 		} else if (strcasecmp(option, "APP") == 0) {
 			dest_s = &connection->app_name;
 		} else if (strcasecmp(option, "WSID") == 0) {
-			dest_s = &connection->host_name;
+			dest_s = &connection->client_host_name;
 		} else if (strcasecmp(option, "LANGUAGE") == 0) {
 			dest_s = &connection->language;
 		} else if (strcasecmp(option, "Port") == 0) {
 			connection->port = atoi(tds_dstr_cstr(&value));
 		} else if (strcasecmp(option, "TDS_Version") == 0) {
 			tds_config_verstr(tds_dstr_cstr(&value), connection);
-		} else if (strcasecmp(option, "Domain") == 0) {
-			dest_s = &connection->default_domain;
 		} else if (strcasecmp(option, "TextSize") == 0) {
 			connection->text_size = atoi(tds_dstr_cstr(&value));
 		} else if (strcasecmp(option, "PacketSize") == 0) {
@@ -302,8 +293,9 @@ odbc_parse_connect_string(const char *connect_string, const char *connect_string
 
 		/* copy to destination */
 		if (dest_s) {
-			tds_dstr_set(dest_s, tds_dstr_cstr(&value));
-			tds_dstr_init(&value);
+			DSTR tmp = *dest_s;
+			*dest_s = value;
+			value = tmp;
 		}
 
 		p = end;
@@ -341,8 +333,7 @@ tdoParseProfile(const char *option, const char *value, void *param)
 	ProfileParam *p = (ProfileParam *) param;
 
 	if (strcasecmp(p->entry, option) == 0) {
-		strncpy(p->buffer, value, p->buffer_len);
-		p->buffer[p->buffer_len - 1] = '\0';
+		tds_strlcpy(p->buffer, value, p->buffer_len);
 
 		p->ret_val = strlen(p->buffer);
 		p->found = 1;
@@ -356,23 +347,23 @@ SQLGetPrivateProfileString(LPCSTR pszSection, LPCSTR pszEntry, LPCSTR pszDefault
 	FILE *hFile;
 	ProfileParam param;
 
+	tdsdump_log(TDS_DBG_FUNC, "SQLGetPrivateProfileString(%p, %p, %p, %p, %d, %p)\n", 
+			pszSection, pszEntry, pszDefault, pRetBuffer, nRetBuffer, pszFileName);
+
 	if (!pszSection) {
 		/* spec says return list of all section names - but we will just return nothing */
-		fprintf(stderr, "[FreeTDS][ODBC][%s][%d] WARNING: Functionality for NULL pszSection not implemented.\n", __FILE__,
-			__LINE__);
+		tdsdump_log(TDS_DBG_WARN, "WARNING: Functionality for NULL pszSection not implemented.\n");
 		return 0;
 	}
 
 	if (!pszEntry) {
 		/* spec says return list of all key names in section - but we will just return nothing */
-		fprintf(stderr, "[FreeTDS][ODBC][%s][%d] WARNING: Functionality for NULL pszEntry not implemented.\n", __FILE__,
-			__LINE__);
+		tdsdump_log(TDS_DBG_WARN, "WARNING: Functionality for NULL pszEntry not implemented.\n");
 		return 0;
 	}
 
 	if (nRetBuffer < 1)
-		fprintf(stderr, "[FreeTDS][ODBC][%s][%d] WARNING: No space to return a value because nRetBuffer < 1.\n", __FILE__,
-			__LINE__);
+		tdsdump_log(TDS_DBG_WARN, "WARNING: No space to return a value because nRetBuffer < 1.\n");
 
 	if (pszFileName && *pszFileName == '/')
 		hFile = fopen(pszFileName, "r");
@@ -380,7 +371,7 @@ SQLGetPrivateProfileString(LPCSTR pszSection, LPCSTR pszEntry, LPCSTR pszDefault
 		hFile = tdoGetIniFileName();
 
 	if (hFile == NULL) {
-		fprintf(stderr, "[FreeTDS][ODBC][%s][%d] ERROR: Could not open configuration file\n", __FILE__, __LINE__);
+		tdsdump_log(TDS_DBG_ERROR, "ERROR: Could not open configuration file\n");
 		return 0;
 	}
 
@@ -394,8 +385,7 @@ SQLGetPrivateProfileString(LPCSTR pszSection, LPCSTR pszEntry, LPCSTR pszDefault
 	tds_read_conf_section(hFile, pszSection, tdoParseProfile, &param);
 
 	if (pszDefault && !param.found) {
-		strncpy(pRetBuffer, pszDefault, nRetBuffer);
-		pRetBuffer[nRetBuffer - 1] = '\0';
+		tds_strlcpy(pRetBuffer, pszDefault, nRetBuffer);
 
 		param.ret_val = strlen(pRetBuffer);
 	}
@@ -513,8 +503,8 @@ ODBCINSTGetProperties(HODBCINSTPROPERTY hLastProperty)
 	hLastProperty = hLastProperty->pNext;
 	memset(hLastProperty, 0, sizeof(ODBCINSTPROPERTY));
 	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_TEXTEDIT;
-	strncpy(hLastProperty->szName, "Servername", INI_MAX_PROPERTY_NAME);
-	strncpy(hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE);
+	tds_strlcpy(hLastProperty->szName, "Servername", INI_MAX_PROPERTY_NAME);
+	tds_strlcpy(hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE);
 	hLastProperty->pszHelp = (char *) strdup("Name of FreeTDS connection to connect to.\n"
 						 "This server name refer to entry in freetds.conf file, not real server name.\n"
 						 "This property cannot be used with Server property.");
@@ -523,8 +513,8 @@ ODBCINSTGetProperties(HODBCINSTPROPERTY hLastProperty)
 	hLastProperty = hLastProperty->pNext;
 	memset(hLastProperty, 0, sizeof(ODBCINSTPROPERTY));
 	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_TEXTEDIT;
-	strncpy(hLastProperty->szName, "Server", INI_MAX_PROPERTY_NAME);
-	strncpy(hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE);
+	tds_strlcpy(hLastProperty->szName, "Server", INI_MAX_PROPERTY_NAME);
+	tds_strlcpy(hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE);
 	hLastProperty->pszHelp = (char *) strdup("Name of server to connect to.\n"
 						 "This should be the name of real server.\n"
 						 "This property cannot be used with Servername property.");
@@ -533,24 +523,24 @@ ODBCINSTGetProperties(HODBCINSTPROPERTY hLastProperty)
 	hLastProperty = hLastProperty->pNext;
 	memset(hLastProperty, 0, sizeof(ODBCINSTPROPERTY));
 	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_TEXTEDIT;
-	strncpy(hLastProperty->szName, "Address", INI_MAX_PROPERTY_NAME);
-	strncpy(hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE);
+	tds_strlcpy(hLastProperty->szName, "Address", INI_MAX_PROPERTY_NAME);
+	tds_strlcpy(hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE);
 	hLastProperty->pszHelp = (char *) strdup("The hostname or ip address of the server.");
 
 	hLastProperty->pNext = (HODBCINSTPROPERTY) malloc(sizeof(ODBCINSTPROPERTY));
 	hLastProperty = hLastProperty->pNext;
 	memset(hLastProperty, 0, sizeof(ODBCINSTPROPERTY));
 	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_TEXTEDIT;
-	strncpy(hLastProperty->szName, "Port", INI_MAX_PROPERTY_NAME);
-	strncpy(hLastProperty->szValue, "1433", INI_MAX_PROPERTY_VALUE);
+	tds_strlcpy(hLastProperty->szName, "Port", INI_MAX_PROPERTY_NAME);
+	tds_strlcpy(hLastProperty->szValue, "1433", INI_MAX_PROPERTY_VALUE);
 	hLastProperty->pszHelp = (char *) strdup("TCP/IP Port to connect to.");
 
 	hLastProperty->pNext = (HODBCINSTPROPERTY) malloc(sizeof(ODBCINSTPROPERTY));
 	hLastProperty = hLastProperty->pNext;
 	memset(hLastProperty, 0, sizeof(ODBCINSTPROPERTY));
 	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_TEXTEDIT;
-	strncpy(hLastProperty->szName, "Database", INI_MAX_PROPERTY_NAME);
-	strncpy(hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE);
+	tds_strlcpy(hLastProperty->szName, "Database", INI_MAX_PROPERTY_NAME);
+	tds_strlcpy(hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE);
 	hLastProperty->pszHelp = (char *) strdup("Default database.");
 
 	hLastProperty->pNext = (HODBCINSTPROPERTY) malloc(sizeof(ODBCINSTPROPERTY));
@@ -559,8 +549,8 @@ ODBCINSTGetProperties(HODBCINSTPROPERTY hLastProperty)
 	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_LISTBOX;
 	hLastProperty->aPromptData = malloc(sizeof(aTDSver));
 	memcpy(hLastProperty->aPromptData, aTDSver, sizeof(aTDSver));
-	strncpy(hLastProperty->szName, "TDS_Version", INI_MAX_PROPERTY_NAME);
-	strncpy(hLastProperty->szValue, "4.2", INI_MAX_PROPERTY_VALUE);
+	tds_strlcpy(hLastProperty->szName, "TDS_Version", INI_MAX_PROPERTY_NAME);
+	tds_strlcpy(hLastProperty->szValue, "4.2", INI_MAX_PROPERTY_VALUE);
 	hLastProperty->pszHelp = (char *) strdup("The TDS protocol version.\n"
 						 " 4.2 MSSQL 6.5 or Sybase < 10.x\n"
 						 " 5.0 Sybase >= 10.x\n" " 7.0 MSSQL 7 or MSSQL 2000\n" " 8.0 MSSQL 2000");
@@ -571,16 +561,16 @@ ODBCINSTGetProperties(HODBCINSTPROPERTY hLastProperty)
 	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_COMBOBOX;
 	hLastProperty->aPromptData = malloc(sizeof(aLanguage));
 	memcpy(hLastProperty->aPromptData, aLanguage, sizeof(aLanguage));
-	strncpy(hLastProperty->szName, "Language", INI_MAX_PROPERTY_NAME);
-	strncpy(hLastProperty->szValue, "us_english", INI_MAX_PROPERTY_VALUE);
+	tds_strlcpy(hLastProperty->szName, "Language", INI_MAX_PROPERTY_NAME);
+	tds_strlcpy(hLastProperty->szValue, "us_english", INI_MAX_PROPERTY_VALUE);
 	hLastProperty->pszHelp = (char *) strdup("The default language setting.");
 
 	hLastProperty->pNext = (HODBCINSTPROPERTY) malloc(sizeof(ODBCINSTPROPERTY));
 	hLastProperty = hLastProperty->pNext;
 	memset(hLastProperty, 0, sizeof(ODBCINSTPROPERTY));
 	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_HIDDEN;
-	strncpy(hLastProperty->szName, "TextSize", INI_MAX_PROPERTY_NAME);
-	strncpy(hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE);
+	tds_strlcpy(hLastProperty->szName, "TextSize", INI_MAX_PROPERTY_NAME);
+	tds_strlcpy(hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE);
 	hLastProperty->pszHelp = (char *) strdup("Text datatype limit.");
 
 	/* ??? in odbc.ini ??? */
@@ -589,16 +579,16 @@ ODBCINSTGetProperties(HODBCINSTPROPERTY hLastProperty)
 	hLastProperty = hLastProperty->pNext;
 	memset(hLastProperty, 0, sizeof(ODBCINSTPROPERTY));
 	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_TEXTEDIT;
-	strncpy(hLastProperty->szName, "UID", INI_MAX_PROPERTY_NAME);
-	strncpy(hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE);
+	tds_strlcpy(hLastProperty->szName, "UID", INI_MAX_PROPERTY_NAME);
+	tds_strlcpy(hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE);
 	hLastProperty->pszHelp = (char *) strdup("User ID (Beware of security issues).");
 
 	hLastProperty->pNext = (HODBCINSTPROPERTY) malloc(sizeof(ODBCINSTPROPERTY));
 	hLastProperty = hLastProperty->pNext;
 	memset(hLastProperty, 0, sizeof(ODBCINSTPROPERTY));
 	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_TEXTEDIT;
-	strncpy(hLastProperty->szName, "PWD", INI_MAX_PROPERTY_NAME);
-	strncpy(hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE);
+	tds_strlcpy(hLastProperty->szName, "PWD", INI_MAX_PROPERTY_NAME);
+	tds_strlcpy(hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE);
 	hLastProperty->pszHelp = (char *) strdup("Password (Beware of security issues).");
 */
 
@@ -609,8 +599,8 @@ ODBCINSTGetProperties(HODBCINSTPROPERTY hLastProperty)
 	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_LISTBOX;
 	hLastProperty->aPromptData = malloc(sizeof(aAuth));
 	memcpy(hLastProperty->aPromptData, aAuth, sizeof(aAuth));
-	strncpy(hLastProperty->szName, "Authentication", INI_MAX_PROPERTY_NAME);
-	strncpy(hLastProperty->szValue, "Server", INI_MAX_PROPERTY_VALUE);
+	tds_strlcpy(hLastProperty->szName, "Authentication", INI_MAX_PROPERTY_NAME);
+	tds_strlcpy(hLastProperty->szValue, "Server", INI_MAX_PROPERTY_VALUE);
 	hLastProperty->pszHelp = (char *) strdup("The server authentication mechanism.");
 */
 
@@ -618,16 +608,16 @@ ODBCINSTGetProperties(HODBCINSTPROPERTY hLastProperty)
 	hLastProperty = hLastProperty->pNext;
 	memset(hLastProperty, 0, sizeof(ODBCINSTPROPERTY));
 	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_TEXTEDIT;
-	strncpy(hLastProperty->szName, "Domain", INI_MAX_PROPERTY_NAME);
-	strncpy(hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE);
+	tds_strlcpy(hLastProperty->szName, "Domain", INI_MAX_PROPERTY_NAME);
+	tds_strlcpy(hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE);
 	hLastProperty->pszHelp = (char *) strdup("The default domain to use when using Domain Authentication.");
 
 	hLastProperty->pNext = (HODBCINSTPROPERTY) malloc(sizeof(ODBCINSTPROPERTY));
 	hLastProperty = hLastProperty->pNext;
 	memset(hLastProperty, 0, sizeof(ODBCINSTPROPERTY));
 	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_TEXTEDIT;
-	strncpy(hLastProperty->szName, "PacketSize", INI_MAX_PROPERTY_NAME);
-	strncpy(hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE);
+	tds_strlcpy(hLastProperty->szName, "PacketSize", INI_MAX_PROPERTY_NAME);
+	tds_strlcpy(hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE);
 	hLastProperty->pszHelp = (char *) strdup("Size of network packets.");
 
 	return 1;
