@@ -1,6 +1,6 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
  * Copyright (C) 1998-1999  Brian Bruns
- * Copyright (C) 2003-2004  Frediano Ziglio
+ * Copyright (C) 2003-2005  Frediano Ziglio
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -32,21 +32,18 @@
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
-#include "tds.h"
 #include "tdsodbc.h"
 #include "tdsconvert.h"
-#include "odbc_util.h"
-#include "convert_tds2sql.h"
 
 #ifdef DMALLOC
 #include <dmalloc.h>
 #endif
 
-static const char software_version[] = "$Id: convert_tds2sql.c,v 1.40 2004/10/28 12:42:12 freddy77 Exp $";
-static const void *const no_unused_var_warn[] = { software_version, no_unused_var_warn };
+TDS_RCSID(var, "$Id: convert_tds2sql.c,v 1.49.2.1 2008/01/29 10:14:28 freddy77 Exp $");
 
 TDS_INT
-convert_tds2sql(TDSCONTEXT * context, int srctype, TDS_CHAR * src, TDS_UINT srclen, int desttype, TDS_CHAR * dest, SQLULEN destlen)
+convert_tds2sql(TDSCONTEXT * context, int srctype, TDS_CHAR * src, TDS_UINT srclen, int desttype, TDS_CHAR * dest, SQLULEN destlen,
+		const struct _drecord *drec_ixd)
 {
 	TDS_INT nDestSybType;
 	TDS_INT nRetVal = TDS_FAIL;
@@ -59,25 +56,29 @@ convert_tds2sql(TDSCONTEXT * context, int srctype, TDS_CHAR * src, TDS_UINT srcl
 	TIMESTAMP_STRUCT *tssp;
 	SQL_NUMERIC_STRUCT *num;
 
-	TDS_UINT *uip;
-	TDS_USMALLINT *usip;
-
-	int ret = TDS_FAIL;
+	int ret = TDS_CONVERT_FAIL;
 	int i, cplen;
 
 	tdsdump_log(TDS_DBG_FUNC, "convert_tds2sql: src is %d dest = %d\n", srctype, desttype);
 
 	assert(desttype != SQL_C_DEFAULT);
 
+	nDestSybType = odbc_c_to_server_type(desttype);
+	if (nDestSybType == TDS_FAIL)
+		return TDS_CONVERT_NOAVAIL;
+
 	/* special case for binary type */
 	if (desttype == SQL_C_BINARY) {
 		tdsdump_log(TDS_DBG_FUNC, "convert_tds2sql: outputting binary data destlen = %lu \n", (unsigned long) destlen);
 
 		if (is_numeric_type(srctype)) {
 			desttype = SQL_C_NUMERIC;
+			nDestSybType = SYBNUMERIC;
 			/* prevent buffer overflow */
 			if (destlen < sizeof(SQL_NUMERIC_STRUCT))
 				return TDS_CONVERT_FAIL;
+			ores.n.precision = ((TDS_NUMERIC *) src)->precision;
+			ores.n.scale = ((TDS_NUMERIC *) src)->scale;
 		} else {
 			ret = srclen;
 			if (destlen > 0) {
@@ -88,24 +89,39 @@ convert_tds2sql(TDSCONTEXT * context, int srctype, TDS_CHAR * src, TDS_UINT srcl
 			} else {
 				/* if destlen == 0 we return only length */
 				if (destlen != 0)
-					ret = TDS_FAIL;
+					ret = TDS_CONVERT_FAIL;
 			}
 			return ret;
 		}
-	}
-
-	nDestSybType = odbc_c_to_server_type(desttype);
-	if (nDestSybType == TDS_FAIL)
-		return TDS_CONVERT_NOAVAIL;
-
-	if (is_numeric_type(nDestSybType)) {
+	} else if (is_numeric_type(nDestSybType)) {
 		/* TODO use descriptor information (APD) ?? However APD can contain SQL_C_DEFAULT... */
-		ores.n.precision = 38;
+		if (drec_ixd)
+			ores.n.precision = drec_ixd->sql_desc_precision;
+		else
+			ores.n.precision = 38;
 		ores.n.scale = 0;
 	}
 
+	if (desttype == SQL_C_CHAR) {
+		nDestSybType = TDS_CONVERT_CHAR;
+		ores.cc.len = destlen >= 0 ? destlen : 0;
+		ores.cc.c = dest;
+	}
 
-	nRetVal = tds_convert(context, srctype, src, srclen, nDestSybType, &ores);
+	if (desttype == SQL_C_CHAR && (srctype == SYBDATETIME || srctype == SYBDATETIME4)) {
+		char buf[40];
+		TDSDATEREC when;
+
+		memset(&when, 0, sizeof(when));
+
+		tds_datecrack(srctype, src, &when);
+		tds_strftime(buf, sizeof(buf), srctype == SYBDATETIME ? "%Y-%m-%d %H:%M:%S.%z" : "%Y-%m-%d %H:%M:%S", &when);
+
+		nRetVal = strlen(buf);
+		memcpy(dest, buf, destlen >= 0 ? (destlen < nRetVal ? destlen : nRetVal) : 0);
+	} else {
+		nRetVal = tds_convert(context, srctype, src, srclen, nDestSybType, &ores);
+	}
 	if (nRetVal < 0)
 		return nRetVal;
 
@@ -119,17 +135,16 @@ convert_tds2sql(TDSCONTEXT * context, int srctype, TDS_CHAR * src, TDS_UINT srcl
 		if (destlen > 0) {
 			cplen = (destlen - 1) > nRetVal ? nRetVal : (destlen - 1);
 			assert(cplen >= 0);
-			/* odbc always terminate but do not overwrite 
-			 * destination buffer more than needed */
-			memcpy(dest, ores.c, cplen);
+			/*
+			 * odbc always terminate but do not overwrite 
+			 * destination buffer more than needed
+			 */
 			dest[cplen] = 0;
 		} else {
 			/* if destlen == 0 we return only length */
 			if (destlen != 0)
-				ret = TDS_FAIL;
+				ret = TDS_CONVERT_FAIL;
 		}
-
-		free(ores.c);
 		break;
 
 	case SQL_C_TYPE_DATE:
@@ -190,50 +205,40 @@ convert_tds2sql(TDSCONTEXT * context, int srctype, TDS_CHAR * src, TDS_UINT srcl
 #ifdef SQL_C_SBIGINT
 	case SQL_C_SBIGINT:
 	case SQL_C_UBIGINT:
-		memcpy(dest, &(ores.bi), sizeof(TDS_INT8));
+		*((TDS_INT8 *) dest) = ores.bi;
 		ret = sizeof(TDS_INT8);
 		break;
 #endif
 
 	case SQL_C_LONG:
 	case SQL_C_SLONG:
-		memcpy(dest, &(ores.i), sizeof(TDS_INT));
-		ret = sizeof(TDS_INT);
-		break;
-
 	case SQL_C_ULONG:
-		uip = (TDS_UINT *) dest;
-		*uip = ores.i;
-		ret = sizeof(TDS_UINT);
+		*((TDS_INT *) dest) = ores.i;
+		ret = sizeof(TDS_INT);
 		break;
 
 	case SQL_C_SHORT:
 	case SQL_C_SSHORT:
-		memcpy(dest, &(ores.si), sizeof(TDS_SMALLINT));
-		ret = sizeof(TDS_SMALLINT);
-		break;
-
 	case SQL_C_USHORT:
-		usip = (TDS_USMALLINT *) dest;
-		*usip = ores.si;
-		ret = sizeof(TDS_USMALLINT);
+		*((TDS_SMALLINT *) dest) = ores.si;
+		ret = sizeof(TDS_SMALLINT);
 		break;
 
 	case SQL_C_TINYINT:
 	case SQL_C_STINYINT:
 	case SQL_C_UTINYINT:
 	case SQL_C_BIT:
-		memcpy(dest, &(ores.ti), sizeof(TDS_TINYINT));
+		*((TDS_TINYINT *) dest) = ores.ti;
 		ret = sizeof(TDS_TINYINT);
 		break;
 
 	case SQL_C_DOUBLE:
-		memcpy(dest, &(ores.f), sizeof(TDS_FLOAT));
+		*((TDS_FLOAT *) dest) = ores.f;
 		ret = sizeof(TDS_FLOAT);
 		break;
 
 	case SQL_C_FLOAT:
-		memcpy(dest, &(ores.r), sizeof(TDS_REAL));
+		*((TDS_REAL *) dest) = ores.r;
 		ret = sizeof(TDS_REAL);
 		break;
 
@@ -243,8 +248,12 @@ convert_tds2sql(TDSCONTEXT * context, int srctype, TDS_CHAR * src, TDS_UINT srcl
 		num->precision = ores.n.precision;
 		num->scale = ores.n.scale;
 		num->sign = ores.n.array[0] ^ 1;
-		/* FIXME can i be greater than SQL_MAX_NUMERIC_LEN ?? */
-		i = tds_numeric_bytes_per_prec[ores.n.precision] - 1;
+		/*
+		 * TODO can be greater than SQL_MAX_NUMERIC_LEN ?? 
+		 * seeing Sybase manual wire support bigger numeric but currently
+		 * DBs so not support such precision
+		 */
+		i = ODBC_MIN(tds_numeric_bytes_per_prec[ores.n.precision] - 1, SQL_MAX_NUMERIC_LEN);
 		memcpy(num->val, ores.n.array + 1, i);
 		tds_swap_bytes(num->val, i);
 		if (i < SQL_MAX_NUMERIC_LEN)
