@@ -1,6 +1,6 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
  * Copyright (C) 1998-1999  Brian Bruns
- * Copyright (C) 2003  Frediano Ziglio
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007  Frediano Ziglio
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -22,6 +22,7 @@
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
 
+#include <stdarg.h>
 #include <stdio.h>
 
 #if HAVE_STDLIB_H
@@ -34,11 +35,9 @@
 
 #include <assert.h>
 
-#include "tds.h"
+#include "tdsodbc.h"
 #include "tdsstring.h"
 #include "replacements.h"
-#include "tdsodbc.h"
-#include "odbc_util.h"
 
 #if HAVE_ODBCSS_H
 #include <odbcss.h>
@@ -48,14 +47,12 @@
 #include <dmalloc.h>
 #endif
 
-static const char software_version[] = "$Id: error.c,v 1.34 2004/10/28 12:42:12 freddy77 Exp $";
-static const void *const no_unused_var_warn[] = { software_version, no_unused_var_warn };
+TDS_RCSID(var, "$Id: error.c,v 1.49 2007/10/16 15:12:21 freddy77 Exp $");
 
 static void odbc_errs_pop(struct _sql_errors *errs);
 static const char *odbc_get_msg(const char *sqlstate);
 static void odbc_get_v2state(const char *sqlstate, char *dest_state);
 static void sqlstate2to3(char *state);
-SQLRETURN _SQLRowCount(SQLHSTMT hstmt, SQLLEN FAR * pcrow);
 
 struct s_SqlMsgMap
 {
@@ -155,6 +152,7 @@ static const struct s_SqlMsgMap SqlMsgMap[] = {
 	ODBCERR("HY105", "Invalid parameter type"),
 	ODBCERR("HY106", "Fetch type out of range"),
 	ODBCERR("HY107", "Row value out of range"),
+	ODBCERR("HY108", "Concurrency option out of range"),
 	ODBCERR("HY109", "Invalid cursor position"),
 	ODBCERR("HY110", "Invalid driver completion"),
 	ODBCERR("HY111", "Invalid bookmark value"),
@@ -237,6 +235,80 @@ static const struct s_v3to2map v3to2map[] = {
 	{"", ""}
 };
 
+/* 
+ * ODBC messages must be sorted by importance
+ * 1. Errors regarding the status of a transaction
+ * 2. Other errors (level ordered)
+ * 3. No-Data messages with a state class of 02 ??
+ * 4. Warning
+ * 5. Informational
+ */
+static void
+rank_errors(struct _sql_errors *errs)
+{
+	int settled, current, best;
+	struct _sql_error swapbuf;
+	char istrans;
+
+	if (errs->ranked == 0 && errs->num_errors > 1) {
+		/* Find the highest of all unranked errors until there are none left */
+		for (settled = 0; settled < errs->num_errors; settled++) {
+			best = -1;
+			for (current = settled; current < errs->num_errors; current++) {
+				istrans = 0;
+				switch (errs->errs[current].native) {
+				case 1205:
+				case 1211:
+				case 2625:
+				case 3309:
+				case 7112:
+				case 266:
+				case 277:
+				case 611:
+				case 628:
+				case 3902:
+				case 3903:
+				case 3906:
+				case 3908:
+				case 6401:
+					istrans = 1;
+					break;
+				}
+
+				if (istrans == 0) {
+					if (strcmp(errs->errs[current].state3,"25000") == 0)
+						istrans = 1;
+					else if (strcmp(errs->errs[current].state3,"S1012") == 0)
+						istrans = 1;
+					else if (strcmp(errs->errs[current].state3,"08007") == 0)
+						istrans = 1;
+				}
+
+				/* Transaction errors are always best */
+				if (istrans == 1 && errs->errs[current].msgstate >= 10)	{
+					best = current;
+					break;
+				}
+
+				if (best == -1)
+					best = current;
+
+				/* Non-terminating comparisons only below this point */
+				if (errs->errs[current].msgstate > errs->errs[best].msgstate)
+					best = current;
+			}
+
+			/* swap settled position with best */
+			if (best != settled) {
+				swapbuf = errs->errs[settled];
+				errs->errs[settled] = errs->errs[best];
+				errs->errs[best] = swapbuf;
+			}
+		}
+	}
+	errs->ranked = 1;
+}
+
 static const char *
 odbc_get_msg(const char *sqlstate)
 {
@@ -257,16 +329,15 @@ odbc_get_v2state(const char *sqlstate, char *dest_state)
 {
 	const struct s_v3to2map *pmap = v3to2map;
 
-	dest_state[5] = 0;
 	while (pmap->v3[0]) {
 		if (!strcasecmp(pmap->v3, sqlstate)) {
-			strncpy(dest_state, pmap->v2, 5);
+			tds_strlcpy(dest_state, pmap->v2, 6);
 			return;
 		}
 		++pmap;
 	}
 	/* return the original if a v2 state is not found */
-	strncpy(dest_state, sqlstate, 5);
+	tds_strlcpy(dest_state, sqlstate, 6);
 }
 
 void
@@ -277,15 +348,14 @@ odbc_errs_reset(struct _sql_errors *errs)
 	if (errs->errs) {
 		for (i = 0; i < errs->num_errors; ++i) {
 			/* TODO see flags */
-			if (errs->errs[i].msg)
-				free((char *) errs->errs[i].msg);
-			if (errs->errs[i].server)
-				free(errs->errs[i].server);
+			free((char *) errs->errs[i].msg);
+			free(errs->errs[i].server);
 		}
 		TDS_ZERO_FREE(errs->errs);
 		errs->num_errors = 0;
 	}
 	errs->lastrc = SQL_SUCCESS;
+	errs->ranked = 0;
 	assert(errs->num_errors == 0);
 }
 
@@ -302,17 +372,15 @@ odbc_errs_pop(struct _sql_errors *errs)
 	}
 
 	/* TODO see flags */
-	if (errs->errs[0].msg)
-		free((char *) errs->errs[0].msg);
-	if (errs->errs[0].server)
-		free(errs->errs[0].server);
+	free((char *) errs->errs[0].msg);
+	free(errs->errs[0].server);
 
 	--errs->num_errors;
 	memmove(&(errs->errs[0]), &(errs->errs[1]), errs->num_errors * sizeof(errs->errs[0]));
 }
 
 void
-odbc_errs_add(struct _sql_errors *errs, const char *sqlstate, const char *msg, const char *server)
+odbc_errs_add(struct _sql_errors *errs, const char *sqlstate, const char *msg)
 {
 	struct _sql_error *p;
 	int n = errs->num_errors;
@@ -329,12 +397,11 @@ odbc_errs_add(struct _sql_errors *errs, const char *sqlstate, const char *msg, c
 
 	memset(&errs->errs[n], 0, sizeof(struct _sql_error));
 	errs->errs[n].native = 0;
-	strncpy(errs->errs[n].state3, sqlstate, 5);
-	errs->errs[n].state3[5] = '\0';
+	tds_strlcpy(errs->errs[n].state3, sqlstate, 6);
 	odbc_get_v2state(errs->errs[n].state3, errs->errs[n].state2);
 
 	/* TODO why driver ?? -- freddy77 */
-	errs->errs[n].server = (server) ? strdup(server) : strdup("DRIVER");
+	errs->errs[n].server = strdup("DRIVER");
 	errs->errs[n].msg = msg ? strdup(msg) : odbc_get_msg(errs->errs[n].state3);
 	++errs->num_errors;
 }
@@ -357,10 +424,9 @@ odbc_errs_add_rdbms(struct _sql_errors *errs, TDS_UINT native, const char *sqlst
 
 	memset(&errs->errs[n], 0, sizeof(struct _sql_error));
 	errs->errs[n].native = native;
-	if (sqlstate) {
-		strncpy(errs->errs[n].state2, sqlstate, 5);
-		errs->errs[n].state2[5] = '\0';
-	} else
+	if (sqlstate)
+		tds_strlcpy(errs->errs[n].state2, sqlstate, 6);
+	else
 		errs->errs[n].state2[0] = '\0';
 	strcpy(errs->errs[n].state3, errs->errs[n].state2);
 	sqlstate2to3(errs->errs[n].state3);
@@ -441,6 +507,9 @@ _SQLGetDiagRec(SQLSMALLINT handleType, SQLHANDLE handle, SQLSMALLINT numRecord,
 
 	SQLINTEGER odbc_ver = SQL_OV_ODBC2;
 
+	tdsdump_log(TDS_DBG_FUNC, "_SQLGetDiagRec(%d, %p, %d, %p, %p, %p, %d, %p)\n", 
+			handleType, handle, numRecord, szSqlState, pfNativeError, szErrorMsg, cbErrorMsgMax, pcbErrorMsg);
+
 	if (numRecord <= 0 || cbErrorMsgMax < 0)
 		return SQL_ERROR;
 
@@ -471,6 +540,8 @@ _SQLGetDiagRec(SQLSMALLINT handleType, SQLHANDLE handle, SQLSMALLINT numRecord,
 		return SQL_NO_DATA_FOUND;
 	--numRecord;
 
+	rank_errors(errs);
+
 	if (szSqlState) {
 		if (odbc_ver == SQL_OV_ODBC3)
 			strcpy((char *) szSqlState, errs->errs[numRecord].state3);
@@ -491,7 +562,7 @@ _SQLGetDiagRec(SQLSMALLINT handleType, SQLHANDLE handle, SQLSMALLINT numRecord,
 	return result;
 }
 
-SQLRETURN SQL_API
+SQLRETURN ODBC_API
 SQLError(SQLHENV henv, SQLHDBC hdbc, SQLHSTMT hstmt, SQLCHAR FAR * szSqlState, SQLINTEGER FAR * pfNativeError,
 	 SQLCHAR FAR * szErrorMsg, SQLSMALLINT cbErrorMsgMax, SQLSMALLINT FAR * pcbErrorMsg)
 {
@@ -500,6 +571,9 @@ SQLError(SQLHENV henv, SQLHDBC hdbc, SQLHSTMT hstmt, SQLCHAR FAR * szSqlState, S
 	SQLSMALLINT type;
 	SQLHANDLE handle;
 
+	tdsdump_log(TDS_DBG_FUNC, "SQLError(%p, %p, %p, %p, %p, %p, %d, %p)\n", 
+			henv, hdbc, hstmt, szSqlState, pfNativeError, szErrorMsg, cbErrorMsgMax, pcbErrorMsg);
+
 	if (hstmt) {
 		errs = &((TDS_STMT *) hstmt)->errs;
 		handle = hstmt;
@@ -515,6 +589,7 @@ SQLError(SQLHENV henv, SQLHDBC hdbc, SQLHSTMT hstmt, SQLCHAR FAR * szSqlState, S
 	} else
 		return SQL_INVALID_HANDLE;
 
+	rank_errors(errs);
 
 	result = _SQLGetDiagRec(type, handle, 1, szSqlState, pfNativeError, szErrorMsg, cbErrorMsgMax, pcbErrorMsg);
 
@@ -527,14 +602,17 @@ SQLError(SQLHENV henv, SQLHDBC hdbc, SQLHSTMT hstmt, SQLCHAR FAR * szSqlState, S
 }
 
 #if (ODBCVER >= 0x0300)
-SQLRETURN SQL_API
+SQLRETURN ODBC_API
 SQLGetDiagRec(SQLSMALLINT handleType, SQLHANDLE handle, SQLSMALLINT numRecord, SQLCHAR FAR * szSqlState,
 	      SQLINTEGER FAR * pfNativeError, SQLCHAR * szErrorMsg, SQLSMALLINT cbErrorMsgMax, SQLSMALLINT FAR * pcbErrorMsg)
 {
+	tdsdump_log(TDS_DBG_FUNC, "SQLGetDiagRec(%d, %p, %d, %p, %p, %p, %d, %p)\n", 
+			handleType, handle, numRecord, szSqlState, pfNativeError, szErrorMsg, cbErrorMsgMax, pcbErrorMsg);
 	return _SQLGetDiagRec(handleType, handle, numRecord, szSqlState, pfNativeError, szErrorMsg, cbErrorMsgMax, pcbErrorMsg);
 }
 
-SQLRETURN SQL_API
+
+SQLRETURN ODBC_API
 SQLGetDiagField(SQLSMALLINT handleType, SQLHANDLE handle, SQLSMALLINT numRecord, SQLSMALLINT diagIdentifier, SQLPOINTER buffer,
 		SQLSMALLINT cbBuffer, SQLSMALLINT FAR * pcbBuffer)
 {
@@ -549,6 +627,9 @@ SQLGetDiagField(SQLSMALLINT handleType, SQLHANDLE handle, SQLSMALLINT numRecord,
 	TDS_ENV *env = NULL;
 	char tmp[16];
 
+	tdsdump_log(TDS_DBG_FUNC, "SQLGetDiagField(%d, %p, %d, %d, %p, %d, %p)\n", 
+			handleType, handle, numRecord, diagIdentifier, buffer, cbBuffer, pcbBuffer);
+
 	if (cbBuffer < 0)
 		return SQL_ERROR;
 
@@ -685,8 +766,10 @@ SQLGetDiagField(SQLSMALLINT handleType, SQLHANDLE handle, SQLSMALLINT numRecord,
 			break;
 		case SQL_HANDLE_STMT:
 			msg = tds_dstr_cstr(&stmt->dbc->server);
-			/* if dbc->server is not initialized, init it
-			 * from the errs structure */
+			/*
+			 * if dbc->server is not initialized, init it
+			 * from the errs structure
+			 */
 			if (!msg[0] && errs->errs[numRecord].server) {
 				tds_dstr_copy(&stmt->dbc->server, errs->errs[numRecord].server);
 				msg = errs->errs[numRecord].server;
