@@ -1,5 +1,5 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 2003 Frediano Ziglio
+ * Copyright (C) 2003, 2004, 2005 Frediano Ziglio
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -20,6 +20,7 @@
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
 
+#include <stdarg.h>
 #include <stdio.h>
 
 #if HAVE_STDLIB_H
@@ -32,18 +33,14 @@
 
 #include <assert.h>
 
-#include "tds.h"
 #include "tdsodbc.h"
 #include "tdsstring.h"
-#include "odbc_util.h"
-#include "odbc_checks.h"
 
 #ifdef DMALLOC
 #include <dmalloc.h>
 #endif
 
-static const char software_version[] = "$Id: odbc_checks.c,v 1.13 2004/10/28 12:42:12 freddy77 Exp $";
-static const void *const no_unused_var_warn[] = { software_version, no_unused_var_warn };
+TDS_RCSID(var, "$Id: odbc_checks.c,v 1.19 2007/05/17 10:33:21 freddy77 Exp $");
 
 #if ENABLE_EXTRA_CHECKS
 
@@ -71,6 +68,16 @@ odbc_check_stmt_extra(TDS_STMT * stmt)
 	odbc_check_desc_extra(stmt->ipd);
 	assert(!stmt->prepared_query_is_func || stmt->prepared_query_is_rpc);
 	assert(stmt->param_num <= stmt->param_count + 1);
+	assert(stmt->num_param_rows >= 1);
+	assert(stmt->curr_param_row >= 0);
+	assert(stmt->curr_param_row <= stmt->num_param_rows);
+	if (stmt->prepared_query_is_rpc) {
+		char *query = stmt->prepared_query ? stmt->prepared_query : stmt->query;
+		assert(query);
+		assert(stmt->prepared_pos == NULL || (stmt->prepared_pos >= query && stmt->prepared_pos <= strchr(query,0)));
+	} else {
+		assert(stmt->prepared_pos == NULL);
+	}
 	/* TODO assert dbc has this statement in list */
 }
 
