@@ -1,5 +1,6 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004  Brian Bruns
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005  Brian Bruns
+ * Copyright (C) 2005, 2006 Frediano Ziglio
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -31,17 +32,13 @@
 
 #include <assert.h>
 
-#include "tds.h"
 #include "tdsodbc.h"
-#include "odbc_util.h"
-#include "convert_tds2sql.h"
 
 #ifdef DMALLOC
 #include <dmalloc.h>
 #endif
 
-static const char software_version[] = "$Id: odbc_util.c,v 1.70 2004/10/28 12:42:12 freddy77 Exp $";
-static const void *const no_unused_var_warn[] = { software_version, no_unused_var_warn };
+TDS_RCSID(var, "$Id: odbc_util.c,v 1.94.2.1 2008/01/29 10:14:31 freddy77 Exp $");
 
 /**
  * \ingroup odbc_api
@@ -51,7 +48,7 @@ static const void *const no_unused_var_warn[] = { software_version, no_unused_va
 
 /**
  * \addtogroup odbc_util
- *  \@{ 
+ * @{ 
  */
 
 static int
@@ -73,6 +70,9 @@ odbc_set_stmt(TDS_STMT * stmt, char **dest, const char *sql, int sql_len)
 	stmt->param_count = 0;
 	stmt->prepared_query_is_func = 0;
 	stmt->prepared_query_is_rpc = 0;
+	stmt->prepared_pos = NULL;
+	stmt->curr_param_row = 0;
+	stmt->num_param_rows = 1;
 
 	if (stmt->prepared_query)
 		TDS_ZERO_FREE(stmt->prepared_query);
@@ -109,36 +109,49 @@ odbc_set_stmt_prepared_query(TDS_STMT * stmt, const char *sql, int sql_len)
 
 
 void
-odbc_set_return_status(struct _hstmt *stmt)
+odbc_set_return_status(struct _hstmt *stmt, unsigned int n_row)
 {
 	TDSSOCKET *tds = stmt->dbc->tds_socket;
 	TDSCONTEXT *context = stmt->dbc->env->tds_ctx;
 
-#if 0
-	TDSLOCALE *locale = context->locale;
-#endif
-
+	/* TODO handle different type results (functions) on mssql2k */
 	if (stmt->prepared_query_is_func && tds->has_status) {
 		struct _drecord *drec;
 		int len;
+		const TDS_DESC* axd = stmt->apd;
+		TDS_INTPTR len_offset;
+		char *data_ptr;
 
-		if (stmt->apd->header.sql_desc_count < 1)
+		if (axd->header.sql_desc_count < 1)
 			return;
-		drec = &stmt->apd->records[0];
+		drec = &axd->records[0];
+		data_ptr = drec->sql_desc_data_ptr;
+
+		if (axd->header.sql_desc_bind_type != SQL_BIND_BY_COLUMN) {
+			len_offset = axd->header.sql_desc_bind_type * n_row;
+			if (axd->header.sql_desc_bind_offset_ptr)
+				len_offset += *axd->header.sql_desc_bind_offset_ptr;
+			data_ptr += len_offset;
+		} else {
+			len_offset = sizeof(SQLLEN) * n_row;
+			data_ptr += sizeof(SQLINTEGER) * n_row;
+		}
+#define LEN(ptr) *((SQLLEN*)(((char*)(ptr)) + len_offset))
 
 		len = convert_tds2sql(context, SYBINT4, (TDS_CHAR *) & tds->ret_status, sizeof(TDS_INT),
-				      drec->sql_desc_concise_type, drec->sql_desc_data_ptr, drec->sql_desc_octet_length);
-		if (TDS_FAIL == len)
+				      drec->sql_desc_concise_type, (void *) data_ptr, drec->sql_desc_octet_length, NULL);
+		if (len < 0)
 			return /* SQL_ERROR */ ;
 		if (drec->sql_desc_indicator_ptr)
-			*drec->sql_desc_indicator_ptr = 0;
-		*drec->sql_desc_octet_length_ptr = len;
+			LEN(drec->sql_desc_indicator_ptr) = 0;
+		if (drec->sql_desc_octet_length_ptr)
+			LEN(drec->sql_desc_octet_length_ptr) = len;
 	}
-
+#undef LEN
 }
 
 void
-odbc_set_return_params(struct _hstmt *stmt)
+odbc_set_return_params(struct _hstmt *stmt, unsigned int n_row)
 {
 	TDSSOCKET *tds = stmt->dbc->tds_socket;
 	TDSPARAMINFO *info = tds->current_results;
@@ -154,62 +167,83 @@ odbc_set_return_params(struct _hstmt *stmt)
 		return;
 
 	for (i = 0; i < info->num_cols; ++i) {
-		struct _drecord *drec_apd, *drec_ipd;
+		const TDS_DESC* axd = stmt->apd;
+		const struct _drecord *drec_apd, *drec_ipd;
 		TDSCOLUMN *colinfo = info->columns[i];
 		TDS_CHAR *src;
 		int srclen;
 		SQLINTEGER len;
 		int c_type;
+		char *data_ptr;
+		TDS_INTPTR len_offset;
 
 		/* find next output parameter */
 		for (;;) {
 			drec_apd = NULL;
 			/* TODO best way to stop */
-			if (nparam >= stmt->apd->header.sql_desc_count || nparam >= stmt->ipd->header.sql_desc_count)
+			if (nparam >= axd->header.sql_desc_count || nparam >= stmt->ipd->header.sql_desc_count)
 				return;
-			drec_apd = &stmt->apd->records[nparam];
+			drec_apd = &axd->records[nparam];
 			drec_ipd = &stmt->ipd->records[nparam];
 			if (stmt->ipd->records[nparam++].sql_desc_parameter_type != SQL_PARAM_INPUT)
 				break;
 		}
 
+		data_ptr = drec_apd->sql_desc_data_ptr;
+		if (axd->header.sql_desc_bind_type != SQL_BIND_BY_COLUMN) {
+			len_offset = axd->header.sql_desc_bind_type * n_row;
+			if (axd->header.sql_desc_bind_offset_ptr)
+				len_offset += *axd->header.sql_desc_bind_offset_ptr;
+			data_ptr += len_offset;
+		} else {
+			len_offset = sizeof(SQLLEN) * n_row;
+			data_ptr += odbc_get_octet_len(drec_apd->sql_desc_concise_type, drec_apd) * n_row;
+		}
+#define LEN(ptr) *((SQLLEN*)(((char*)(ptr)) + len_offset))
+
 		/* null parameter ? */
-		if (tds_get_null(info->current_row, i)) {
+		if (colinfo->column_cur_size < 0) {
 			/* FIXME error if NULL */
 			if (drec_apd->sql_desc_indicator_ptr)
-				*drec_apd->sql_desc_indicator_ptr = SQL_NULL_DATA;
+				LEN(drec_apd->sql_desc_indicator_ptr) = SQL_NULL_DATA;
 			continue;
 		}
 
-		src = (TDS_CHAR *) & info->current_row[colinfo->column_offset];
+		src = (TDS_CHAR *) colinfo->column_data;
 		if (is_blob_type(colinfo->column_type))
 			src = ((TDSBLOB *) src)->textvalue;
 		srclen = colinfo->column_cur_size;
 		c_type = drec_apd->sql_desc_concise_type;
 		if (c_type == SQL_C_DEFAULT)
 			c_type = odbc_sql_to_c_type_default(drec_ipd->sql_desc_concise_type);
+		/* 
+		 * TODO why IPD ?? perhaps SQLBindParameter it's not correct ??
+		 * Or tests are wrong ??
+		 */
 		len = convert_tds2sql(context, tds_get_conversion_type(colinfo->column_type, colinfo->column_size), src, srclen,
-				      c_type, drec_apd->sql_desc_data_ptr, drec_apd->sql_desc_octet_length);
+				      c_type, (void *) data_ptr, drec_apd->sql_desc_octet_length, drec_ipd);
 		/* TODO error handling */
 		if (len < 0)
 			return /* SQL_ERROR */ ;
 		if (drec_apd->sql_desc_indicator_ptr)
-			*drec_apd->sql_desc_indicator_ptr = 0;
-		*drec_apd->sql_desc_octet_length_ptr = len;
+			LEN(drec_apd->sql_desc_indicator_ptr) = 0;
+		if (drec_apd->sql_desc_octet_length_ptr)
+			LEN(drec_apd->sql_desc_octet_length_ptr) = len;
+#undef LEN
 	}
 }
 
 int
 odbc_get_string_size(int size, SQLCHAR * str)
 {
-	if (!str) {
-		return 0;
-	}
-	if (size == SQL_NTS) {
-		return strlen((const char *) str);
-	} else {
-		return size;
+	if (str) {
+		if (size == SQL_NTS)
+			return strlen((const char *) str);
+		if (size >= 0)
+			return size;
 	}
+	/* SQL_NULL_DATA or any other strange value */
+	return 0;
 }
 
 /**
@@ -269,11 +303,9 @@ odbc_server_to_sql_type(int col_type, int col_size)
 		}
 		break;
 	case SYBMONEY:
-		return SQL_DOUBLE;
 	case SYBMONEY4:
-		return SQL_DOUBLE;
 	case SYBMONEYN:
-		return SQL_DOUBLE;
+		return SQL_FLOAT;
 	case SYBDATETIME:
 	case SYBDATETIME4:
 	case SYBDATETIMN:
@@ -310,12 +342,25 @@ odbc_server_to_sql_type(int col_type, int col_size)
 	case SYBVARIANT:
 		break;
 #endif
-		/* TODO what should I do with these types ?? */
+		/*
+		 * TODO what should I do with these types ?? 
+		 * return other types can cause additional problems
+		 */
 	case SYBVOID:
 	case SYBSINT1:
 	case SYBUINT2:
 	case SYBUINT4:
 	case SYBUINT8:
+	case SYBUINT1:
+	case SYBDATE:
+	case SYBDATEN:
+	case SYB5INT8:
+	case SYBINTERVAL:
+	case SYBTIME:
+	case SYBTIMEN:
+	case SYBUINTN:
+	case SYBUNITEXT:
+	case SYBXML:
 		break;
 	}
 	return SQL_UNKNOWN_TYPE;
@@ -396,13 +441,16 @@ odbc_c_to_server_type(int c_type)
 }
 
 void
-odbc_set_sql_type_info(TDSCOLUMN * col, int odbc_ver, struct _drecord *drec)
+odbc_set_sql_type_info(TDSCOLUMN * col, struct _drecord *drec, SQLINTEGER odbc_ver)
 {
 #define SET_INFO(type, prefix, suffix) \
-	drec->sql_desc_literal_prefix = ""; \
-	drec->sql_desc_literal_suffix = ""; \
+	drec->sql_desc_literal_prefix = prefix; \
+	drec->sql_desc_literal_suffix = suffix; \
 	drec->sql_desc_type_name = type; \
 	return;
+#define SET_INFO2(type, prefix, suffix, len) \
+	drec->sql_desc_length = len; \
+	SET_INFO(type, prefix, suffix)
 
 	/* FIXME finish, support for N type (nvarchar) */
 	switch (tds_get_conversion_type(col->column_type, col->column_size)) {
@@ -416,28 +464,30 @@ odbc_set_sql_type_info(TDSCOLUMN * col, int odbc_ver, struct _drecord *drec)
 		SET_INFO("text", "'", "'");
 	case SYBBIT:
 	case SYBBITN:
-		SET_INFO("bit", "", "");
+		SET_INFO2("bit", "", "", 1);
 #if (ODBCVER >= 0x0300)
 	case SYBINT8:
 		/* TODO return numeric for odbc2 and convert bigint to numeric */
-		SET_INFO("bigint", "", "");
+		SET_INFO2("bigint", "", "", 19);
 #endif
 	case SYBINT4:
-		SET_INFO("int", "", "");
+		SET_INFO2("int", "", "", 10);
 	case SYBINT2:
-		SET_INFO("smallint", "", "");
+		SET_INFO2("smallint", "", "", 5);
 	case SYBINT1:
-		SET_INFO("tinyint", "", "");
+		SET_INFO2("tinyint", "", "", 3);
 	case SYBREAL:
-		SET_INFO("real", "", "");
+		SET_INFO2("real", "", "", odbc_ver == SQL_OV_ODBC3 ? 24 : 7);
 	case SYBFLT8:
-		SET_INFO("float", "", "");
+		SET_INFO2("float", "", "", odbc_ver == SQL_OV_ODBC3 ? 53 : 15);
 	case SYBMONEY:
+		SET_INFO2("money", "$", "", 19);
 	case SYBMONEY4:
-		SET_INFO("money", "$", "");
+		SET_INFO2("money", "$", "", 10);
 	case SYBDATETIME:
+		SET_INFO2("datetime", "'", "'", 23);
 	case SYBDATETIME4:
-		SET_INFO("datetime", "'", "'");
+		SET_INFO2("datetime", "'", "'", 16);
 	case SYBBINARY:
 		/* handle TIMESTAMP using usertype */
 		if (col->column_usertype == 80)
@@ -448,9 +498,9 @@ odbc_set_sql_type_info(TDSCOLUMN * col, int odbc_ver, struct _drecord *drec)
 	case SYBVARBINARY:
 		SET_INFO("varbinary", "0x", "");
 	case SYBNUMERIC:
-		SET_INFO("numeric", "", "");
+		SET_INFO2("numeric", "", "", col->column_prec);
 	case SYBDECIMAL:
-		SET_INFO("decimal", "", "");
+		SET_INFO2("decimal", "", "", col->column_prec);
 	case SYBINTN:
 	case SYBDATETIMN:
 	case SYBFLTN:
@@ -473,10 +523,11 @@ odbc_set_sql_type_info(TDSCOLUMN * col, int odbc_ver, struct _drecord *drec)
 	}
 	SET_INFO("", "", "");
 #undef SET_INFO
+#undef SET_INFO2
 }
 
 SQLINTEGER
-odbc_sql_to_displaysize(int sqltype, int column_size, int column_prec)
+odbc_sql_to_displaysize(int sqltype, TDSCOLUMN *col)
 {
 	SQLINTEGER size = 0;
 
@@ -484,7 +535,12 @@ odbc_sql_to_displaysize(int sqltype, int column_size, int column_prec)
 	case SQL_CHAR:
 	case SQL_VARCHAR:
 	case SQL_LONGVARCHAR:
-		size = column_size;
+		size = col->column_size;
+		break;
+	case SQL_BINARY:
+	case SQL_VARBINARY:
+	case SQL_LONGVARBINARY:
+		size = col->column_size * 2;
 		break;
 	case SQL_BIGINT:
 		size = 20;
@@ -495,12 +551,15 @@ odbc_sql_to_displaysize(int sqltype, int column_size, int column_prec)
 	case SQL_SMALLINT:
 		size = 6;	/* -10000 */
 		break;
+	case SQL_BIT:
+		size = 1;
+		break;
 	case SQL_TINYINT:
 		size = 3;	/* 255 */
 		break;
 	case SQL_DECIMAL:
 	case SQL_NUMERIC:
-		size = column_prec + 2;
+		size = col->column_prec + 2;
 		break;
 	case SQL_DATE:
 		/* FIXME check always yyyy-mm-dd ?? */
@@ -513,14 +572,24 @@ odbc_sql_to_displaysize(int sqltype, int column_size, int column_prec)
 	case SQL_TYPE_TIMESTAMP:
 	case SQL_TIMESTAMP:
 		/* TODO dependent on precision (decimal second digits) */
-		size = 24;	/* FIXME check, always format 
-				 * yyyy-mm-dd hh:mm:ss[.fff] ?? */
-		/* spinellia@acm.org: int token.c it is 30 should we comply? */
+		/* we always format using yyyy-mm-dd hh:mm:ss[.fff], see convert_tds2sql.c */
+		size = 19;
+		if (col->column_type == SYBDATETIME || (col->column_type == SYBDATETIMN && col->column_size == 8))
+			size = 23;
 		break;
 	case SQL_FLOAT:
 	case SQL_REAL:
 	case SQL_DOUBLE:
-		size = 24;	/* FIXME -- what should the correct size be? */
+		/* TODO check REAL/FLOAT format */
+		if (col->column_type == SYBREAL || (col->column_type == SYBFLTN && col->column_size == 4))
+			size = 14;
+		/* TODO check money format returned by propretary ODBC, scale == 4 but we use 2 digits */
+		else if (col->column_type == SYBMONEY || (col->column_type == SYBMONEYN && col->column_size == 8))
+			size = 21;
+		else if (col->column_type == SYBMONEY4 || (col->column_type == SYBMONEYN && col->column_size == 4))
+			size = 12;
+		else 
+			size = 24;	/* FIXME -- what should the correct size be? */
 		break;
 #ifdef SQL_GUID
 	case SQL_GUID:
@@ -528,7 +597,7 @@ odbc_sql_to_displaysize(int sqltype, int column_size, int column_prec)
 		break;
 #endif
 	default:
-		/* FIXME TODO finish, should support ALL types (interval) */
+		/* FIXME TODO finish, should support ALL types (interval, binary) */
 		size = 40;
 		tdsdump_log(TDS_DBG_INFO1, "odbc_sql_to_displaysize: unknown sql type %d\n", (int) sqltype);
 		break;
@@ -612,7 +681,9 @@ odbc_sql_to_server_type(TDSSOCKET * tds, int sql_type)
 		return 0;
 #endif
 	case SQL_BIT:
-		return SYBBITN;
+		if (IS_TDS7_PLUS(tds))
+			return SYBBITN;
+		return SYBBIT;
 	case SQL_TINYINT:
 		return SYBINT1;
 	case SQL_SMALLINT:
@@ -709,26 +780,36 @@ odbc_rdbms_version(TDSSOCKET * tds, char *pversion_string)
 
 /** Return length of parameter from parameter information */
 SQLINTEGER
-odbc_get_param_len(TDSSOCKET * tds, const struct _drecord *drec_apd, const struct _drecord *drec_ipd)
+odbc_get_param_len(const struct _drecord *drec_axd, const struct _drecord *drec_ixd, const TDS_DESC* axd, unsigned int n_row)
 {
 	SQLINTEGER len;
 	int size;
+	TDS_INTPTR len_offset;
 
-	if (drec_apd->sql_desc_indicator_ptr && *drec_apd->sql_desc_indicator_ptr == SQL_NULL_DATA)
+	if (axd->header.sql_desc_bind_type != SQL_BIND_BY_COLUMN) {
+		len_offset = axd->header.sql_desc_bind_type * n_row;
+		if (axd->header.sql_desc_bind_offset_ptr)
+			len_offset += *axd->header.sql_desc_bind_offset_ptr;
+	} else {
+		len_offset = sizeof(SQLLEN) * n_row;
+	}
+#define LEN(ptr) *((SQLLEN*)(((char*)(ptr)) + len_offset))
+
+	if (drec_axd->sql_desc_indicator_ptr && LEN(drec_axd->sql_desc_indicator_ptr) == SQL_NULL_DATA)
 		len = SQL_NULL_DATA;
-	else if (drec_apd->sql_desc_octet_length_ptr)
-		len = *drec_apd->sql_desc_octet_length_ptr;
+	else if (drec_axd->sql_desc_octet_length_ptr)
+		len = LEN(drec_axd->sql_desc_octet_length_ptr);
 	else {
 		len = 0;
 		/* TODO add XML if defined */
 		/* FIXME, other types available */
-		if (drec_apd->sql_desc_concise_type == SQL_C_CHAR || drec_apd->sql_desc_concise_type == SQL_C_BINARY) {
+		if (drec_axd->sql_desc_concise_type == SQL_C_CHAR || drec_axd->sql_desc_concise_type == SQL_C_BINARY) {
 			len = SQL_NTS;
 		} else {
-			int type = drec_apd->sql_desc_concise_type;
+			int type = drec_axd->sql_desc_concise_type;
 
 			if (type == SQL_C_DEFAULT)
-				type = odbc_sql_to_c_type_default(drec_ipd->sql_desc_concise_type);
+				type = odbc_sql_to_c_type_default(drec_ixd->sql_desc_concise_type);
 			type = odbc_c_to_server_type(type);
 
 			/* FIXME check what happen to DATE/TIME types */
@@ -738,6 +819,7 @@ odbc_get_param_len(TDSSOCKET * tds, const struct _drecord *drec_apd, const struc
 		}
 	}
 	return len;
+#undef LEN
 }
 
 #ifdef SQL_GUID
@@ -961,4 +1043,37 @@ odbc_set_concise_c_type(SQLSMALLINT concise_type, struct _drecord * drec, int ch
 #undef TYPE_VERBOSE_END
 }
 
-/** \@} */
+SQLLEN
+odbc_get_octet_len(int c_type, const struct _drecord *drec)
+{
+	SQLLEN len;
+
+	/* this shit is mine -- freddy77 */
+	switch (c_type) {
+	case SQL_C_CHAR:
+	case SQL_C_BINARY:
+		len = drec->sql_desc_octet_length;
+		break;
+	case SQL_C_DATE:
+	case SQL_C_TYPE_DATE:
+		len = sizeof(DATE_STRUCT);
+		break;
+	case SQL_C_TIME:
+	case SQL_C_TYPE_TIME:
+		len = sizeof(TIME_STRUCT);
+		break;
+	case SQL_C_TIMESTAMP:
+	case SQL_C_TYPE_TIMESTAMP:
+		len = sizeof(TIMESTAMP_STRUCT);
+		break;
+	case SQL_C_NUMERIC:
+		len = sizeof(SQL_NUMERIC_STRUCT);
+		break;
+	default:
+		len = tds_get_size_by_type(odbc_c_to_server_type(c_type));
+		break;
+	}
+	return len;
+}
+
+/** @} */
