@@ -1,5 +1,6 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004  Brian Bruns
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005  Brian Bruns
+ * Copyright (C) 2005 Frediano Ziglio
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -31,18 +32,27 @@
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
-#include "tds.h"
+#if TIME_WITH_SYS_TIME
+# if HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# endif
+# include <time.h>
+#else
+# if HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+
 #include "tdsodbc.h"
 #include "tdsconvert.h"
-#include "sql2tds.h"
-#include "odbc_util.h"
 
 #ifdef DMALLOC
 #include <dmalloc.h>
 #endif
 
-static const char software_version[] = "$Id: sql2tds.c,v 1.37 2004/10/28 12:42:12 freddy77 Exp $";
-static const void *const no_unused_var_warn[] = { software_version, no_unused_var_warn };
+TDS_RCSID(var, "$Id: sql2tds.c,v 1.65 2008/01/07 21:00:09 freddy77 Exp $");
 
 static TDS_INT
 convert_datetime2server(int bindtype, const void *src, TDS_DATETIME * dt)
@@ -51,25 +61,41 @@ convert_datetime2server(int bindtype, const void *src, TDS_DATETIME * dt)
 	unsigned int dt_time;
 	TDS_INT dt_days;
 	int i;
+	int got_date = 1;
+	time_t curr_time;
 
 	const DATE_STRUCT *src_date = (const DATE_STRUCT *) src;
 	const TIME_STRUCT *src_time = (const TIME_STRUCT *) src;
 	const TIMESTAMP_STRUCT *src_timestamp = (const TIMESTAMP_STRUCT *) src;
 
-	memset(&src_tm, 0, sizeof(src_tm));
-
 	switch (bindtype) {
 	case SQL_C_DATE:
 	case SQL_C_TYPE_DATE:
 		src_tm.tm_year = src_date->year - 1900;
 		src_tm.tm_mon = src_date->month - 1;
 		src_tm.tm_mday = src_date->day;
+		src_tm.tm_hour = 0;
+		src_tm.tm_min = 0;
+		src_tm.tm_sec = 0;
+		src_tm.tm_ms = 0;
 		break;
 	case SQL_C_TIME:
 	case SQL_C_TYPE_TIME:
+		got_date = 0;
+#if HAVE_GETTIMEOFDAY
+		{
+			struct timeval tv;
+		        gettimeofday(&tv, NULL);
+		        curr_time = tv.tv_sec;
+		}
+#else
+		curr_time = time(NULL);
+#endif
+		dt_days = (curr_time / 86400u) + (70u * 365u + 17u);
 		src_tm.tm_hour = src_time->hour;
 		src_tm.tm_min = src_time->minute;
 		src_tm.tm_sec = src_time->second;
+		src_tm.tm_ms = 0;
 		break;
 	case SQL_C_TIMESTAMP:
 	case SQL_C_TYPE_TIMESTAMP:
@@ -86,13 +112,16 @@ convert_datetime2server(int bindtype, const void *src, TDS_DATETIME * dt)
 	}
 
 	/* TODO code copied from convert.c, function */
-	i = (src_tm.tm_mon - 13) / 12;
-	dt_days = 1461 * (src_tm.tm_year + 300 + i) / 4 +
-		(367 * (src_tm.tm_mon - 1 - 12 * i)) / 12 - (3 * ((src_tm.tm_year + 400 + i) / 100)) / 4 + src_tm.tm_mday - 109544;
+	if (got_date) {
+		i = (src_tm.tm_mon - 13) / 12;
+		dt_days = 1461 * (src_tm.tm_year + 300 + i) / 4 +
+			(367 * (src_tm.tm_mon - 1 - 12 * i)) / 12 - (3 * ((src_tm.tm_year + 400 + i) / 100)) / 4 +
+			src_tm.tm_mday - 109544;
+	}
 
 	dt->dtdays = dt_days;
 	dt_time = (src_tm.tm_hour * 60 + src_tm.tm_min) * 60 + src_tm.tm_sec;
-	dt->dttime = dt_time * 300 + (src_tm.tm_ms * 300 / 1000);
+	dt->dttime = dt_time * 300 + (src_tm.tm_ms * 3 + 5) / 10;
 	return sizeof(TDS_DATETIME);
 }
 
@@ -101,14 +130,15 @@ convert_datetime2server(int bindtype, const void *src, TDS_DATETIME * dt)
  * @return SQL_SUCCESS, SQL_ERROR or SQL_NEED_DATA
  */
 SQLRETURN
-sql2tds(TDS_DBC * dbc, const struct _drecord *drec_ipd, const struct _drecord *drec_apd, TDSPARAMINFO * info, int nparam, int compute_row)
+sql2tds(TDS_STMT * stmt, const struct _drecord *drec_ipd, const struct _drecord *drec_apd, TDSCOLUMN *curcol,
+	int compute_row, const TDS_DESC* axd, unsigned int n_row)
 {
+	TDS_DBC * dbc = stmt->dbc;
 	int dest_type, src_type, sql_src_type, res;
 	CONV_RESULT ores;
 	TDSBLOB *blob;
 	char *src;
 	unsigned char *dest;
-	TDSCOLUMN *curcol = info->columns[nparam];
 	int len;
 	TDS_DATETIME dt;
 	TDS_NUMERIC num;
@@ -117,13 +147,13 @@ sql2tds(TDS_DBC * dbc, const struct _drecord *drec_ipd, const struct _drecord *d
 	int need_data = 0, i;
 
 	/* TODO handle bindings of char like "{d '2002-11-12'}" */
-	tdsdump_log(TDS_DBG_INFO2, "%s:%d type=%d\n", __FILE__, __LINE__, drec_ipd->sql_desc_concise_type);
+	tdsdump_log(TDS_DBG_INFO2, "type=%d\n", drec_ipd->sql_desc_concise_type);
 
 	/* what type to convert ? */
 	dest_type = odbc_sql_to_server_type(dbc->tds_socket, drec_ipd->sql_desc_concise_type);
 	if (dest_type == TDS_FAIL)
 		return SQL_ERROR;
-	tdsdump_log(TDS_DBG_INFO2, "%s:%d\n", __FILE__, __LINE__);
+	tdsdump_log(TDS_DBG_INFO2, "trace\n");
 
 	/* TODO what happen for unicode types ?? */
 	tds_set_param_type(dbc->tds_socket, curcol, dest_type);
@@ -145,13 +175,13 @@ sql2tds(TDS_DBC * dbc, const struct _drecord *drec_ipd, const struct _drecord *d
 		 * attention to fill correctly blob/fixed type/variable type
 		 */
 		/* TODO location of this test is correct here ?? */
-		if (dest_type != SYBUNIQUE) {
+		if (dest_type != SYBUNIQUE && dest_type != SYBBITN && !is_fixed_type(dest_type)) {
 			curcol->column_cur_size = 0;
 			curcol->column_size = drec_ipd->sql_desc_length;
 			if (curcol->column_size < 0)
 				curcol->column_size = 0x7FFFFFFFl;
 		}
-	} else {
+	} else if (dest_type != SYBBIT) {
 		/* TODO only a trick... */
 		tds_set_param_type(dbc->tds_socket, curcol, tds_get_null_type(dest_type));
 	}
@@ -171,11 +201,34 @@ sql2tds(TDS_DBC * dbc, const struct _drecord *drec_ipd, const struct _drecord *d
 	if (!compute_row)
 		return TDS_SUCCEED;
 
+	src = drec_apd->sql_desc_data_ptr;
+	if (src && n_row) {
+		SQLLEN len;
+		if (axd->header.sql_desc_bind_type != SQL_BIND_BY_COLUMN) {
+			len = axd->header.sql_desc_bind_type;
+			if (axd->header.sql_desc_bind_offset_ptr)
+				src += *axd->header.sql_desc_bind_offset_ptr;
+		} else {
+			len = odbc_get_octet_len(sql_src_type, drec_apd);
+			if (len < 0)
+				/* TODO sure ? what happen to upper layer ?? */
+				return SQL_ERROR;
+		}
+		src += len * n_row;
+	}
+
 	/* if only output assume input is NULL */
-	if (drec_ipd->sql_desc_parameter_type == SQL_PARAM_OUTPUT)
+	if (drec_ipd->sql_desc_parameter_type == SQL_PARAM_OUTPUT) {
 		sql_len = SQL_NULL_DATA;
-	else
-		sql_len = odbc_get_param_len(dbc->tds_socket, drec_apd, drec_ipd);
+	} else {
+		sql_len = odbc_get_param_len(drec_apd, drec_ipd, axd, n_row);
+
+		/* special case, MS ODBC handle conversion from "\0" to any to NULL, DBD::ODBC require it */
+		if (src_type == SYBVARCHAR && sql_len == 1 && drec_ipd->sql_desc_parameter_type == SQL_PARAM_INPUT_OUTPUT
+		    && src && *src == 0) {
+			sql_len = SQL_NULL_DATA;
+		}
+	}
 
 	/* compute source length */
 	switch (sql_len) {
@@ -183,7 +236,12 @@ sql2tds(TDS_DBC * dbc, const struct _drecord *drec_ipd, const struct _drecord *d
 		len = 0;
 		break;
 	case SQL_NTS:
-		len = strlen(drec_apd->sql_desc_data_ptr);
+		/* check that SQLBindParameter::ParameterValuePtr is not zero for input parameters */
+		if (!src) {
+			odbc_errs_add(&stmt->errs, "HY090", NULL);
+			return SQL_ERROR;
+		}
+		len = strlen(src);
 		break;
 	case SQL_DEFAULT_PARAM:
 	case SQL_DATA_AT_EXEC:
@@ -216,7 +274,7 @@ sql2tds(TDS_DBC * dbc, const struct _drecord *drec_ipd, const struct _drecord *d
 	}
 
 	/* allocate given space */
-	if (!tds_alloc_param_row(info, curcol))
+	if (!tds_alloc_param_data(curcol))
 		return SQL_ERROR;
 
 	if (need_data) {
@@ -227,13 +285,16 @@ sql2tds(TDS_DBC * dbc, const struct _drecord *drec_ipd, const struct _drecord *d
 	/* set null */
 	assert(drec_ipd->sql_desc_parameter_type != SQL_PARAM_OUTPUT || sql_len == SQL_NULL_DATA);
 	if (sql_len == SQL_NULL_DATA) {
-		curcol->column_cur_size = 0;
-		tds_set_null(info->current_row, nparam);
+		curcol->column_cur_size = -1;
 		return TDS_SUCCEED;
 	}
 
+	if (!src) {
+		odbc_errs_add(&stmt->errs, "HY090", NULL);
+		return SQL_ERROR;
+	}
+
 	/* convert special parameters (not libTDS compatible) */
-	src = drec_apd->sql_desc_data_ptr;
 	switch (src_type) {
 	case SYBDATETIME:
 		convert_datetime2server(drec_apd->sql_desc_concise_type, src, &dt);
@@ -245,64 +306,56 @@ sql2tds(TDS_DBC * dbc, const struct _drecord *drec_ipd, const struct _drecord *d
 		num.precision = sql_num->precision;
 		num.scale = sql_num->scale;
 		num.array[0] = sql_num->sign ^ 1;
-		/* TODO test precision so client do not crash our library ?? */
+		/* test precision so client do not crash our library */
+		if (num.precision <= 0 || num.precision > 38 || num.scale > num.precision)
+			/* TODO add proper error */
+			return SQL_ERROR;
 		i = tds_numeric_bytes_per_prec[num.precision];
-		memcpy(num.array + 1, sql_num->val, i);
-		tds_swap_bytes(num.array + 1, i);
-		++i;
+		memcpy(num.array + 1, sql_num->val, i - 1);
+		tds_swap_bytes(num.array + 1, i - 1);
 		if (i < sizeof(num.array))
 			memset(num.array + i, 0, sizeof(num.array) - i);
 		src = (char *) &num;
-
-		/* set output */
-		/* TODO use descriptors informations ?? */
-		ores.n.precision = 18;
-		ores.n.scale = 2;
 		break;
 		/* TODO intervals */
 	}
 
-	res = tds_convert(dbc->env->tds_ctx, src_type, src, len, dest_type, &ores);
-	if (res < 0)
-		return SQL_ERROR;
-	tdsdump_log(TDS_DBG_INFO2, "%s:%d\n", __FILE__, __LINE__);
-
-	/* truncate ?? */
-	/* TODO what happen for blobs ?? */
-	if (res > curcol->column_size)
-		res = curcol->column_size;
-	curcol->column_cur_size = res;
-
-	/* free allocated memory */
-	dest = &info->current_row[curcol->column_offset];
+	dest = curcol->column_data;
 	switch ((TDS_SERVER_TYPE) dest_type) {
 	case SYBCHAR:
 	case SYBVARCHAR:
 	case XSYBCHAR:
 	case XSYBVARCHAR:
-		memcpy(&info->current_row[curcol->column_offset], ores.c, res);
-		free(ores.c);
-		break;
-	case SYBTEXT:
-		blob = (TDSBLOB *) dest;
-		if (blob->textvalue)
-			free(blob->textvalue);
-		blob->textvalue = ores.c;
+		ores.cc.c = (TDS_CHAR*) dest;
+		ores.cc.len = curcol->column_size;
+		res = tds_convert(dbc->env->tds_ctx, src_type, src, len, TDS_CONVERT_CHAR, &ores);
+		if (res > curcol->column_size)
+			res = curcol->column_size;
 		break;
 	case SYBBINARY:
 	case SYBVARBINARY:
 	case XSYBBINARY:
 	case XSYBVARBINARY:
-		memcpy(&info->current_row[curcol->column_offset], ores.ib, res);
-		free(ores.ib);
+		ores.cb.ib = (TDS_CHAR*) dest;
+		ores.cb.len = curcol->column_size;
+		res = tds_convert(dbc->env->tds_ctx, src_type, src, len, TDS_CONVERT_BINARY, &ores);
+		if (res > curcol->column_size)
+			res = curcol->column_size;
 		break;
+	case SYBTEXT:
 	case SYBLONGBINARY:
 	case SYBIMAGE:
+		res = tds_convert(dbc->env->tds_ctx, src_type, src, len, dest_type, &ores);
+		if (res < 0)
+			return SQL_ERROR;
 		blob = (TDSBLOB *) dest;
-		if (blob->textvalue)
-			free(blob->textvalue);
+		free(blob->textvalue);
 		blob->textvalue = ores.ib;
 		break;
+	case SYBNUMERIC:
+	case SYBDECIMAL:
+		((TDS_NUMERIC *) dest)->precision = drec_ipd->sql_desc_precision;
+		((TDS_NUMERIC *) dest)->scale = drec_ipd->sql_desc_scale;
 	case SYBINTN:
 	case SYBINT1:
 	case SYBINT2:
@@ -316,8 +369,6 @@ sql2tds(TDS_DBC * dbc, const struct _drecord *drec_ipd, const struct _drecord *d
 	case SYBDATETIME4:
 	case SYBREAL:
 	case SYBBITN:
-	case SYBNUMERIC:
-	case SYBDECIMAL:
 	case SYBFLTN:
 	case SYBMONEYN:
 	case SYBDATETIMN:
@@ -326,8 +377,9 @@ sql2tds(TDS_DBC * dbc, const struct _drecord *drec_ipd, const struct _drecord *d
 	case SYBUINT4:
 	case SYBUINT8:
 	case SYBUNIQUE:
-		memcpy(&info->current_row[curcol->column_offset], &ores, res);
+		res = tds_convert(dbc->env->tds_ctx, src_type, src, len, dest_type, (CONV_RESULT*) dest);
 		break;
+	default:
 	case XSYBNVARCHAR:
 	case XSYBNCHAR:
 	case SYBNVARCHAR:
@@ -336,8 +388,14 @@ sql2tds(TDS_DBC * dbc, const struct _drecord *drec_ipd, const struct _drecord *d
 	case SYBVARIANT:
 		/* TODO ODBC 3.5 */
 		assert(0);
+		res = -1;
 		break;
 	}
 
+	if (res < 0)
+		return SQL_ERROR;
+
+	curcol->column_cur_size = res;
+
 	return SQL_SUCCESS;
 }
