@@ -6,9 +6,13 @@
 
 #include <ctype.h>
 
+#ifndef WIN32
 #include "tds_sysdep_private.h"
+#else
+#define TDS_SDIR_SEPARATOR "\\"
+#endif
 
-static char software_version[] = "$Id: common.c,v 1.33 2004/05/04 07:45:26 freddy77 Exp $";
+static char software_version[] = "$Id: common.c,v 1.42 2007/12/27 10:22:18 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 HENV Environment;
@@ -22,6 +26,7 @@ char PASSWORD[512];
 char DATABASE[512];
 char DRIVER[1024];
 
+#ifndef WIN32
 static int
 check_lib(char *path, const char *file)
 {
@@ -37,6 +42,7 @@ check_lib(char *path, const char *file)
 	path[len] = 0;
 	return 0;
 }
+#endif
 
 /* some platforms do not have setenv, define a replacement */
 #if !HAVE_SETENV
@@ -55,13 +61,20 @@ odbc_setenv(const char *name, const char *value, int overwrite)
 int
 read_login_info(void)
 {
+	static const char *PWD = "../../../PWD";
 	FILE *in;
 	char line[512];
 	char *s1, *s2;
+#ifndef WIN32
 	char path[1024];
 	int len;
+#endif
 
+#if 0
 	in = fopen(".." TDS_SDIR_SEPARATOR ".." TDS_SDIR_SEPARATOR ".." TDS_SDIR_SEPARATOR "PWD", "r");
+#else
+	in = fopen(PWD, "r");
+#endif
 	if (!in)
 		in = fopen("PWD", "r");
 
@@ -134,13 +147,14 @@ ReportError(const char *errmsg, int line, const char *file)
 	}
 	if (errmsg[0]) {
 		if (line)
-			printf("%s:%d %s\n", file, line, errmsg);
+			fprintf(stderr, "%s:%d %s\n", file, line, errmsg);
 		else
-			printf("%s\n", errmsg);
+			fprintf(stderr, "%s\n", errmsg);
 	}
 	ret = SQLGetDiagRec(handletype, handle, 1, sqlstate, NULL, msg, sizeof(msg), NULL);
 	if (ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO)
 		fprintf(stderr, "SQL error %s -- %s\n", sqlstate, msg);
+	Disconnect();
 	exit(1);
 }
 
@@ -200,6 +214,11 @@ Connect(void)
 		CheckReturn();
 		exit(1);
 	}
+
+#ifndef TDS_NO_DM
+	/* unixODBC seems to require it */
+	SQLMoreResults(Statement);
+#endif
 	return 0;
 }
 
@@ -280,3 +299,83 @@ driver_is_freetds(void)
 	}
 	return freetds_driver;
 }
+
+void
+CheckCols(int n, int line, const char * file)
+{
+	SQLSMALLINT cols;
+	SQLRETURN res;
+
+	res = SQLNumResultCols(Statement, &cols);
+	if (res != SQL_SUCCESS) {
+		if (res == SQL_ERROR && n < 0)
+			return;
+		fprintf(stderr, "%s:%d: Unable to get column numbers\n", file, line);
+		CheckReturn();
+		Disconnect();
+		exit(1);
+	}
+
+	if (cols != n) {
+		fprintf(stderr, "%s:%d: Expected %d columns returned %d\n", file, line, n, (int) cols);
+		Disconnect();
+		exit(1);
+	}
+}
+
+void
+CheckRows(int n, int line, const char * file)
+{
+	SQLLEN rows;
+	SQLRETURN res;
+
+	res = SQLRowCount(Statement, &rows);
+	if (res != SQL_SUCCESS) {
+		if (res == SQL_ERROR && n < -1)
+			return;
+		fprintf(stderr, "%s:%d: Unable to get row\n", file, line);
+		CheckReturn();
+		Disconnect();
+		exit(1);
+	}
+
+	if (rows != n) {
+		fprintf(stderr, "%s:%d: Expected %d rows returned %d\n", file, line, n, (int) rows);
+		Disconnect();
+		exit(1);
+	}
+}
+
+void
+ResetStatement(void)
+{
+	SQLFreeStmt(Statement, SQL_DROP);
+	Statement = SQL_NULL_HSTMT;
+	if (SQLAllocStmt(Connection, &Statement) != SQL_SUCCESS)
+		ODBC_REPORT_ERROR("Unable to allocate statement");
+}
+
+void
+CheckCursor(void)
+{
+	SQLRETURN retcode;
+
+	retcode = SQLSetStmtAttr(Statement, SQL_ATTR_CONCURRENCY, (SQLPOINTER) SQL_CONCUR_ROWVER, 0);
+	if (retcode != SQL_SUCCESS) {
+		char output[256];
+		unsigned char sqlstate[6];
+
+		retcode = SQLGetDiagRec(SQL_HANDLE_STMT, Statement, 1, sqlstate, NULL, (SQLCHAR *) output, sizeof(output), NULL);
+		if (retcode != SQL_SUCCESS)
+			ODBC_REPORT_ERROR("SQLGetDiagRec");
+		sqlstate[5] = 0;
+		if (strcmp((const char*) sqlstate, "01S02") == 0) {
+			printf("Your connection seems to not support cursors, probably you are using wrong protocol version or Sybase\n");
+			Disconnect();
+			exit(0);
+		}
+		ODBC_REPORT_ERROR("SQLSetStmtAttr");
+	}
+	ResetStatement();
+}
+
