@@ -7,6 +7,7 @@
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
 
+#include <stdarg.h>
 #include <stdio.h>
 
 #if HAVE_STDLIB_H
@@ -17,11 +18,10 @@
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
-#include <sqltypes.h>
 #include <sql.h>
 #include <sqlext.h>
 
-static char rcsid_common_h[] = "$Id: common.h,v 1.15 2004/10/28 13:16:18 freddy77 Exp $";
+static char rcsid_common_h[] = "$Id: common.h,v 1.21 2007/12/21 10:39:10 freddy77 Exp $";
 static void *no_unused_common_h_warn[] = { rcsid_common_h, no_unused_common_h_warn };
 
 #ifndef HAVE_SQLLEN
@@ -48,6 +48,13 @@ int read_login_info(void);
 void CheckReturn(void);
 void ReportError(const char *msg, int line, const char *file);
 
+void CheckCols(int n, int line, const char * file);
+void CheckRows(int n, int line, const char * file);
+#define CHECK_ROWS(n) CheckRows(n, __LINE__, __FILE__)
+#define CHECK_COLS(n) CheckCols(n, __LINE__, __FILE__)
+void ResetStatement(void);
+void CheckCursor(void);
+
 #define ODBC_REPORT_ERROR(msg) ReportError(msg, __LINE__, __FILE__)
 int Connect(void);
 int Disconnect(void);
@@ -56,6 +63,9 @@ SQLRETURN CommandWithResult(HSTMT stmt, const char *command);
 int db_is_microsoft(void);
 int driver_is_freetds(void);
 
+#define int2ptr(i) ((void*)(((char*)0)+(i)))
+#define ptr2int(p) ((int)(((char*)(p))-((char*)0)))
+
 #if !HAVE_SETENV
 void odbc_setenv(const char *name, const char *value, int overwrite);
 
