@@ -1,7 +1,7 @@
 #include "common.h"
 #include <assert.h>
 
-static char software_version[] = "$Id: type.c,v 1.6 2004/10/28 13:16:18 freddy77 Exp $";
+static char software_version[] = "$Id: type.c,v 1.8 2007/11/26 18:12:31 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 struct type
@@ -17,8 +17,10 @@ struct type
 #define TYPE_C(s) {s, #s, FLAG_C }
 #define TYPE_SQL(s) {s, #s, FLAG_SQL }
 #define TYPE_BOTH(s,s2) {s, #s, FLAG_SQL|FLAG_C }
-/* Same define with test for constants
- *  #define TYPE_BOTH(s,s2) {s, #s, (FLAG_SQL|FLAG_C)+((1/!(s-s2))-1) } */
+/* 
+ * Same define with test for constants
+ *  #define TYPE_BOTH(s,s2) {s, #s, (FLAG_SQL|FLAG_C)+((1/!(s-s2))-1) }
+ */
 static const struct type types[] = {
 	TYPE_BOTH(SQL_C_CHAR, SQL_CHAR),
 	TYPE_BOTH(SQL_C_LONG, SQL_INTEGER),
@@ -104,9 +106,11 @@ main(int argc, char **argv)
 
 	Connect();
 
-	/* test setting two time a descriptor
+	/*
+	 * test setting two time a descriptor
 	 * success all user allocated are ARD or APD so type cheching can be done
-	 * TODO freeing descriptor dissociate it from statements */
+	 * TODO freeing descriptor dissociate it from statements
+	 */
 
 	/* test C types */
 	for (p = types; p->name; ++p) {
@@ -137,7 +141,7 @@ main(int argc, char **argv)
 			concise_type = p->type;
 			SQLGetStmtAttr(Statement, SQL_ATTR_APP_PARAM_DESC, &desc, sizeof(desc), &ind);
 			if (SQL_SUCCEEDED
-			    (SQLSetDescField(desc, 1, SQL_DESC_CONCISE_TYPE, (SQLPOINTER) (int) concise_type, sizeof(SQLSMALLINT))))
+			    (SQLSetDescField(desc, 1, SQL_DESC_CONCISE_TYPE, int2ptr(concise_type), sizeof(SQLSMALLINT))))
 			{
 				SQLGetDescField(desc, 1, SQL_DESC_TYPE, &type, sizeof(SQLSMALLINT), &ind);
 				SQLGetDescField(desc, 1, SQL_DESC_CONCISE_TYPE, &concise_type, sizeof(SQLSMALLINT), &ind);
@@ -160,7 +164,7 @@ main(int argc, char **argv)
 		SQLSMALLINT concise_type = p->type;
 
 		if (SQL_SUCCEEDED
-		    (SQLSetDescField(desc, 1, SQL_DESC_CONCISE_TYPE, (SQLPOINTER) (int) concise_type, sizeof(SQLSMALLINT)))) {
+		    (SQLSetDescField(desc, 1, SQL_DESC_CONCISE_TYPE, int2ptr(concise_type), sizeof(SQLSMALLINT)))) {
 			SQLSMALLINT concise_type, type, code;
 
 			concise_type = type = code = 0;
