@@ -18,17 +18,18 @@
  */
 
 /*
-** Note on terminology: a pool member is a connection to the database,
-** a pool user is a client connection that is temporarily assigned to a
-** pool member.
-*/
+ * Note on terminology: a pool member is a connection to the database,
+ * a pool user is a client connection that is temporarily assigned to a
+ * pool member.
+ */
 
 #if HAVE_CONFIG_H
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
 
-#include <signal.h>
+#include <stdarg.h>
 #include <stdio.h>
+#include <signal.h>
 
 #if HAVE_STDLIB_H
 #include <stdlib.h>
@@ -56,21 +57,18 @@
 
 #include "pool.h"
 
-static char software_version[] = "$Id: main.c,v 1.16 2003/04/30 18:51:38 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
-/* this will go away...starting with just 1 global pool */
-TDS_POOL *g_pool = NULL;
+TDS_RCSID(var, "$Id: main.c,v 1.24 2007/09/17 08:46:03 freddy77 Exp $");
 
 /* to be set by sig term */
-int term = 0;
+static int term = 0;
 
 /* number of users in wait state */
 int waiters = 0;
-int hack = 0;
 
 static void term_handler(int sig);
 static void pool_schedule_waiters(TDS_POOL * pool);
+static TDS_POOL *pool_init(char *name);
+static void pool_main_loop(TDS_POOL * pool);
 
 static void
 term_handler(int sig)
@@ -80,27 +78,25 @@ term_handler(int sig)
 }
 
 /*
-** pool_init creates a named pool and opens connections to the database
-*/
-TDS_POOL *
+ * pool_init creates a named pool and opens connections to the database
+ */
+static TDS_POOL *
 pool_init(char *name)
 {
 	TDS_POOL *pool;
 
 	/* initialize the pool */
 
-	g_pool = (TDS_POOL *) malloc(sizeof(TDS_POOL));
-	pool = g_pool;
+	pool = (TDS_POOL *) malloc(sizeof(TDS_POOL));
 	memset(pool, '\0', sizeof(TDS_POOL));
-	/* FIX ME -- read this from the conf file */
+	/* FIXME -- read this from the conf file */
 	if (!pool_read_conf_file(name, pool)) {
 		fprintf(stderr, "Configuration for pool ``%s'' not found.\n", name);
 		exit(EXIT_FAILURE);
 	}
 	pool->num_members = pool->max_open_conn;
 
-	pool->name = (char *) malloc(strlen(name) + 1);
-	strcpy(pool->name, name);
+	pool->name = strdup(name);
 
 	pool_mbr_init(pool);
 	pool_user_init(pool);
@@ -140,11 +136,11 @@ pool_schedule_waiters(TDS_POOL * pool)
 }
 
 /* 
-** pool_main_loop
-** Accept new connections from clients, and handle all input from clients and
-** pool members.
-*/
-void
+ * pool_main_loop
+ * Accept new connections from clients, and handle all input from clients and
+ * pool members.
+ */
+static void
 pool_main_loop(TDS_POOL * pool)
 {
 	TDS_POOL_USER *puser;
@@ -153,16 +149,19 @@ pool_main_loop(TDS_POOL * pool)
 	int s, maxfd, i;
 	int retval;
 	fd_set rfds;
+	int socktrue = 1;
 
-/* fix me -- read the interfaces file and bind accordingly */
+	/* FIXME -- read the interfaces file and bind accordingly */
 	sin.sin_addr.s_addr = INADDR_ANY;
 	sin.sin_port = htons(pool->port);
 	sin.sin_family = AF_INET;
 
-	if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+	if (TDS_IS_SOCKET_INVALID(s = socket(AF_INET, SOCK_STREAM, 0))) {
 		perror("socket");
 		exit(1);
 	}
+	/* don't keep addr in use from s.craig@andronics.com */
+	setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (const void *) &socktrue, sizeof(socktrue));
 
 	fprintf(stderr, "Listening on port %d\n", pool->port);
 	if (bind(s, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
@@ -176,14 +175,14 @@ pool_main_loop(TDS_POOL * pool)
 	maxfd = s;
 
 	while (!term) {
-		fprintf(stderr, "waiting for a connect\n");
+		/* fprintf(stderr, "waiting for a connect\n"); */
 		retval = select(maxfd + 1, &rfds, NULL, NULL, NULL);
 		if (term)
-			continue;
+			break;
 
 		/* process the sockets */
 		if (FD_ISSET(s, &rfds)) {
-			puser = pool_user_create(pool, s, &sin);
+			pool_user_create(pool, s, &sin);
 		}
 		pool_process_users(pool, &rfds);
 		pool_process_members(pool, &rfds);
@@ -218,7 +217,7 @@ pool_main_loop(TDS_POOL * pool)
 			}
 		}
 	}			/* while !term */
-	close(s);
+	CLOSESOCKET(s);
 	for (i = 0; i < pool->max_users; i++) {
 		puser = (TDS_POOL_USER *) & pool->users[i];
 		if (!IS_TDSDEAD(puser->tds)) {
@@ -244,11 +243,10 @@ main(int argc, char **argv)
 	signal(SIGINT, term_handler);
 	if (argc < 2) {
 		fprintf(stderr, "Usage: tdspool <pool name>\n");
-		exit(1);
+		return 1;
 	}
 	pool = pool_init(argv[1]);
 	pool_main_loop(pool);
 	fprintf(stdout, "tdspool Shutdown\n");
-	exit(EXIT_SUCCESS);
-	return 0;
+	return EXIT_SUCCESS;
 }
