@@ -1,5 +1,5 @@
 /* TDSPool - Connection pooling for TDS based databases
- * Copyright (C) 2001, 2002, 2003, 2004  Brian Bruns
+ * Copyright (C) 2001, 2002, 2003, 2004, 2005  Brian Bruns
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
 
+#include <stdarg.h>
 #include <stdio.h>
 
 #if HAVE_STDLIB_H
@@ -39,6 +40,10 @@
 #include <sys/param.h>
 #endif /* HAVE_SYS_PARAM_H */
 
+#if HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif /* HAVE_SYS_SOCKET_H */
+
 #if HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif /* HAVE_NETINET_IN_H */
@@ -48,22 +53,22 @@
 #endif /* HAVE_ARPA_INET_H */
 
 #include "pool.h"
+#include "replacements.h"
 
 #ifndef MAXHOSTNAMELEN
 #define MAXHOSTNAMELEN 256
 #endif /* MAXHOSTNAMELEN */
 
-static char software_version[] = "$Id: member.c,v 1.28 2004/02/03 19:28:11 jklowden Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+TDS_RCSID(var, "$Id: member.c,v 1.41 2006/12/26 14:56:20 freddy77 Exp $");
 
 static int pool_packet_read(TDS_POOL_MEMBER * pmbr);
-static void pool_free_member(TDS_POOL_MEMBER * pmbr);
+static TDSSOCKET *pool_mbr_login(TDS_POOL * pool);
 
 /*
-** pool_mbr_login open a single pool login, to be call at init time or
-** to reconnect.
-*/
-TDSSOCKET *
+ * pool_mbr_login open a single pool login, to be call at init time or
+ * to reconnect.
+ */
+static TDSSOCKET *
 pool_mbr_login(TDS_POOL * pool)
 {
 	TDSCONTEXT *context;
@@ -81,27 +86,28 @@ pool_mbr_login(TDS_POOL * pool)
 #if HAVE_GETHOSTNAME
 	if (gethostname(hostname, MAXHOSTNAMELEN) < 0)
 #endif
-		strncpy(hostname, "tdspool", MAXHOSTNAMELEN);
-	hostname[MAXHOSTNAMELEN - 1] = '\0';
+		tds_strlcpy(hostname, "tdspool", MAXHOSTNAMELEN);
 	tds_set_host(login, hostname);
 	tds_set_library(login, "TDS-Library");
 	tds_set_server(login, pool->server);
 	tds_set_client_charset(login, "iso_1");
 	tds_set_language(login, "us_english");
 	tds_set_packet(login, 512);
-	context = tds_alloc_context();
+	context = tds_alloc_context(NULL);
 	tds = tds_alloc_socket(context, 512);
-	tds_set_parent(tds, NULL);
 	connection = tds_read_config_info(NULL, login, context->locale);
 	if (!connection || tds_connect(tds, connection) == TDS_FAIL) {
+		tds_free_socket(tds);
 		tds_free_connection(connection);
 		/* what to do? */
 		fprintf(stderr, "Could not open connection to server %s\n", pool->server);
 		return NULL;
 	}
 	tds_free_connection(connection);
-	/* FIX ME -- tds_connect no longer preallocates the in_buf need to 
-	 * ** do something like what tds_read_packet does */
+	/*
+	 * FIXME -- tds_connect no longer preallocates the in_buf need to 
+	 * do something like what tds_read_packet does
+	 */
 	tds->in_buf = (unsigned char *) malloc(BLOCKSIZ);
 	memset(tds->in_buf, 0, BLOCKSIZ);
 
@@ -123,18 +129,46 @@ pool_mbr_login(TDS_POOL * pool)
 	return tds;
 }
 
-static void
+void
+pool_assign_member(TDS_POOL_MEMBER * pmbr, TDS_POOL_USER *puser)
+{
+	pmbr->current_user = puser;
+	puser->assigned_member = pmbr;
+}
+
+void
+pool_deassign_member(TDS_POOL_MEMBER * pmbr)
+{
+	if (pmbr->current_user)
+		pmbr->current_user->assigned_member = NULL;
+	pmbr->current_user = NULL;
+}
+
+/*
+ * if a dead connection on the client side left this member in a questionable
+ * state, let's clean it up.
+ */
+void
+pool_reset_member(TDS_POOL_MEMBER * pmbr)
+{
+	pool_free_member(pmbr);
+}
+
+void
 pool_free_member(TDS_POOL_MEMBER * pmbr)
 {
 	if (!IS_TDSDEAD(pmbr->tds)) {
 		tds_close_socket(pmbr->tds);
 	}
 	pmbr->tds = NULL;
-	/* if he is allocated disconnect the client 
-	 * ** otherwise we end up with broken client.
+	/*
+	 * if he is allocated disconnect the client 
+	 * otherwise we end up with broken client.
 	 */
-	if (pmbr->current_user)
+	if (pmbr->current_user) {
 		pool_free_user(pmbr->current_user);
+		pmbr->current_user = NULL;
+	}
 	pmbr->state = TDS_IDLE;
 }
 
@@ -167,17 +201,17 @@ pool_mbr_init(TDS_POOL * pool)
 }
 
 /* 
-** pool_process_members
-** check the fd_set for members returning data to the client, lookup the 
-** client holding this member and forward the results.
-*/
+ * pool_process_members
+ * check the fd_set for members returning data to the client, lookup the 
+ * client holding this member and forward the results.
+ */
 int
 pool_process_members(TDS_POOL * pool, fd_set * fds)
 {
 	TDS_POOL_MEMBER *pmbr;
 	TDS_POOL_USER *puser;
 	TDSSOCKET *tds;
-	int i, age;
+	int i, age, ret;
 	int cnt = 0;
 	unsigned char *buf;
 	time_t time_now;
@@ -202,19 +236,43 @@ pool_process_members(TDS_POOL * pool, fd_set * fds)
 				/* mark as dead */
 				pool_free_member(pmbr);
 			} else if (tds->in_len == -1) {
+				fprintf(stderr, "Uh oh! member %d disconnected\n", i);
 				perror("read");
+				pool_free_member(pmbr);
 			} else {
-				fprintf(stderr, "read %d bytes from member %d\n", tds->in_len, i);
+				/* fprintf(stderr, "read %d bytes from member %d\n", tds->in_len, i); */
 				if (pmbr->current_user) {
 					puser = pmbr->current_user;
 					buf = tds->in_buf;
-					if (pool_find_end_token(pmbr, buf + 8, tds->in_len - 8)) {
+					/* 
+					 * check the netlib final packet flag
+					 * instead of looking for done tokens.
+					 * It's more efficient and generic to 
+					 * all protocol versions. -- bsb 
+					 * 2004-12-12 
+					 */
+					if (buf[1]) {
+					/* if (pool_find_end_token(pmbr, buf + 8, tds->in_len - 8)) { */
 						/* we are done...deallocate member */
-						pmbr->current_user = NULL;
+						fprintf(stdout, "deassigning user from member %d\n",i);
+						pool_deassign_member(pmbr);
+
 						pmbr->state = TDS_IDLE;
 						puser->user_state = TDS_SRV_IDLE;
 					}
-					write(puser->tds->s, buf, tds->in_len);
+#ifdef MSG_NOSIGNAL	
+					/* cf. net.c for better technique.  */
+					ret = send(puser->tds->s, buf, tds->in_len, MSG_NOSIGNAL);
+#else
+					/* write(puser->tds->s, buf, tds->in_len); */
+					ret = send(puser->tds->s, buf, tds->in_len, 0);
+#endif
+					if (ret==-1) { /* couldn't write, ditch the user */
+						fprintf(stdout, "member %d received error while writing\n",i);
+						pool_free_user(pmbr->current_user);
+						pool_deassign_member(pmbr);
+						pool_reset_member(pmbr);
+					}
 				}
 			}
 		}
@@ -228,9 +286,9 @@ pool_process_members(TDS_POOL * pool, fd_set * fds)
 }
 
 /*
-** pool_find_idle_member
-** returns the first pool member in TDS_IDLE state
-*/
+ * pool_find_idle_member
+ * returns the first pool member in TDS_IDLE state
+ */
 TDS_POOL_MEMBER *
 pool_find_idle_member(TDS_POOL * pool)
 {
@@ -243,9 +301,11 @@ pool_find_idle_member(TDS_POOL * pool)
 		if (pmbr->tds) {
 			active_members++;
 			if (pmbr->state == TDS_IDLE) {
-				/* make sure member wasn't idle more that the timeout 
-				 * ** otherwise it'll send the query and close leaving a
-				 * ** hung client */
+				/*
+				 * make sure member wasn't idle more that the timeout 
+				 * otherwise it'll send the query and close leaving a
+				 * hung client
+				 */
 				pmbr->last_used_tm = time(NULL);
 				return pmbr;
 			}
