@@ -8,18 +8,17 @@
 #include <config.h>
 #endif
 
-#include <stdio.h>
 #include <stdarg.h>
+#include <stdio.h>
 
 #if HAVE_STRING_H
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
+#include "tds_sysdep_private.h"
 #include "replacements.h"
 
-static char software_version[] = "$Id: asprintf.c,v 1.5 2003/02/18 09:54:58 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
+TDS_RCSID(var, "$Id: asprintf.c,v 1.7 2006/08/23 14:26:20 freddy77 Exp $");
 
 int
 asprintf(char **ret, const char *fmt, ...)
