@@ -8,14 +8,13 @@
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
 
+#include <stdarg.h>
 #include <stdio.h>
 
 #if HAVE_STDLIB_H
 #include <stdlib.h>
 #endif /* HAVE_STDLIB_H */
 
-#include <stdarg.h>
-
 #if HAVE_STRING_H
 #include <string.h>
 #endif /* HAVE_STRING_H */
@@ -31,8 +30,14 @@
 #include "tds_sysdep_private.h"
 #include "replacements.h"
 
-static char software_version[] = "$Id: vasprintf.c,v 1.14 2004/03/15 09:15:44 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+TDS_RCSID(var, "$Id: vasprintf.c,v 1.18 2007/12/27 13:45:23 freddy77 Exp $");
+
+#if defined(HAVE__VSNPRINTF) && !defined(HAVE_VSNPRINTF)
+#undef HAVE_VSNPRINTF
+#undef vsnprintf
+#define HAVE_VSNPRINTF 1
+#define vsnprintf _vsnprintf
+#endif
 
 #ifndef _PATH_DEVNULL
 #define _PATH_DEVNULL "/dev/null"
@@ -56,7 +61,7 @@ vasprintf(char **ret, const char *fmt, va_list ap)
 			return -1;
 		}
 		len = vsnprintf(buf, buflen, fmt, ap);
-		if (len >= 0 && len < (buflen - 1)) {
+		if (len >= 0 && (size_t) len < (buflen - 1)) {
 			break;
 		}
 		free(buf);
@@ -65,7 +70,7 @@ vasprintf(char **ret, const char *fmt, va_list ap)
 		 * len >= 0 are required for vsnprintf implementation that 
 		 * return -1 of buffer insufficient
 		 */
-		if (len >= 0 && len >= buflen) {
+		if (len >= 0 && (size_t) len >= buflen) {
 			buflen = len + 1;
 		}
 	}
