@@ -20,6 +20,8 @@
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
 
+#include <assert.h>
+
 #if HAVE_STDLIB_H
 #include <stdlib.h>
 #endif /* HAVE_STDLIB_H */
@@ -27,32 +29,36 @@
 #include "tds.h"
 #include "tdssrv.h"
 
-static char software_version[] = "$Id: query.c,v 1.13 2004/05/27 14:50:06 freddy77 Exp $";
+static char software_version[] = "$Id: query.c,v 1.17 2007/09/24 10:02:14 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
+static char *query;
+static size_t query_buflen = 0;
+
 /**
  * Read a TDS5 tokenized query.
  */
 char *
 tds_get_query(TDSSOCKET * tds)
 {
-	static unsigned char *query;
-	static size_t query_buflen = 0;
 	int len;
 
 	if (query_buflen == 0) {
 		query_buflen = 1024;
-		query = (unsigned char *) malloc(query_buflen);
+		query = (char *) malloc(query_buflen);
 	}
 	tds_get_byte(tds);	/* 33 */
-	len = tds_get_smallint(tds);	/* query size +1 */
-	tds_get_n(tds, NULL, 3);
+	len = tds_get_int(tds);	/* query size +1 */
+	tds_get_byte(tds);	/* has args, ignored TODO */
+	assert(len >= 1);	/* TODO handle correctly */
 	if (len > query_buflen) {
 		query_buflen = len;
-		query = (unsigned char *) realloc(query, query_buflen);
+		query = (char *) realloc(query, query_buflen);
 	}
-	tds_get_n(tds, query, len - 1);
-	return (char *) query;
+	--len;
+	tds_get_n(tds, query, len);
+	query[len] = 0;
+	return query;
 }
 
 /**
@@ -67,8 +73,6 @@ tds_get_query(TDSSOCKET * tds)
  */
 char *tds_get_generic_query(TDSSOCKET * tds)
 {
-	static char *query;
-	static int query_buflen;
  	int token, byte;
 	int len, more, i, j;
 
@@ -85,21 +89,24 @@ char *tds_get_generic_query(TDSSOCKET * tds)
 
 		/* Queries can arrive in a couple different formats. */
 		switch (tds->in_flag) {
-		case 0x03: /* RPC packet */
-		case 0x0f: /* TDS5 query packet */
+		case TDS_RPC:
+			/* TODO */
+		case TDS_NORMAL: /* TDS5 query packet */
 			/* get the token */
 			token = tds_get_byte(tds);
 			switch (token) {
 			case TDS_LANGUAGE_TOKEN:
 				/* SQL query */
-				len = tds_get_smallint(tds); /* query size +1 */
-				tds_get_n(tds, NULL, 3);
+				len = tds_get_int(tds); /* query size +1 */
+				assert(len >= 1);	/* TODO handle */
+				tds_get_byte(tds);	/* has args, ignored TODO */
 				if (len > query_buflen) {
 					query_buflen = len;
 					query = (char *) realloc(query, query_buflen);
 				}
-				tds_get_n(tds, query, len - 1);
-				query[len - 1] = '\0';
+				--len;
+				tds_get_n(tds, query, len);
+				query[len] = 0;
 				return query;
 
 			case TDS_DBRPC_TOKEN:
@@ -112,7 +119,7 @@ char *tds_get_generic_query(TDSSOCKET * tds)
 				len = tds_get_byte(tds) + 1;/* sproc name size +1 */
 				if (len > query_buflen) {
 					query_buflen = len;
-					query = (unsigned char *) realloc(query, query_buflen);
+					query = (char *) realloc(query, query_buflen);
 				}
 
 				/*
@@ -145,15 +152,19 @@ char *tds_get_generic_query(TDSSOCKET * tds)
 			}
 			break;
 
-		case 0x01:
+		case TDS_QUERY:
 			/* TDS4 and TDS7+ fill the whole packet with a query */
 			len = 0;
-			do {
+			for (;;) {
+				const char *src;
+
 				/* If buffer needs to grow, then grow */
 				more = tds->in_len - tds->in_pos;
+				src = (char *) (tds->in_buf + tds->in_pos);
 				if ((size_t)(len + more + 1) > query_buflen)
 				{
-					query_buflen += 1024;
+					query_buflen = len + more + 1024u;
+					query_buflen -= query_buflen % 1024u;
 					query = (char *)realloc(query, query_buflen);
 				}
 
@@ -164,23 +175,23 @@ char *tds_get_generic_query(TDSSOCKET * tds)
 				 */
 				while (--more >= 0)
 				{
-					query[len] = tds_get_byte(tds);
+					query[len] = *src++;
 					if (query[len] != '\0')
 						len++;
 				}
 
 				/* if more then read it */
-				more = !tds->last_packet;
-				if (more && tds_read_packet(tds) < 0) {
+				if (tds->last_packet)
+					break;
+				if (tds_read_packet(tds) < 0)
 					return NULL;
-				}
-			} while (more);
+			}
 
 			/* add a NUL to mark the end */
 			query[len] = '\0';
 			return query;
 
-		  case 0x06:
+		case TDS_CANCEL:
 			/*
 			 * ignore cancel requests -- if we're waiting
 			 * for the next query then it's obviously too
@@ -189,7 +200,7 @@ char *tds_get_generic_query(TDSSOCKET * tds)
 			/* TODO it's not too late -- freddy77 */
 			break;
 
-		  default:
+		default:
 			/* not a query packet */
 			return NULL;
 		}
