@@ -1,5 +1,6 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004  Brian Bruns
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005  Brian Bruns
+ * Copyright (C) 2006, 2007  Frediano Ziglio
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,9 +22,15 @@
 #include <config.h>
 #endif
 
+#include <stdarg.h>
+#include <stdio.h>
+
+#if HAVE_ERRNO_H
+#include <errno.h>
+#endif /* HAVE_ERRNO_H */
+
 #include <assert.h>
 #include <ctype.h>
-#include <stdio.h>
 
 #if HAVE_STDLIB_H
 #include <stdlib.h>
@@ -57,6 +64,10 @@
 #include <arpa/inet.h>
 #endif /* HAVE_ARPA_INET_H */
 
+#ifdef WIN32
+#include <process.h>
+#endif
+
 #include "tds.h"
 #include "tds_configs.h"
 #include "tdsstring.h"
@@ -65,12 +76,9 @@
 #include <dmalloc.h>
 #endif
 
-static char software_version[] = "$Id: config.c,v 1.97 2004/11/07 08:59:33 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
+TDS_RCSID(var, "$Id: config.c,v 1.132 2007/12/23 21:12:02 jklowden Exp $");
 
 static void tds_config_login(TDSCONNECTION * connection, TDSLOGIN * login);
-static void tds_config_env_tdsquery(TDSCONNECTION * connection);
 static void tds_config_env_tdsdump(TDSCONNECTION * connection);
 static void tds_config_env_tdsver(TDSCONNECTION * connection);
 static void tds_config_env_tdsport(TDSCONNECTION * connection);
@@ -81,6 +89,7 @@ static void tds_read_interfaces(const char *server, TDSCONNECTION * connection);
 static int tds_config_boolean(const char *value);
 static int parse_server_name_for_port(TDSCONNECTION * connection, TDSLOGIN * login);
 static int tds_lookup_port(const char *portname);
+static void tds_config_encryption(const char * value, TDSCONNECTION * connection);
 
 extern int tds_g_append_mode;
 
@@ -88,14 +97,31 @@ static char *interf_file = NULL;
 
 #define TDS_ISSPACE(c) isspace((unsigned char ) (c))
 
+#if !defined(WIN32) && !defined(DOS32X)
+       const char STD_DATETIME_FMT[] = "%b %e %Y %I:%M%p";
+static const char pid_config_logpath[] = "/tmp/tdsconfig.log.%d";
+static const char freetds_conf[] = "%s/etc/freetds.conf";
+static const char location[] = "(from $FREETDS/etc)";
+static const char pid_logpath[] = "/tmp/freetds.log.%d";
+static const char interfaces_path[] = "/etc/freetds";
+#else
+       const char STD_DATETIME_FMT[] = "%b %d %Y %I:%M%p"; /* msvcr80.dll does not support %e */
+static const char pid_config_logpath[] = "c:\\tdsconfig.log.%d";
+static const char freetds_conf [] = "%s\\freetds.conf";
+static const char location[] = "(from $FREETDS)";
+static const char pid_logpath[] = "c:\\freetds.log.%d";
+static const char interfaces_path[] = "c:\\";
+#endif
+
 /**
  * \ingroup libtds
  * \defgroup config Configuration
  * Handle reading of configuration
  */
 
-/** \addtogroup config
- *  \@{ 
+/**
+ * \addtogroup config
+ * @{ 
  */
 
 /**
@@ -134,11 +160,7 @@ tds_read_config_info(TDSSOCKET * tds, TDSLOGIN * login, TDSLOCALE * locale)
 			opened = tdsdump_open(s);
 		} else {
 			pid = getpid();
-#ifndef WIN32
-			if (asprintf(&path, "/tmp/tdsconfig.log.%d", pid) >= 0) {
-#else
-			if (asprintf(&path, "c:\\tdsconfig.log.%d", pid) >= 0) {
-#endif
+			if (asprintf(&path, pid_config_logpath, pid) >= 0) {
 				if (*path) {
 					opened = tdsdump_open(path);
 				}
@@ -147,6 +169,15 @@ tds_read_config_info(TDSSOCKET * tds, TDSLOGIN * login, TDSLOCALE * locale)
 		}
 	}
 
+	tdsdump_log(TDS_DBG_INFO1, "Getting connection information for [%s].\n", 
+			    tds_dstr_cstr(&login->server_name));	/* (The server name is set in login.c.) */
+
+	if (parse_server_name_for_port(connection, login)) {
+		tdsdump_log(TDS_DBG_INFO1, "Parsed servername, now %s on %d.\n", 
+			    tds_dstr_cstr(&connection->server_name), login->port);
+	}
+
+	/* Read the config files. */
 	tdsdump_log(TDS_DBG_INFO1, "Attempting to read conf files.\n");
 	if (!tds_read_conf_file(connection, tds_dstr_cstr(&login->server_name))) {
 		/* fallback to interfaces file */
@@ -154,16 +185,45 @@ tds_read_config_info(TDSSOCKET * tds, TDSLOGIN * login, TDSLOCALE * locale)
 		tds_read_interfaces(tds_dstr_cstr(&login->server_name), connection);
 	}
 
-	if (parse_server_name_for_port(connection, login)) {
-		tdsdump_log(TDS_DBG_INFO1, "Parsed servername, now %s on %d.\n", tds_dstr_cstr(&connection->server_name), login->port);
-	}
-
+	/* Override config file settings with environment variables. */
 	tds_fix_connection(connection);
 
-	/* And finally the login structure */
+	/* And finally apply anything from the login structure */
 	tds_config_login(connection, login);
 
 	if (opened) {
+		tdsdump_log(TDS_DBG_INFO1, "Final connection parameters:\n");
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %s\n", "server_name", tds_dstr_cstr(&connection->server_name));
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %d\n", "port", connection->port);
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %d\n", "major_version", (int)connection->major_version);
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %d\n", "minor_version", (int)connection->minor_version);
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %d\n", "block_size", connection->block_size);
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %s\n", "language", tds_dstr_cstr(&connection->language));
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %s\n", "server_charset", tds_dstr_cstr(&connection->server_charset));
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %d\n", "connect_timeout", connection->connect_timeout);
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %s\n", "client_host_name", tds_dstr_cstr(&connection->client_host_name));
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %s\n", "app_name", tds_dstr_cstr(&connection->app_name));
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %s\n", "user_name", tds_dstr_cstr(&connection->user_name));
+		/* tdsdump_log(TDS_DBG_PASSWD, "\t%20s = %s\n", "password", tds_dstr_cstr(&connection->password)); 
+			(no such flag yet) */
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %s\n", "library", tds_dstr_cstr(&connection->library));
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %d\n", "bulk_copy", (int)connection->bulk_copy);
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %d\n", "suppress_language", (int)connection->suppress_language);
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %d\n", "encrypt level", (int)connection->encryption_level);
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %d\n", "query_timeout", connection->query_timeout);
+		/* tdsdump_log(TDS_DBG_INFO1, "\t%20s = %s\n", "capabilities", tds_dstr_cstr(&connection->capabilities)); 
+			(not null terminated) */
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %s\n", "client_charset", tds_dstr_cstr(&connection->client_charset));
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %s\n", "ip_addr", tds_dstr_cstr(&connection->ip_addr));
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %s\n", "instance_name", tds_dstr_cstr(&connection->instance_name));
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %s\n", "database", tds_dstr_cstr(&connection->database));
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %s\n", "dump_file", tds_dstr_cstr(&connection->dump_file));
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %x\n", "debug_flags", connection->debug_flags);
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %d\n", "text_size", connection->text_size);
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %d\n", "broken_dates", connection->broken_dates);
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %d\n", "broken_money", connection->broken_money);
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %d\n", "emul_little_endian", connection->emul_little_endian);
+
 		tdsdump_close();
 	}
 	return connection;
@@ -180,7 +240,6 @@ tds_fix_connection(TDSCONNECTION * connection)
 	tds_config_env_tdsver(connection);
 	tds_config_env_tdsdump(connection);
 	tds_config_env_tdsport(connection);
-	tds_config_env_tdsquery(connection);
 	tds_config_env_tdshost(connection);
 }
 
@@ -191,13 +250,13 @@ tds_try_conf_file(const char *path, const char *how, const char *server, TDSCONN
 	FILE *in;
 
 	if ((in = fopen(path, "r")) != NULL) {
-		tdsdump_log(TDS_DBG_INFO1, "Found conf file '%s' %s. Reading section '%s'.\n", path, how, server);
+		tdsdump_log(TDS_DBG_INFO1, "Found conf file '%s' %s.\n", path, how);
 		found = tds_read_conf_sections(in, server, connection);
 
 		if (found) {
-			tdsdump_log(TDS_DBG_INFO1, "...Success.\n");
+			tdsdump_log(TDS_DBG_INFO1, "Success: [%s] defined in %s.\n", server, path);
 		} else {
-			tdsdump_log(TDS_DBG_INFO2, "...'%s' not found.\n", server);
+			tdsdump_log(TDS_DBG_INFO2, "[%s] not found.\n", server);
 		}
 
 		fclose(in);
@@ -248,7 +307,7 @@ tds_read_conf_file(TDSCONNECTION * connection, const char *server)
 		if (path) {
 			found = tds_try_conf_file(path, "(from $FREETDSCONF)", server, connection);
 		} else {
-			tdsdump_log(TDS_DBG_INFO2, "...$FREETDSCONF not set.  Trying $FREETDS/etc.\n");
+			tdsdump_log(TDS_DBG_INFO2, "... $FREETDSCONF not set.  Trying $FREETDS/etc.\n");
 		}
 	}
 
@@ -256,15 +315,12 @@ tds_read_conf_file(TDSCONNECTION * connection, const char *server)
 	if (!found) {
 		eptr = getenv("FREETDS");
 		if (eptr) {
-#ifndef WIN32
-			asprintf(&path, "%s/etc/freetds.conf", eptr);
-			found = tds_try_conf_file(path, "(from $FREETDS/etc)", server, connection);
-#else
-			asprintf(&path, "%s\\freetds.conf", eptr);
-			found = tds_try_conf_file(path, "(from $FREETDS)", server, connection);
-#endif
+			if (asprintf(&path, freetds_conf, eptr) >= 0) {
+				found = tds_try_conf_file(path, location, server, connection);
+				free(path);
+			}
 		} else {
-			tdsdump_log(TDS_DBG_INFO2, "...$FREETDS not set.  Trying $HOME.\n");
+			tdsdump_log(TDS_DBG_INFO2, "... $FREETDS not set.  Trying $HOME.\n");
 		}
 	}
 
@@ -274,7 +330,7 @@ tds_read_conf_file(TDSCONNECTION * connection, const char *server)
 			found = tds_try_conf_file(path, "(.freetds.conf)", server, connection);
 			free(path);
 		} else {
-			tdsdump_log(TDS_DBG_INFO2, "...Error getting ~/.freetds.conf.  Trying %s.\n", FREETDS_SYSCONFFILE);
+			tdsdump_log(TDS_DBG_INFO2, "... Error getting ~/.freetds.conf.  Trying %s.\n", FREETDS_SYSCONFFILE);
 		}
 	}
 
@@ -296,13 +352,29 @@ tds_read_conf_sections(FILE * in, const char *server, TDSCONNECTION * connection
 static int
 tds_config_boolean(const char *value)
 {
-	if (!strcmp(value, "yes") || !strcmp(value, "on") || !strcmp(value, "true") || !strcmp(value, "1")) {
-		tdsdump_log(TDS_DBG_INFO1, "%s is a 'yes/on/true'.\n", value);
+	if (!strcmp(value, "yes") || !strcmp(value, "on") || !strcmp(value, "true") || !strcmp(value, "1"))
 		return 1;
-	} else {
-		tdsdump_log(TDS_DBG_INFO1, "%s is a 'no/off/false'.\n", value);
+	if (!strcmp(value, "no") || !strcmp(value, "off") || !strcmp(value, "false") || !strcmp(value, "0"))
 		return 0;
-	}
+	tdsdump_log(TDS_DBG_INFO1, "UNRECOGNIZED boolean value: '%s'. Treating as 'no'.\n", value);
+	return 0;
+}
+
+static void
+tds_config_encryption(const char * value, TDSCONNECTION * connection)
+{
+	TDS_ENCRYPTION_LEVEL lvl = TDS_ENCRYPTION_OFF;
+
+	if (!strcasecmp(value, TDS_STR_ENCRYPTION_OFF))
+		;
+	else if (!strcasecmp(value, TDS_STR_ENCRYPTION_REQUEST))
+		lvl = TDS_ENCRYPTION_REQUEST;
+	else if (!strcasecmp(value, TDS_STR_ENCRYPTION_REQUIRE))
+		lvl = TDS_ENCRYPTION_REQUIRE;
+	else
+		tdsdump_log(TDS_DBG_INFO1, "UNRECOGNIZED option value '%s'...ignoring.\n", value);
+
+	connection->encryption_level = lvl;
 }
 
 /**
@@ -315,7 +387,8 @@ tds_config_boolean(const char *value)
 int
 tds_read_conf_section(FILE * in, const char *section, TDSCONFPARSE tds_conf_parse, void *param)
 {
-	char line[256], option[256], value[256];
+	char line[256], *value;
+#define option line
 	char *s;
 	char p;
 	int i;
@@ -323,7 +396,7 @@ tds_read_conf_section(FILE * in, const char *section, TDSCONFPARSE tds_conf_pars
 	int found = 0;
 
 	tdsdump_log(TDS_DBG_INFO1, "Looking for section %s.\n", section);
-	while (fgets(line, 256, in)) {
+	while (fgets(line, sizeof(line), in)) {
 		s = line;
 
 		/* skip leading whitespace */
@@ -338,15 +411,20 @@ tds_read_conf_section(FILE * in, const char *section, TDSCONFPARSE tds_conf_pars
 		p = 0;
 		i = 0;
 		while (*s && *s != '=') {
-			if (!TDS_ISSPACE(*s) && TDS_ISSPACE(p))
-				option[i++] = ' ';
-			if (!TDS_ISSPACE(*s))
+			if (!TDS_ISSPACE(*s)) {
+				if (TDS_ISSPACE(p))
+					option[i++] = ' ';
 				option[i++] = tolower((unsigned char) *s);
+			}
 			p = *s;
 			s++;
 		}
 		option[i] = '\0';
 
+		/* skip if empty option */
+		if (!i)
+			continue;
+
 		/* skip the = */
 		if (*s)
 			s++;
@@ -356,30 +434,25 @@ tds_read_conf_section(FILE * in, const char *section, TDSCONFPARSE tds_conf_pars
 			s++;
 
 		/* read up to a # ; or null ignoring duplicate spaces */
+		value = s;
 		p = 0;
 		i = 0;
 		while (*s && *s != ';' && *s != '#') {
-			if (!TDS_ISSPACE(*s) && TDS_ISSPACE(p))
-				value[i++] = ' ';
-			if (!TDS_ISSPACE(*s))
+			if (!TDS_ISSPACE(*s)) {
+				if (TDS_ISSPACE(p))
+					value[i++] = ' ';
 				value[i++] = *s;
+			}
 			p = *s;
 			s++;
 		}
 		value[i] = '\0';
 
-		if (!strlen(option))
-			continue;
-
 		if (option[0] == '[') {
-			s = &option[1];
-			while (*s) {
-				if (*s == ']')
-					*s = '\0';
-				*s = tolower((unsigned char) *s);
-				s++;
-			}
-			tdsdump_log(TDS_DBG_INFO1, "... Found section %s.\n", &option[1]);
+			s = strchr(option, ']');
+			if (s)
+				*s = '\0';
+			tdsdump_log(TDS_DBG_INFO1, "\tFound section %s.\n", &option[1]);
 
 			if (!strcasecmp(section, &option[1])) {
 				tdsdump_log(TDS_DBG_INFO1, "Got a match.\n");
@@ -393,16 +466,17 @@ tds_read_conf_section(FILE * in, const char *section, TDSCONFPARSE tds_conf_pars
 		}
 
 	}
+	tdsdump_log(TDS_DBG_INFO1, "\tReached EOF\n");
 	return found;
+#undef option
 }
 
 static void
 tds_parse_conf_section(const char *option, const char *value, void *param)
 {
 	TDSCONNECTION *connection = (TDSCONNECTION *) param;
-	char tmp[256];
 
-	tdsdump_log(TDS_DBG_INFO1, "option = '%s' value = '%s'.\n", option, value);
+	tdsdump_log(TDS_DBG_INFO1, "\t%s = '%s'\n", option, value);
 
 	if (!strcmp(option, TDS_STR_VERSION)) {
 		tds_config_verstr(value, connection);
@@ -413,27 +487,26 @@ tds_parse_conf_section(const char *option, const char *value, void *param)
 		connection->broken_dates = tds_config_boolean(value);
 	} else if (!strcmp(option, TDS_STR_SWAPMNY)) {
 		connection->broken_money = tds_config_boolean(value);
-	} else if (!strcmp(option, TDS_STR_TRYSVR)) {
-		connection->try_server_login = tds_config_boolean(value);
-	} else if (!strcmp(option, TDS_STR_TRYDOM)) {
-		connection->try_domain_login = tds_config_boolean(value);
-	} else if (!strcmp(option, TDS_STR_DOMAIN)) {
-		tds_dstr_copy(&connection->default_domain, value);
-	} else if (!strcmp(option, TDS_STR_XDOMAUTH)) {
-		connection->xdomain_auth = tds_config_boolean(value);
 	} else if (!strcmp(option, TDS_STR_DUMPFILE)) {
 		tds_dstr_copy(&connection->dump_file, value);
-	} else if (!strcmp(option, TDS_STR_DEBUGLVL)) {
+	} else if (!strcmp(option, TDS_STR_DEBUGFLAGS)) {
+		char *end;
+		long l;
+		errno = 0;
+		l = strtol(value, &end, 0);
+		if (errno == 0 && *end == 0)
+			connection->debug_flags = l;
+	} else if (!strcmp(option, TDS_STR_TIMEOUT) || !strcmp(option, TDS_STR_QUERY_TIMEOUT)) {
 		if (atoi(value))
-			connection->debug_level = atoi(value);
-	} else if (!strcmp(option, TDS_STR_TIMEOUT)) {
-		if (atoi(value))
-			connection->timeout = atoi(value);
+			connection->query_timeout = atoi(value);
 	} else if (!strcmp(option, TDS_STR_CONNTIMEOUT)) {
 		if (atoi(value))
 			connection->connect_timeout = atoi(value);
 	} else if (!strcmp(option, TDS_STR_HOST)) {
+		char tmp[256];
+
 		tdsdump_log(TDS_DBG_INFO1, "Found host entry %s.\n", value);
+		tds_dstr_copy(&connection->server_host_name, value);
 		tds_lookup_host(value, tmp);
 		tds_dstr_copy(&connection->ip_addr, tmp);
 		tdsdump_log(TDS_DBG_INFO1, "IP addr is %s.\n", tds_dstr_cstr(&connection->ip_addr));
@@ -450,15 +523,17 @@ tds_parse_conf_section(const char *option, const char *value, void *param)
 		tdsdump_log(TDS_DBG_INFO1, "%s is %s.\n", option, tds_dstr_cstr(&connection->server_charset));
 	} else if (!strcmp(option, TDS_STR_CLCHARSET)) {
 		tds_dstr_copy(&connection->client_charset, value);
-		tdsdump_log(TDS_DBG_INFO1, "tds_config_login:%d: %s is %s.\n", __LINE__, option, tds_dstr_cstr(&connection->client_charset));
+		tdsdump_log(TDS_DBG_INFO1, "tds_config_login: %s is %s.\n", option, tds_dstr_cstr(&connection->client_charset));
 	} else if (!strcmp(option, TDS_STR_LANGUAGE)) {
 		tds_dstr_copy(&connection->language, value);
 	} else if (!strcmp(option, TDS_STR_APPENDMODE)) {
 		tds_g_append_mode = tds_config_boolean(value);
+	} else if (!strcmp(option, TDS_STR_INSTANCE)) {
+		tds_dstr_copy(&connection->instance_name, value);
+	} else if (!strcmp(option, TDS_STR_ENCRYPTION)) {
+		tds_config_encryption(value, connection);
 	} else {
-		tdsdump_log(TDS_DBG_INFO1, "UNRECOGNIZED option '%s'...ignoring.\n", option);
-		fprintf(stderr, "FreeTDS: %s:%d: ignoring unrecognized option '%s'\n", 
-				__FILE__, __LINE__, option);
+		tdsdump_log(TDS_DBG_INFO1, "UNRECOGNIZED option '%s' ... ignoring.\n", option);
 	}
 }
 
@@ -466,52 +541,42 @@ static void
 tds_config_login(TDSCONNECTION * connection, TDSLOGIN * login)
 {
 	if (!tds_dstr_isempty(&login->server_name)) {
-		tds_dstr_copy(&connection->server_name, tds_dstr_cstr(&login->server_name));
+		tds_dstr_dup(&connection->server_name, &login->server_name);
 	}
 	if (login->major_version || login->minor_version) {
 		connection->major_version = login->major_version;
 		connection->minor_version = login->minor_version;
 	}
 	if (!tds_dstr_isempty(&login->language)) {
-		tds_dstr_copy(&connection->language, tds_dstr_cstr(&login->language));
+		tds_dstr_dup(&connection->language, &login->language);
 	}
 	if (!tds_dstr_isempty(&login->server_charset)) {
-		tds_dstr_copy(&connection->server_charset, tds_dstr_cstr(&login->server_charset));
+		tds_dstr_dup(&connection->server_charset, &login->server_charset);
 	}
 	if (!tds_dstr_isempty(&login->client_charset)) {
-		tds_dstr_copy(&connection->client_charset, tds_dstr_cstr(&login->client_charset));
-		tdsdump_log(TDS_DBG_INFO1, "tds_config_login:%d: %s is %s.\n", __LINE__, "client_charset",
+		tds_dstr_dup(&connection->client_charset, &login->client_charset);
+		tdsdump_log(TDS_DBG_INFO1, "tds_config_login: %s is %s.\n", "client_charset",
 			    tds_dstr_cstr(&connection->client_charset));
 	}
-	if (!tds_dstr_isempty(&login->host_name)) {
-		tds_dstr_copy(&connection->host_name, tds_dstr_cstr(&login->host_name));
-		/* DBSETLHOST and it's equivilants are commentary fields
-		 * ** they don't affect connection->ip_addr (the server) but they show
-		 * ** up in an sp_who as the *clients* hostname.  (bsb, 11/10) 
-		 */
-		/* should work with IP (mlilback, 11/7/01) */
-		/*
-		 * if (connection->ip_addr) free(connection->ip_addr);
-		 * connection->ip_addr = calloc(sizeof(char),18);
-		 * tds_lookup_host(connection->host_name, NULL, connection->ip_addr, NULL);
-		 */
+	if (!tds_dstr_isempty(&login->client_host_name)) {
+		tds_dstr_dup(&connection->client_host_name, &login->client_host_name);
 	}
 	if (!tds_dstr_isempty(&login->app_name)) {
-		tds_dstr_copy(&connection->app_name, tds_dstr_cstr(&login->app_name));
+		tds_dstr_dup(&connection->app_name, &login->app_name);
 	}
 	if (!tds_dstr_isempty(&login->user_name)) {
-		tds_dstr_copy(&connection->user_name, tds_dstr_cstr(&login->user_name));
+		tds_dstr_dup(&connection->user_name, &login->user_name);
 	}
 	if (!tds_dstr_isempty(&login->password)) {
 		/* for security reason clear memory */
 		tds_dstr_zero(&connection->password);
-		tds_dstr_copy(&connection->password, tds_dstr_cstr(&login->password));
+		tds_dstr_dup(&connection->password, &login->password);
 	}
 	if (!tds_dstr_isempty(&login->library)) {
-		tds_dstr_copy(&connection->library, tds_dstr_cstr(&login->library));
+		tds_dstr_dup(&connection->library, &login->library);
 	}
-	if (login->encrypted) {
-		connection->encrypted = 1;
+	if (login->encryption_level) {
+		connection->encryption_level = login->encryption_level;
 	}
 	if (login->suppress_language) {
 		connection->suppress_language = 1;
@@ -524,33 +589,18 @@ tds_config_login(TDSCONNECTION * connection, TDSLOGIN * login)
 	}
 	if (login->port) {
 		connection->port = login->port;
+		tds_dstr_copy(&connection->instance_name, "");
 	}
 	if (login->connect_timeout)
 		connection->connect_timeout = login->connect_timeout;
 
+	if (login->query_timeout)
+		connection->query_timeout = login->query_timeout;
+
 	/* copy other info not present in configuration file */
-	connection->query_timeout = login->query_timeout;
-	connection->longquery_timeout = login->longquery_timeout;
-	connection->longquery_func = login->longquery_func;
-	connection->longquery_param = login->longquery_param;
 	memcpy(connection->capabilities, login->capabilities, TDS_MAX_CAPABILITY);
 }
 
-static void
-tds_config_env_tdsquery(TDSCONNECTION * connection)
-{
-	char *s;
-
-	if ((s = getenv("TDSQUERY")) != NULL && s[0]) {
-		tds_dstr_copy(&connection->server_name, s);
-		tdsdump_log(TDS_DBG_INFO1, "Setting 'server_name' to '%s' from $TDSQUERY.\n", s);
-		return;
-	}
-	if ((s = getenv("DSQUERY")) != NULL && s[0]) {
-		tds_dstr_copy(&connection->server_name, s);
-		tdsdump_log(TDS_DBG_INFO1, "Setting 'server_name' to '%s' from $DSQUERY.\n", s);
-	}
-}
 static void
 tds_config_env_tdsdump(TDSCONNECTION * connection)
 {
@@ -561,11 +611,7 @@ tds_config_env_tdsdump(TDSCONNECTION * connection)
 	if ((s = getenv("TDSDUMP"))) {
 		if (!strlen(s)) {
 			pid = getpid();
-#ifndef WIN32
-			if (asprintf(&path, "/tmp/freetds.log.%d", pid) >= 0)
-#else
-			if (asprintf(&path, "c:\\freetds.log.%d", pid) >= 0)
-#endif
+			if (asprintf(&path, pid_logpath, pid) >= 0)
 				tds_dstr_set(&connection->dump_file, path);
 		} else {
 			tds_dstr_copy(&connection->dump_file, s);
@@ -579,7 +625,8 @@ tds_config_env_tdsport(TDSCONNECTION * connection)
 	char *s;
 
 	if ((s = getenv("TDSPORT"))) {
-		connection->port = atoi(s);
+		connection->port = tds_lookup_port(s);
+		tds_dstr_copy(&connection->instance_name, "");
 		tdsdump_log(TDS_DBG_INFO1, "Setting 'port' to %s from $TDSPORT.\n", s);
 	}
 	return;
@@ -605,12 +652,11 @@ tds_config_env_tdshost(TDSCONNECTION * connection)
 	char tmp[256];
 
 	if ((tdshost = getenv("TDSHOST"))) {
+		tds_dstr_copy(&connection->server_host_name, tdshost);
 		tds_lookup_host(tdshost, tmp);
 		tds_dstr_copy(&connection->ip_addr, tmp);
 		tdsdump_log(TDS_DBG_INFO1, "Setting 'ip_addr' to %s (%s) from $TDSHOST.\n", tmp, tdshost);
-
 	}
-	return;
 }
 
 /**
@@ -641,6 +687,16 @@ tds_config_verstr(const char *tdsver, TDSCONNECTION * connection)
 		connection->major_version = 8;
 		connection->minor_version = 0;
 		return;
+#ifdef ENABLE_DEVELOPING
+	} else if (!strcmp(tdsver, "90") || !strcmp(tdsver, "9.0")) {
+		connection->major_version = 9;
+		connection->minor_version = 0;
+		return;
+#endif
+	} else if (!strcmp(tdsver, "0.0")) {
+		connection->major_version = 0;
+		connection->minor_version = 0;
+		return;
 	}
 }
 
@@ -666,20 +722,18 @@ tds_set_interfaces_file_loc(const char *interf)
 }
 
 /**
- * Given a servername and port name or number, lookup the
- * hostname and service.  The server ip will be stored in the
- * string 'servername' in dotted-decimal notation.  The service port
- * number will be stored in string form in the 'port' parameter.
+ * Given a servername lookup the hostname. The server ip will be stored 
+ * in the string 'ip' in dotted-decimal notation.
  *
- * If we can't determine both the IP address and port number then
- * 'ip' and 'port' will be set to empty strings.
+ * If we can't determine the IP address then 'ip' will be set to empty
+ * string.
  */
 /* TODO callers seem to set always connection info... change it */
 void
 tds_lookup_host(const char *servername,	/* (I) name of the server                  */
 		char *ip	/* (O) dotted-decimal ip address of server */
 	)
-{				/* (O) port number of the service          */
+{
 	struct hostent *host = NULL;
 	unsigned int ip_addr = 0;
 
@@ -688,12 +742,14 @@ tds_lookup_host(const char *servername,	/* (I) name of the server
 	char buffer[4096];
 	int h_errnop;
 
-	/* Only call gethostbyname if servername is not an ip address. 
+	/*
+	 * Only call gethostbyname if servername is not an ip address. 
 	 * This call take a while and is useless for an ip address.
-	 * mlilback 3/2/02 */
+	 * mlilback 3/2/02
+	 */
 	ip_addr = inet_addr(servername);
 	if (ip_addr != INADDR_NONE) {
-		strncpy(ip, servername, 17);
+		tds_strlcpy(ip, servername, 17);
 		return;
 	}
 
@@ -703,16 +759,15 @@ tds_lookup_host(const char *servername,	/* (I) name of the server
 	if (host) {
 		struct in_addr *ptr = (struct in_addr *) host->h_addr;
 
-#if defined(AF_INET) && HAVE_INET_NTOP
-		inet_ntop(AF_INET, ptr, ip, 17);
-#elif HAVE_INET_NTOA_R
-		inet_ntoa_r(*ptr, ip, 17);
-#else
-		strncpy(ip, inet_ntoa(*ptr), 17);
-#endif
+		tds_inet_ntoa_r(*ptr, ip, 17);
 	}
 }				/* tds_lookup_host()  */
 
+/**
+ * Given a portname lookup the port.
+ *
+ * If we can't determine the port number than function return 0.
+ */
 static int
 tds_lookup_port(const char *portname)
 {
@@ -734,40 +789,35 @@ tds_lookup_port(const char *portname)
 
 /* TODO same code in convert.c ?? */
 static int
-hexdigit(char c)
+hexdigit(int c)
 {
-	if (c >= 'a' && c <= 'f') {
-		return c - 'a' + 10;
-	} else if (c >= 'A' && c <= 'F') {
-		return c - 'A' + 10;
-	} else if (c >= '0' && c <= '9') {
+	if (c >= '0' && c <= '9')
 		return c - '0';
-	} else {
-		return 0;	/* bad hex digit */
-	}
+	/* ascii optimization, 'A' -> 'a', 'a' -> 'a' */
+	c |= 0x20;
+	if (c >= 'a' && c <= 'f')
+		return c - 'a' + 10;
+	return 0;	/* bad hex digit */
 }
+
 static int
 hex2num(char *hex)
 {
 	return hexdigit(hex[0]) * 16 + hexdigit(hex[1]);
 }
 
-/* ========================= search_interface_file() =========================
+/**
+ * Open and read the file 'file' searching for a logical server
+ * by the name of 'host'.  If one is found then lookup
+ * the IP address and port number and store them in 'connection'
  *
- * Def:  Open and read the file 'file' searching for a logical server
- *       by the name of 'host'.  If one is found then lookup
- *       the IP address and port number and store them in 'ip_addr', and
- *       'ip_port'.
- *
- * Ret:  void
- *
- * ===========================================================================
+ * \param dir name of base directory for interface file
+ * \param file name of the interface file
+ * \param host logical host to search for
+ * \return 0 if not fount 1 if found
  */
 static int
-search_interface_file(TDSCONNECTION * connection, const char *dir,	/* (I) Name of base directory for interface file */
-		      const char *file,	/* (I) Name of the interface file                */
-		      const char *host	/* (I) Logical host to search for                */
-	)
+search_interface_file(TDSCONNECTION * connection, const char *dir, const char *file, const char *host)
 {
 	char *pathname;
 	char line[255];
@@ -791,7 +841,7 @@ search_interface_file(TDSCONNECTION * connection, const char *dir,	/* (I) Name o
 		return 0;
 
 	/*
-	 * * create the full pathname to the interface file
+	 * create the full pathname to the interface file
 	 */
 	if (file[0] == '\0') {
 		pathname[0] = '\0';
@@ -807,7 +857,7 @@ search_interface_file(TDSCONNECTION * connection, const char *dir,	/* (I) Name o
 
 
 	/*
-	 * *  parse the interfaces file and find the server and port
+	 * parse the interfaces file and find the server and port
 	 */
 	if ((in = fopen(pathname, "r")) == NULL) {
 		tdsdump_log(TDS_DBG_INFO1, "Couldn't open %s.\n", pathname);
@@ -863,6 +913,7 @@ search_interface_file(TDSCONNECTION * connection, const char *dir,	/* (I) Name o
 	 * Look up the host and service
 	 */
 	if (server_found) {
+		tds_dstr_copy(&connection->server_host_name, tmp_ip);
 		tds_lookup_host(tmp_ip, line);
 		tdsdump_log(TDS_DBG_INFO1, "Resolved IP as '%s'.\n", line);
 		tds_dstr_copy(&connection->ip_addr, line);
@@ -878,8 +929,6 @@ search_interface_file(TDSCONNECTION * connection, const char *dir,	/* (I) Name o
  * Try to find the IP number and port for a (possibly) logical server name.
  *
  * @note This function uses only the interfaces file and is deprecated.
- *
- * ===========================================================================
  */
 static void
 tds_read_interfaces(const char *server, TDSCONNECTION * connection)
@@ -931,11 +980,8 @@ tds_read_interfaces(const char *server, TDSCONNECTION * connection)
 		if ( (int)unixspec != 0 && (int)unixspec != -1 ) sybase = unixspec;
 #endif
 		if (!sybase || !sybase[0])
-#ifndef WIN32
-			sybase = "/etc/freetds";
-#else
-			sybase = "c:\\";
-#endif
+			sybase = interfaces_path;
+
 		tdsdump_log(TDS_DBG_INFO1, "Looking for server in %s/interfaces.\n", sybase);
 		founded = search_interface_file(connection, sybase, "interfaces", server);
 	}
@@ -989,42 +1035,38 @@ tds_read_interfaces(const char *server, TDSCONNECTION * connection)
 
 /**
  * Check the server name to find port info first
- * return 1 when found, else 0
  * Warning: connection-> & login-> are all modified when needed
+ * \return 1 when found, else 0
  */
 static int
 parse_server_name_for_port(TDSCONNECTION * connection, TDSLOGIN * login)
 {
-	char *pSep, *pEnd;
-	char *server;
+	const char *pSep;
+	const char *server;
 
 	/* seek the ':' in login server_name */
 	server = tds_dstr_cstr(&login->server_name);
-	pEnd = server + strlen(server);
-	for (pSep = server; pSep < pEnd; pSep++)
-		if (*pSep == ':')
-			break;
-
-	if ((pSep < pEnd) && (pSep != server)) {	/* yes, i found it! */
-		if (!tds_dstr_copyn(&connection->server_name, server, pSep - server))	/* end the server_name before the ':' */
-			return 0;	/* FALSE */
+	pSep = strrchr(server, ':');
 
+	if (pSep && pSep != server) {	/* yes, i found it! */
 		/* modify connection-> && login->server_name & ->port */
 		login->port = connection->port = atoi(pSep + 1);
-		*pSep = 0;
+		tds_dstr_copy(&connection->instance_name, "");
+	} else {
+		/* handle instance name */
+		pSep = strrchr(server, '\\');
+		if (!pSep || pSep == server)
+			return 0;
 
-		/* connection->ip_addr needed */
-		{
-			char tmp[256];
+		login->port = connection->port = 0;
+		tds_dstr_copy(&connection->instance_name, pSep + 1);
+	}
 
-			tds_lookup_host(tds_dstr_cstr(&connection->server_name), tmp);
-			if (!tds_dstr_copy(&connection->ip_addr, tmp))
-				return 0;	/* FALSE */
-		}
+	tds_dstr_setlen(&login->server_name, pSep - server);
+	if (!tds_dstr_dup(&connection->server_name, &login->server_name))
+		return 0;
 
-		return 1;	/* TRUE */
-	} else
-		return 0;	/* FALSE */
+	return 1;
 }
 
 
@@ -1036,14 +1078,20 @@ parse_server_name_for_port(TDSCONNECTION * connection, TDSLOGIN * login)
 const TDS_COMPILETIME_SETTINGS *
 tds_get_compiletime_settings(void)
 {
-	static TDS_COMPILETIME_SETTINGS settings = {
-		TDS_VERSION_NO, "unknown"	/* need fancy script in makefile */
+	static const TDS_COMPILETIME_SETTINGS settings = {
+		  TDS_VERSION_NO
+		, FREETDS_SYSCONFDIR
+		, "unknown"	/* need fancy script in makefile */
 #		ifdef MSDBLIB
 			, 1
 #		else
 			, 0
 #		endif
-			, -1	/* unknown: sybase_compat only a makefile setting, so far. */
+#		ifdef TDS_SYBASE_COMPAT
+			, 1
+#		else
+			, 0
+#		endif
 #		ifdef _REENTRANT
 			, 1
 #		else
@@ -1088,4 +1136,4 @@ tds_get_compiletime_settings(void)
 	return &settings;
 }
 
-/** \@} */
+/** @} */
