@@ -1,5 +1,5 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004  Brian Bruns
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005  Brian Bruns
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -22,12 +22,14 @@
  * all over the other code
  */
 
-#include <assert.h>
-
 #if HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <stdarg.h>
+#include <stdio.h>
+#include <assert.h>
+
 #if HAVE_STRING_H
 #include <string.h>
 #endif /* HAVE_STRING_H */
@@ -47,14 +49,11 @@
 /* define this for now; remove when done testing */
 #define HAVE_ICONV_ALWAYS 1
 
-static char software_version[] = "$Id: iconv.c,v 1.114.2.1 2005/02/27 08:48:23 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+TDS_RCSID(var, "$Id: iconv.c,v 1.132 2007/01/02 20:47:05 jklowden Exp $");
 
 #define CHARSIZE(charset) ( ((charset)->min_bytes_per_char == (charset)->max_bytes_per_char )? \
 				(charset)->min_bytes_per_char : 0 )
 
-#define SAFECPY(d, s) 	strncpy((d), (s), sizeof(d)); (d)[sizeof(d) - 1] = '\0'
-
 
 #if !HAVE_ICONV_ALWAYS
 static int bytes_per_char(TDS_ENCODING * charset);
@@ -80,7 +79,6 @@ static void tds_iconv_info_close(TDSICONV * char_conv);
 /* this will contain real iconv names */
 static const char *iconv_names[sizeof(canonic_charsets) / sizeof(canonic_charsets[0])];
 static int iconv_initialized = 0;
-static unsigned int ucs2pos;
 static const char *ucs2name;
 
 enum
@@ -193,8 +191,7 @@ tds_iconv_init(void)
 	if (!iconv_names[POS_UCS2LE] && !iconv_names[POS_UCS2BE])
 		return 2;
 
-	ucs2pos = iconv_names[POS_UCS2LE] ? POS_UCS2LE : POS_UCS2BE;
-	ucs2name = iconv_names[ucs2pos];
+	ucs2name = iconv_names[POS_UCS2LE] ? iconv_names[POS_UCS2LE] : iconv_names[POS_UCS2BE];
 
 	for (i = 0; i < 4; ++i)
 		tdsdump_log(TDS_DBG_INFO1, "names for %s: %s\n", canonic_charsets[i].name,
@@ -284,7 +281,7 @@ tds_iconv_alloc(TDSSOCKET * tds)
 
 	assert(!tds->char_convs);
 	if (!(tds->char_convs = (TDSICONV **) malloc(sizeof(TDSICONV *) * (initial_char_conv_count + 1))))
-		return 1;
+	return 1;
 	char_conv = (TDSICONV *) malloc(sizeof(TDSICONV) * initial_char_conv_count);
 	if (!char_conv) {
 		TDS_ZERO_FREE(tds->char_convs);
@@ -306,7 +303,7 @@ tds_iconv_alloc(TDSSOCKET * tds)
 
 /**
  * \addtogroup conv
- * \@{ 
+ * @{ 
  * Set up the initial iconv conversion descriptors.
  * When the socket is allocated, three TDSICONV structures are attached to iconv.  
  * They have fixed meanings:
@@ -317,20 +314,20 @@ tds_iconv_alloc(TDSSOCKET * tds)
  * Other designs that use less data are possible, but these three conversion needs are 
  * very often needed.  By reserving them, we avoid searching the array for our most common purposes.
  *
- * To solve different iconv names and portability problem FreeTDS use a complex
- * method. It maintain a list of all alias of a given charset.
- * First it discover some needed charset (UTF-8, ISO8859-1 and UCS2) and then
- * try to discover others from those characters (this discover happen only when
- * required).
+ * To solve different iconv names and portability problems FreeTDS maintains 
+ * a list of aliases each charset.  
+ * 
+ * First we discover the names of our minimum required charsets (UTF-8, ISO8859-1 and UCS2).  
+ * Later, as and when it's needed, we try to discover others.
  *
- * There are a list of canonic names (GNU iconv names) and a set of aliases
- * (one for others iconv implementations and another for Sybase). For every
- * canonic charset name we cache iconv name found during discovery.
+ * There is one list of canonic names (GNU iconv names) and two sets of aliases
+ * (one for other iconv implementations and another for Sybase). For every
+ * canonic charset name we cache the iconv name found during discovery. 
  */
 void
 tds_iconv_open(TDSSOCKET * tds, const char *charset)
 {
-	static const char *UCS_2LE = "UCS-2LE";
+	static const char UCS_2LE[] = "UCS-2LE";
 	const char *name;
 	int fOK, ret;
 
@@ -349,7 +346,7 @@ tds_iconv_open(TDSSOCKET * tds, const char *charset)
 	/* initialize */
 	if (!iconv_initialized) {
 		if ((ret = tds_iconv_init()) > 0) {
-			static char *names[] = { "ISO 8859-1", "UTF-8" };
+			static const char names[][12] = { "ISO 8859-1", "UTF-8" };
 			assert(ret < 3);
 			tdsdump_log(TDS_DBG_FUNC, "error: tds_iconv_init() returned %d; "
 						  "could not find a name for %s that your iconv accepts.\n"
@@ -383,8 +380,9 @@ tds_iconv_open(TDSSOCKET * tds, const char *charset)
 	 * TODO: the server hasn't reported its charset yet, so this logic can't work here.  
 	 *       not sure what to do about that yet.  
 	 */
-	if (tds->env && tds->env->charset) {
-		fOK = tds_iconv_info_init(tds->char_convs[client2server_chardata], charset, tds->env->charset);
+	tds->char_convs[client2server_chardata]->flags = TDS_ENCODING_MEMCPY;
+	if (tds->env.charset) {
+		fOK = tds_iconv_info_init(tds->char_convs[client2server_chardata], charset, tds->env.charset);
 		if (!fOK)
 			return;
 	}
@@ -395,8 +393,8 @@ tds_iconv_open(TDSSOCKET * tds, const char *charset)
 	name = UCS_2LE;
 	if (tds->major_version < 7) {
 		name = "ISO-8859-1";
-		if (tds->env && tds->env->charset)
-			name = tds->env->charset;
+		if (tds->env.charset)
+			name = tds->env.charset;
 	}
 	fOK = tds_iconv_info_init(tds->char_convs[iso2server_metadata], "ISO-8859-1", name);
 
@@ -575,20 +573,21 @@ tds_iconv_free(TDSSOCKET * tds)
 
 /** 
  * Wrapper around iconv(3).  Same parameters, with slightly different behavior.
+ * \param tds state information for the socket and the TDS protocol
  * \param io Enumerated value indicating whether the data are being sent to or received from the server. 
- * \param iconv information about the encodings involved, including the iconv(3) conversion descriptors. 
+ * \param conv information about the encodings involved, including the iconv(3) conversion descriptors. 
  * \param inbuf address of pointer to the input buffer of data to be converted.  
  * \param inbytesleft address of count of bytes in \a inbuf.
  * \param outbuf address of pointer to the output buffer.  
  * \param outbytesleft address of count of bytes in \a outbuf.
- * \retval number of irreversible conversions performed.  \i -1 on error, see iconv(3) documentation for 
- * a description of the possible values of \i errno.  
+ * \retval number of irreversible conversions performed.  -1 on error, see iconv(3) documentation for 
+ * a description of the possible values of \e errno.  
  * \remarks Unlike iconv(3), none of the arguments can be nor point to NULL.  Like iconv(3), all pointers will 
- *  	be updated.  Succcess is signified by a nonnegative return code and \a *inbytesleft == 0.  
+ *  	be updated.  Success is signified by a nonnegative return code and \a *inbytesleft == 0.  
  * 	If the conversion descriptor in \a iconv is -1 or NULL, \a inbuf is copied to \a outbuf, 
  *	and all parameters updated accordingly. 
  * 
- * 	In the event that a character in \a inbuf cannot be converted because no such cbaracter exists in the
+ * 	If a character in \a inbuf cannot be converted because no such cbaracter exists in the
  * 	\a outbuf character set, we emit messages similar to the ones Sybase emits when it fails such a conversion. 
  * 	The message varies depending on the direction of the data.  
  * 	On a read error, we emit Msg 2403, Severity 16 (EX_INFO):
@@ -596,11 +595,11 @@ tds_iconv_free(TDSSOCKET * tds)
  *			Unconverted bytes were changed to question marks ('?')."
  * 	On a write error we emit Msg 2402, Severity 16 (EX_USER):
  *		"Error converting client characters into server's character set. Some character(s) could not be converted."
- *  	  and return an error code.  Client libraries relying on this routine should reflect an error back to the appliction.  
+ *  	  and return an error code.  Client libraries relying on this routine should reflect an error back to the application.  
  * 	
  * \todo Check for variable multibyte non-UTF-8 input character set.  
  * \todo Use more robust error message generation.  
- * \todo For reads, cope with \outbuf encodings that don't have the equivalent of an ASCII '?'.  
+ * \todo For reads, cope with \a outbuf encodings that don't have the equivalent of an ASCII '?'.  
  * \todo Support alternative to '?' for the replacement character.  
  */
 size_t
@@ -798,18 +797,13 @@ end_loop:
 		/* invalid multibyte input sequence encountered */
 		if (io == to_client) {
 			if (irreversible == (size_t) - 1) {
-				tds_client_msg(tds->tds_ctx, tds, 2404, 16, 0, 0,
-					       "WARNING! Some character(s) could not be converted into client's character set. ");
+				tdserror(tds->tds_ctx, tds, TDSEICONV2BIG, 0);
 			} else {
-				tds_client_msg(tds->tds_ctx, tds, 2403, 16, 0, 0,
-					       "WARNING! Some character(s) could not be converted into client's character set. "
-					       "Unconverted bytes were changed to question marks ('?').");
+				tdserror(tds->tds_ctx, tds, TDSEICONVI, 0);
 				errno = 0;
 			}
 		} else {
-			tds_client_msg(tds->tds_ctx, tds, 2402, 16, 0, 0,
-				       "Error converting client characters into server's character set. "
-				       "Some character(s) could not be converted.");
+			tdserror(tds->tds_ctx, tds, TDSEICONVO, 0);
 		}
 		suppress->eilseq = 1;
 	}
@@ -819,16 +813,13 @@ end_loop:
 		if (suppress->einval)
 			break;
 		/* in chunk conversion this can mean we end a chunk inside a character */
-		tds_client_msg(tds->tds_ctx, tds, 2401, 16, *inbytesleft, 0,
-			       "iconv EINVAL: Error converting between character sets. "
-			       "Conversion abandoned at offset indicated by the \"state\" value of this message.");
+		tdserror(tds->tds_ctx, tds, TDSEICONVAVAIL, 0);
 		suppress->einval = 1;
 		break;
 	case E2BIG:		/* output buffer has no more room */
 		if (suppress->e2big)
 			break;
-		tds_client_msg(tds->tds_ctx, tds, 2400, 16, *inbytesleft, 0,
-			       "iconv E2BIG: Error converting between character sets. " "Output buffer exhausted.");
+		tdserror(tds->tds_ctx, tds, TDSEICONVIU, 0);
 		suppress->e2big = 1;
 		break;
 	default:
@@ -849,14 +840,18 @@ end_loop:
 size_t
 tds_iconv_fread(iconv_t cd, FILE * stream, size_t field_len, size_t term_len, char *outbuf, size_t * outbytesleft)
 {
+#ifdef ENABLE_EXTRA_CHECKS
+	char buffer[16];
+#else
 	char buffer[16000];
+#endif
 	char *ib;
 	size_t isize = 0, nonreversible_conversions = 0;
 
 	/*
 	 * If cd isn't valid, it's just an indication that this column needs no conversion.  
 	 */
-	if (cd == (iconv_t) - 1 || cd == NULL) {
+	if (cd == (iconv_t) - 1) {
 		assert(field_len <= *outbytesleft);
 		if (field_len > 0) {
 			if (1 != fread(outbuf, field_len, 1, stream)) {
@@ -882,23 +877,21 @@ tds_iconv_fread(iconv_t cd, FILE * stream, size_t field_len, size_t term_len, ch
 	 */
 	isize = (sizeof(buffer) < field_len) ? sizeof(buffer) : field_len;
 
-	for (ib = buffer; isize && 1 == fread(ib, isize, 1, stream);) {
+	for (ib = buffer; isize && (isize = fread(ib, 1, isize, stream)) > 0;) {
 
 		tdsdump_log(TDS_DBG_FUNC, "tds_iconv_fread: read %u of %u bytes; outbuf has %u left.\n", (unsigned int) isize,
 			    (unsigned int) field_len, (unsigned int) *outbytesleft);
 		field_len -= isize;
 
+		isize += ib - buffer;
+		ib = buffer;
 		nonreversible_conversions += tds_sys_iconv(cd, (ICONV_CONST char **) &ib, &isize, &outbuf, outbytesleft);
 
 		if (isize != 0) {
+			memmove(buffer, ib, isize);
 			switch (errno) {
 			case EINVAL:	/* incomplete multibyte sequence encountered in input */
-				memmove(buffer, buffer + sizeof(buffer) - isize, isize);
-				ib = buffer + isize;
-				isize = sizeof(buffer) - isize;
-				if (isize < field_len)
-					isize = field_len;
-				continue;
+				break;
 			case E2BIG:	/* insufficient room in output buffer */
 			case EILSEQ:	/* invalid multibyte sequence encountered in input */
 			default:
@@ -907,7 +900,10 @@ tds_iconv_fread(iconv_t cd, FILE * stream, size_t field_len, size_t term_len, ch
 				break;
 			}
 		}
-		isize = (sizeof(buffer) < field_len) ? sizeof(buffer) : field_len;
+		ib = buffer + isize;
+		isize = sizeof(buffer) - isize;
+		if (isize > field_len)
+			isize = field_len;
 	}
 	
 	READ_TERMINATOR:
@@ -973,13 +969,20 @@ tds_srv_charset_changed(TDSSOCKET * tds, const char *charset)
 #if HAVE_ICONV_ALWAYS
 	TDSICONV *char_conv = tds->char_convs[client2server_chardata];
 
-	const char *canonic_charset = tds_canonical_charset_name(charset);
+	int canonic_charset_num = tds_canonical_charset(charset);
+	const char *canonic_charset;
+
+	if (tds->major_version >= 7 && canonic_charset_num == TDS_CHARSET_ISO_8859_1)
+		canonic_charset_num = TDS_CHARSET_CP1252;
 
 	/* ignore request to change to unknown charset */
-	if (!canonic_charset) {
+	if (canonic_charset_num < 0) {
 		tdsdump_log(TDS_DBG_FUNC, "tds_srv_charset_changed: what is charset \"%s\"?\n", charset);
 		return;
 	}
+	canonic_charset = canonic_charsets[canonic_charset_num].name;
+
+	tdsdump_log(TDS_DBG_FUNC, "setting server single-byte charset to \"%s\"\n", canonic_charset);
 
 	if (strcmp(canonic_charset, char_conv->server_charset.name) == 0)
 		return;
@@ -1064,7 +1067,8 @@ skip_one_input_sequence(iconv_t cd, const TDS_ENCODING * charset, const char **i
 	}
 
 	if (0 == strcmp(charset->name, "UTF-8")) {
-		/* Deal with UTF-8.  
+		/*
+		 * Deal with UTF-8.  
 		 * bytes | bits | representation
 		 *     1 |    7 | 0vvvvvvv
 		 *     2 |   11 | 110vvvvv 10vvvvvv
@@ -1211,7 +1215,8 @@ tds_sybase_charset_name(const char *charset_name)
 static const char *
 collate2charset(int sql_collate, int lcid)
 {
-	/* The table from the MSQLServer reference "Windows Collation Designators" 
+	/*
+	 * The table from the MSQLServer reference "Windows Collation Designators" 
 	 * and from " NLS Information for Microsoft Windows XP"
 	 */
 
@@ -1444,4 +1449,4 @@ tds_iconv_from_collate(TDSSOCKET * tds, int sql_collate, int lcid)
 	return tds_iconv_get_info(tds, charset);
 }
 
-/** \@} */
+/** @} */
