@@ -1,5 +1,6 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004  Brian Bruns
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005  Brian Bruns
+ * Copyright (C) 2005, 2006, 2007  Ziglio Frediano
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,65 +22,26 @@
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
 
-#if TIME_WITH_SYS_TIME
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# endif
-# include <time.h>
-#else
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
-
-#if HAVE_ERRNO_H
-#include <errno.h>
-#endif
+#include <stdarg.h>
+#include <stdio.h>
 
 #if HAVE_STDLIB_H
 #include <stdlib.h>
 #endif /* HAVE_STDLIB_H */
 
-#include <stdio.h>
 #include <assert.h>
 
 #if HAVE_STRING_H
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
-#if HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
-#endif /* HAVE_SYS_SOCKET_H */
-
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif /* HAVE_SYS_TYPES_H */
-
-#if HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif /* HAVE_NETINET_IN_H */
-
-#if HAVE_NETINET_TCP_H
-#include <netinet/tcp.h>
-#endif /* HAVE_NETINET_TCP_H */
-
-#if HAVE_ARPA_INET_H
-#include <arpa/inet.h>
-#endif /* HAVE_ARPA_INET_H */
-
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
 
-#if HAVE_SYS_IOCTL_H
-#include <sys/ioctl.h>
-#endif /* HAVE_SYS_IOCTL_H */
-
-#if HAVE_SELECT_H
-#include <sys/select.h>
-#endif /* HAVE_SELECT_H */
+#ifdef WIN32
+#include <process.h>
+#endif
 
 #include "tds.h"
 #include "tdsiconv.h"
@@ -89,14 +51,14 @@
 #include <dmalloc.h>
 #endif
 
-static char software_version[] = "$Id: login.c,v 1.126 2004/07/29 10:22:41 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+TDS_RCSID(var, "$Id: login.c,v 1.172 2007/12/31 10:06:50 freddy77 Exp $");
 
 static int tds_send_login(TDSSOCKET * tds, TDSCONNECTION * connection);
+static int tds8_do_login(TDSSOCKET * tds, TDSCONNECTION * connection);
 static int tds7_send_login(TDSSOCKET * tds, TDSCONNECTION * connection);
 
 void
-tds_set_version(TDSLOGIN * tds_login, short major_ver, short minor_ver)
+tds_set_version(TDSLOGIN * tds_login, TDS_TINYINT major_ver, TDS_TINYINT minor_ver)
 {
 	tds_login->major_version = major_ver;
 	tds_login->minor_version = minor_ver;
@@ -137,7 +99,7 @@ tds_set_user(TDSLOGIN * tds_login, const char *username)
 void
 tds_set_host(TDSLOGIN * tds_login, const char *hostname)
 {
-	tds_dstr_copy(&tds_login->host_name, hostname);
+	tds_dstr_copy(&tds_login->client_host_name, hostname);
 }
 
 void
@@ -146,18 +108,43 @@ tds_set_app(TDSLOGIN * tds_login, const char *application)
 	tds_dstr_copy(&tds_login->app_name, application);
 }
 
+/**
+ * \brief Set the servername in a TDSLOGIN structure
+ *
+ * Normally copies \a server into \a tds_login.  If \a server does not point to a plausible name, the environment 
+ * variables TDSQUERY and DSQUERY are used, in that order.  If they don't exist, the "default default" servername
+ * is "SYBASE" (although the utility of that choice is a bit murky).  
+ *
+ * \param tds_login	points to a TDSLOGIN structure
+ * \param server	the servername, or NULL, or a zero-length string
+ * \todo open the log file earlier, so these messages can be seen.  
+ */
 void
 tds_set_server(TDSLOGIN * tds_login, const char *server)
 {
+	if (!server || strlen(server) == 0) {
+		server = getenv("TDSQUERY");
+		tdsdump_log(TDS_DBG_INFO1, "Setting 'server_name' to '%s' from $TDSQUERY.\n", server);
+	}
 	if (!server || strlen(server) == 0) {
 		server = getenv("DSQUERY");
-		if (!server || strlen(server) == 0) {
-			server = "SYBASE";
-		}
+		tdsdump_log(TDS_DBG_INFO1, "Setting 'server_name' to '%s' from $DSQUERY.\n", server);
+	}
+	if (!server || strlen(server) == 0) {
+		server = "SYBASE";
+		tdsdump_log(TDS_DBG_INFO1, "Setting 'server_name' to '%s' (compiled-in default).\n", server);
 	}
 	tds_dstr_copy(&tds_login->server_name, server);
 }
 
+void
+tds_set_server_addr(TDSLOGIN * tds_login, const char *server_addr)
+{
+	if (server_addr) {
+		tds_dstr_copy(&tds_login->server_addr, server_addr);
+	}
+}
+
 void
 tds_set_library(TDSLOGIN * tds_login, const char *library)
 {
@@ -176,49 +163,209 @@ tds_set_language(TDSLOGIN * tds_login, const char *language)
 	tds_dstr_copy(&tds_login->language, language);
 }
 
-void
-tds_set_longquery_handler(TDSLOGIN * tds_login, void (*longquery_func) (void *), void *longquery_param)
-{				/* Jeff's hack */
-	tds_login->longquery_func = longquery_func;
-	tds_login->longquery_param = longquery_param;
-}
-
 void
 tds_set_capabilities(TDSLOGIN * tds_login, unsigned char *capabilities, int size)
 {
 	memcpy(tds_login->capabilities, capabilities, size > TDS_MAX_CAPABILITY ? TDS_MAX_CAPABILITY : size);
 }
 
+struct tds_save_msg
+{
+	TDSMESSAGE msg;
+	char type;
+};
+
+struct tds_save_env
+{
+	char *oldval;
+	char *newval;
+	int type;
+};
+
+typedef struct tds_save_context
+{
+	/* must be first !!! */
+	TDSCONTEXT ctx;
+
+	unsigned num_msg;
+	struct tds_save_msg msgs[10];
+
+	unsigned num_env;
+	struct tds_save_env envs[10];
+} TDSSAVECONTEXT;
+
+static void
+tds_save(TDSSAVECONTEXT *ctx, char type, TDSMESSAGE *msg)
+{
+	struct tds_save_msg *dest_msg;
+
+	if (ctx->num_msg >= TDS_VECTOR_SIZE(ctx->msgs))
+		return;
+
+	dest_msg = &ctx->msgs[ctx->num_msg];
+	dest_msg->type = type;
+	dest_msg->msg = *msg;
+#define COPY(name) if (msg->name) dest_msg->msg.name = strdup(msg->name);
+	COPY(server);
+	COPY(message);
+	COPY(proc_name);
+	COPY(sql_state);
+#undef COPY
+	++ctx->num_msg;
+}
+
+static int
+tds_save_msg(const TDSCONTEXT *ctx, TDSSOCKET *tds, TDSMESSAGE *msg)
+{
+	tds_save((TDSSAVECONTEXT *) ctx, 0, msg);
+	return 0;
+}
+
+static int
+tds_save_err(const TDSCONTEXT *ctx, TDSSOCKET *tds, TDSMESSAGE *msg)
+{
+	tds_save((TDSSAVECONTEXT *) ctx, 1, msg);
+	return TDS_INT_CANCEL;
+}
+
+static void
+tds_save_env(TDSSOCKET * tds, int type, char *oldval, char *newval)
+{
+	TDSSAVECONTEXT *ctx;
+	struct tds_save_env *env;
+
+	if (tds->tds_ctx->msg_handler != tds_save_msg)
+		return;
+
+	ctx = (TDSSAVECONTEXT *) tds->tds_ctx;
+	if (ctx->num_env >= TDS_VECTOR_SIZE(ctx->envs))
+		return;
+
+	env = &ctx->envs[ctx->num_env];
+	env->type = type;
+	env->oldval = oldval ? strdup(oldval) : NULL;
+	env->newval = newval ? strdup(newval) : NULL;
+	++ctx->num_env;
+}
+
+static void
+init_save_context(TDSSAVECONTEXT *ctx, const TDSCONTEXT *old_ctx)
+{
+	memset(ctx, 0, sizeof(*ctx));
+	ctx->ctx.locale = old_ctx->locale;
+	ctx->ctx.msg_handler = tds_save_msg;
+	ctx->ctx.err_handler = tds_save_err;
+}
+
+static void
+replay_save_context(TDSSOCKET *tds, TDSSAVECONTEXT *ctx)
+{
+	unsigned n;
+
+	/* replay all recorded messages */
+	for (n = 0; n < ctx->num_msg; ++n)
+		if (ctx->msgs[n].type == 0) {
+			if (tds->tds_ctx->msg_handler)
+				tds->tds_ctx->msg_handler(tds->tds_ctx, tds, &ctx->msgs[n].msg);
+		} else {
+			if (tds->tds_ctx->err_handler)
+				tds->tds_ctx->err_handler(tds->tds_ctx, tds, &ctx->msgs[n].msg);
+		}
+
+	/* replay all recorded envs */
+	for (n = 0; n < ctx->num_env; ++n)
+		if (tds->env_chg_func)
+			tds->env_chg_func(tds, ctx->envs[n].type, ctx->envs[n].oldval, ctx->envs[n].newval);
+}
+
+static void
+reset_save_context(TDSSAVECONTEXT *ctx)
+{
+	unsigned n;
+
+	/* free all messages */
+	for (n = 0; n < ctx->num_msg; ++n)
+		tds_free_msg(&ctx->msgs[n].msg);
+	ctx->num_msg = 0;
+
+	/* free all envs */
+	for (n = 0; n < ctx->num_env; ++n) {
+		free(ctx->envs[n].oldval);
+		free(ctx->envs[n].newval);
+	}
+	ctx->num_env = 0;
+}
+
+static void
+free_save_context(TDSSAVECONTEXT *ctx)
+{
+	reset_save_context(ctx);
+}
+
 /**
  * Do a connection to socket
  * @param tds connection structure. This should be a non-connected connection.
- *        If the function fail connection is deleted
  * @param connection info for connection
  * @return TDS_FAIL or TDS_SUCCEED
  */
 int
 tds_connect(TDSSOCKET * tds, TDSCONNECTION * connection)
 {
-	struct sockaddr_in sin;
-
-	/* Jeff's hack - begin */
-	unsigned long ioctl_blocking = 1;
-	struct timeval selecttimeout;
-	fd_set fds;
 	int retval;
-	time_t start, now;
 	int connect_timeout = 0;
 	int db_selected = 0;
-	char version[256];
+	char version[64];
 	char *str;
 	int len;
+	
+	/*
+	 * A major version of 0 means try to guess the TDS version. 
+	 * We try them in an order that should work. 
+	 */
+	const static struct tdsver {
+		TDS_TINYINT major_version;
+		TDS_TINYINT minor_version;
+	} versions[] =
+		{ { 8, 0 }
+		, { 7, 0 }
+		, { 5, 0 }
+		, { 4, 2 }
+		};
 
-	FD_ZERO(&fds);
+	if (connection->major_version == 0) {
+		unsigned int i;
+		TDSSAVECONTEXT save_ctx;
+		const TDSCONTEXT *old_ctx = tds->tds_ctx;
+		typedef void (*env_chg_func_t) (TDSSOCKET * tds, int type, char *oldval, char *newval);
+		env_chg_func_t old_env_chg = tds->env_chg_func;
+
+		init_save_context(&save_ctx, old_ctx);
+		tds->tds_ctx = &save_ctx.ctx;
+		tds->env_chg_func = tds_save_env;
+		for (i=0; i < TDS_VECTOR_SIZE(versions); ++i) {
+			connection->major_version = versions[i].major_version;
+			connection->minor_version = versions[i].minor_version;
+			/* fprintf(stdout, "trying TDSVER %d.%d\n", connection->major_version, connection->minor_version); */
+			reset_save_context(&save_ctx);
+			retval = tds_connect(tds, connection);
+			if (TDS_SUCCEED == retval)
+				break;
+			tds_close_socket(tds);
+		}
+		tds->env_chg_func = old_env_chg;
+		tds->tds_ctx = old_ctx;
+		replay_save_context(tds, &save_ctx);
+		free_save_context(&save_ctx);
+		return retval;
+	}
+	
 
 	/*
 	 * If a dump file has been specified, start logging
 	 */
 	if (!tds_dstr_isempty(&connection->dump_file)) {
+		if (connection->debug_flags)
+			tds_debug_flags = connection->debug_flags;
 		tdsdump_open(tds_dstr_cstr(&connection->dump_file));
 	}
 
@@ -234,9 +381,11 @@ tds_connect(TDSSOCKET * tds, TDSCONNECTION * connection)
 	}
 #endif
 
-	/* set up iconv */
-	if (connection->client_charset) {
-		tds_iconv_open(tds, tds_dstr_cstr(&connection->client_charset));
+	/* set up iconv if not already initialized*/
+	if (tds->char_convs[client2ucs2]->to_wire == (iconv_t) -1) {
+		if (!tds_dstr_isempty(&connection->client_charset)) {
+			tds_iconv_open(tds, tds_dstr_cstr(&connection->client_charset));
+		}
 	}
 
 	/* specified a date format? */
@@ -248,128 +397,60 @@ tds_connect(TDSSOCKET * tds, TDSCONNECTION * connection)
 	connect_timeout = connection->connect_timeout;
 
 	/* Jeff's hack - begin */
-	tds->timeout = (connect_timeout) ? connection->query_timeout : 0;
-	tds->longquery_timeout = (connect_timeout) ? connection->longquery_timeout : 0;
-	tds->longquery_func = connection->longquery_func;
-	tds->longquery_param = connection->longquery_param;
+	tds->query_timeout = connect_timeout ? connect_timeout : connection->query_timeout;
 	/* end */
 
-	/* verify that ip_addr is not NULL */
+	/* verify that ip_addr is not empty */
 	if (tds_dstr_isempty(&connection->ip_addr)) {
-		tdsdump_log(TDS_DBG_ERROR, "IP address pointer is NULL\n");
-		if (connection->server_name) {
+		tdsdump_log(TDS_DBG_ERROR, "IP address pointer is empty\n");
+		if (!tds_dstr_isempty(&connection->server_name)) {
 			tdsdump_log(TDS_DBG_ERROR, "Server %s not found!\n", tds_dstr_cstr(&connection->server_name));
 		} else {
 			tdsdump_log(TDS_DBG_ERROR, "No server specified!\n");
 		}
-		tds_free_socket(tds);
-		return TDS_FAIL;
-	}
-	sin.sin_addr.s_addr = inet_addr(tds_dstr_cstr(&connection->ip_addr));
-	if (sin.sin_addr.s_addr == INADDR_NONE) {
-		tdsdump_log(TDS_DBG_ERROR, "inet_addr() failed, IP = %s\n", tds_dstr_cstr(&connection->ip_addr));
-		tds_free_socket(tds);
 		return TDS_FAIL;
 	}
 
-	sin.sin_family = AF_INET;
-	sin.sin_port = htons(connection->port);
+	if (!IS_TDS50(tds) && !tds_dstr_isempty(&connection->instance_name))
+		connection->port = tds7_get_instance_port(tds_dstr_cstr(&connection->ip_addr), tds_dstr_cstr(&connection->instance_name));
+
+	if (connection->port < 1) {
+		tdsdump_log(TDS_DBG_ERROR, "invalid port number\n");
+		return TDS_FAIL;
+	}
 
 	memcpy(tds->capabilities, connection->capabilities, TDS_MAX_CAPABILITY);
 
-
 	retval = tds_version(tds, version);
 	if (!retval)
 		version[0] = '\0';
 
-	tdsdump_log(TDS_DBG_INFO1, "Connecting to %s port %d, TDS %s.\n", inet_ntoa(sin.sin_addr), ntohs(sin.sin_port), version);
-	if (TDS_IS_SOCKET_INVALID(tds->s = socket(AF_INET, SOCK_STREAM, 0))) {
-		perror("socket");
-		tds_free_socket(tds);
+	if (tds_open_socket(tds, tds_dstr_cstr(&connection->ip_addr), connection->port, connect_timeout) != TDS_SUCCEED)
 		return TDS_FAIL;
-	}
+	tds_set_state(tds, TDS_IDLE);
 
-#ifdef SO_KEEPALIVE
-	len = 1;
-	setsockopt(tds->s, SOL_SOCKET, SO_KEEPALIVE, (const void *) &len, sizeof(len));
-#endif
-
-#if defined(TCP_NODELAY) && (defined(IPPROTO_TCP) || defined(SOL_TCP))
-	len = 1;
-#ifdef SOL_TCP
-	setsockopt(tds->s, SOL_TCP, TCP_NODELAY, (const void *) &len, sizeof(len));
-#else
-	setsockopt(tds->s, IPPROTO_TCP, TCP_NODELAY, (const void *) &len, sizeof(len));
-#endif
-#endif
-	
-	/* Jeff's hack *** START OF NEW CODE *** */
-	if (connect_timeout) {
-		start = time(NULL);
-		ioctl_blocking = 1;	/* ~0; //TRUE; */
-		if (IOCTLSOCKET(tds->s, FIONBIO, &ioctl_blocking) < 0) {
-			tds_free_socket(tds);
-			return TDS_FAIL;
-		}
-		retval = connect(tds->s, (struct sockaddr *) &sin, sizeof(sin));
-		if (retval < 0 && sock_errno == TDSSOCK_EINPROGRESS)
-			retval = 0;
-		if (retval < 0) {
-			perror("src/tds/login.c: tds_connect (timed)");
-			tds_free_socket(tds);
-			return TDS_FAIL;
-		}
-		/* Select on writeability for connect_timeout */
-		now = start;
-		while ((retval == 0) && ((now - start) < connect_timeout)) {
-			FD_SET(tds->s, &fds);
-			selecttimeout.tv_sec = connect_timeout - (now - start);
-			selecttimeout.tv_usec = 0;
-			retval = select(tds->s + 1, NULL, &fds, NULL, &selecttimeout);
-			if (retval < 0 && sock_errno == TDSSOCK_EINTR)
-				retval = 0;
-			now = time(NULL);
-		}
-
-		if ((now - start) >= connect_timeout) {
-			tds_client_msg(tds->tds_ctx, tds, 20009, 9, 0, 0, "Server is unavailable or does not exist.");
-			tds_free_socket(tds);
-			return TDS_FAIL;
-		}
-	} else if (connect(tds->s, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
-		char *message;
-
-		if (asprintf(&message, "src/tds/login.c: tds_connect: %s:%d", inet_ntoa(sin.sin_addr), ntohs(sin.sin_port)) >= 0) {
-			perror(message);
-			free(message);
-		}
-		tds_client_msg(tds->tds_ctx, tds, 20009, 9, 0, 0, "Server is unavailable or does not exist.");
-		tds_free_socket(tds);
-		return TDS_FAIL;
-	}
-	/* END OF NEW CODE */
-
-	if (IS_TDS7_PLUS(tds)) {
-		tds->out_flag = 0x10;
-		tds7_send_login(tds, connection);
+	if (IS_TDS8_PLUS(tds)) {
+		retval = tds8_do_login(tds, connection);
+		db_selected = 1;
+	} else if (IS_TDS7_PLUS(tds)) {
+		retval = tds7_send_login(tds, connection);
 		db_selected = 1;
 	} else {
-		tds->out_flag = 0x02;
-		tds_send_login(tds, connection);
+		tds->out_flag = TDS_LOGIN;
+		retval = tds_send_login(tds, connection);
 	}
-	if (!tds_process_login_tokens(tds)) {
+	if (retval == TDS_FAIL || !tds_process_login_tokens(tds)) {
 		tds_close_socket(tds);
-		tds_client_msg(tds->tds_ctx, tds, 20014, 9, 0, 0, "Login incorrect.");
-		tds_free_socket(tds);
+		tdserror(tds->tds_ctx, tds, TDSEFCON, 0); 	/* "Adaptive Server connection failed" */
+								/* If it's a bad login, the server will send that mesasge */
 		return TDS_FAIL;
 	}
 
 	if (connection->text_size || (!db_selected && !tds_dstr_isempty(&connection->database))) {
 		len = 64 + tds_quote_id(tds, NULL, tds_dstr_cstr(&connection->database),-1);
-		if ((str = (char *) malloc(len)) == NULL) {
-			tds_free_socket(tds);
+		if ((str = (char *) malloc(len)) == NULL)
 			return TDS_FAIL;
-		}
+
 		str[0] = 0;
 		if (connection->text_size) {
 			sprintf(str, "set textsize %d ", connection->text_size);
@@ -380,17 +461,14 @@ tds_connect(TDSSOCKET * tds, TDSCONNECTION * connection)
 		}
 		retval = tds_submit_query(tds, str);
 		free(str);
-		if (retval != TDS_SUCCEED) {
-			tds_free_socket(tds);
+		if (retval != TDS_SUCCEED)
 			return TDS_FAIL;
-		}
 
-		if (tds_process_simple_query(tds) != TDS_SUCCEED) {
-			tds_free_socket(tds);
+		if (tds_process_simple_query(tds) != TDS_SUCCEED)
 			return TDS_FAIL;
-		}
 	}
 
+	tds->query_timeout = connection->query_timeout;
 	tds->connection = NULL;
 	return TDS_SUCCEED;
 }
@@ -455,6 +533,11 @@ tds_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 	int len;
 	char blockstr[16];
 
+	if (strchr(tds_dstr_cstr(&connection->user_name), '\\') != NULL) {
+		tdsdump_log(TDS_DBG_ERROR, "NT login not support using TDS 4.x or 5.0\n");
+		return TDS_FAIL;
+	}
+
 	if (IS_TDS42(tds)) {
 		memcpy(protocol_version, "\004\002\000\000", 4);
 		memcpy(program_version, "\004\002\000\000", 4);
@@ -474,7 +557,7 @@ tds_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 	 * do this, (well...mine was a kludge actually) so here's mostly his
 	 */
 
-	tds_put_login_string(tds, tds_dstr_cstr(&connection->host_name), TDS_MAX_LOGIN_STR_SZ);	/* client host name */
+	tds_put_login_string(tds, tds_dstr_cstr(&connection->client_host_name), TDS_MAX_LOGIN_STR_SZ);	/* client host name */
 	tds_put_login_string(tds, tds_dstr_cstr(&connection->user_name), TDS_MAX_LOGIN_STR_SZ);	/* account name */
 	tds_put_login_string(tds, tds_dstr_cstr(&connection->password), TDS_MAX_LOGIN_STR_SZ);	/* account password */
 	tds_put_login_string(tds, "37876", TDS_MAX_LOGIN_STR_SZ);	/* host process */
@@ -505,7 +588,7 @@ tds_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 			len = 0;
 		tds_put_byte(tds, 0);
 		tds_put_byte(tds, len);
-		tds_put_n(tds, connection->password, len);
+		tds_put_n(tds, tds_dstr_cstr(&connection->password), len);
 		tds_put_n(tds, NULL, 253 - len);
 		tds_put_byte(tds, len + 2);
 	}
@@ -529,7 +612,7 @@ tds_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 	tds_put_login_string(tds, tds_dstr_cstr(&connection->language), TDS_MAX_LOGIN_STR_SZ);	/* language */
 	tds_put_byte(tds, connection->suppress_language);
 	tds_put_n(tds, magic5, 2);
-	tds_put_byte(tds, connection->encrypted);
+	tds_put_byte(tds, connection->encryption_level ? 1 : 0);
 	tds_put_n(tds, magic6, 10);
 
 	/* use charset nearest to client or nothing */
@@ -566,102 +649,6 @@ tds_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 	return tds_flush_packet(tds);
 }
 
-
-int
-tds7_send_auth(TDSSOCKET * tds, const unsigned char *challenge)
-{
-	int current_pos;
-	TDSANSWER answer;
-
-	/* FIXME: stuff duplicate in tds7_send_login */
-	const char *domain;
-	const char *user_name;
-	const char *p;
-	int user_name_len;
-	int host_name_len;
-	int password_len;
-	int domain_len;
-
-	TDSCONNECTION *connection = tds->connection;
-
-	/* check connection */
-	if (!connection)
-		return TDS_FAIL;
-
-	/* parse a bit of config */
-	domain = tds_dstr_cstr(&connection->default_domain);
-	user_name = tds_dstr_cstr(&connection->user_name);
-	user_name_len = user_name ? strlen(user_name) : 0;
-	host_name_len = tds_dstr_len(&connection->host_name);
-	password_len = tds_dstr_len(&connection->password);
-	domain_len = strlen(domain);
-
-	/* check override of domain */
-	if (user_name && (p = strchr(user_name, '\\')) != NULL) {
-		domain = user_name;
-		domain_len = p - user_name;
-
-		user_name = p + 1;
-		user_name_len = strlen(user_name);
-	}
-
-	tds->out_flag = 0x11;
-	tds_put_n(tds, "NTLMSSP", 8);
-	tds_put_int(tds, 3);	/* sequence 3 */
-
-	current_pos = 64 + (domain_len + user_name_len + host_name_len) * 2;
-
-	tds_put_smallint(tds, 24);	/* lan man resp length */
-	tds_put_smallint(tds, 24);	/* lan man resp length */
-	tds_put_int(tds, current_pos);	/* resp offset */
-	current_pos += 24;
-
-	tds_put_smallint(tds, 24);	/* nt resp length */
-	tds_put_smallint(tds, 24);	/* nt resp length */
-	tds_put_int(tds, current_pos);	/* nt resp offset */
-
-	current_pos = 64;
-
-	/* domain */
-	tds_put_smallint(tds, domain_len * 2);
-	tds_put_smallint(tds, domain_len * 2);
-	tds_put_int(tds, current_pos);
-	current_pos += domain_len * 2;
-
-	/* username */
-	tds_put_smallint(tds, user_name_len * 2);
-	tds_put_smallint(tds, user_name_len * 2);
-	tds_put_int(tds, current_pos);
-	current_pos += user_name_len * 2;
-
-	/* hostname */
-	tds_put_smallint(tds, host_name_len * 2);
-	tds_put_smallint(tds, host_name_len * 2);
-	tds_put_int(tds, current_pos);
-	current_pos += host_name_len * 2;
-
-	/* unknown */
-	tds_put_smallint(tds, 0);
-	tds_put_smallint(tds, 0);
-	tds_put_int(tds, current_pos + (24 * 2));
-
-	/* flags */
-	tds_put_int(tds, 0x8201);
-
-	tds_put_string(tds, domain, domain_len);
-	tds_put_string(tds, user_name, user_name_len);
-	tds_put_string(tds, tds_dstr_cstr(&connection->host_name), host_name_len);
-
-	tds_answer_challenge(tds_dstr_cstr(&connection->password), challenge, &answer);
-	tds_put_n(tds, answer.lm_resp, 24);
-	tds_put_n(tds, answer.nt_resp, 24);
-
-	/* for security reason clear structure */
-	memset(&answer, 0, sizeof(TDSANSWER));
-
-	return tds_flush_packet(tds);
-}
-
 /**
  * tds7_send_login() -- Send a TDS 7.0 login packet
  * TDS 7.0 login packet is vastly different and so gets its own function
@@ -676,6 +663,7 @@ tds7_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 
 	static const unsigned char tds7Version[] = { 0x00, 0x00, 0x00, 0x70 };
 	static const unsigned char tds8Version[] = { 0x01, 0x00, 0x00, 0x71 };
+	static const unsigned char tds9Version[] = { 0x02, 0x00, 0x09, 0x72 };
 
 	static const unsigned char connection_id[] = { 0x00, 0x00, 0x00, 0x00 };
 	unsigned char option_flag1 = 0x00;
@@ -695,58 +683,76 @@ tds7_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 	int packet_size;
 	int block_size;
 	int current_pos;
-	static const unsigned char ntlm_id[] = "NTLMSSP";
-	int domain_login = connection->try_domain_login ? 1 : 0;
 
-	const char *domain = tds_dstr_cstr(&connection->default_domain);
 	const char *user_name = tds_dstr_cstr(&connection->user_name);
-	const char *p;
 	int user_name_len = strlen(user_name);
-	int host_name_len = tds_dstr_len(&connection->host_name);
+	int host_name_len = tds_dstr_len(&connection->client_host_name);
 	int app_name_len = tds_dstr_len(&connection->app_name);
 	size_t password_len = tds_dstr_len(&connection->password);
 	int server_name_len = tds_dstr_len(&connection->server_name);
 	int library_len = tds_dstr_len(&connection->library);
 	int language_len = tds_dstr_len(&connection->language);
 	int database_len = tds_dstr_len(&connection->database);
-	int domain_len = strlen(domain);
 	int auth_len = 0;
 
+	tds->out_flag = TDS7_LOGIN;
+
+	/* discard possible previous authentication */
+	if (tds->authentication) {
+		tds->authentication->free(tds, tds->authentication);
+		tds->authentication = NULL;
+	}
+
 	/* avoid overflow limiting password */
 	if (password_len > 128)
 		password_len = 128;
 
-	/* check override of domain */
-	if (user_name && (p = strchr(user_name, '\\')) != NULL) {
-		domain = user_name;
-		domain_len = p - user_name;
+	current_pos = IS_TDS90(tds) ? 86 + 8 : 86;	/* ? */
 
-		user_name = p + 1;
-		user_name_len = strlen(user_name);
+	packet_size = current_pos + (host_name_len + app_name_len + server_name_len + library_len + language_len + database_len) * 2;
 
-		domain_login = 1;
-	}
-
-	packet_size = 86 + (host_name_len + app_name_len + server_name_len + library_len + language_len + database_len) * 2;
-	if (domain_login) {
-		auth_len = 32 + host_name_len + domain_len;
+	/* check ntlm */
+	if (strchr(user_name, '\\') != NULL) {
+		tds->authentication = tds_ntlm_get_auth(tds);
+		if (!tds->authentication)
+			return TDS_FAIL;
+		auth_len = tds->authentication->packet_len;
 		packet_size += auth_len;
+	} else if (user_name_len == 0) {
+#ifdef ENABLE_KRB5
+		/* try kerberos */
+		tds->authentication = tds_gss_get_auth(tds);
+		if (!tds->authentication)
+			return TDS_FAIL;
+		auth_len = tds->authentication->packet_len;
+		packet_size += auth_len;
+#else
+		return TDS_FAIL;
+#endif
 	} else
 		packet_size += (user_name_len + password_len) * 2;
 
+	tdsdump_log(TDS_DBG_INFO2, "quietly sending TDS 7+ login packet\n");
+	tdsdump_off();
+
 	tds_put_int(tds, packet_size);
-	if (IS_TDS80(tds)) {
+	if (IS_TDS90(tds)) {
+		tds_put_n(tds, tds9Version, 4);
+	} else if (IS_TDS8_PLUS(tds)) {
 		tds_put_n(tds, tds8Version, 4);
 	} else {
 		tds_put_n(tds, tds7Version, 4);
 	}
 
-	if (connection->block_size < 1000000)
+	if (connection->block_size < 1000000 && connection->block_size >= 512)
 		block_size = connection->block_size;
 	else
 		block_size = 4096;	/* SQL server default */
 	tds_put_int(tds, block_size);	/* desired packet size being requested by client */
 
+	if (block_size > tds->env.block_size)
+		tds_realloc_socket(tds, block_size);
+
 	tds_put_n(tds, client_progver, 4);	/* client program version ? */
 
 	tds_put_int(tds, getpid());	/* process id of this process */
@@ -759,7 +765,7 @@ tds7_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 
 	tds_put_byte(tds, option_flag1);
 
-	if (domain_login)
+	if (tds->authentication)
 		option_flag2 |= 0x80;	/* enable domain login security                     */
 
 	tds_put_byte(tds, option_flag2);
@@ -770,12 +776,11 @@ tds7_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 	tds_put_n(tds, time_zone, 4);
 	tds_put_n(tds, collation, 4);
 
-	current_pos = 86;	/* ? */
 	/* host name */
 	tds_put_smallint(tds, current_pos);
 	tds_put_smallint(tds, host_name_len);
 	current_pos += host_name_len * 2;
-	if (domain_login) {
+	if (tds->authentication) {
 		tds_put_smallint(tds, 0);
 		tds_put_smallint(tds, 0);
 		tds_put_smallint(tds, 0);
@@ -820,18 +825,24 @@ tds7_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 
 	/* authentication stuff */
 	tds_put_smallint(tds, current_pos);
-	if (domain_login) {
-		tds_put_smallint(tds, auth_len);	/* this matches numbers at end of packet */
-		current_pos += auth_len;
-	} else
-		tds_put_smallint(tds, 0);
+	tds_put_smallint(tds, auth_len);	/* this matches numbers at end of packet */
+	current_pos += auth_len;
 
 	/* unknown */
 	tds_put_smallint(tds, current_pos);
 	tds_put_smallint(tds, 0);
 
-	tds_put_string(tds, tds_dstr_cstr(&connection->host_name), host_name_len);
-	if (!domain_login) {
+	if (IS_TDS90(tds)) {
+		tds_put_smallint(tds, current_pos);
+		tds_put_smallint(tds, 0);
+
+		tds_put_int(tds, 0);
+	}
+
+	/* FIXME here we assume single byte, do not use *2 to compute bytes, convert before !!! */
+	tds_put_string(tds, tds_dstr_cstr(&connection->client_host_name), host_name_len);
+	if (!tds->authentication) {
+		const char *p;
 		TDSICONV *char_conv = tds->char_convs[client2ucs2];
 		tds_put_string(tds, tds_dstr_cstr(&connection->user_name), user_name_len);
 		p = tds_dstr_cstr(&connection->password);
@@ -854,30 +865,9 @@ tds7_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 	tds_put_string(tds, tds_dstr_cstr(&connection->language), language_len);
 	tds_put_string(tds, tds_dstr_cstr(&connection->database), database_len);
 
-	if (domain_login) {
-		/* from here to the end of the packet is the NTLMSSP authentication */
-		tds_put_n(tds, ntlm_id, 8);
-		/* sequence 1 client -> server */
-		tds_put_int(tds, 1);
-		/* flags */
-		tds_put_int(tds, 0xb201);
+	if (tds->authentication)
+		tds_put_n(tds, tds->authentication->packet, auth_len);
 
-		/* domain info */
-		tds_put_smallint(tds, domain_len);
-		tds_put_smallint(tds, domain_len);
-		tds_put_int(tds, 32 + host_name_len);
-
-		/* hostname info */
-		tds_put_smallint(tds, host_name_len);
-		tds_put_smallint(tds, host_name_len);
-		tds_put_int(tds, 32);
-
-		/* hostname and domain */
-		tds_put_n(tds, connection->host_name, host_name_len);
-		tds_put_n(tds, domain, domain_len);
-	}
-
-	tdsdump_off();
 	rc = tds_flush_packet(tds);
 	tdsdump_on();
 
@@ -893,13 +883,148 @@ unsigned char *
 tds7_crypt_pass(const unsigned char *clear_pass, int len, unsigned char *crypt_pass)
 {
 	int i;
-	unsigned char xormask = 0x5A;
-	unsigned char hi_nibble, lo_nibble;
 
-	for (i = 0; i < len; i++) {
-		lo_nibble = (clear_pass[i] ^ xormask) >> 4;
-		hi_nibble = (clear_pass[i] ^ xormask) << 4;
-		crypt_pass[i] = hi_nibble | lo_nibble;
-	}
+	for (i = 0; i < len; i++)
+		crypt_pass[i] = ((clear_pass[i] << 4) | (clear_pass[i] >> 4)) ^ 0xA5;
 	return crypt_pass;
 }
+
+static int
+tds8_do_login(TDSSOCKET * tds, TDSCONNECTION * connection)
+{
+#if !defined(HAVE_GNUTLS) && !defined(HAVE_OPENSSL)
+	/*
+	 * In case we do not have SSL enabled do not send pre-login so
+	 * if server has certificate but no force encryption login success
+	 */
+	return tds7_send_login(tds, connection);
+#else
+	int i, len, ret;
+	const char *instance_name = tds_dstr_isempty(&connection->instance_name) ? "MSSQLServer" : tds_dstr_cstr(&connection->instance_name);
+	int instance_name_len = strlen(instance_name) + 1;
+	TDS_CHAR crypt_flag;
+	unsigned int start_pos = 21;
+
+#define START_POS 21
+#define UI16BE(n) ((n) >> 8), ((n) & 0xffu)
+#define SET_UI16BE(i,n) do { buf[i] = ((n) >> 8); buf[i+1] = ((n) & 0xffu); } while(0)
+	TDS_UCHAR buf[] = {
+		/* netlib version */
+		0, UI16BE(START_POS), UI16BE(6),
+		/* encryption */
+		1, UI16BE(START_POS + 6), UI16BE(1),
+		/* instance */
+		2, UI16BE(START_POS + 6 + 1), UI16BE(0),
+		/* process id */
+		3, UI16BE(0), UI16BE(4),
+		/* ???? unknown ??? */
+		4, UI16BE(0), UI16BE(1),
+		/* end */
+		0xff
+	};
+	static const TDS_UCHAR netlib8[] = { 8, 0, 1, 0x55, 0, 0 };
+	static const TDS_UCHAR netlib9[] = { 9, 0, 0,    0, 0, 0 };
+
+	TDS_UCHAR *p;
+
+	SET_UI16BE(13, instance_name_len);
+	if (!IS_TDS90(tds)) {
+		SET_UI16BE(16, START_POS + 6 + 1 + instance_name_len);
+		buf[20] = 0xff;
+	} else {
+		start_pos += 5;
+#undef  START_POS
+#define START_POS 26
+		SET_UI16BE(1, START_POS);
+		SET_UI16BE(6, START_POS + 6);
+		SET_UI16BE(11, START_POS + 6 + 1);
+		SET_UI16BE(16, START_POS + 6 + 1 + instance_name_len);
+		SET_UI16BE(21, START_POS + 6 + 1 + instance_name_len + 4);
+	}
+
+	assert(start_pos >= 21 && start_pos <= sizeof(buf));
+	assert(buf[start_pos-1] == 0xff);
+
+	/* do prelogin */
+	tds->out_flag = TDS8_PRELOGIN;
+
+	tds_put_n(tds, buf, start_pos);
+	/* netlib version */
+	tds_put_n(tds, IS_TDS90(tds) ? netlib9 : netlib8, 6);
+	/* encryption */
+	tds_put_byte(tds, connection->encryption_level ? 1 : 0);
+	/* instance */
+	tds_put_n(tds, instance_name, instance_name_len);
+	/* pid */
+	tds_put_int(tds, getpid());
+	/* ???? unknown ??? */
+	if (IS_TDS90(tds))
+		tds_put_byte(tds, 0);
+	if (tds_flush_packet(tds) == TDS_FAIL)
+		return TDS_FAIL;
+
+	/* now process reply from server */
+	len = tds_read_packet(tds);
+	if (len <= 0 || tds->in_flag != 4)
+		return TDS_FAIL;
+	len = tds->in_len - tds->in_pos;
+
+	/* the only thing we care is flag */
+	p = tds->in_buf + tds->in_pos;
+	/* default 2, no certificate, no encryption */
+	crypt_flag = 2;
+	for (i = 0;; i += 5) {
+		TDS_UCHAR type;
+		int off, l;
+
+		if (i >= len)
+			return TDS_FAIL;
+		type = p[i];
+		if (type == 0xff)
+			break;
+		/* check packet */
+		if (i+4 >= len)
+			return TDS_FAIL;
+		off = (((int) p[i+1]) << 8) | p[i+2];
+		l = (((int) p[i+3]) << 8) | p[i+4];
+		if (off > len || (off+l) > len)
+			return TDS_FAIL;
+		if (type == 1 && l >= 1) {
+			crypt_flag = p[off];
+		}
+	}
+	/* we readed all packet */
+	tds->in_pos += len;
+	/* TODO some mssql version do not set last packet, update tds according */
+
+	tdsdump_log(TDS_DBG_INFO1, "detected flag %d\n", crypt_flag);
+
+	/* if server do not has certificate do normal login */
+	if (crypt_flag == 2) {
+		/* unless we wanted encryption and got none, then fail */
+		if (connection->encryption_level >= TDS_ENCRYPTION_REQUIRE)
+			return TDS_FAIL;
+
+		return tds7_send_login(tds, connection);
+	}
+
+	/*
+	 * if server has a certificate it require at least a crypted login
+	 * (even if data is not encrypted)
+	 */
+
+	/* here we have to do encryption ... */
+
+	if (tds_ssl_init(tds) != TDS_SUCCEED)
+		return TDS_FAIL;
+
+	ret = tds7_send_login(tds, connection);
+
+	/* if flag is 0 it means that after login server continue not encrypted */
+	if (crypt_flag == 0 || ret != TDS_SUCCEED)
+		tds_ssl_deinit(tds);
+
+	return ret;
+#endif
+}
+
