@@ -1,4 +1,5 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
+ * Copyright (C) 2004-2005  Frediano Ziglio
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -20,7 +21,9 @@
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
 
+#include <stdarg.h>
 #include <stdio.h>
+#include <assert.h>
 
 #if HAVE_STDLIB_H
 #include <stdlib.h>
@@ -37,8 +40,7 @@
 #include "tds.h"
 #include "tdsstring.h"
 
-static char software_version[] = "$Id: tdsstring.c,v 1.10 2004/02/11 16:13:19 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+TDS_RCSID(var, "$Id: tdsstring.c,v 1.19 2007/07/01 10:10:52 freddy77 Exp $");
 
 
 /**
@@ -49,27 +51,28 @@ static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
  */
 
 /* This is in a separate module because we use the pointer to discriminate allocated and not allocated */
-char tds_str_empty[] = "";
+const char tds_str_empty[1] = "";
 
-/** \addtogroup dstring
- *  \@{ 
+/**
+ * \addtogroup dstring
+ * @{ 
  */
 
 /** clear all string filling with zeroes (mainly for security reason) */
 void
 tds_dstr_zero(DSTR * s)
 {
-	if (*s)
-		memset((*s)->dstr_s, 0, strlen((*s)->dstr_s));
+	memset(s->dstr_s, 0, s->dstr_size);
 }
 
 /** free string */
 void
 tds_dstr_free(DSTR * s)
 {
-	if (*s != (DSTR) tds_str_empty)
-		free(*s);
-	*s = (DSTR) tds_str_empty;
+	if (s->dstr_s != tds_str_empty)
+		free(s->dstr_s);
+	s->dstr_size = 0;
+	s->dstr_s = (char*) tds_str_empty;
 }
 
 /**
@@ -79,21 +82,26 @@ tds_dstr_free(DSTR * s)
  * @param length length of source buffer
  * @return string copied or NULL on memory error
  */
-DSTR
+DSTR*
 tds_dstr_copyn(DSTR * s, const char *src, unsigned int length)
 {
-	if (*s != (DSTR) tds_str_empty)
-		free(*s);
+	if (s->dstr_s != tds_str_empty)
+		free(s->dstr_s);
 	if (!length) {
-		*s = (DSTR) tds_str_empty;
+		s->dstr_s = (char *) tds_str_empty;
+		s->dstr_size = 0;
 	} else {
-		*s = (struct DSTR_STRUCT *) malloc(length + 1);
-		if (!*s)
+		s->dstr_s = (char *) malloc(length + 1);
+		if (!s->dstr_s) {
+			s->dstr_s = (char *) tds_str_empty;
+			s->dstr_size = 0;
 			return NULL;
-		memcpy(*s, src, length);
-		(*s)->dstr_s[length] = 0;
+		}
+		s->dstr_size = length;
+		memcpy(s->dstr_s, src, length);
+		s->dstr_s[length] = 0;
 	}
-	return *s;
+	return s;
 }
 
 /**
@@ -104,13 +112,19 @@ tds_dstr_copyn(DSTR * s, const char *src, unsigned int length)
  * @param src    source buffer
  * @return string copied or NULL on memory error
  */
-DSTR
+DSTR*
 tds_dstr_set(DSTR * s, char *src)
 {
-	if (*s != (DSTR) tds_str_empty)
-		free(*s);
-	*s = (DSTR) src;
-	return *s;
+	size_t len = strlen(src);
+
+	if (s->dstr_s != tds_str_empty)
+		free(s->dstr_s);
+
+	s->dstr_s = len ? src : (char *) tds_str_empty;
+	if (!len)
+		free(src);
+	s->dstr_size = len;
+	return s;
 }
 
 /**
@@ -119,42 +133,86 @@ tds_dstr_set(DSTR * s, char *src)
  * @param src    source buffer
  * @return string copied or NULL on memory error
  */
-DSTR
+DSTR*
 tds_dstr_copy(DSTR * s, const char *src)
 {
-	if (*s != (DSTR) tds_str_empty)
-		free(*s);
-	if (!src[0])
-		*s = (DSTR) tds_str_empty;
-	else
-		*s = (DSTR) strdup(src);
-	return *s;
+	return tds_dstr_copyn(s, src, strlen(src));
+}
+
+DSTR*
+tds_dstr_dup(DSTR * s, const DSTR * src)
+{
+	return tds_dstr_copyn(s, src->dstr_s, src->dstr_size);
+}
+
+/**
+ * limit length of string, MUST be <= current length
+ * @param s        dynamic string
+ * @param length   new length 
+ */
+DSTR*
+tds_dstr_setlen(DSTR *s, unsigned int length)
+{
+#if ENABLE_EXTRA_CHECKS
+	assert(s->dstr_size >= length);
+#endif
+	/* test required for empty strings */
+	if (s->dstr_size >= length) {
+		s->dstr_size = length;
+		s->dstr_s[length] = 0;
+	}
+	return s;
+}
+
+/**
+ * allocate space for length char
+ * @param s        dynamic string
+ * @param length   new length 
+ * @return string allocated or NULL on memory error
+ */
+DSTR*
+tds_dstr_alloc(DSTR *s, unsigned int length)
+{
+	char *p;
+	if (s->dstr_s != tds_str_empty)
+		free(s->dstr_s);
+	p = (char *) malloc(length + 1);
+	if (!p) {
+		s->dstr_s = (char *) tds_str_empty;
+		s->dstr_size = 0;
+		return NULL;
+	}
+	s->dstr_s = p;
+	s->dstr_s[0] = 0;
+	s->dstr_size = length;
+	return s;
 }
 
 #if ENABLE_EXTRA_CHECKS
 void
 tds_dstr_init(DSTR * s)
 {
-	*s = (DSTR) tds_str_empty;
+	s->dstr_s = (char *) tds_str_empty;
+	s->dstr_size = 0;
 }
 
 int
 tds_dstr_isempty(DSTR * s)
 {
-	return (*s)->dstr_s[0] == 0;
+	return s->dstr_size == 0;
 }
 
 char *
-tds_dstr_cstr(DSTR * s)
+tds_dstr_buf(DSTR * s)
 {
-	return (*s)->dstr_s;
+	return s->dstr_s;
 }
 
 size_t
 tds_dstr_len(DSTR * s)
 {
-	return strlen((*s)->dstr_s);
+	return s->dstr_size;
 }
 #endif
 
-/** \@} */
+/** @} */
