@@ -1,5 +1,6 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
  * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004  Brian Bruns
+ * Copyright (C) 2005  Frediano Ziglio
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,6 +22,9 @@
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
 
+#include <stdarg.h>
+#include <stdio.h>
+
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
@@ -42,8 +46,6 @@
 #include <pwd.h>
 #endif
 
-#include <stdio.h>
-
 #if HAVE_STDLIB_H
 #include <stdlib.h>
 #endif /* HAVE_STDLIB_H */
@@ -68,21 +70,26 @@
 #include <netinet/in.h>
 #endif /* HAVE_NETINET_IN_H */
 
+#if HAVE_ARPA_INET_H
+#include <arpa/inet.h>
+#endif /* HAVE_ARPA_INET_H */
+
 #ifdef WIN32
 #include <shlobj.h>
 #endif
 
 #include "tds.h"
+#include "replacements.h"
 #ifdef DMALLOC
 #include <dmalloc.h>
 #endif
 
-static char software_version[] = "$Id: threadsafe.c,v 1.35 2004/05/03 20:02:47 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+TDS_RCSID(var, "$Id: threadsafe.c,v 1.46 2007/05/30 07:56:38 freddy77 Exp $");
 
 char *
 tds_timestamp_str(char *str, int maxlen)
 {
+#if !defined(WIN32)
 	struct tm *tm;
 	time_t t;
 
@@ -90,7 +97,7 @@ tds_timestamp_str(char *str, int maxlen)
 	struct timeval tv;
 	char usecs[10];
 #endif
-#ifdef _REENTRANT
+#if defined(_REENTRANT)
 	struct tm res;
 #endif
 
@@ -105,7 +112,7 @@ tds_timestamp_str(char *str, int maxlen)
 	time(&t);
 #endif
 
-#if defined(_REENTRANT) && !defined(WIN32)
+#if defined(_REENTRANT)
 #if HAVE_FUNC_LOCALTIME_R_TM
 	tm = localtime_r(&t, &res);
 #else
@@ -125,6 +132,14 @@ tds_timestamp_str(char *str, int maxlen)
 	strcat(str, usecs);
 #endif
 
+#else /* WIN32 */
+	SYSTEMTIME st;
+
+	GetLocalTime(&st);
+	_snprintf(str, maxlen - 1, "%02u:%02u:%02u.%03u", st.wHour, st.wMinute, st.wSecond, st.wMilliseconds);
+	str[maxlen - 1] = 0;
+#endif
+
 	return str;
 }
 
@@ -133,12 +148,19 @@ tds_timestamp_str(char *str, int maxlen)
  * just assume the standard BSD netdb interface is reentrant and use it.
  */
 #ifndef _REENTRANT
-# ifndef NETDB_REENTRANT
-#  define NETDB_REENTRANT 1
-# endif	/* NETDB_REENTRANT */
+# undef NETDB_REENTRANT
+# define NETDB_REENTRANT 1
 #endif /* _REENTRANT */
 
-#if !defined(NETDB_REENTRANT) && (defined(HAVE_GETIPNODEBYNAME) || defined(HAVE_GETIPNODEBYADDR))
+
+#if defined(NETDB_REENTRANT)
+struct hostent *
+tds_gethostbyname_r(const char *servername, struct hostent *result, char *buffer, int buflen, int *h_errnop)
+{
+	return gethostbyname(servername);
+}
+
+#elif defined(HAVE_GETIPNODEBYNAME) || defined(HAVE_GETIPNODEBYADDR)
 /**
  * Copy a hostent structure to an allocated buffer
  * @return 0 on success, -1 otherwise
@@ -218,16 +240,10 @@ tds_copy_hostent(struct hostent *he, struct hostent *result, char *buffer, int b
 	}
 	return 0;
 }
-#endif
 
 struct hostent *
 tds_gethostbyname_r(const char *servername, struct hostent *result, char *buffer, int buflen, int *h_errnop)
 {
-#if defined(NETDB_REENTRANT)
-	return gethostbyname(servername);
-
-/* we have a better replacements */
-#elif defined(HAVE_GETIPNODEBYNAME)
 	struct hostent *he = getipnodebyname(servername, AF_INET, 0, h_errnop);
 
 	if (!he)
@@ -241,17 +257,29 @@ tds_gethostbyname_r(const char *servername, struct hostent *result, char *buffer
 	}
 	freehostent(he);
 	return result;
+}
 
 #elif defined(HAVE_FUNC_GETHOSTBYNAME_R_6)
+struct hostent *
+tds_gethostbyname_r(const char *servername, struct hostent *result, char *buffer, int buflen, int *h_errnop)
+{
 	if (gethostbyname_r(servername, result, buffer, buflen, &result, h_errnop))
 		return NULL;
 	return result;
+}
 
 #elif defined(HAVE_FUNC_GETHOSTBYNAME_R_5)
+struct hostent *
+tds_gethostbyname_r(const char *servername, struct hostent *result, char *buffer, int buflen, int *h_errnop)
+{
 	result = gethostbyname_r(servername, result, buffer, buflen, h_errnop);
 	return result;
+}
 
 #elif defined(HAVE_FUNC_GETHOSTBYNAME_R_3)
+struct hostent *
+tds_gethostbyname_r(const char *servername, struct hostent *result, char *buffer, int buflen, int *h_errnop)
+{
 	struct hostent_data *data = (struct hostent_data *) buffer;
 
 	memset(buffer, 0, buflen);
@@ -260,12 +288,113 @@ tds_gethostbyname_r(const char *servername, struct hostent *result, char *buffer
 		result = NULL;
 	}
 	return result;
+}
+
+#elif defined(HAVE_GETADDRINFO) && \
+	(!defined(__FreeBSD__) && !defined(__NetBSD__) && !defined(__OpenBSD__) && \
+	!defined(__bsdi__) && !defined(__DragonFly__))
+
+static int
+tds_addrinfo_to_hostent(struct addrinfo *ai, struct hostent *result, char *buffer, int buflen)
+{
+#define CHECK_BUF(len) \
+	if (p + sizeof(struct hostent) - buffer > buflen) return -1;
+#define ALIGN_P do { p += TDS_ALIGN_SIZE - 1; p -= (p-buffer) % TDS_ALIGN_SIZE; } while(0)
+
+	int n;
+	char *p = buffer;
+	int len;
+	char **addresses;
+	struct addrinfo *curr_ai;
+
+	memset(result, 0, sizeof(struct hostent));
+	result->h_addrtype = sizeof(struct sockaddr_in);
+
+	/* count addresses */
+	for (n = 0, curr_ai = ai; curr_ai; curr_ai = curr_ai->ai_next) {
+		if (curr_ai->ai_family != PF_INET)
+			continue;
+		++n;
+	}
+
+	/* copy addresses */
+	addresses = (char **) p;
+	result->h_addr_list = addresses;
+	result->h_length = sizeof(struct in_addr);
+	len = sizeof(char *) * (n + 1);
+	CHECK_BUF(len);
+	p += len;
+	ALIGN_P;
+	for (n = 0, curr_ai = ai; curr_ai; curr_ai = curr_ai->ai_next) {
+		if (curr_ai->ai_family != PF_INET)
+			continue;
+		addresses[n++] = p;
+
+		len = sizeof(struct in_addr);
+		CHECK_BUF(len);
+		memcpy(p, &((struct sockaddr_in *) curr_ai->ai_addr)->sin_addr, len);
+		p += len;
+		ALIGN_P;
+	}
+	addresses[n] = NULL;
+
+	/* copy name */
+	if (ai->ai_canonname) {
+		n = strlen(ai->ai_canonname) + 1;
+		result->h_name = p;
+		CHECK_BUF(n);
+		memcpy(p, ai->ai_canonname, n);
+		p += n;
+		ALIGN_P;
+	}
+	return 0;
+}
+
+struct hostent *
+tds_gethostbyname_r(const char *servername, struct hostent *result, char *buffer, int buflen, int *h_errnop)
+{
+	struct addrinfo hints, *res;
+
+	memset(&hints, 0, sizeof(hints));
+	hints.ai_family = PF_INET;
+	hints.ai_socktype = SOCK_STREAM;
+	hints.ai_flags = 0;
+	hints.ai_protocol = IPPROTO_TCP;
+
+	res = NULL;
+	/* default error */
+	if (h_errnop)
+		*h_errnop = HOST_NOT_FOUND;
+	if (getaddrinfo(servername, NULL, &hints, &res))
+		return NULL;
+	if (res->ai_family != PF_INET || !res->ai_addr) {
+		freeaddrinfo(res);
+		return NULL;
+	}
+	if (tds_addrinfo_to_hostent(res, result, buffer, buflen)) {
+		errno = ENOMEM;
+		if (h_errnop)
+			*h_errnop = NETDB_INTERNAL;
+		freeaddrinfo(res);
+		return NULL;
+	}
+	freeaddrinfo(res);
+	return result;
+}
+
+#elif defined(TDS_NO_THREADSAFE)
+struct hostent *
+tds_gethostbyname_r(const char *servername, struct hostent *result, char *buffer, int buflen, int *h_errnop)
+{
+	return gethostbyname(servername);
+}
 
 #else
 #error gethostbyname_r style unknown
 #endif
-}
 
+/* not used by FreeTDS, uncomment if needed */
+#ifdef ENABLE_DEVELOPING
 struct hostent *
 tds_gethostbyaddr_r(const char *addr, int len, int type, struct hostent *result, char *buffer, int buflen, int *h_errnop)
 {
@@ -306,10 +435,27 @@ tds_gethostbyaddr_r(const char *addr, int len, int type, struct hostent *result,
 	}
 	return result;
 
+#elif defined(TDS_NO_THREADSAFE)
+	return gethostbyaddr(addr, len, type);
+
 #else
 #error gethostbyaddr_r style unknown
 #endif
 }
+#endif
+
+const char *
+tds_inet_ntoa_r(struct in_addr iaddr, char *ip, size_t len)
+{
+#if defined(AF_INET) && HAVE_INET_NTOP
+	inet_ntop(AF_INET, &iaddr, ip, len);
+#elif HAVE_INET_NTOA_R
+	inet_ntoa_r(iaddr, ip, len);
+#else
+	tds_strlcpy(ip, inet_ntoa(iaddr), len);
+#endif
+	return ip;
+}
 
 struct servent *
 tds_getservbyname_r(const char *name, const char *proto, struct servent *result, char *buffer, int buflen)
@@ -340,6 +486,7 @@ tds_getservbyname_r(const char *name, const char *proto, struct servent *result,
 	hints.ai_family = PF_INET;
 	hints.ai_socktype = SOCK_STREAM;
 	hints.ai_flags = AI_PASSIVE;
+	res = NULL;
 	if (getaddrinfo(NULL, name, &hints, &res))
 		return NULL;
 	if (res->ai_family != PF_INET || !res->ai_addr) {
@@ -351,6 +498,8 @@ tds_getservbyname_r(const char *name, const char *proto, struct servent *result,
 	freeaddrinfo(res);
 	return result;
 
+#elif defined(TDS_NO_THREADSAFE)
+	return getservbyname(name, proto);
 #else
 #error getservbyname_r style unknown
 #endif
