@@ -18,7 +18,7 @@
  */
 #include "common.h"
 
-static char software_version[] = "$Id: dynamic1.c,v 1.11 2004/02/03 19:28:12 jklowden Exp $";
+static char software_version[] = "$Id: dynamic1.c,v 1.18 2007/09/13 06:58:48 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 static int discard_result(TDSSOCKET * tds);
@@ -47,26 +47,25 @@ test(TDSSOCKET * tds, TDSDYNAMIC * dyn, TDS_INT n, const char *s)
 	curcol->column_type = SYBINTN;
 	curcol->on_server.column_type = SYBINTN;
 	curcol->column_size = sizeof(TDS_INT);
+	curcol->on_server.column_size = sizeof(TDS_INT);
 	curcol->column_varint_size = 1;
 	curcol->column_cur_size = sizeof(TDS_INT);
 
 	/* TODO test error */
-	tds_alloc_param_row(params, curcol);
-	memcpy(&params->current_row[curcol->column_offset], &n, sizeof(n));
+	tds_alloc_param_data(curcol);
+	memcpy(curcol->column_data, &n, sizeof(n));
 
 	if (!(params = tds_alloc_param_result(dyn->params)))
 		fatal_error("out of memory!");
 	dyn->params = params;
 
 	curcol = params->columns[1];
-	curcol->column_type = SYBVARCHAR;
-	curcol->on_server.column_type = SYBVARCHAR;
+	tds_set_param_type(tds, curcol, SYBVARCHAR);
 	curcol->column_size = 40;
-	curcol->column_varint_size = 1;
 	curcol->column_cur_size = len;
 
-	tds_alloc_param_row(params, curcol);
-	memcpy(&params->current_row[curcol->column_offset], s, len);
+	tds_alloc_param_data(curcol);
+	memcpy(curcol->column_data, s, len);
 
 	if (tds_submit_execute(tds, dyn) != TDS_SUCCEED)
 		fatal_error("tds_submit_execute() error");
@@ -145,7 +144,7 @@ discard_result(TDSSOCKET * tds)
 	int rc;
 	int result_type;
 
-	while ((rc = tds_process_result_tokens(tds, &result_type, NULL)) == TDS_SUCCEED) {
+	while ((rc = tds_process_tokens(tds, &result_type, NULL, TDS_TOKEN_RESULTS)) == TDS_SUCCEED) {
 
 		switch (result_type) {
 		case TDS_DONE_RESULT:
@@ -161,10 +160,10 @@ discard_result(TDSSOCKET * tds)
 		}
 	}
 	if (rc == TDS_FAIL) {
-		fprintf(stderr, "tds_process_result_tokens() returned TDS_FAIL\n");
+		fprintf(stderr, "tds_process_tokens() returned TDS_FAIL\n");
 		return TDS_FAIL;
 	} else if (rc != TDS_NO_MORE_RESULTS) {
-		fprintf(stderr, "tds_process_result_tokens() unexpected return\n");
+		fprintf(stderr, "tds_process_tokens() unexpected return\n");
 		return TDS_FAIL;
 	}
 
