@@ -20,7 +20,7 @@
 #include "common.h"
 #include <tdsconvert.h>
 
-static char software_version[] = "$Id: t0007.c,v 1.12 2003/04/21 16:06:11 freddy77 Exp $";
+static char software_version[] = "$Id: t0007.c,v 1.14.2.1 2008/02/06 08:49:44 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 static TDSCONTEXT ctx;
@@ -39,6 +39,7 @@ test0(const char *src, int len, int dsttype, const char *result)
 	if (res < 0)
 		strcpy(buf, "error");
 	else {
+		buf[0] = 0;
 		switch (dsttype) {
 		case SYBINT1:
 			sprintf(buf, "%d", cr.ti);
@@ -49,6 +50,9 @@ test0(const char *src, int len, int dsttype, const char *result)
 		case SYBINT4:
 			sprintf(buf, "%d", cr.i);
 			break;
+		case SYBINT8:
+			sprintf(buf, "0x%08x%08x", (unsigned int) ((cr.bi >> 32) & 0xfffffffflu), (unsigned int) (cr.bi & 0xfffffffflu));
+			break;
 		case SYBUNIQUE:
 			sprintf(buf, "%08X-%04X-%04X-%02X%02X%02X%02X"
 				"%02X%02X%02X%02X",
@@ -63,11 +67,16 @@ test0(const char *src, int len, int dsttype, const char *result)
 				sprintf(strchr(buf, 0), " %02X", (TDS_UCHAR) cr.ib[i]);
 			free(cr.ib);
 			break;
+		case SYBDATETIME:
+			sprintf(buf, "%ld %ld", (long int) cr.dt.dtdays, (long int) cr.dt.dttime);
+			break;
 		}
 	}
 	printf("%s\n", buf);
-	if (strcmp(buf, result) != 0)
+	if (strcmp(buf, result) != 0) {
+		fprintf(stderr, "Expected %s\n", result);
 		exit(1);
+	}
 }
 
 void
@@ -87,9 +96,42 @@ main(int argc, char **argv)
 	test("123", SYBINT1, "123");
 	test("  -    1234   ", SYBINT2, "-1234");
 	test("  -    1234   a", SYBINT2, "error");
+	test("", SYBINT4, "0");
+	test("    ", SYBINT4, "0");
+	test("    123", SYBINT4, "123");
+	test("    123    ", SYBINT4, "123");
+	test("  +  123  ", SYBINT4, "123");
+	test("+", SYBINT4, "error");
+	test("   +", SYBINT4, "error");
+	test("+   ", SYBINT4, "error");
+	test("   +   ", SYBINT4, "error");
+	test("-", SYBINT4, "error");
+	test("   -", SYBINT4, "error");
+	test("-   ", SYBINT4, "error");
+	test("   -   ", SYBINT4, "error");
+
+	test("  -    1234   ", SYBINT8, "0xfffffffffffffb2e");
+	test("  -    1234   a", SYBINT8, "error");
+	test("", SYBINT8, "0x0000000000000000");
+	test("    ", SYBINT8, "0x0000000000000000");
+	test("    123", SYBINT8, "0x000000000000007b");
+	test("    123    ", SYBINT8, "0x000000000000007b");
+	test("  +  123  ", SYBINT8, "0x000000000000007b");
+	test("+", SYBINT8, "error");
+	test("   +", SYBINT8, "error");
+	test("+   ", SYBINT8, "error");
+	test("   +   ", SYBINT8, "error");
+	test("-", SYBINT8, "error");
+	test("   -", SYBINT8, "error");
+	test("-   ", SYBINT8, "error");
+	test("   -   ", SYBINT8, "error");
 
 	/* test for overflow */
 	printf("overflow checks...\n");
+	test("9223372036854775807", SYBINT8, "0x7fffffffffffffff");
+	test("9223372036854775808", SYBINT8, "error");
+	test("-9223372036854775808", SYBINT8, "0x8000000000000000");
+	test("-9223372036854775809", SYBINT8, "error");
 	test("2147483647", SYBINT4, "2147483647");
 	test("2147483648", SYBINT4, "error");
 	test("-2147483648", SYBINT4, "-2147483648");
@@ -103,10 +145,11 @@ main(int argc, char **argv)
 	test("0", SYBINT1, "0");
 	test("-1", SYBINT1, "error");
 
-	/* test overflow on very big numbers 
+	/*
+	 * test overflow on very big numbers 
 	 * i use increment of 10^9 to be sure lower 32bit be correct
 	 * in a case
-	 * */
+	 */
 	printf("overflow on big number checks...\n");
 	test("62147483647", SYBINT4, "error");
 	test("63147483647", SYBINT4, "error");
@@ -115,6 +158,7 @@ main(int argc, char **argv)
 
 	/* test not terminated string */
 	test0("1234", 2, SYBINT4, "12");
+	test0("123456", 4, SYBINT8, "0x00000000000004d2");
 
 	/* some test for unique */
 	printf("unique type...\n");
@@ -136,5 +180,16 @@ main(int argc, char **argv)
 	test("0x100", SYBBINARY, "len=2 01 00");
 	test("0x1", SYBBINARY, "len=1 01");
 
+	test("Jan 01 2006", SYBDATETIME, "38716 0");
+	test("January 01 2006", SYBDATETIME, "38716 0");
+	test("March 05 2005", SYBDATETIME, "38414 0");
+	test("may 13 2001", SYBDATETIME, "37022 0");
+
+	test("02 Jan 2006", SYBDATETIME, "38717 0");
+	test("2 Jan 2006", SYBDATETIME, "38717 0");
+	test("02Jan2006", SYBDATETIME, "38717 0");
+	test("20060102", SYBDATETIME, "38717 0");
+	test("060102", SYBDATETIME, "38717 0");
+
 	return 0;
 }
