@@ -35,8 +35,7 @@
 #include "tds.h"
 #include "replacements.h"
 
-static char software_version[] = "$Id: vstrbuild.c,v 1.12 2004/08/31 14:04:09 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+TDS_RCSID(var, "$Id: vstrbuild.c,v 1.14 2007/10/16 15:12:23 freddy77 Exp $");
 
 struct string_linked_list
 {
@@ -200,9 +199,7 @@ tds_vstrbuild(char *buffer, int buflen, int *resultlen, char *text, int textlen,
 	rc = TDS_SUCCEED;
 
       out:
-	if (string_array != NULL) {
-		free(string_array);
-	}
+	free(string_array);
 	for (item = head; item != NULL; item = head) {
 		head = head->next;
 		free(item);
